@@ -1,8 +1,8 @@
 //
 // File:        BabelConfiguration.java
 // Package:     gov.llnl.babel
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:    @(#) $Revision: 6212 $
+// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
 // Description: Store configuration info shared by parsers and backend
 // 
 // Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -33,14 +33,19 @@ package gov.llnl.babel;
 
 import gov.llnl.babel.symbols.RegexMatch;
 import gov.llnl.babel.symbols.SymbolID;
-import java.util.List;
+
 import java.util.LinkedList;
+import java.util.List;
 
 /**
  * This class provides access to configuration information (including user
  * options) required by the parsers and backend.
+ *
+ * This class originally envisaged as a singleton. However, it makes sense
+ * to have several of them when trying to do several runs in a single
+ * command line.
  */
-public class BabelConfiguration 
+public class BabelConfiguration implements Cloneable
 {
   /*
    * The basic configuration literals.
@@ -50,49 +55,75 @@ public class BabelConfiguration
   private static final String BASE_CLASSINFOI= "sidl.ClassInfoI";
   private static final String BASE_EXCEPTION = "sidl.BaseException";
   private static final String BASE_INTERFACE = "sidl.BaseInterface";
+  private static final String RUNTIME_EXCEPTION = "sidl.RuntimeException";
+  private static final String MEMORYALLOCATION_EXCEPTION = "sidl.MemAllocException";
+  private static final String NOTIMPLEMENTED_EXCEPTION = "sidl.NotImplementedException";
+  private static final String LANGSPECIFIC_EXCEPTION = "sidl.LangSpecificException";
   private static final String MAKEFILE_NAME  = "babel.make";
   private static final int    MAXIMUM_ARRAY  = 7;
   private static final String SIDL_EXCEPTION = "sidl.SIDLException";
+  private static final String SIDL_RMI_TICKET = "sidl.rmi.Ticket";
+  private static final String SIDL_SERIALIZABLE = "sidl.io.Serializable";
 
+  
   public  static final String FUND_EXCEPTION = "sidl.BaseInterface";
-  public  static final String PRE_EXCEPTION  = "sidl.PreconditionViolation";
-  public  static final String POST_EXCEPTION = "sidl.PostconditionViolation";
-  public  static final String INV_EXCEPTION  = "sidl.InvariantViolation";
+  public  static final String PRE_EXCEPTION  = "sidl.PreViolation";
+  public  static final String POST_EXCEPTION = "sidl.PostViolation";
+  public  static final String INV_EXCEPTION  = "sidl.InvViolation";
 
+
+  public static final String OPTIONAL_RMI = "WITH_RMI";
   /*
    * Overall level of assertion statistics support.
    */
   public  static final String NO_CHECKING         = "0";
-  public  static final String MIN_CHECKING_STATS  = "1";
-  public  static final String FULL_CHECKING_STATS = "2";
+  public  static final String SUPPORT_CHECKING    = "1";
+  public  static final String ADVANCED_CHECKING   = "2";
+
+  /**
+   * Static options -- not dependent on command line settings
+   * or represent global settings.
+   */
+  private boolean d_multi_mode             = false;
+  private StringBuffer d_repository_path   = new StringBuffer ("");
+  private List    d_included_symbols       = new LinkedList();
 
   /*
    * User options -- extractable from the command line via UserOptions.
+   * Each of these must be copied in clone().
    */
-  private String  d_generate_assertions    = FULL_CHECKING_STATS;
-  private boolean d_generate_client        = false;
-  private boolean d_generate_interceptors  = false;
-  private boolean d_generate_server        = false;
-  private boolean d_generate_stdlib        = false;
-  private boolean d_generate_text          = false;
+  private String  d_assertion_level        = SUPPORT_CHECKING;
+  private boolean d_generate_hooks         = false;
   private boolean d_package_subdirs        = false;
   private boolean d_language_subdir        = false;
   private boolean d_glue_subdirs           = false;
   private boolean d_checkNullIOR           = false;
-  private boolean d_parser_check           = false;
-  private boolean d_suppress_timestamps    = false;
+  private boolean d_suppress_timestamps    = true;
   private boolean d_comment_local_only     = false;
   private boolean d_short_file_names       = false;
-  private String  d_target_language        = "";
   private String  d_output_directory       = ".";
   private String  d_vpath_directory        = null;
-  private StringBuffer d_repository_path   = new StringBuffer ("");
-  private List    d_excluded_symbols       = new LinkedList();
-  private boolean d_exclude_external       = false;
-  private List    d_included_symbols       = new LinkedList();
+  private LinkedList d_excluded_symbols       = new LinkedList();
+  private boolean d_exclude_external       = true;
   private String  d_make_prefix            = "";
   private String  d_makefile               = MAKEFILE_NAME;
-  private boolean d_sidl_file_input        = false;
+  private boolean d_suppress_ior           = false;
+  private boolean d_suppress_stub          = false;
+  private boolean d_rename_splicers        = false;
+  private boolean d_cca_mode               = false; 
+  private boolean d_protect_last_time_mod  = true;
+
+  /*
+   * State variables that are deliberately not copied during clone.
+   */
+  private boolean d_parser_check           = false;
+  private boolean d_verbose                = false;
+  private boolean d_generate_stdlib        = false;
+  private boolean d_generate_client        = false;
+  private boolean d_generate_text          = false;
+  private boolean d_generate_server        = false;
+  private String  d_target_language        = null;
+  private boolean d_has_bind_c             = true;
 
   /**
    *  These are the values returned by calling the _type function 
@@ -112,25 +143,45 @@ public class BabelConfiguration
   public static final int sidl_interface_array = 11;
     /* an array of sidl.BaseInterface's */
 
-  /*
-   * My handle.
-   */
-  private static BabelConfiguration s_my_instance = null;
+  /** This array corrosponds to the enumeration found in sidlArray.h*/
+  private static final String s_ensureOrderConstant[] = {
+    "sidl_general_order",
+    "sidl_column_major_order",
+    "sidl_row_major_order"
+  };
+
 
   /**
    * Since this is a singleton class, its constructor is protected.
    */
-  protected BabelConfiguration() {
+  public BabelConfiguration() {
   }
 
   /**
-   * Return the singleton instance of this class.
+   * Make a copy of the configuration.
    */
-  public static BabelConfiguration getInstance() {
-    if (s_my_instance == null) {
-      s_my_instance = new BabelConfiguration();
-    }
-    return s_my_instance;
+  public Object clone()
+  {
+    BabelConfiguration result = new BabelConfiguration();
+    result.d_assertion_level = this.d_assertion_level;
+    result.d_generate_hooks = this.d_generate_hooks;
+    result.d_package_subdirs = this.d_package_subdirs;
+    result.d_language_subdir = this.d_language_subdir;
+    result.d_glue_subdirs = this.d_glue_subdirs;
+    result.d_checkNullIOR = this.d_checkNullIOR;
+    result.d_suppress_timestamps = this.d_suppress_timestamps;
+    result.d_comment_local_only = this.d_comment_local_only;
+    result.d_short_file_names = this.d_short_file_names;
+    result.d_output_directory = this.d_output_directory;
+    result.d_vpath_directory = this.d_vpath_directory;
+    result.d_excluded_symbols = (LinkedList)this.d_excluded_symbols.clone();
+    result.d_exclude_external = this.d_exclude_external;
+    result.d_make_prefix = this.d_make_prefix;
+    result.d_makefile = this.d_makefile;
+    result.d_suppress_ior = this.d_suppress_ior;
+    result.d_suppress_stub = this.d_suppress_stub;
+    result.d_rename_splicers = this.d_rename_splicers;
+    return result;
   }
 
   public static String arrayType(int type) {
@@ -207,6 +258,25 @@ public class BabelConfiguration
     return SIDL_EXCEPTION;
   }
 
+  /**
+   * Return the full name of the base exception class.
+   */
+  public static String getOptionalRmiMacro() {
+    return OPTIONAL_RMI;
+  }
+
+  /**
+   * The full name of the exception type implicit in all methods.
+   */
+  public static String getRuntimeException() { 
+    return RUNTIME_EXCEPTION;
+  }
+
+  public static String getLangSpecific(){ return LANGSPECIFIC_EXCEPTION; }
+  public static String getNotImplemented(){ return NOTIMPLEMENTED_EXCEPTION; }
+  public static String getMemoryAllocationException(){ return MEMORYALLOCATION_EXCEPTION; }
+
+
   /**
    * The full name of the fundamental exception type.
    */
@@ -214,6 +284,21 @@ public class BabelConfiguration
     return BASE_INTERFACE;
   }
 
+  /**
+   * 
+   */
+  public static String getRMITicket() { 
+    return SIDL_RMI_TICKET;
+  }
+
+  /**
+   * 
+   */
+  public static String getSerializableType() { 
+    return SIDL_SERIALIZABLE;
+  }
+
+  
   /**
    * The full name of the file associated with generated makefile fragments.
    */
@@ -228,6 +313,15 @@ public class BabelConfiguration
     return MAXIMUM_ARRAY;
   }
 
+  public void setMultiMode(boolean multi_mode) {
+    d_multi_mode = multi_mode;
+  }
+
+  public boolean getMultiMode()
+  {
+    return d_multi_mode;
+  }
+
   /**
    * Return whether the specified symbol belongs to the SIDL namespace.
    */
@@ -244,27 +338,96 @@ public class BabelConfiguration
     return is_sidl;
   }
 
+  /**
+   * Set the IOR suppression to <code>true</code> or <code>false</code>.
+   */
+  public void setSuppressIOR(boolean value)
+  {
+    d_suppress_ior = value;
+  }
+
+  public boolean getSuppressIOR()
+  {
+    return d_suppress_ior;
+  }
+
+  /** Set the stub suppression to <code>true</code> or <code>false</code>.
+   */
+  public void setSuppressStub(boolean value)
+  {
+    d_suppress_stub = value;
+  }
+
+  public boolean getSuppressStub()
+  {
+    return d_suppress_stub;
+  }
+
+  /** Set the rename splicers flag to <code>true</code> or <code>false</code>.
+   */
+  public void setRenameSplicers(boolean value)
+  {
+    d_rename_splicers = value;
+  }
+
+  public boolean getRenameSplicers()
+  {
+    return d_rename_splicers;
+  }
+
+  /** Set the rename splicers flag to <code>true</code> or <code>false</code>.
+   */
+  public void setCCAMode(boolean value)
+  {
+    d_cca_mode = value;
+  }
+
+  public boolean getProtectLastTimeModified()
+  {
+    return d_protect_last_time_mod;
+  }
+
+  public void setProtectLastTimeModified(boolean value)
+  {
+    d_protect_last_time_mod = value;
+  }
+
+  public boolean getCCAMode()
+  {
+    return d_cca_mode; 
+  }
+
   /**
    * Set the level of the assertion generation user option.
    */
-  public void setGenerateAssertions(String lvl) {
-    d_generate_assertions = lvl;
+  public void setAssertionLevel(String lvl) {
+    d_assertion_level = lvl;
   }
 
   /**
    * Return the level of assertion checking.
    */
   public String getAssertionLevel() {
-    return d_generate_assertions;
+    return d_assertion_level;
   }
   
   /**
    * Return whether or not user has indicated assertions should be generated.
    */
   public boolean generateAssertions() {
-    return (d_generate_assertions != NO_CHECKING);
+    return (!d_assertion_level.equals(NO_CHECKING));
   }
   
+  /**
+   *
+   */
+  public static String getArrayOrderName(int i) {
+    if(i >=0 && i<= 3) {
+      return s_ensureOrderConstant[i];
+    }
+    return null;
+  }
+
   /**
    * Set the value of the client generation user option.
    */
@@ -280,17 +443,17 @@ public class BabelConfiguration
   }
   
   /**
-   * Set the value of the interceptor generation user option.
+   * Set the value of the pre/post method hooks generation user option.
    */
-  public void setGenerateInterceptors(boolean generate) {
-    d_generate_interceptors = generate;
+  public void setGenerateHooks(boolean generate) {
+    d_generate_hooks = generate;
   }
 
   /**
-   * Return the value of the interceptor generation user option.
+   * Return the value of the pre/post method hooks generation user option.
    */
-  public boolean generateInterceptors() {
-    return d_generate_interceptors;
+  public boolean generateHooks() {
+    return d_generate_hooks;
   }
   
   /**
@@ -349,6 +512,20 @@ public class BabelConfiguration
     return d_parser_check;
   }
 
+  /**
+   * Set the value of the parser check user option.
+   */
+  public void setVerbose(boolean verbose) { 
+    d_verbose = verbose;
+  }
+
+  /**
+   * Return the value of the parser check user option.
+   */
+  public boolean isVerbose() {
+    return d_verbose;
+  }
+  
   /**
    * Set the value of the comment local methods only option.
    */
@@ -486,25 +663,49 @@ public class BabelConfiguration
     d_target_language = lang;
   }
 
-  /**
-   * Return the value of the target language user option.
-   */
   public String getTargetLanguage() {
     return d_target_language;
   }
-  
+
+  /**
+   * Set if this Fortran compiler supports the iso_c_binding module
+   */
+  public void setHasBindC(boolean bindc) {
+    d_has_bind_c = bindc;
+  }
+
+  public boolean getHasBindC() {
+    return d_has_bind_c;
+  }
+
   /**
    * Adds a new entry to the repository path.
    */
   public void addToRepositoryPath(String new_path) {
-    if ( d_repository_path != null ) {
-      if ( d_repository_path.length() == 0 ) {
-        d_repository_path.insert (0, new_path);
-      } else {
-        d_repository_path.insert (0, new_path + ";");
-      }
+    if ( d_repository_path.length() == 0 ) {
+      d_repository_path.insert (0, new_path);
     } else {
-      d_repository_path = new StringBuffer(new_path);
+      d_repository_path.insert (0, new_path + ";");
+    }
+  }
+
+  public void removeFromRepositoryPath(String path) {
+    String curPath = getRepositoryPath();
+    int index;
+    if (curPath.equals(path)) {
+      d_repository_path = new StringBuffer();
+    }
+    else if (curPath.endsWith(";" + path)) {
+      d_repository_path = new 
+        StringBuffer(curPath.substring(0, curPath.length() - path.length()-1));
+    }
+    else if (curPath.startsWith(path + ";")) {
+      d_repository_path = new StringBuffer(curPath.substring(path.length()+1));
+    }
+    else if ((index = curPath.indexOf(";" + path + ";")) >= 0) {
+      d_repository_path = new StringBuffer
+        (curPath.substring(0, index+1) +
+         curPath.substring(index+2+path.length()));
     }
   }
 
@@ -577,18 +778,4 @@ public class BabelConfiguration
     d_make_prefix = ((prefix != null) ? prefix : "");
     d_makefile = d_make_prefix + MAKEFILE_NAME;
   }
-
-  /**
-   * Return the value of the sidl file input flag.
-   */
-  public boolean getSIDLFileInput() {
-    return d_sidl_file_input;
-  }
-
-  /**
-   * Set the value of the sidl file input flag.
-   */
-  public void setSIDLFileInput(boolean sf) {
-    d_sidl_file_input = sf;
-  }
 }
