@@ -1,8 +1,8 @@
 //
 // File:        ArrayMethods.java
 // Package:     gov.llnl.babel.backend
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:    @(#) $Revision: 6196 $
+// Date:        $Date: 2007-10-24 16:42:37 -0700 (Wed, 24 Oct 2007) $
 // Description: 
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,9 +30,10 @@
 // Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 package gov.llnl.babel.backend.c;
+import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.writers.LanguageWriter;
-import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Version;
 
@@ -53,19 +54,19 @@ public class ArrayMethods {
 
   private SymbolID d_implID;
 
-  public ArrayMethods(SymbolID id, boolean isEnum) {
+  public ArrayMethods(SymbolID id, boolean isEnum, Context context) {
     d_id = id;
     d_isEnum = isEnum;
-    d_implArrayType = isEnum ? "struct sidl_int__array *" 
+    d_implArrayType = isEnum ? "struct sidl_long__array *" 
                              : "struct sidl_interface__array *";
-    d_implDataType = isEnum ? "int32_t" : "struct sidl_BaseInterface__object *";
-    d_initDataType = isEnum ? "int32_t" : C.getObjectName(id);
+    d_implDataType = isEnum ? "int64_t" : "struct sidl_BaseInterface__object *";
+    d_initDataType = isEnum ? "int64_t" : C.getObjectName(id);
     if (isEnum) {
-      d_implID = new SymbolID("sidl.int", new Version());
+      d_implID = new SymbolID("sidl.long", new Version());
     } else {
       d_implID = new SymbolID("sidl.interface", new Version());
     }
-    d_comments = !BabelConfiguration.getInstance().getCommentLocalOnly();
+    d_comments = !context.getConfig().getCommentLocalOnly();
   }
 
   private String getTypeName() {
@@ -183,7 +184,7 @@ public class ArrayMethods {
     String methodName = generateBorrowName(d_id);
     String firstType;
     if (d_isEnum) {
-      firstType = "int32_t *";
+      firstType = "int64_t *";
     } else {
       firstType = C.getObjectName(d_id) + "*";
     }
@@ -193,7 +194,7 @@ public class ArrayMethods {
       lw.println("source. The initial contents are determined by the");
       lw.println("data being borrowed.");
       if (d_isEnum){
-        lw.println("The borrowed data must be a pointer to int32_t.");
+        lw.println("The borrowed data must be a pointer to int64_t.");
       } else {
         lw.println("Any time an element in the borrowed array is replaced");
         lw.println("via a set call, deleteRef will be called on the");
@@ -218,7 +219,6 @@ public class ArrayMethods {
 
   private void generateSmartCopySig(LanguageWriter lw, String terminator) {
     String methodName = generateSmartCopyName(d_id);
-    String firstType;
     if (d_comments) {
       lw.beginBlockComment(true);
       lw.println("If array is borrowed, allocate a new self-sufficient");
