@@ -1,7 +1,7 @@
 //
 // File:        C.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: C.java 4462 2005-03-23 19:29:24Z leek2 $
+// Revision:    @(#) $Id: C.java 6196 2007-10-24 23:42:37Z epperly $
 // Description: common C binding routines shared by C code generators
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,13 +31,17 @@
 package gov.llnl.babel.backend.c;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.symbols.Argument;
-import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.CExprString;
 import gov.llnl.babel.symbols.Comment;
+import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
@@ -52,39 +56,10 @@ import java.util.List;
  */
 public class C 
 {
-  public final static String FUNCTION_RESULT   = "_result";
+  public final static String FUNCTION_RESULT   = "_retval";
   public final static String NULL              = "NULL";
   public final static String RAW_ARRAY_EXT     = "_tmp";
 
-  private final static String s_types[] = 
-  {
-    "void", 
-    "sidl_bool",
-    "char",
-    "struct sidl_dcomplex",
-    "double",
-    "struct sidl_fcomplex", 
-    "float",
-    "int32_t",
-    "int64_t",
-    "void*",
-    "char*"
-  };
-
-  private final static String s_array_types[] = {
-    null,                 
-    "struct sidl_bool__array*",
-    "struct sidl_char__array*",   
-    "struct sidl_dcomplex__array*",
-    "struct sidl_double__array*", 
-    "struct sidl_fcomplex__array*",
-    "struct sidl_float__array*",  
-    "struct sidl_int__array*",
-    "struct sidl_long__array*",   
-    "struct sidl_opaque__array*",
-    "struct sidl_string__array*"
-  };
-
   private static String s_exceptionType   = null;
   private static String s_exceptionObjPtr = null;
   private static String s_self            = Utilities.s_self;
@@ -254,6 +229,22 @@ public class C
     return "struct " + getObjectName(id) + "__object";
   }
 
+  /**
+   * Calculate the maximum name length of struct items.
+   */
+  public static int getLongestNameLen(Struct strct)
+  {
+    int result = 0;
+    Iterator i = strct.getItems().iterator();
+    while (i.hasNext()) {
+      final int maxLen = ((Struct.Item)i.next()).getName().length();
+      if (maxLen > result) {
+        result = maxLen;
+      }
+    }
+    return result;
+  }
+
   /**
    * Convert a SIDL symbol into the name of its associated set EPV
    * method, which is the symbol name appended with "__set_epv".
@@ -264,6 +255,37 @@ public class C
     return getSymbolName(id) + "__set_epv";
   }
 
+  /**
+    * Convert a SIDL symbol into the name of its associated remote
+    * connector.  This requires both the SybmolID of the class this is being
+    * defined in (sourceid) and the SymbolID of the target class to be
+    * connected (targetid) 
+    */
+  public static String getImplFConnectName(SymbolID sourceid, SymbolID targetid) {
+      return "impl_"+getSymbolName(sourceid)+"_fconnect_" + getSymbolName(targetid);
+   }
+
+  /**
+    * Convert a SIDL symbol into the name of its associated remote
+    * connector.  This requires both the SybmolID of the class this is being
+    * defined in (sourceid) and the SymbolID of the target class to be
+    * connected (targetid) 
+    */
+  public static String getImplFCastName(SymbolID sourceid, SymbolID targetid) {
+      return "impl_"+getSymbolName(sourceid)+"_fcast_" + getSymbolName(targetid);
+   }
+
+
+  /**
+    * Convert a SIDL symbol into the name of its associated get URL function.
+    * This requires both the SybmolID of the class this is being
+    * defined in (sourceid) and the SymbolID of the target class to be
+    * connected (targetid) 
+    */
+  public static String getImplFGetURLName(SymbolID sourceid, SymbolID targetid) {
+      return "impl_"+getSymbolName(sourceid)+"_fgetURL_" + getSymbolName(targetid);
+   }
+
   /**
    * Convert a SIDL symbol into the name of its associated set static
    * EPV method, which is the symbol name appended with "__set_sepv".
@@ -280,13 +302,30 @@ public class C
    * @param id the <code>SymbolID</code> of the <code>Symbol</code> 
    *           associated with the method.
    *
-   * @param method the <code>String</code> version of the name of the
+   * @param methodName the <code>String</code> version of the name of the
    *               method whose impl name is being built.
    */
   public static String getMethodImplName(SymbolID id, String methodName) {
     return "impl_" + getSymbolName(id) + '_' + methodName;
   }
 
+  private static boolean hasEnum(Method m)
+  {
+    boolean result = (Type.ENUM ==   m.getReturnType().getDetailedType());
+    Iterator i = m.getArgumentList().iterator();
+    while (!result && i.hasNext()) {
+      Argument arg = (Argument)i.next();
+      result = (arg.getType().getDetailedType() == Type.ENUM);
+    }
+    return result;
+  }
+
+  public static boolean methodNeedsSkel(Method method)
+  {
+    return method.hasArrayOrderSpec() || method.hasRarray() ||
+      hasEnum(method);
+  }
+
   /**
    * Generate the skel method's name.  In most cases, the skel name is the
    * impl name except when the method has an array with an ordering
@@ -298,7 +337,7 @@ public class C
    * @param method the method
    */
   public static String getMethodSkelName(SymbolID id, Method method) {
-    return method.hasArrayOrderSpec() 
+    return methodNeedsSkel(method)
       ? ("skel_" + getSymbolName(id) + '_' + method.getLongMethodName())
       : getMethodImplName(id, method.getLongMethodName());
   }
@@ -326,7 +365,7 @@ public class C
    * @param id the <code>SymbolID</code> of the <code>Symbol</code> 
    *           associated with the method.
    *
-   * @param method the <code>String</code> version of the name of the
+   * @param methodName the <code>String</code> version of the name of the
    *               method whose full name is being built.
    */
   public static String getFullMethodName(SymbolID id, String methodName) {
@@ -339,7 +378,7 @@ public class C
     case Type.INTERFACE:
       return ArrayMethods.generateEnsureName(arrayType.getSymbolID());
     case Type.ENUM:
-      return "sidl_int__array_ensure";
+      return "sidl_long__array_ensure";
     default:
       return "sidl_" + arrayType.getTypeString() + "__array_ensure";
     }
@@ -351,7 +390,7 @@ public class C
     case Type.INTERFACE:
       return ArrayMethods.generateDelRefName(arrayType.getSymbolID());
     case Type.ENUM:
-      return "sidl_int__array_deleteRef";
+      return "sidl_long__array_deleteRef";
     default:
       return "sidl_" + arrayType.getTypeString() + "__array_deleteRef";
     }
@@ -361,55 +400,34 @@ public class C
     return s_exceptionType;
   }
 
-  /**
-   * Creates a Method that represents the Exec method
-   */
-  public static Method getExecMethod() throws CodeGenerationException {
-    Method m_exec = new Method();
-    String[] cmmnt = {"Exec method for rmi and reflexity."};
-    m_exec.setMethodName("_exec");
-    m_exec.setComment(new Comment(cmmnt));
-    m_exec.setReturnType(new Type(Type.VOID));
-    Argument a = new Argument(false, Argument.IN, new Type(Type.STRING), 
-                              "methodName");
-    m_exec.addArgument(a);
-    
-    Symbol tmpSym = Utilities.lookupSymbol("sidl.io.Deserializer");
-    a = new Argument(false, Argument.IN, new Type(tmpSym.getSymbolID()), 
-                     "inArgs");
-    m_exec.addArgument(a);
-    tmpSym = Utilities.lookupSymbol("sidl.io.Serializer");
-    a = new Argument(false, Argument.IN, new Type(tmpSym.getSymbolID()), 
-                     "outArgs");
-    m_exec.addArgument(a);
-    return m_exec;
-  }
-
   /**
    * Creates a Method that represents the static Exec method
    */
-  public static Method getSExecMethod() throws CodeGenerationException {
-    Method m_exec = new Method();
+  public static Method getSExecMethod(Context context) 
+    throws CodeGenerationException 
+  {
+    Method m_exec = new Method(context);
     String[] cmmnt = {"static Exec method for reflexity."};
     m_exec.setMethodName("_sexec");
     m_exec.setComment(new Comment(cmmnt));
     m_exec.setDefinitionModifier(Method.STATIC);
     m_exec.setReturnType(new Type(Type.VOID));
-    Argument a = new Argument(false, Argument.IN, new Type(Type.STRING), 
+    Argument a = new Argument(Argument.IN, new Type(Type.STRING), 
                               "methodName");
     m_exec.addArgument(a);
     
-    Symbol tmpSym = Utilities.lookupSymbol("sidl.io.Deserializer");
-    a = new Argument(false, Argument.IN, new Type(tmpSym.getSymbolID()), 
-                     "inArgs");
-    m_exec.addArgument(a);
-    tmpSym = Utilities.lookupSymbol("sidl.io.Serializer");
-    a = new Argument(false, Argument.IN, new Type(tmpSym.getSymbolID()), 
+    Symbol tmpSym = Utilities.lookupSymbol(context, "sidl.rmi.Call");
+    a = new Argument(Argument.IN, new Type(tmpSym, context), 
+                     "inArgs"); 
+    m_exec.addArgument(a);  
+    tmpSym = Utilities.lookupSymbol(context, "sidl.rmi.Return");
+    a = new Argument(Argument.IN, new Type(tmpSym, context), 
                      "outArgs");
     m_exec.addArgument(a);
     return m_exec;
   }
 
+
  /**
   * Generate the method's argument list.
   *
@@ -445,6 +463,7 @@ public class C
   *    violations of the data type invariants.
   */
   public static void generateArgumentList(LanguageWriterForC writer, 
+                                          Context context,
                                           String self, boolean is_interface,
                                           SymbolID id, Method method,
                                           boolean in_signature,
@@ -456,7 +475,8 @@ public class C
   {
     String excVar = do_throws ? Utilities.s_exception : "";
 
-    generateArgumentList(writer, self, is_interface, id, method, in_signature,
+    generateArgumentList(writer, context, 
+                         self, is_interface, id, method, in_signature,
                          add_type, obj_ptr, excVar, do_return, do_indices, 
                          do_rarrays);
   }
@@ -496,6 +516,7 @@ public class C
   *    violations of the data type invariants.
   */
   public static void generateArgumentList(LanguageWriterForC writer, 
+                                          Context context,
                                           String self, boolean is_interface,
                                           SymbolID id, Method method,
                                           boolean in_signature,
@@ -532,8 +553,9 @@ public class C
     } else {
       m_self = s_self;
     }
-    generateArguments(writer, m_self, args, method.isStatic(), excVar,
-                      returnType, add_type, obj_ptr, do_rarrays);
+    generateArguments(writer, context,
+                      m_self, args, method.isStatic(), excVar,
+                      returnType, add_type, obj_ptr, do_rarrays, false);
   }
 
  /**
@@ -559,21 +581,90 @@ public class C
   * @param doRaw the boolean indicating if special raw array argument handling
   *              is needed.
   *
+  * @param deref_inout if true, out and inout arguments are dereferenced in
+  *                    passing.  (Used for pre and post hooks)
+  *
   * @exception gov.llnl.babel.backend.CodeGenerationException
   *    this is a catch all exception. It can be caused by I/O trouble or
   *    violations of the data type invariants.
   */
-  public static void generateArguments(LanguageWriterForC writer, String self,
+  public static void generateArguments(LanguageWriterForC writer, 
+                                       Context context,
+                                       String self,
                                        List args, boolean isStatic,
                                        boolean doThrows, Type returnType,
                                        boolean addType, boolean objPtr, 
-                                       boolean doRaw)
+                                       boolean doRaw, boolean deref_inout)
     throws CodeGenerationException
   {
     String excVar = doThrows ? Utilities.s_exception : "";
 
-    generateArguments(writer, self, args, isStatic, excVar, returnType, 
-                      addType, objPtr, doRaw);
+    generateArguments(writer, context, 
+                      self, args, isStatic, excVar, returnType, 
+                      addType, objPtr, doRaw, deref_inout);
+  }
+
+  /**
+   * Create a comment to describe the SIDL type for the C signature.
+   * 
+   * @param arg the argument to make a comment from
+   * @return usually this is just the mode as a string. For arrays
+   * and rarrays more information is returned.
+   */
+  public static String argComment(Argument arg)
+  {
+    final Type argType = arg.getType();
+    if (Type.ARRAY == argType.getDetailedType()) {
+      StringBuffer buf = new StringBuffer(arg.getModeString());
+      if ( argType.isRarray()) {
+        buf.append(" rarray[");
+        Iterator i = argType.getArrayIndexExprs().iterator();
+        while (i.hasNext()) {
+          AssertionExpression ae = (AssertionExpression)i.next();
+          buf.append(ae.accept(new CExprString(), null).toString());
+          if (i.hasNext()) buf.append(',');
+        }
+        buf.append(']');
+      }
+      else {
+        buf.append(' ');
+        buf.append(argType.getTypeString());
+      }
+      return buf.toString();
+    }
+    else {
+      return arg.getModeString();
+    }
+  }
+
+  public static String getReturnString(Type type,
+                                       Context context,
+                                       boolean objPtr,
+                                       boolean inStub)
+    throws CodeGenerationException
+  {
+    return (Type.ENUM == type.getDetailedType())
+      ? getEnumName(type.getSymbolID())
+      : IOR.getReturnString(type, context, objPtr, inStub);
+  }
+
+  public static String
+    getArgumentWithFormal(Argument arg,
+                          Context context,
+                          boolean objPtr,
+                          boolean inStub,
+                          boolean isExec)
+    throws CodeGenerationException
+  {
+    final Type type = arg.getType();
+    if (Type.ENUM == type.getDetailedType() ) {
+      return getEnumName(type.getSymbolID()) +
+        ((arg.getMode() != Argument.IN) ? "* " : " ") + 
+        arg.getFormalName();
+    }
+    else {
+      return IOR.getArgumentWithFormal(arg, context, objPtr, inStub, isExec);
+    }
   }
 
  /**
@@ -598,22 +689,26 @@ public class C
   *
   * @param doRaw the boolean indicating if special raw array argument handling
   *              is needed.
-  *
+  * 
+  * @param deref_inout if true, out and inout arguments are dereferenced in
+  *                    passing.  (Used for pre and post hooks)
   * 
   * @exception gov.llnl.babel.backend.CodeGenerationException
   *    this is a catch all exception. It can be caused by I/O trouble or
   *    violations of the data type invariants.
   */
-  public static void generateArguments(LanguageWriterForC writer, String self,
+  public static void generateArguments(LanguageWriterForC writer, 
+                                       Context context,
+                                       String self,
                                        List args, boolean isStatic,
                                        String excVar, Type returnType,
                                        boolean addType, boolean objPtr, 
-                                       boolean doRaw)
+                                       boolean doRaw, boolean deref_inout)
     throws CodeGenerationException
   {
     boolean doThrows = (excVar != null) && !excVar.equals("");
-    boolean doReturn = (returnType != null)
-                       && (returnType.getType() != Type.VOID);
+    boolean doReturn = false; //(returnType != null)
+    //&& (returnType.getType() != Type.VOID);
 
     /*
      * If the method is not static, then it will begin with an
@@ -637,14 +732,24 @@ public class C
       boolean  isRaw = doRaw && arg.getType().isRarray();
 
       if (addType) {
-        writer.print("/* " + arg.getModeString() + " */ ");
-        writer.print(IOR.getArgumentWithFormal(arg, objPtr, isRaw, false));
+        writer.print("/* " + argComment(arg) + " */ ");
+        if(deref_inout && arg.getMode() != Argument.IN) {
+          writer.print(" *");
+        }
+        writer.print(C.getArgumentWithFormal(arg, context, objPtr, isRaw, false));
       } else if (isRaw) {
         if (arg.getMode() == Argument.INOUT) {
           writer.print("&");
         }
         writer.print(arg.getFormalName() + RAW_ARRAY_EXT);
       } else {
+        if(deref_inout && arg.getMode() != Argument.IN) {
+          writer.print(" *");
+        }
+        if ((arg.getMode() != Argument.IN) &&
+            (arg.getType().getDetailedType() == Type.ENUM)) {
+          writer.print("&_proxy_");
+        }
         writer.print(arg.getFormalName());
       }
       if (a.hasNext() || doThrows || doReturn) {
@@ -677,10 +782,11 @@ public class C
      */
     if (doReturn) {
       if (addType) {
-        writer.print("/* out */ ");
-        writer.print(IOR.getReturnString(returnType, objPtr, false));
+        writer.print("/* in */ ");
+        writer.print(C.getReturnString(returnType, context, objPtr, false));
       }
       writer.print(" " + FUNCTION_RESULT);
     }
   }
+
 }
