@@ -1,7 +1,7 @@
 //
 // File:        ImplHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: ImplHeader.java 4462 2005-03-23 19:29:24Z leek2 $
+// Revision:    @(#) $Id: ImplHeader.java 6201 2007-10-26 06:15:35Z kumfert $
 // Description: generate C implementation header to a pretty writer stream
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,7 +30,7 @@
 
 package gov.llnl.babel.backend.c;
 
-import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
@@ -42,12 +42,9 @@ import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
-import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
-import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.Type;
 import java.util.Collection;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
 
@@ -66,8 +63,6 @@ public class ImplHeader {
    */
   private LanguageWriterForC d_writer;
 
-  private SymbolTable        d_symtab;
-
   /**
    * The splicer is used when replacing an existing file. Sections from the
    * previous file are stored in the splicer and can be grafted back into
@@ -75,6 +70,8 @@ public class ImplHeader {
    */
   private CodeSplicer        d_splicer;
 
+  private Context d_context;
+
   /**
    * Create a <code>ImplHeader</code> object that will write symbol
    * information to the provided output language writer stream.
@@ -85,10 +82,11 @@ public class ImplHeader {
    * @param writer  the output writer for the new implementation header
    *                file. This writer will not be closed by this method.
    */
-  public ImplHeader(LanguageWriterForC writer, CodeSplicer splicer) {
+  public ImplHeader(LanguageWriterForC writer, CodeSplicer splicer,
+                    Context context) {
     d_splicer = splicer;
     d_writer  = writer;
-    d_symtab  = SymbolTable.getInstance();
+    d_context = context;
   }
 
   /**
@@ -106,10 +104,11 @@ public class ImplHeader {
    *
    */
   public static void generateCode(Class cls, LanguageWriterForC writer,
-                                  CodeSplicer splicer)
+                                  CodeSplicer splicer,
+                                  Context context)
     throws CodeGenerationException
   {
-    ImplHeader header = new ImplHeader(writer, splicer);
+    ImplHeader header = new ImplHeader(writer, splicer, context);
     header.generateCode(cls);
   }
 
@@ -119,10 +118,9 @@ public class ImplHeader {
    * @param cls the <code>Class</code> whose header will be written.
    */
   private void writePrologue(Class cls) {
-    final SymbolID id = cls.getSymbolID();
-    String header = C.getImplHeaderFile(id);
+    String header = C.getImplHeaderFile(cls);
     d_writer.writeBanner(cls, header, true, CodeConstants.C_DESC_IMPL_PREFIX 
-      + id.getFullName());
+      + cls.getFullName());
     d_writer.openHeaderGuard(header);
   }
 
@@ -132,14 +130,13 @@ public class ImplHeader {
    * @param cls  the <code>Class</code> whose header will be written.
    */
   private void writeIncludeSection(Class cls, Collection localRefs) {
-    final SymbolID id = cls.getSymbolID();
-    String includes = id.getFullName() + "." + "_includes";
+    String includes = cls.getFullName() + "." + "_hincludes";
     d_writer.generateInclude("sidl_header.h", true);
     if(cls.hasOverwrittenMethods()) {
-      d_writer.generateInclude(IOR.getHeaderFile(id), true);
+      d_writer.generateInclude(IOR.getHeaderFile(cls), true);
     }
 
-    Iterator i = localRefs.iterator();
+    Iterator i = Utilities.sort(localRefs).iterator();
     while (i.hasNext()) {
       SymbolID sid = (SymbolID)i.next();
       d_writer.generateInclude(C.getHeaderFile(sid), true);
@@ -151,26 +148,16 @@ public class ImplHeader {
     d_writer.println();
   }
 
-  /**
-   * Write any necessary forward struct declarations.
-   *
-   * @param cls  the <code>Class</code> whose header will be written.
-   */
-  private void writeForwardStructDecls(Class cls, Collection localRefs) {
-    // no forward struct declarations are needed
-  }
-
   /**
    * Output the signatures for the get private data access methods.
    *
    * @param cls  the <code>Class</code> whose header will be written.
    */
   private void writeGetPrivateDataDecl(Class cls) {
-    final SymbolID id = cls.getSymbolID();
-    d_writer.println("extern " + C.getDataName(id) + "*");
-    d_writer.println(C.getDataGetName(id) + "(");
+      d_writer.println("extern " + C.getDataName(cls) + "*");
+    d_writer.println(C.getDataGetName(cls) + "(");
     d_writer.tab();
-    d_writer.println(C.getObjectName(id)+ ");");
+    d_writer.println(C.getObjectName(cls)+ ");");
     d_writer.backTab();
     d_writer.println();
   }
@@ -181,12 +168,11 @@ public class ImplHeader {
    * @param cls  the <code>Class</code> whose header will be written.
    */
   private void writeSetPrivateDataDecl(Class cls) {
-    final SymbolID id = cls.getSymbolID();
     d_writer.println("extern void");
-    d_writer.println(C.getDataSetName(id) + "(");
+    d_writer.println(C.getDataSetName(cls) + "(");
     d_writer.tab();
-    d_writer.println(C.getObjectName(id) + ",");
-    d_writer.println(C.getDataName(id) + "*);");
+    d_writer.println(C.getObjectName(cls) + ",");
+    d_writer.println(C.getDataName(cls) + "*);");
     d_writer.backTab();
     d_writer.println();
   }
@@ -206,8 +192,8 @@ public class ImplHeader {
   private void writeIORWrapperDecl(Class cls, String funcName, Method iorMethod)
     throws CodeGenerationException
   {
-    final SymbolID id = cls.getSymbolID();
-    Iterator args = Utilities.extendArgs(id, iorMethod, false).iterator();
+    Iterator args = Utilities.extendArgs(cls, iorMethod,
+                                         false, d_context).iterator();
     d_writer.println("extern void");
     d_writer.print(funcName);
     d_writer.println("(");
@@ -234,10 +220,10 @@ public class ImplHeader {
     *           is needed leaving this one for all other cases (especially
     *           back ends that rely on it).
     */
-   private static String getArgumentString(Argument arg)
+  private String getArgumentString(Argument arg)
      throws CodeGenerationException
    {
-     return IOR.getArgumentString(arg, false, false, false);
+     return IOR.getArgumentString(arg, d_context, false, false, false);
    }
 
   /**
@@ -248,8 +234,6 @@ public class ImplHeader {
    *
    * @param lw        the target language writer 
    * @param meth      the method 
-   * @param suffix    the suffix to attach to the name, typically for
-   *                  the pre- and post-method hooks.
    * @param self      the self variable, if appropriate
    * @param addExtern TRUE if the signature is to include the "extern" 
    *                  modifier (as in the header file case).
@@ -259,29 +243,33 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void writeMethodSignature(LanguageWriterForC lw, Method meth,
-                                          String suffix, String self,
+  public static void writeMethodSignature(LanguageWriterForC lw, 
+                                          Method meth,
+                                          String self,
                                           boolean addExtern, boolean isIfc, 
-                                          boolean addType, SymbolID id)
+                                          boolean addType, SymbolID id,
+                                          Context context)
     throws CodeGenerationException
   {
-    boolean isOrig        = suffix.equals("");
     Type    returnType    = meth.getReturnType();
-    String  returnTypeStr = getReturnString(returnType);
+    String  returnTypeStr = getReturnString(returnType, context);
     if (addExtern) {
-      lw.print("extern ");
+      lw.println("extern");
     }
-    if (isOrig) {
-      lw.println(returnTypeStr);
-    } else {
-      lw.println("void");
+    else {
+      lw.printlnUnformatted("#ifdef __cplusplus");
+      lw.println("extern \"C\"");
+      lw.printlnUnformatted("#endif");
     }
-    lw.print(C.getMethodImplName(id, meth.getLongMethodName()) + suffix);
+
+    lw.println(returnTypeStr);
+
+    lw.print(C.getMethodImplName(id, meth.getLongMethodName()));
     lw.println("(");
     lw.tab();
-    C.generateArgumentList(lw, self, isIfc, id, meth, true, addType, false, 
-                           isOrig, suffix.equals(IOR.GENERIC_POST_SUFFIX), true,
-                           true);
+    C.generateArgumentList(lw, context,
+                           self, isIfc, id, meth, true, addType, false, 
+                           true, false, true, true);
     lw.print(")");
   }
 
@@ -291,10 +279,11 @@ public class ImplHeader {
     * at the start of this class.  Symbol types and array types require
     * special processing.
     */
-   private static String getReturnString(Type type)
+   private static String getReturnString(Type type,
+                                         Context context)
      throws CodeGenerationException
    {
-     return IOR.getReturnString(type, false, false);
+     return IOR.getReturnString(type, context, false, false);
    }
 
   /**
@@ -303,19 +292,17 @@ public class ImplHeader {
    * @param lw       the language writer to write to
    * @param isIfc     TRUE if the method belongs to an interface
    * @param m        the method description
-   * @param suffix   the suffix to attach to the name, typically for
-   *                 the pre- and post-method hooks.
    * @param id       the name of the symbol owning the method
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
   private static void writeMethodDecl(LanguageWriterForC lw, boolean isIfc, 
-                                      Method m, String suffix, SymbolID id)
+                                      Method m, SymbolID id, Context context)
     throws CodeGenerationException
   {
-    writeMethodSignature(lw, m, suffix, Utilities.s_self, true, isIfc, true, 
-                         id);
+    writeMethodSignature(lw, m, Utilities.s_self, true, isIfc, true, 
+                         id, context);
     lw.println(";");
     lw.backTab();
     lw.println();
@@ -334,11 +321,12 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void writeMethodDecls(LanguageWriterForC lw, Class cls,
-                                      boolean isStatic)
+  public static void writeMethodDecls(LanguageWriterForC lw, 
+                                      Class cls,
+                                      boolean isStatic,
+                                      Context context)
     throws CodeGenerationException
   {
-    final SymbolID id = cls.getSymbolID();
     Iterator i = null;
     if (isStatic) {
       i = cls.getStaticMethods(false).iterator();
@@ -348,17 +336,75 @@ public class ImplHeader {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        if (IOR.supportInterceptors(cls)) {
-          writeMethodDecl(lw, false, m, IOR.GENERIC_PRE_SUFFIX, id);
-          writeMethodDecl(lw, false, m, "", id);
-          writeMethodDecl(lw, false, m, IOR.GENERIC_POST_SUFFIX, id);
+        if (IOR.supportHooks(cls, context)) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          writeMethodDecl(lw, false, pre, cls, context);
+          writeMethodDecl(lw, false, m, cls, context);
+          writeMethodDecl(lw, false, post, cls, context);
+
         } else {
-          writeMethodDecl(lw, false, m, "", id);
+          writeMethodDecl(lw, false, m, cls, context);
         }
       }
     }
+    writeRMIAccessDecls(lw,cls,isStatic);
   }
 
+
+  /**
+   * Write external declarations for the RMI fconnects methods in the class.
+   *
+   * @param lw       the language writer to write to.
+   * @param cls      the <code>Class</code> whose header will be written.
+   * @param isStatic <code>true</code> means write only static methods;
+   *                 <code>false</code> means write only non-static
+   *                 methods.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *    this is a catch all exception. It can be caused by I/O trouble or
+   *    violations of the data type invariants.
+   */
+  public static void writeRMIAccessDecls(LanguageWriterForC lw, Class cls,
+                                      boolean isStatic)
+    throws CodeGenerationException
+  {
+    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
+    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
+
+    lw.printlnUnformatted("#ifdef WITH_RMI");
+
+    for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      lw.println("extern " + C.getSymbolObjectPtr(l_id) + " " +C.getImplFConnectName(cls,l_id) + 
+                     "(const char* url, sidl_bool ar, sidl_BaseInterface *_ex);");
+    }
+
+    for (Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      lw.println("extern void");
+      lw.println(IOR.getSkelSerializationName(cls,l_id, true) +
+                 "(const " +
+                 IOR.getStructName(l_id) + " *strct, " +
+                 "struct sidl_rmi_Return__object *pipe, const char *name, sidl_bool copyArg, " +
+                 IOR.getExceptionFundamentalType() + "*exc);");
+    }
+    for (Iterator i = deserializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      lw.println("extern void");
+      lw.println(IOR.getSkelSerializationName(cls,l_id, false) +
+                 "(" +
+                 IOR.getStructName(l_id) + " *strct, " +
+                 "struct sidl_rmi_Call__object *pipe, const char *name, sidl_bool copyArg, " +
+                 IOR.getExceptionFundamentalType() + "*exc);");
+    }
+
+   lw.printlnUnformatted("#endif /*WITH_RMI*/");
+
+  }
+
+
+
   /**
    * Write external declarations for the <code>Class</code> constructor and
    * destructor.
@@ -369,50 +415,40 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void writeBuiltinDecls(LanguageWriterForC lw, Class cls)
+  public static void writeBuiltinDecls(LanguageWriterForC lw, 
+                                       Class cls,
+                                       Context context)
     throws CodeGenerationException
   {
-    final SymbolID id = cls.getSymbolID();
     if(cls.hasOverwrittenMethods()) {
       //function from the Impl file that sets the superEPV
-      lw.println("extern void " + C.getObjectName(id) + "__superEPV(");
-      lw.println("struct " + C.getObjectName(cls.getParentClass().getSymbolID())
+      lw.println("extern void " + C.getObjectName(cls) + "__superEPV(");
+      lw.println("struct " + C.getObjectName(cls.getParentClass())
         + "__epv*);");
       lw.println();
     }
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.LOAD, id), "", 
-                    id);
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id), "", 
-                    id);
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id), "", 
-                    id);
-    if (IOR.supportAssertions(cls)) {
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.LOAD, cls, context), 
+                    cls, context);
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, cls, context), 
+                    cls, context);
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, cls, context), 
+                    cls, context);
+
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.DESTRUCTOR, cls, context),  
+                    cls, context);
+    if (IOR.supportAssertions(cls, context)) {
       if (cls.hasStaticMethod(true)) {
-        writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id, 
-                        true), "", id);
+        writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, 
+                        context, true), cls, context);
       }
-      writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id), "",
-                      id);
+      writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, context), 
+                      cls, context);
     }
   }
 
-  public Collection getLocalReferences(Extendable ext) {
-    boolean hasThrows = false;
-    Set refs = new HashSet();
-    Iterator i = ext.getMethods(false).iterator();
-    refs.add(ext.getSymbolID());
-    while (i.hasNext()) {
-      Method m = (Method)i.next();
-      refs.addAll(m.getSymbolReferences());
-      if (!(hasThrows || m.getThrows().isEmpty())) {
-        Symbol sym =
-          d_symtab.lookupSymbol(BabelConfiguration.getBaseExceptionInterface());
-        if (sym != null) {
-          hasThrows = true;
-          refs.add(sym.getSymbolID());
-        }
-      }
-    }
+  public Collection getLocalReferences(Extendable ext) throws CodeGenerationException{
+    Set refs = ext.getObjectDependencies();
+    refs.add(ext); 
     return refs;
   }
 
@@ -424,11 +460,10 @@ public class ImplHeader {
    * @param cls  the <code>Class</code> whose data header will be written.
    */
   private void writePrivateDataType(Class cls) {
-    final SymbolID id = cls.getSymbolID();
-    String name = id.getFullName();
+    String name = cls.getFullName();
     d_writer.writeComment("Private data for class " + name, false);
 
-    d_writer.println(C.getDataName(id) +" {");
+    d_writer.println(C.getDataName(cls) +" {");
     d_writer.tab();
 
     String data = cls.getFullName() + "." + "_data";
@@ -451,11 +486,15 @@ public class ImplHeader {
    *    violations of the data type invariants.
    */
   public void generateCode(Class cls) throws CodeGenerationException {
-    final SymbolID id = cls.getSymbolID();
+     final String fullname = cls.getFullName();
     Collection localRefs = getLocalReferences(cls);
+    if (d_context.getConfig().getRenameSplicers()) {
+      d_splicer.renameSymbol(fullname + "._includes", 
+                             fullname + "._hincludes");
+    }
     writePrologue(cls);
     writeIncludeSection(cls, localRefs);
-    writeForwardStructDecls(cls, localRefs);
+    //writeForwardStructDecls(cls, localRefs);
     writePrivateDataType(cls);
     d_writer.openCxxExtern();
     d_writer.writeComment("Access functions for class private data and built-in"
@@ -463,13 +502,17 @@ public class ImplHeader {
     writeGetPrivateDataDecl(cls);
     writeSetPrivateDataDecl(cls);
     if (cls.getParentClass() == null) {
-      writeIORWrapperDecl(cls, C.getPrivateDestructor(id),
-                          IOR.getBuiltinMethod(IOR.DELETE, id));
+      writeIORWrapperDecl(cls, C.getPrivateDestructor(cls),
+                          IOR.getBuiltinMethod(IOR.DELETE, cls, d_context));
     }
-    writeBuiltinDecls(d_writer, cls);
+    writeBuiltinDecls(d_writer, cls, d_context);
     d_writer.writeComment("User-defined object methods", false);
-    writeMethodDecls(d_writer, cls, true);  // static methods
-    writeMethodDecls(d_writer, cls, false); // object methods
+    writeMethodDecls(d_writer, cls, true, d_context);  // static methods
+    writeMethodDecls(d_writer, cls, false, d_context); // object methods
+
+    d_writer.println();
+    d_splicer.splice("_hmisc", d_writer, "miscellaneous things");
+    d_writer.println();
 
     /*
      * Close the header file and include unused splicer symbols, if any.
@@ -483,7 +526,7 @@ public class ImplHeader {
       d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT2);
       d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT3);
       d_writer.endBlockComment(true);
-      d_splicer.outputUnusedSymbolEdits(d_writer.getPrintWriter());
+      d_splicer.outputUnusedSymbolEdits(d_writer);
       d_writer.writeCommentLine(CodeConstants.C_END_UNREFERENCED_METHODS);
     }
     d_writer.closeCxxExtern();
