@@ -1,8 +1,8 @@
 //
 // File:        SkelSource.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Revision: 4462 $
-// Date:        $Date: 2005-03-23 11:29:24 -0800 (Wed, 23 Mar 2005) $
+// Revision:    @(#) $Revision: 6212 $
+// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
 // Description: generate C skeleton based for a particular class/interface
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,6 +31,7 @@
 
 package gov.llnl.babel.backend.c;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
@@ -39,15 +40,19 @@ import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.CExprString;
 import gov.llnl.babel.symbols.Class;
+import gov.llnl.babel.symbols.Inverter;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
-import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 /**
  * <p>
@@ -80,6 +85,13 @@ public class SkelSource {
 
   private static final String s_self = Utilities.s_self;
 
+  private static final String s_epv = "epv";
+  private static final String s_pre_epv = "pre_epv";
+  private static final String s_post_epv = "post_epv";
+  private static final String s_sepv = "sepv";
+  private static final String s_pre_sepv = "pre_sepv";
+  private static final String s_post_sepv = "post_sepv";
+
 
   /**
    * Write the skeleton file for a particular class to the language writer
@@ -94,17 +106,50 @@ public class SkelSource {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void generateCode(Class cls, LanguageWriterForC writer)
+  public static void generateCode(Class cls, LanguageWriterForC writer,
+                                  Context context)
     throws CodeGenerationException
   {
-    final SymbolID id = cls.getSymbolID();
-    writer.writeBanner(cls, C.getSkelFile(cls.getSymbolID()), false,
-      CodeConstants.C_DESC_SKEL_PREFIX + id.getFullName());
-    writer.generateInclude(IOR.getHeaderFile(cls.getSymbolID()), false);
-    writer.generateInclude(C.getHeaderFile(cls.getSymbolID()), false);
+    writer.writeBanner(cls, C.getSkelFile(cls), false,
+      CodeConstants.C_DESC_SKEL_PREFIX + cls.getFullName());
+
+    generateIncludes(cls, writer, context);
+
+    if (cls.getParentClass() == null) {
+      writeIORCall(cls, C.getPrivateDestructor(cls),
+                   IOR.getBuiltinMethod(IOR.DELETE, cls, context), writer,
+                   context);
+    }
+
+    ImplHeader.writeBuiltinDecls(writer,cls, context);
+    ImplHeader.writeMethodDecls(writer, cls, true, context);
+    ImplHeader.writeMethodDecls(writer, cls, false, context);
+
+    writeSkelMethods(cls, writer, context);
+    
+    writeInitializeEPV(cls, writer, context);
+    if (cls.hasStaticMethod(true)) {
+      writeInitializeSEPV(cls, writer, context);
+    }
+    writer.openCxxExtern();
+    writeCallLoad(cls,writer);
+    writeRMIAccessFunctions(cls,writer, context);
+    
+    writeGetDataPointer(cls, writer);
+    writeSetDataPointer(cls, writer);
+    writer.closeCxxExtern();
+  }
+
+  public static void generateIncludes(Class cls, 
+                                      LanguageWriterForC writer,
+                                      Context context)
+    throws CodeGenerationException
+  {
+    writer.generateInclude(IOR.getHeaderFile(cls), false);
+    writer.generateInclude(C.getHeaderFile(cls), false);
     writer.printlnUnformatted("#include <stddef.h>");
    if(cls.hasOverwrittenMethods()) {
-      writer.generateInclude(IOR.getHeaderFile(id), false);
+      writer.generateInclude(IOR.getHeaderFile(cls), false);
       writer.printlnUnformatted("#ifdef SIDL_DYNAMIC_LIBRARY");
       writer.printlnUnformatted("#include <stdio.h>");
       writer.printlnUnformatted("#include <stdlib.h>");
@@ -113,33 +158,48 @@ public class SkelSource {
 
       //function from the Impl file that sets the superEPV
       writer.println();
-      writer.println("extern void " + C.getObjectName(id) + "__superEPV(");
+      writer.println("extern void " + C.getObjectName(cls) + "__superEPV(");
       writer.println("struct " 
-       + C.getObjectName(cls.getParentClass().getSymbolID()) + "__epv*);");
+       + C.getObjectName(cls.getParentClass()) + "__epv*);");
 
-      StubSource.generateGetExternals(cls,writer);
+      StubSource.generateGetExternals(cls,writer, context);
     }
     writer.println();
 
-    if (cls.getParentClass() == null) {
-      writeIORCall(cls, C.getPrivateDestructor(id),
-                   IOR.getBuiltinMethod(IOR.DELETE, id), writer);
-    }
-
-    ImplHeader.writeBuiltinDecls(writer,cls);
-    ImplHeader.writeMethodDecls(writer, cls, true);
-    ImplHeader.writeMethodDecls(writer, cls, false);
-
-    writeSkelMethods(cls, writer);
+    writer.printlnUnformatted("#ifdef WITH_RMI");
     
-    writeInitializeEPV(cls, writer);
-    if (cls.hasStaticMethod(true)) {
-      writeInitializeSEPV(cls, writer);
+    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
+    
+    for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.generateInclude(C.getHeaderFile(l_id), true);
     }
-    writeCallLoad(cls,writer);
 
-    writeGetDataPointer(cls, writer);
-    writeSetDataPointer(cls, writer);
+    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
+
+    if (!serializeSIDs.isEmpty()) {
+      writer.generateInclude("sidl_io_Serializer.h", true);
+      writer.println();
+    }
+
+    for(Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.generateInclude(C.getHeaderFile(l_id), true);
+      writer.println();
+    }
+
+    if (!deserializeSIDs.isEmpty()) {
+      writer.generateInclude("sidl_io_Deserializer.h", true);
+      writer.println();
+    }
+
+    for(Iterator i = deserializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.generateInclude(C.getHeaderFile(l_id), true);
+      writer.println();
+    }
+    writer.printlnUnformatted("#endif /* WITH_RMI */");
   }
 
   /**
@@ -154,19 +214,23 @@ public class SkelSource {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private static void writeSkelMethods(Class cls, LanguageWriterForC writer) 
+  private static void writeSkelMethods(Class cls, 
+                                       LanguageWriterForC writer,
+                                       Context context) 
     throws CodeGenerationException
   {
     Iterator i = cls.getMethods(false).iterator();
     while (i.hasNext()){
       Method m = (Method)i.next();
-      if (m.hasArrayOrderSpec() || m.hasRarray()) {
-        if (IOR.supportInterceptors(cls)) {
-          writeSkelMethod(cls.getSymbolID(), m, IOR.GENERIC_PRE_SUFFIX, writer);
-          writeSkelMethod(cls.getSymbolID(), m, "", writer);
-          writeSkelMethod(cls.getSymbolID(), m, IOR.GENERIC_POST_SUFFIX,writer);
+      if (C.methodNeedsSkel(m)) {
+        if (IOR.supportHooks(cls, context)) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          writeSkelMethod(cls, pre, writer, context);
+          writeSkelMethod(cls, m,  writer, context);
+          writeSkelMethod(cls, post,writer, context);
         } else {
-          writeSkelMethod(cls.getSymbolID(), m, "", writer);
+          writeSkelMethod(cls, m, writer, context);
         }
       }
     }
@@ -183,33 +247,32 @@ public class SkelSource {
    *
    * @param id	   the name of the class who owns the method.
    * @param m      the method itself.
-   * @param suffix the method name suffix, if any, for the interceptor method.
    * @param writer the output device where the skeleton function
    *               will be written.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private static void writeSkelMethod(SymbolID id, Method m, String suffix,
-                                      LanguageWriterForC writer)
+  private static void writeSkelMethod(SymbolID id, 
+                                      Method m, 
+                                      LanguageWriterForC writer,
+                                      Context context)
     throws CodeGenerationException
   {
-    boolean isNormal     = suffix.equals("");
     Type    methodReturn = m.getReturnType();
-    boolean useReturn    = isNormal && (methodReturn.getType() != Type.VOID);
-    final String method_name = C.getMethodSkelName(id, m) + suffix;
+    boolean useReturn    = (methodReturn.getType() != Type.VOID);
+    boolean hasOrderSpec = false;
+    final String method_name = C.getMethodSkelName(id, m);
 
     //Begin method signature
     writer.print("static ");
-    if (isNormal) {
-      writer.println(getReturnString(methodReturn));
-    } else {
-      writer.println("void");
-    }
+    writer.println(getReturnString(methodReturn, context));
+    
     writer.println(method_name + "(");
     writer.tab();
-    C.generateArgumentList(writer, s_self, false, id, m, true, true, false, 
-                           true, false, false, false);
+    IOR.generateArgumentList(writer, context,
+                             s_self, false, id, m, true, true, false, 
+                             true, false, false, false);
     writer.backTab();
     writer.println(")");
 
@@ -217,21 +280,25 @@ public class SkelSource {
     writer.println("{");
     writer.tab();
     if (useReturn) {
-      writer.println(getReturnString(methodReturn) + " _return;");
+      writer.println(getReturnString(methodReturn, context) + " _return;");
       if (methodReturn.hasArrayOrderSpec()) {
-        writer.println(getReturnString(methodReturn) + " _return_proxy;");
+        hasOrderSpec = hasOrderSpec || !methodReturn.isRarray();
+        writer.println(getReturnString(methodReturn, context) + " _return_proxy;");
       }
     }
-    List extArgs = Utilities.extendArgs(id, m, false);
+    List extArgs = Utilities.extendArgs(id, m, false, context);
     Iterator args = extArgs.iterator();
-    HashMap index_args = m.getRarrayInfo();
+    Map index_args = m.getRarrayInfo();
 
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
       //declare ensured array and ensure it
       if (arg.hasArrayOrderSpec()) {
         final Type argType = arg.getType();
-        writer.print(getReturnString(argType) + " " + arg.getFormalName() 
+        hasOrderSpec = hasOrderSpec || 
+          (!argType.isRarray() && (arg.getMode() != Argument.IN));
+        writer.print(getReturnString(argType, context) +
+                     " " + arg.getFormalName() 
           + "_proxy");
         if (arg.getMode()!= Argument.OUT) {
           writer.println(" = " + C.getEnsureArray(argType.getArrayType())
@@ -243,53 +310,21 @@ public class SkelSource {
       }
       //declare rarray proxies.
       if(arg.getType().isRarray()) {
-        Type argType = arg.getType();
-        Type arrayType = argType.getArrayType();
-
-        /*
-        //This part creates a native array and initializes it
-        String first_func_name = IOR.getArrayNameForFunctions(
-                                       arrayType.getType()) + "_first";
-        */
-
-        writer.print(IOR.getArgumentWithFormal(arg, false, true, false) 
+        writer.print(IOR.getArgumentWithFormal(arg, context, 
+                                               false, true, false) 
           + C.RAW_ARRAY_EXT + " = ");
-        //writer.print(first_func_name+"(");
-
         writer.println(arg.getFormalName() + "_proxy->d_firstElement;");
-        /*
-        //This part creates the index variables and initializes THEM.
-        int x = 0;
-        for(Iterator r = argType.getArrayIndices().iterator(); r.hasNext(); ++x)
-        {
-          String index = (String) r.next();
-          boolean contained = false;
-          //Check to make sure this index has not already been created
-          for(Iterator con = used_index_args.iterator(); con.hasNext();) {
-            String loc_name = (String) con.next();
-            if(loc_name.compareTo(index) == 0)
-              contained = true;
-          }
-          if(!contained) {
-            Argument index_arg = m.getArgumentByName(index);
-            String upper_func_name = IOR.getArrayNameForFunctions(
-                                          arrayType.getType()) + "_length";
-            if(index_arg.getType().getType() == Type.LONG)
-              writer.print("int64_t ");
-            else
-              writer.print("int32_t ");
-            writer.print(index_arg.getFormalName() + " = " + upper_func_name 
-              + "(");
-            if(arg.getMode() == Argument.INOUT)
-              writer.print("*");
-            writer.println(arg.getFormalName() + "," + x + ");");
-            //Add this index to the created list
-            used_index_args.add(index_arg.getFormalName());
-
-          }
-          contained = false;
+      }
+      if ((arg.getMode() != Argument.IN) &&
+          (arg.getType().getDetailedType() == Type.ENUM)) {
+        writer.print(C.getEnumName(arg.getType().getSymbolID()) +
+                     " _proxy_" + arg.getFormalName());
+        if (arg.getMode() == Argument.INOUT) {
+          writer.print(" = (" +
+                       C.getEnumName(arg.getType().getSymbolID()) +
+                       ")*" + arg.getFormalName());
         }
-        */
+        writer.println(";");
       }
     }
     //Create rarray index variables and initialize them.
@@ -301,8 +336,6 @@ public class SkelSource {
       Iterator tmp = index_collection.iterator();
       Method.RarrayInfo info = (Method.RarrayInfo) tmp.next();
       Argument index = info.index;
-      Argument rarray = info.rarray;
-      Type arrayType = rarray.getType().getArrayType();
       int dim = info.dim;
 
       String upper_func_name = "sidlLength";
@@ -311,19 +344,28 @@ public class SkelSource {
         writer.print("int64_t ");
       else
         writer.print("int32_t ");
-      writer.print(index.getFormalName() + " = " + upper_func_name + "(");
-      writer.println(info.rarray.getFormalName() + "_proxy," + dim + ");");
-
+      writer.print(index.getFormalName() + " = ");
+      writer.print(CExprString.
+                   toCString(Inverter.invertExpr
+                             ((AssertionExpression)
+                              info.rarray.getType().
+                              getArrayIndexExprs().get(dim),
+                              upper_func_name + "(" +
+                              info.rarray.getFormalName() + "_proxy," + 
+                              dim + ")", context)));
+      writer.println(";");
     }
 
     args = extArgs.iterator();
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
       if (  arg.hasArrayOrderSpec() 
-         && arg.getMode() == Argument.INOUT 
-         && !arg.getType().isRarray() ) {
+            && arg.getMode() == Argument.INOUT ) {
         writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
           + "(" + getArgValue(arg) + ");");
+        if (!arg.getType().isRarray()) {
+          writer.println(getArgValue(arg) + " = NULL;");
+        }
       }
     }
     if (useReturn) {
@@ -335,10 +377,10 @@ public class SkelSource {
       writer.tab();
     }
     //Write Impl call
-    writer.println(C.getMethodImplName(id, m.getLongMethodName()) + suffix 
+    writer.println(C.getMethodImplName(id, m.getLongMethodName()) 
       + "(");
     writer.tab();
-    List callArgs = Utilities.extendArgs(id, m, true);
+    List callArgs = Utilities.extendArgs(id, m, true, context);
     args = callArgs.iterator();
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
@@ -349,6 +391,9 @@ public class SkelSource {
         writer.print(arg.getFormalName() + "_proxy");
       } else if (arg.getType().isRarray()) {
         writer.print(arg.getFormalName() + C.RAW_ARRAY_EXT);
+      } else if ((arg.getMode() != Argument.IN) &&
+                 (arg.getType().getDetailedType() == Type.ENUM)) {
+        writer.print("&_proxy_" + arg.getFormalName());
       } else {
         writer.print(arg.getFormalName());
       }
@@ -361,30 +406,36 @@ public class SkelSource {
     if (useReturn) {
        writer.backTab();
     }
+    if (hasOrderSpec && !m.getThrows().isEmpty()) {
+      writer.println("if (!(*_ex)) {");
+      writer.tab();
+    }
     args = extArgs.iterator();
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
-      if (arg.hasArrayOrderSpec() && !arg.getType().isRarray()) {
+      if (arg.hasArrayOrderSpec()) {
         final Type argType = arg.getType();
-        if (arg.getMode() != Argument.IN) {
-          writer.println(getArgValue(arg) + " = " 
-            + C.getEnsureArray(argType.getArrayType()) + "(" 
-            + arg.getFormalName() + "_proxy, " 
-            + arg.getType().getArrayDimension() + ", " 
-            + s_ensureOrderConstant[argType.getArrayOrder()] + ");");
+        if (!argType.isRarray()) {
+          if ((arg.getMode() != Argument.IN)) {
+            writer.println(getArgValue(arg) + " = " 
+                      + C.getEnsureArray(argType.getArrayType()) + "(" 
+                      + arg.getFormalName() + "_proxy, " 
+                      + arg.getType().getArrayDimension() + ", " 
+                      + s_ensureOrderConstant[argType.getArrayOrder()] + ");");
+            writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
+                           + "(" + arg.getFormalName() + "_proxy);");
+          }
+        }
+        else if (arg.getMode() == Argument.INOUT) {
+          writer.println(getArgValue(arg) + " = " + 
+                         arg.getFormalName() + "_proxy;");
+          writer.println(arg.getFormalName() + "_proxy = NULL;");
         }
-        writer.println(C.getDelRefArray(argType.getArrayType()) 
-          + "(" + arg.getFormalName() + "_proxy);");
       }
-      if(arg.getType().isRarray() && arg.getMode() == Argument.INOUT) {
-        //We need to copy the rarray back into the sidl array
-        String r_name = arg.getFormalName();
-        String init_func_name = IOR.getArrayNameForFunctions(
-                              arg.getType().getArrayType().getType()) + "_init";
-        writer.println(init_func_name + "(" + r_name + C.RAW_ARRAY_EXT + ", *"
-          + r_name + ", " + arg.getType().getArrayDimension() + ", (*" + r_name
-          + ")->d_metadata.d_lower, (*" + r_name + ")->d_metadata.d_upper, (*" 
-          + r_name + ")->d_metadata.d_stride);\n");
+      else if ((arg.getMode() != Argument.IN) &&
+               (arg.getType().getDetailedType() == Type.ENUM)) {
+        writer.println("*" + arg.getFormalName() + " = _proxy_" +
+                       arg.getFormalName() + ";");
       }
     }
     if (useReturn) {
@@ -396,6 +447,30 @@ public class SkelSource {
         writer.println(C.getDelRefArray(methodReturn.getArrayType()) 
           + "(_return_proxy);");
       }
+    }
+    if (hasOrderSpec && !m.getThrows().isEmpty()) {
+      writer.backTab();
+      writer.println("}");
+      if (useReturn) {
+        writer.println("else {");
+        writer.tab();
+        writer.println("_return = " + IOR.getInitialValue(methodReturn) + ";");
+        writer.backTab();
+        writer.println("}");
+      }
+    }
+    args = extArgs.iterator();
+    while (args.hasNext()) {
+      Argument arg = (Argument)args.next();
+      if ((arg.getType().isRarray() && 
+                (arg.getMode() == Argument.INOUT)) ||
+               (arg.getType().hasArrayOrderSpec() &&
+                (arg.getMode() == Argument.IN))) {
+        writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
+                       + "(" + arg.getFormalName() + "_proxy);");
+      }
+    }
+    if (useReturn) {
       writer.println("return _return;");
     }
     writer.backTab();
@@ -409,22 +484,22 @@ public class SkelSource {
    * at the start of this class.  Symbol types and array types require
    * special processing.
    */
-  private static String getReturnString(Type type)
+  private static String getReturnString(Type type,
+                                        Context context)
     throws CodeGenerationException
   {
-    return IOR.getReturnString(type, false, false);
+    return IOR.getReturnString(type, context, false, false);
   }
 
   /**
    * Write the support code to get the data pointer from the class.
    */
   private static void writeGetDataPointer(Class cls, LanguageWriter writer) {
-    SymbolID id = cls.getSymbolID();
-    writer.println(C.getDataName(id) + "*");
-    writer.println(C.getDataGetName(id) + "(" + C.getObjectName(id) + " self)");
+    writer.println(C.getDataName(cls) + "*");
+    writer.println(C.getDataGetName(cls) + "(" + C.getObjectName(cls) + " self)");
     writer.println("{");
     writer.tab();
-    writer.println("return (" + C.getDataName(id) + "*)" 
+    writer.println("return (" + C.getDataName(cls) + "*)" 
       + "(self ? self->d_data : " + C.NULL + ");");
     writer.backTab();
     writer.println("}");
@@ -435,11 +510,10 @@ public class SkelSource {
    * Write the support code to set the data pointer in the class.
    */
   private static void writeSetDataPointer(Class cls, LanguageWriter writer) {
-    SymbolID id = cls.getSymbolID();
-    writer.println("void " + C.getDataSetName(id) + "(");
+    writer.println("void " + C.getDataSetName(cls) + "(");
     writer.tab();
-    writer.println(C.getObjectName(id) + " self,");
-    writer.println(C.getDataName(id) + "* data)");
+    writer.println(C.getObjectName(cls) + " self,");
+    writer.println(C.getDataName(cls) + "* data)");
     writer.backTab();
     writer.println("{");
     writer.tab();
@@ -460,32 +534,48 @@ public class SkelSource {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private static void writeInitializeEPV(Class cls, LanguageWriterForC writer)
+  private static void writeInitializeEPV(Class cls, 
+                                         LanguageWriterForC writer,
+                                         Context context)
     throws CodeGenerationException
   {
-    SymbolID id = cls.getSymbolID();
     writer.openCxxExtern();
     writer.println("void");
-    writer.println(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *epv)");
+    if(IOR.supportHooks(cls, context)) {
+      writer.println(C.getSetEPVName(cls) + "(" + IOR.getEPVName(cls) + " *"+s_epv+",");
+      writer.tab();
+      writer.println(IOR.getPreEPVName(cls) + " *"+s_pre_epv+", "+
+                     IOR.getPostEPVName(cls) + " *"+s_post_epv+")");
+      writer.backTab();
+    } else {
+      writer.println(C.getSetEPVName(cls) + "(" + IOR.getEPVName(cls) + " *"+s_epv+")");
+    }
     writer.println("{");
     writer.tab();
-    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id),
-                            id);
-    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id), 
-                            id);
-    if (IOR.supportAssertions(cls)) {
-      initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id),
-                              id);
+    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, cls,
+                                                         context),
+                            cls, context);
+    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, cls,
+                                                         context),
+                            cls, context);
+
+    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.DESTRUCTOR, cls,
+                                                         context), 
+                            cls, context);
+    if (IOR.supportAssertions(cls, context)) {
+      initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls,
+                                                           context),
+                              cls, context);
     }
     Iterator i = cls.getMethods(false).iterator();
     while (i.hasNext()) {
       Method m = (Method)i.next();
-      initializeMethodPointer(writer, m, id);
+      initializeMethodPointer(writer, m, cls, context);
     }
     writer.println();
     //if we have overwritten methods, initialize the super pointer
     if(cls.hasOverwrittenMethods()) {
-      writer.println(C.getObjectName(id) + "__superEPV(_getExternals()->"
+      writer.println(C.getObjectName(cls) + "__superEPV(_getExternals()->"
         + "getSuperEPV());");
     }
     writer.backTab();
@@ -502,16 +592,116 @@ public class SkelSource {
    */
   private static void writeCallLoad(Class cls, LanguageWriterForC writer)
   {
-    SymbolID id = cls.getSymbolID();
-    String name = IOR.getSymbolName(id);
-    writer.openCxxExtern();
-    writer.println("void " + IOR.getCallLoadName(id) + "(void) { ");
+    String name = IOR.getSymbolName(cls);
+
+    writer.println("void " + IOR.getCallLoadName(cls) + "(void) { ");
     writer.tab();
-    writer.println("impl_" + name + "__load();");
+    writer.println("sidl_BaseInterface _throwaway_exception = NULL;");
+    writer.println("impl_" + name + "__load(&_throwaway_exception);");
     writer.backTab();
     writer.println("}");
   }
 
+  /**
+   * Write functions to call each RMI fconnect function
+   *
+   * @param cls    the class for which an routine will be written.
+   * @param writer the output writer to which the routine will be written.
+   */
+  private static void writeRMIAccessFunctions(Class cls, 
+                                              LanguageWriterForC writer,
+                                              Context context) 
+    throws CodeGenerationException
+  {
+
+    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
+    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
+    //Set fcastSIDs = IOR.getFCastSymbolIDs(cls);
+
+    writer.printlnUnformatted("#ifdef WITH_RMI");
+    
+    for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.println(C.getSymbolObjectPtr(l_id) + " " + IOR.getSkelFConnectName(cls,l_id)+ 
+                     "(const char* url, sidl_bool ar, sidl_BaseInterface *_ex) { ");
+      writer.tab();
+      writer.println("return "+ IOR.getRemoteConnectName(l_id) + "(url, ar, _ex);");
+      //      writer.println("return "+C.getImplFConnectName(cls,l_id)+"(url, ar, _ex);");
+      writer.backTab();
+      writer.println("}");
+      writer.println();
+    }
+
+    /*
+    for (Iterator i = fcastSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.println(C.getSymbolObjectPtr(l_id) + " " + IOR.getSkelFCastName(cls,l_id)+ 
+                     "(void* bi, sidl_BaseInterface *_ex) { ");
+      writer.tab();
+      writer.println("return "+C.getImplFCastName(cls,l_id)+"(bi, _ex);");
+      writer.backTab();
+      writer.println("}");
+      writer.println();
+
+    }
+    */
+    for(Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.println("void");
+      writer.println(IOR.getSkelSerializationName(cls,l_id, true));
+      writer.tab();
+      writer.println("(const " +
+                       IOR.getStructName(l_id) + " *strct, " +
+                       "struct sidl_rmi_Return__object *pipe, const char *name, sidl_bool copyArg, " +
+                       IOR.getExceptionFundamentalType() + "*exc)");
+      writer.backTab();
+      writer.println("{");
+      writer.tab();
+      writer.println("struct sidl_io_Serializer__object *__pipe = sidl_io_Serializer__cast(pipe, exc);");
+      writer.println("if (*exc) return;");
+      writer.println("if (__pipe) {");
+      writer.tab();
+      writer.println(C.getExceptionType() + " __throwaway__;");
+      writer.println(IOR.getSymbolName(l_id) + "__serialize(strct, __pipe, name, copyArg, exc);");
+      writer.println("sidl_io_Serializer_deleteRef(__pipe, &__throwaway__);");
+      writer.backTab();
+      writer.println("}");
+      writer.backTab();
+      writer.println("}");
+      writer.println();
+    }
+
+    for(Iterator i = deserializeSIDs.iterator(); i.hasNext(); ) {
+      SymbolID l_id = (SymbolID) i.next();
+      writer.println("void");
+      writer.println(IOR.getSkelSerializationName(cls, l_id, false));
+      writer.tab();
+      writer.println("(" + IOR.getStructName(l_id) + " *strct, " +
+                       "struct sidl_rmi_Call__object *pipe, const char *name, sidl_bool copyArg, " +
+                       IOR.getExceptionFundamentalType() + " *exc)");
+      writer.backTab();
+      writer.println("{");
+      writer.tab();
+      writer.println("struct sidl_io_Deserializer__object *__pipe = sidl_io_Deserializer__cast(pipe, exc);");
+      writer.println("if (*exc) return;");
+      writer.println("if (__pipe) {");
+      writer.tab();
+      writer.println(C.getExceptionType() + " __throwaway__;");
+      writer.println(IOR.getSymbolName(l_id) + "__deserialize(strct, __pipe, name, copyArg, exc);");
+      writer.println("sidl_io_Deserializer_deleteRef(__pipe, &__throwaway__);");
+      writer.backTab();
+      writer.println("}");
+      writer.backTab();
+      writer.println("}");
+      writer.println();
+    }
+
+   writer.printlnUnformatted("#endif /*WITH_RMI*/");
+
+
+  }
+
   /**
    * For non-<code>static</code> methods, write an assignment statement to
    * initialize a particular membor of the entry point
@@ -524,27 +714,31 @@ public class SkelSource {
    * @param id         the name of class that owns the method
    */
   private static void initializeMethodPointer(LanguageWriter writer,
-                                              Method m, SymbolID id)
+                                              Method m, SymbolID id,
+                                              Context context)
+    throws CodeGenerationException
   {
     final String methodName = m.getLongMethodName();
-    final String epv = "epv";
+
     switch (m.getDefinitionModifier()) {
     case Method.FINAL:
     case Method.NORMAL:
       String ename = IOR.getVectorEntry(methodName);
       String sname = C.getMethodSkelName(id, m);
-      if (IOR.supportInterceptors(id) & (!IOR.isBuiltinMethod(methodName)) ) {
-        writeMethodAssignment(writer, epv, ename + IOR.GENERIC_PRE_SUFFIX, 
-                              sname + IOR.GENERIC_PRE_SUFFIX);
-        writeMethodAssignment(writer, epv, ename, sname);
-        writeMethodAssignment(writer, epv, ename + IOR.GENERIC_POST_SUFFIX, 
-                              sname + IOR.GENERIC_POST_SUFFIX);
+      if (IOR.supportHooks(id, context) & (!IOR.isBuiltinMethod(methodName)) ) {
+        Method pre = m.spawnPreHook();
+        Method post = m.spawnPostHook();
+        writeMethodAssignment(writer, s_pre_epv, IOR.getVectorEntry(pre.getLongMethodName()), 
+                              C.getMethodSkelName(id, pre));
+        writeMethodAssignment(writer, s_epv, ename, sname);
+        writeMethodAssignment(writer, s_post_epv, IOR.getVectorEntry(post.getLongMethodName()), 
+                              C.getMethodSkelName(id, post));
       } else {
-        writeMethodAssignment(writer, epv, ename, sname);
+        writeMethodAssignment(writer, s_epv, ename, sname);
       }
       break;
     case Method.ABSTRACT:
-      writeMethodAssignment(writer, epv, IOR.getVectorEntry(methodName), 
+      writeMethodAssignment(writer, s_epv, IOR.getVectorEntry(methodName), 
                             C.NULL);
       break;
     default:
@@ -561,38 +755,51 @@ public class SkelSource {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private static void writeInitializeSEPV(Class cls, LanguageWriterForC writer)
+  private static void writeInitializeSEPV(Class cls, 
+                                          LanguageWriterForC writer,
+                                          Context context)
     throws CodeGenerationException
   {
-    final String epv = "sepv";
-    SymbolID id = cls.getSymbolID();
     writer.openCxxExtern();
     writer.println("void");
-    writer.println(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) 
-      + " *" + epv + ")");
+    if(IOR.supportHooks(cls, context)) {
+      writer.println(C.getSetSEPVName(cls) + "(" + IOR.getSEPVName(cls) + " *"+s_sepv+",");
+      writer.tab();
+      writer.println(IOR.getPreSEPVName(cls) + " *"+s_pre_sepv+", "+
+                     IOR.getPostSEPVName(cls) + " *"+s_post_sepv+")");
+      writer.backTab();
+    } else {
+      writer.println(C.getSetSEPVName(cls) + "(" + IOR.getSEPVName(cls) 
+      + " *" + s_sepv + ")");
+
+    }
+
     writer.println("{");
     writer.tab();
-    if (IOR.supportAssertions(cls)) {
-      writeMethodAssignment(writer, epv, 
+    if (IOR.supportAssertions(cls, context)) {
+      writeMethodAssignment(writer, s_sepv, 
         IOR.getVectorEntry(IOR.getBuiltinName(IOR.CHECK_ERROR, true)), 
-        C.getMethodSkelName(id, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id, 
-                                                     true)));
+        C.getMethodSkelName(cls, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, 
+                                                     context, true)));
     }
     Iterator i = cls.getMethods(false).iterator();
     while (i.hasNext()) {
       Method m = (Method) i.next();
       if (m.isStatic()) {
         String name     = m.getLongMethodName();
-        String skelName = C.getMethodSkelName(id,m);
-        if (IOR.supportInterceptors(cls) && (!IOR.isBuiltinMethod(name,true))) {
-          writeMethodAssignment(writer, epv, IOR.getVectorEntry(name 
-            + IOR.GENERIC_PRE_SUFFIX), skelName + IOR.GENERIC_PRE_SUFFIX);
-          writeMethodAssignment(writer, epv, IOR.getVectorEntry(name), 
+        String skelName = C.getMethodSkelName(cls,m);
+        if (IOR.supportHooks(cls, context) && 
+            (!IOR.isBuiltinMethod(name,true))) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          writeMethodAssignment(writer, s_pre_sepv, IOR.getVectorEntry(pre.getLongMethodName()), 
+                                C.getMethodSkelName(cls, pre));
+          writeMethodAssignment(writer, s_sepv, IOR.getVectorEntry(name), 
                                 skelName);
-          writeMethodAssignment(writer, epv, IOR.getVectorEntry(name 
-            + IOR.GENERIC_POST_SUFFIX), skelName + IOR.GENERIC_POST_SUFFIX);
+          writeMethodAssignment(writer, s_post_sepv, IOR.getVectorEntry(post.getLongMethodName()), 
+                                C.getMethodSkelName(cls, post));
         } else {
-          writeMethodAssignment(writer, epv, IOR.getVectorEntry(name), 
+          writeMethodAssignment(writer, s_sepv, IOR.getVectorEntry(name), 
                                 skelName);
         }
       }
@@ -632,19 +839,21 @@ public class SkelSource {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private static void writeIORCall(Class cls, String funcName, Method iorMethod,
-                                   LanguageWriter writer)
+  private static void writeIORCall(Class cls, 
+                                   String funcName, 
+                                   Method iorMethod,
+                                   LanguageWriter writer,
+                                   Context context)
     throws CodeGenerationException
   {
-    final SymbolID id = cls.getSymbolID();
-    List extArgs = Utilities.extendArgs(id, iorMethod, false);
+    List extArgs = Utilities.extendArgs(cls, iorMethod, false, context);
     Iterator args = extArgs.iterator();
     writer.println("void");
     writer.print(funcName);
     writer.print("(");
     while (args.hasNext()) {
       Argument a = (Argument)args.next();
-      writer.print(IOR.getArgumentWithFormal(a, false, false, false));
+      writer.print(IOR.getArgumentWithFormal(a, context, false, false, false));
       if (args.hasNext()) {
         writer.print(", ");
       }
