@@ -1,7 +1,7 @@
 //
 // File:        StubHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: StubHeader.java 4449 2005-03-21 18:00:39Z leek2 $
+// Revision:    @(#) $Id: StubHeader.java 6196 2007-10-24 23:42:37Z epperly $
 // Description: write C client (i.e., stub) header to a pretty writer stream
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,22 +30,21 @@
 
 package gov.llnl.babel.backend.c;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
-import gov.llnl.babel.backend.ior.IORSource;
+import gov.llnl.babel.backend.rmi.RMIStubHeader;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
-import gov.llnl.babel.symbols.Argument;
-import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Enumeration;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Package;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
-import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.BabelConfiguration;
 import java.util.Collection;
 import java.util.HashSet;
@@ -65,6 +64,8 @@ public class StubHeader {
     BabelConfiguration.getBaseExceptionClass();
   private final static String SIDL_EXCEPTION_INTERFACE =
     BabelConfiguration.getBaseExceptionInterface();
+  private final static String SIDL_RMI_TICKET = 
+    BabelConfiguration.getRMITicket();
 
   public final static String s_epv       = IOR.getEPVVar(IOR.PUBLIC_EPV);
   public final static String s_self      = Utilities.s_self;
@@ -73,14 +74,26 @@ public class StubHeader {
   /**
    * Indices associated with the special, stub-only built-in methods.
    */
-  public final static int DUMP_STATS       = 0;
-  public final static int SET_CHECKING     = 1;
-  public final static int SET_INTERCEPTORS = 2;
+  public final static int DUMP_STATS   = 0;
+  public final static int SET_CHECKING = 1;
+  public final static int SET_HOOKS    = 2;
+
+  /**
+   * Maximum number of rarray arguments allowed in an inlined method.
+   * <= 0 means no inlining of methods with rarray arguments
+   * > 0 allows some inlining of methods with rarray arguments.
+   * It's unclear what a good value of this should be. Each rarray argument
+   * increases the size of the stub which may cause binary file sizes to
+   * increase.
+   */
+  public final static int MAX_RARRAY_INLINE = 1;
 
   private LanguageWriterForC d_writer;
 
   private boolean d_comment_all;
 
+  private Context d_context;
+
   /**
    * This is a convenience utility function that writes the C client
    * header information into the provided language writer output stream.
@@ -97,10 +110,11 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void generateCode(Symbol symbol, LanguageWriterForC writer)
+  public static void generateCode(Symbol symbol, LanguageWriterForC writer,
+                                  Context context)
     throws CodeGenerationException
   {
-    StubHeader header = new StubHeader(writer);
+    StubHeader header = new StubHeader(writer, context);
     header.generateCode(symbol);
   }
 
@@ -111,9 +125,10 @@ public class StubHeader {
    * @param writer the output writer to which the header will
    *               be written. This will not be closed.
    */
-  public StubHeader(LanguageWriterForC writer) {
+  public StubHeader(LanguageWriterForC writer, Context context) {
     d_writer = writer;
-    d_comment_all = !BabelConfiguration.getInstance().getCommentLocalOnly();
+    d_context = context;
+    d_comment_all = !context.getConfig().getCommentLocalOnly();
   }
 
   /**
@@ -134,25 +149,28 @@ public class StubHeader {
   public void generateCode(Symbol symbol) throws CodeGenerationException {
     if (symbol != null) {
       if ( symbol.getSymbolType() == Symbol.PACKAGE ) { 
-	  d_writer.skipIncludeGuard(); 
+        d_writer.skipIncludeGuard(); 
       }
       generatePrologue(symbol);
 
       switch (symbol.getSymbolType()) {
-        case Symbol.CLASS:
-          generateExtendable((Extendable) symbol);
-          break;
-        case Symbol.ENUM:
-          generateEnumeration((Enumeration) symbol);
-          break;
-        case Symbol.INTERFACE:
-          generateExtendable((Extendable) symbol);
-          break;
-        case Symbol.PACKAGE:
-          generatePackage((Package) symbol);
-          break;
-        default:
-          throw new CodeGenerationException("Unsupported symbol type.");
+      case Symbol.CLASS:
+        generateExtendable((Extendable) symbol);
+        break;
+      case Symbol.ENUM:
+        generateEnumeration((Enumeration) symbol);
+        break;
+      case Symbol.INTERFACE:
+        generateExtendable((Extendable) symbol);
+        break;
+      case Symbol.PACKAGE:
+        generatePackage((Package) symbol);
+        break;
+      case Symbol.STRUCT:
+        generateStruct((Struct)symbol);
+        break;
+      default:
+        throw new CodeGenerationException("Unsupported symbol type.");
       }
     } else {
       throw new CodeGenerationException("Unexpected null Symbol.");
@@ -165,11 +183,10 @@ public class StubHeader {
    * @param sym the <code>Symbol</code> whose header will be written.
    */
   private void generatePrologue(Symbol sym) {
-    final SymbolID id     = sym.getSymbolID();
-    String         header = C.getHeaderFile(id);
+     String         header = C.getHeaderFile(sym);
 
     d_writer.writeBanner(sym, header, false, 
-      CodeConstants.C_DESC_STUB_PREFIX + id.getFullName());
+                         CodeConstants.C_DESC_STUB_PREFIX + sym.getFullName());
     d_writer.openHeaderGuard(header);
   }
 
@@ -198,10 +215,9 @@ public class StubHeader {
    * @param enm the <code>Enumeration</code> whose header is being written.
    */
   private void generateEnumeration(Enumeration enm) {
-    SymbolID id = enm.getSymbolID();
-    ArrayMethods ar = new ArrayMethods(id, true);
+    ArrayMethods ar = new ArrayMethods(enm, true, d_context);
 
-    d_writer.generateInclude(IOR.getHeaderFile(id), true);
+    d_writer.generateInclude(IOR.getHeaderFile(enm), true);
     d_writer.println();
 
     d_writer.openCxxExtern();
@@ -226,17 +242,16 @@ public class StubHeader {
    */
   private void generateExtendable(Extendable ext) throws CodeGenerationException
   {
-    SymbolID id = ext.getSymbolID();
 
     /*
      * Output a documentation comment, forward declare the object, and
      * define the symbol typedef.
      */
     d_writer.writeComment(ext, true);
-    d_writer.println(IOR.getObjectName(id) + ";");
-    d_writer.println(IOR.getArrayName(id) + ";");
-    d_writer.print("typedef " + IOR.getObjectName(id) + "* ");
-    d_writer.println(C.getObjectName(id) + ";");
+    d_writer.println(IOR.getObjectName(ext) + ";");
+    d_writer.println(IOR.getArrayName(ext) + ";");
+    d_writer.print("typedef " + IOR.getObjectName(ext) + "* ");
+    d_writer.println(C.getObjectName(ext) + ";");
     d_writer.println();
 
     /*
@@ -253,9 +268,15 @@ public class StubHeader {
     /*
      * Output array method signatures
      */
-    ArrayMethods ar = new ArrayMethods(id, false);
+    ArrayMethods ar = new ArrayMethods(ext, false, d_context);
     ar.generateHeader(d_writer);
 
+    /*
+     * Output any RMI function headers
+     */
+
+    RMIStubHeader.generateCode(ext, d_writer);
+
     /*
      * Conclude the header file by closing block(s) as needed.
      */
@@ -270,8 +291,6 @@ public class StubHeader {
    * @param p the <code>Package</code> whose header is being written.
    */
   private void generatePackage(Package p) {
-    SymbolID id   = p.getSymbolID();
-
     /*
      * Write out the C include files for each of the symbols within
      * the package.
@@ -286,6 +305,86 @@ public class StubHeader {
     generateEpilogue(false);
   }
 
+  private void generateStruct(Struct strct)
+  {
+    d_writer.writeCommentLine("Include IOR for declaration of struct");
+    d_writer.generateInclude(IOR.getHeaderFile(strct), true);
+    d_writer.println("struct sidl_BaseInterface__object;");
+    d_writer.println("struct sidl_io_Serializer__object;");
+    d_writer.println("struct sidl_io_Deserializer__object;");
+    d_writer.println();
+    d_writer.openCxxExtern();
+    d_writer.println();
+    d_writer.beginBlockComment(true);
+    d_writer.println("This function will initializes all pointer types to");
+    d_writer.println("NULL (if any).");
+    d_writer.endBlockComment(true);
+    d_writer.println("void");
+    d_writer.println(IOR.getSymbolName(strct) + "__init(" +
+                     IOR.getStructName(strct) + "* strct);");
+
+    d_writer.println();
+    d_writer.beginBlockComment(true);
+    d_writer.println("This function will delete all resources contained");
+    d_writer.println("inside a struct. It does not free the struct itself.");
+    d_writer.endBlockComment(true);
+    d_writer.println("void");
+    d_writer.println(IOR.getSymbolName(strct) + "__destroy(" +
+                     IOR.getStructName(strct) + "* strct,");
+    d_writer.tab();
+    d_writer.println("struct sidl_BaseInterface__object **exception);");
+    d_writer.backTab();
+
+    d_writer.println();
+    d_writer.beginBlockComment(true);
+    d_writer.println("Copy data from one struct to another.  It is assumed");
+    d_writer.println("that the incoming contents of the destination can");
+    d_writer.println("safely be overwritten without any prior actions.");
+    d_writer.println("If the destination has unfreed strings or active");
+    d_writer.println("object references, this will cause a leak.");
+    d_writer.println("On exit, dest is an independent copy of src.");
+    d_writer.endBlockComment(true);
+    d_writer.println("void");
+    d_writer.println(IOR.getSymbolName(strct) + "__copy(const " +
+                     IOR.getStructName(strct) + "* src, ");
+    d_writer.tab();
+    d_writer.println(IOR.getStructName(strct) + "* dest,");
+    d_writer.println("struct sidl_BaseInterface__object **exception);");
+    d_writer.backTab();
+
+    d_writer.println();
+    d_writer.beginBlockComment(true);
+    d_writer.println("This function will deserialize all resources into");
+    d_writer.println("a struct. It assumes that the contents of arg");
+    d_writer.println("can be safely overridden without any prior action");
+    d_writer.endBlockComment(true);
+    d_writer.println("void");
+    d_writer.println(IOR.getSymbolName(strct) + 
+                     "__deserialize(" + IOR.getStructName(strct) + " *arg,");
+    d_writer.tab();
+    d_writer.println("struct sidl_io_Deserializer__object *pipe,");
+    d_writer.println("const char *name,");
+    d_writer.println("sidl_bool copyArg,");
+    d_writer.println("struct sidl_BaseInterface__object **exception);");
+    d_writer.backTab();
+    d_writer.println();
+    d_writer.beginBlockComment(true);
+    d_writer.println("This function will serialize all resources contained");
+    d_writer.println("inside a struct. It does not free the struct itself.");
+    d_writer.endBlockComment(true);
+    d_writer.println("void");
+    d_writer.println(IOR.getSymbolName(strct) +
+                     "__serialize(const "+ IOR.getStructName(strct) + " *arg,");
+    d_writer.tab();
+    d_writer.println("struct sidl_io_Serializer__object *pipe,");
+    d_writer.println("const char *name,");
+    d_writer.println("sidl_bool copyArg,");
+    d_writer.println("struct sidl_BaseInterface__object **exception);");
+    d_writer.backTab();
+    d_writer.println();
+    generateEpilogue(true);
+  }
+
   /**
    * Generate the list of include files required to satisfy data
    * dependencies within this header file and output the appropriate
@@ -297,16 +396,20 @@ public class StubHeader {
      * Create the set of include symbol identifiers.
      */
     Set includes = new HashSet();
-
+    
     for (Iterator i = ext.getMethods(true).iterator(); i.hasNext(); ) {
       Method method = (Method) i.next();
       includes.addAll(method.getSymbolReferences());
 
       if (!method.getThrows().isEmpty()) {
-        Symbol symbol = Utilities.lookupSymbol(SIDL_EXCEPTION);
-        includes.add(symbol.getSymbolID());
-        symbol = Utilities.lookupSymbol(SIDL_EXCEPTION_INTERFACE);
-        includes.add(symbol.getSymbolID());
+        Symbol symbol = Utilities.lookupSymbol(d_context, SIDL_EXCEPTION);
+        includes.add(symbol);
+        symbol = Utilities.lookupSymbol(d_context, SIDL_EXCEPTION_INTERFACE);
+        includes.add(symbol);
+      }
+      if ( method.getCommunicationModifier() == Method.NONBLOCKING ) { 
+        Symbol symbol = Utilities.lookupSymbol(d_context, SIDL_RMI_TICKET);
+        includes.add(symbol);
       }
     }
 
@@ -320,7 +423,7 @@ public class StubHeader {
      * Remove this symbol from the dependency set and iterate over symbols if
      * there are any remaining symbols in the set.
      */
-    includes.remove(ext.getSymbolID());
+    includes.remove(ext);
     if (!includes.isEmpty()) {
       List entries = Utilities.sort(includes);
       for (Iterator i = entries.iterator(); i.hasNext(); ) {
@@ -330,11 +433,14 @@ public class StubHeader {
       d_writer.println();
     }
     d_writer.generateInclude(C.getHeaderFile(
-              Utilities.lookupSymbol("sidl.io.Serializer").getSymbolID()), 
-              true);
+                                             Utilities.lookupSymbol(d_context, "sidl.rmi.Call")), 
+                             true);
     d_writer.generateInclude(C.getHeaderFile(
-              Utilities.lookupSymbol("sidl.io.Deserializer").getSymbolID()), 
-              true);
+                                             Utilities.lookupSymbol(d_context, "sidl.rmi.Return")), 
+                             true);
+    d_writer.printlnUnformatted("#ifdef SIDL_C_HAS_INLINE");
+    d_writer.generateInclude(IOR.getHeaderFile(ext), true);
+    d_writer.printlnUnformatted("#endif /* SIDL_C_HAS_INLINE */");
   }
 
   /**
@@ -358,10 +464,35 @@ public class StubHeader {
     generateBuiltinPrototypes(id, SET_CHECKING, doStatic);
     generateBuiltinPrototypes(id, DUMP_STATS, doStatic);
   }
+
+  public static boolean stubInlineMethod(Method m)
+  {
+    return
+      !(m.isStatic() || (m.numRarray() > MAX_RARRAY_INLINE));
+  }
+
+  private void generateMethodStub(SymbolID id,
+                                  boolean isInterface,
+                                  Method method,
+                                  boolean docComment)
+    throws CodeGenerationException
+  {
+    final boolean inlineMethod = stubInlineMethod(method);
+    generateMethodSignature(id, isInterface, method, docComment, inlineMethod);
+    if (inlineMethod) {
+      d_writer.printlnUnformatted("#ifdef SIDL_C_HAS_INLINE");
+      StubSource.generateMethodStubBody(d_writer, d_context,
+                                        id, isInterface, method);
+      d_writer.printlnUnformatted("#else");
+      d_writer.println(";");
+      d_writer.printlnUnformatted("#endif /* SIDL_C_HAS_INLINE */");
+      d_writer.println();
+    }
+  }
  
-   /**
-    * Generate method prototypes for the methods in the interface or class.
-    */
+  /**
+   * Generate method prototypes for the methods in the interface or class.
+   */
   private void generateMethodPrototypes(Extendable ext)
     throws CodeGenerationException 
   {
@@ -370,44 +501,65 @@ public class StubHeader {
      * to "self" for this structure.  For classes, self will be the object
      * structure whereas for interfaces it is void*.
      */
-    SymbolID id   = ext.getSymbolID();
 
     /*
      * Output the normal and remote constructors if this extendable is a class.
      */
     if (!ext.isAbstract()) {
       d_writer.writeComment("Constructor function for the class.", true);
-      d_writer.println(C.getSymbolObjectPtr(id));
-      d_writer.println(C.getFullMethodName(id, "_create") + "(void);");
+      d_writer.println(C.getSymbolObjectPtr(ext));
+      d_writer.println(C.getFullMethodName(ext, "_create") + "(sidl_BaseInterface* _ex);");
       d_writer.println();
 
-      d_writer.writeComment("RMI constructor function for the class.", true);
-      d_writer.println(C.getSymbolName(id));
-
-      d_writer.println(C.getFullMethodName(id, "_createRemote") 
-        + "(const char *, sidl_BaseInterface *_ex);");
+      if(!IOR.isSIDLSymbol(ext)) {
+        d_writer.writeComment("Wraps up the private data struct pointer (" +
+                              C.getDataName(ext) +
+                              ") passed in rather than running the constructor.", 
+                              true);
+        d_writer.println(C.getSymbolName(ext));
+        
+        d_writer.println(C.getFullMethodName(ext, "_wrapObj") 
+                         + "(void * data, sidl_BaseInterface *_ex);");
+      }
 
       d_writer.println();
 
     }
 
+    
+    d_writer.printlnUnformatted("#ifdef WITH_RMI");
+    d_writer.println();
+
+    d_writer.writeComment("RMI constructor function for the class.", true);
+    d_writer.println(C.getSymbolName(ext));
+    
+    d_writer.println(C.getFullMethodName(ext, "_createRemote") 
+                     + "(const char * url, sidl_BaseInterface *_ex);");
+    
+    d_writer.println();
+    
+    d_writer.writeComment("RMI connector function for the class.(addrefs)", true);
+    d_writer.println(C.getSymbolName(ext));
+    
+    d_writer.println(C.getFullMethodName(ext, "_connect") 
+                     + "(const char *, sidl_BaseInterface *_ex);");
+    d_writer.println();                 
+ 
+    d_writer.println();
+    d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
+
     /*
-     * Output the assertion and/or interceptor method(s) if appropriate.
+     * Output the assertion and/or hooks method(s) if appropriate.
      */
     boolean hasStatic = ext.hasStaticMethod(true);
     if (!ext.isAbstract() && !ext.isInterface()) {
-      if (IOR.supportAssertions(ext)) {
+      if (IOR.supportAssertions(ext, d_context)) {
         if (hasStatic) {
-          generateAssertionPrototypes(id, true);
+          generateAssertionPrototypes(ext, true);
         }
-        generateAssertionPrototypes(id, false);
-      }
-      if (IOR.supportInterceptors(ext)) {
-        if (hasStatic) {
-          generateBuiltinPrototypes(id, SET_INTERCEPTORS, true);
-        }
-        generateBuiltinPrototypes(id, SET_INTERCEPTORS, false);
+        generateAssertionPrototypes(ext, false);
       }
+
     }
 
     /*
@@ -417,42 +569,75 @@ public class StubHeader {
     Collection methods = ext.getMethods(true);
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method method = (Method) m.next();
-      generateMethodSignature(id, isIfc, method, 
-                              d_comment_all || ext.isLocal(method));
+      generateMethodStub(ext, isIfc, method, d_comment_all || ext.isLocal(method));
       d_writer.println();
+      if ( method.getCommunicationModifier() == Method.NONBLOCKING ) { 
+        Method send = method.spawnNonblockingSend();
+        generateMethodSignature(ext, isIfc, send, d_comment_all || ext.isLocal(method), false);
+        d_writer.println();
+        Method recv = method.spawnNonblockingRecv();
+        generateMethodSignature(ext, isIfc, recv, d_comment_all || ext.isLocal(method), false);
+        d_writer.println();        
+      }
     }
-
+      
     /*
      * Output the cast methods for the class or interface.
      */
-    d_writer.writeComment("Cast method for interface and class type "
-      + "conversions.", true);
-    d_writer.println(C.getSymbolObjectPtr(id));
-    d_writer.println(C.getFullMethodName(id, "_cast") + "(");
+    d_writer.writeComment("Cast method for interface and class type conversions.", true);
+    d_writer.println(C.getSymbolObjectPtr(ext));
+    d_writer.println(C.getFullMethodName(ext, "_cast") + "(");
     d_writer.tab();
-    d_writer.println("void* obj);");
+    d_writer.println("void* obj,");
+    d_writer.println("sidl_BaseInterface* _ex);");
     d_writer.backTab();
     d_writer.println();
 
     d_writer.writeComment("String cast method for interface and class type "
-      + "conversions.", true);
+                          + "conversions.", true);
     d_writer.println("void*");
-    d_writer.println(C.getFullMethodName(id, "_cast2") + "(");
+    d_writer.println(C.getFullMethodName(ext, "_cast2") + "(");
     d_writer.tab();
     d_writer.println("void* obj,");
-    d_writer.println("const char* type);");
+    d_writer.println("const char* type,");
+    d_writer.println("sidl_BaseInterface *_ex);");
     d_writer.backTab();
     d_writer.println();
     
     /*
      * Output the exec methods for the class or interface.
      */
-    Method m_exec = C.getExecMethod();
-    generateMethodSignature(id, isIfc, m_exec, true);
+    Method m_exec = IOR.getBuiltinMethod(IOR.EXEC,ext, d_context,false);
+    generateMethodStub(ext, isIfc, m_exec, true);
+    
+    //Unused and Doesn't work, just comment it out till we can fix it later.
+    //m_exec = C.getSExecMethod();
+    //generateMethodSignature(id, isIfc, m_exec, true);
 
-    m_exec = C.getSExecMethod();
-    generateMethodSignature(id, isIfc, m_exec, true);
+    Method m_getURL = IOR.getBuiltinMethod(IOR.GETURL,ext, d_context,false);
+    generateMethodStub(ext, isIfc, m_getURL, true);
+
+    Method m_raddRef = IOR.getBuiltinMethod(IOR.RADDREF,ext, d_context,false);
+    generateMethodStub(ext, isIfc, m_raddRef, true);
+
+    Method m_isRemote = IOR.getBuiltinMethod(IOR.ISREMOTE,ext, d_context,false);
+    generateMethodStub(ext, isIfc, m_isRemote, true);
+
+    Method m_isLocal = IOR.getBuiltinMethod(IOR.ISREMOTE,ext, d_context,false);
+    m_isLocal.setMethodName("_isLocal", "");
+    generateMethodSignature(ext, isIfc, m_isLocal, true, false);
+    if(IOR.supportHooks(ext, d_context)) {
+      Method m_hook = IOR.getBuiltinMethod(IOR.HOOKS,ext, d_context,false);
+      generateMethodStub(ext, isIfc, m_hook, true);
+      
+      if (hasStatic) {
+        //Static set_hooks
+        Method m_s_hook = IOR.getBuiltinMethod(IOR.HOOKS,ext, d_context,true);
+        generateMethodStub(ext, isIfc, m_s_hook, true);
+      }
+    }
   }
+  
 
   /**
    * Generate the method signature.
@@ -474,20 +659,27 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateMethodSignature(SymbolID id, boolean isInterface,
-                                       Method method, boolean docComment)
+    private void generateMethodSignature(SymbolID id, 
+                                       boolean isInterface,
+                                       Method method, 
+                                       boolean docComment,
+                                       boolean isInline)
     throws CodeGenerationException
   {
     if (docComment) {
       d_writer.writeComment(method, true);
     }
+    if (isInline) {
+      d_writer.println("SIDL_C_INLINE_DECL");
+    }
     d_writer.println(getReturnString(method.getReturnType()));
     d_writer.print(C.getFullMethodName(id, method));
     d_writer.println("(");
     d_writer.tab();
-    C.generateArgumentList(d_writer, s_self, isInterface, id, method, true, 
+    C.generateArgumentList(d_writer, d_context,
+                           s_self, isInterface, id, method, true, 
                            true, false, true, false, true, true);
-    d_writer.println(");");
+    d_writer.println(isInline ? ")" : ");");
     d_writer.backTab();
   }
 
@@ -497,10 +689,12 @@ public class StubHeader {
     * at the start of this class.  Symbol types and array types require
     * special processing.
     */
-   private static String getReturnString(Type type)
+   private String getReturnString(Type type)
      throws CodeGenerationException
    {
-     return IOR.getReturnString(type, false, true);
+     return (Type.ENUM == type.getDetailedType()) 
+       ? C.getEnumName(type.getSymbolID())
+       : IOR.getReturnString(type, d_context, false, true);
    }
 
 
@@ -518,8 +712,8 @@ public class StubHeader {
       case DUMP_STATS:
         cmt = "Method to dump " + desc +  "assertion checking statistics.";
         break;
-      case SET_INTERCEPTORS:
-        cmt = "Method to enable/disable " + desc +  "interceptor execution.";
+      case SET_HOOKS:
+        cmt = "Method to enable/disable " + desc +  "hooks execution.";
         break;
     }
     return cmt;
@@ -541,8 +735,8 @@ public class StubHeader {
       case DUMP_STATS:
         basename = "_dump_stats";
         break;
-      case SET_INTERCEPTORS:
-        basename = "_set_interceptors";
+      case SET_HOOKS:
+        basename = "_set_hooks";
         break;
     }
     lw.println("void");
@@ -555,13 +749,19 @@ public class StubHeader {
       case SET_CHECKING:
         lw.println("int32_t level,");
         lw.println("double  rate,");
-        lw.println("int32_t resetCounters)" + terminator);
+        lw.println("int32_t resetCounters,");
+        lw.println(IOR.getExceptionFundamentalType() +
+                   "*_ex)" + terminator);
         break;
       case DUMP_STATS:
-        lw.println("const char* filename)"+ terminator);
+        lw.println("const char* filename,");
+        lw.println(IOR.getExceptionFundamentalType() +
+                   "*_ex)" + terminator);
         break;
-      case SET_INTERCEPTORS:
-        lw.println("int32_t on)" + terminator);
+      case SET_HOOKS:
+        lw.println("int32_t on,");
+        lw.println(IOR.getExceptionFundamentalType() +
+                   "*_ex)" + terminator);
         break;
     }
     lw.backTab();
@@ -576,13 +776,13 @@ public class StubHeader {
     String args = null;
     switch (type) {
       case SET_CHECKING:
-        args = "level, rate, resetCounters";
+        args = "level, rate, resetCounters, _ex";
         break;
       case DUMP_STATS:
-        args = "filename";
+        args = "filename, _ex";
         break;
-      case SET_INTERCEPTORS:
-        args = "on";
+      case SET_HOOKS:
+        args = "on, _ex";
         break;
     }
     return args;
