@@ -1,9 +1,9 @@
 //
 // File:        ArrayModule.java
 // Package:     gov.llnl.babel.backend.fortran
-// Copyright:   (c) 2003 The Regents of the University of California
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Copyright:   (c) 2003 Lawrence Livermore National Security, LLC
+// Revision:    @(#) $Revision: 6196 $
+// Date:        $Date: 2007-10-24 16:42:37 -0700 (Wed, 24 Oct 2007) $
 // Description: Create a F90 module to provide stubs for arrays
 // 
 // Copyright (c) 2003, The Regents of the University of Calfornia.
@@ -33,32 +33,34 @@
 package gov.llnl.babel.backend.fortran;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.fortran.AbbrevHeader;
 import gov.llnl.babel.backend.mangler.FortranMangler;
 import gov.llnl.babel.backend.mangler.NameMangler;
-import gov.llnl.babel.backend.writers.LanguageWriter;
+import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
 import java.io.UnsupportedEncodingException;
 import java.security.NoSuchAlgorithmException;
+import java.util.TreeMap;
 
 /**
  * Create a FORTRAN 90 module to provide client-side bindings for arrays of 
  * classes and interfaces.
  */
 public class ArrayModule {
-  private final int GENERIC = -1;
-  private final int BASETYPE = 0;
-  private final int BASEINTERFACE = 1;
-  private final int USERDEFINED = 2;
+  private final static int GENERIC = -1;
+  private final static int BASETYPE = 0;
+  private final static int BASEINTERFACE = 1;
+  private final static int USERDEFINED = 2;
 
   static private final int s_maxArray = BabelConfiguration.getMaximumArray();
 
   private NameMangler d_mang;
 
-  private LanguageWriter d_lw;
+  private LanguageWriterForFortran d_lw;
   
   private SymbolID d_id;
 
@@ -68,6 +70,8 @@ public class ArrayModule {
   
   private boolean d_access;
 
+  private Context d_context;
+
   private boolean d_borrow;
 
   private boolean d_is_base_type;
@@ -175,7 +179,7 @@ public class ArrayModule {
       d_is_base_type = false;
       d_access = false;
       d_borrow = false;
-      d_implID = new SymbolID("sidl.int", new Version());
+      d_implID = new SymbolID("sidl.long", new Version());
       break;
     case Type.CHAR:
     case Type.OPAQUE:
@@ -209,23 +213,25 @@ public class ArrayModule {
    */
   public ArrayModule(SymbolID id,
                      int      type,
-                     LanguageWriter lw)
+                     LanguageWriterForFortran lw,
+                     Context context)
     throws CodeGenerationException
   {
     try {
-      boolean isEnum = (Type.ENUM == type);
       d_id = id;
       d_lw = lw;
       d_type = type;
+      d_context = context;
       d_mang = new FortranMangler(AbbrevHeader.MAXNAME,
                                   AbbrevHeader.MAXUNMANGLED);
       if (type <= Type.STRING) {
-        d_implDataType = Fortran.getReturnString(new Type(type));
+        d_implDataType = Fortran.getReturnString(new Type(type), d_context);
       }
       else {
-        d_implDataType = Fortran.getReturnString(new Type(id));
+        d_implDataType = Fortran.getReturnString(new Type(id, d_context), 
+                                                 d_context);
       }
-      d_intType = Fortran.getReturnString(new Type(Type.INT));
+      d_intType = Fortran.getReturnString(new Type(Type.INT), d_context);
       initImplID(type);
       d_casttype = castType();
     }
@@ -243,17 +249,19 @@ public class ArrayModule {
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *  this means the arguments were incorrect
    */
-  public ArrayModule(LanguageWriter lw)
+  public ArrayModule(LanguageWriterForFortran lw,
+                     Context context)
     throws CodeGenerationException
   {
     try {
+      d_context = context;
       d_id = new SymbolID("sidl.array", new Version());
       d_lw = lw;
       d_type = -1;
       d_mang = new FortranMangler(AbbrevHeader.MAXNAME,
                                   AbbrevHeader.MAXUNMANGLED);
       d_implDataType = null;
-      d_intType = Fortran.getReturnString(new Type(Type.INT));
+      d_intType = Fortran.getReturnString(new Type(Type.INT), d_context);
       // Instead of calling initImplID(type), we do this directly.
       d_access = false;
       d_borrow = false;
@@ -415,7 +423,7 @@ public class ArrayModule {
     d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: src");
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: dest");
-    d_lw.println("integer(selected_int_kind(9)) :: dim");
+    d_lw.println("integer(sidl_int) :: dim");
     d_lw.println("external " + function);
     d_lw.println("dim = " + dim);
     d_lw.println("call " + function + "(src, " + dim +", dest)");
@@ -424,13 +432,6 @@ public class ArrayModule {
     d_lw.println();
   }
 
-  private void generateCopies(int dim)
-    throws UnsupportedEncodingException
-  {
-    generateCopy("copy", dim);
-    generateSmartCopy("smartCopy", dim);
-  }
-
   private void generateEnsure(int dim)
     throws UnsupportedEncodingException
   {
@@ -558,7 +559,7 @@ public class ArrayModule {
 
     d_lw.println("subroutine " + funcname + "(oldType, newType)");
     d_lw.tab();    
-    //d_lw.println("use " + "sidl_"+BabelConfiguration.arrayType(type)+"_array");
+    //d_lw.generateUse("sidl_"+BabelConfiguration.arrayType(type)+"_array");
     d_lw.println("type(sidl__array), intent(in) :: oldType");
     d_lw.println("type("+oldTypeName+"), intent(out) :: newType");
     
@@ -913,12 +914,6 @@ public class ArrayModule {
     d_lw.println();
   }
 
-  private void generateUseStatements() {
-    for(int type = 1; type <= BabelConfiguration.sidl_interface_array; ++type) {
-      d_lw.println("use sidl_" + BabelConfiguration.arrayType(type) + "_array"); 
-    }
-  }
-
   private void writeGenericInterface(String func)
   {
     int i;
@@ -964,10 +959,10 @@ public class ArrayModule {
     d_lw.println("end subroutine " + getPrivateName("access", dim));
   }
 
-  private void generateBorrow(int dim)
+  private void generateBorrow(int dim, boolean common)
     throws UnsupportedEncodingException
   {
-    final String func = "borrow";
+    final String func = (common ? "cborrow" : "borrow");
     final String function = d_mang.shortArrayName
       (d_implID.getFullName(), func, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim) +
@@ -987,7 +982,7 @@ public class ArrayModule {
   private void generateDeclarations()
     throws UnsupportedEncodingException
   {
-    int i, dim;
+    int i;
     if (d_is_base_type) {
       TypeModule.writeArrayType(d_lw, d_id, 
                                 (d_access ? d_implDataType : null));
@@ -1016,6 +1011,7 @@ public class ArrayModule {
       }
       if (d_borrow) {
         writeGenericInterface("borrow");
+        writeGenericInterface("cborrow");
       }
     }
   }
@@ -1044,19 +1040,6 @@ public class ArrayModule {
     generateRef("deleteRef", dim);
   }
 
-  private void generateGenericArrayTypes() {
-    
-    d_lw.println();
-    d_lw.println("type sidl__array");
-    d_lw.tab();
-    d_lw.println("sequence");
-    d_lw.println("integer (selected_int_kind(18)) :: d_array");
-    d_lw.backTab();
-    d_lw.println("end type sidl__array");
-    d_lw.println();
-
-  }
-
   private void generateImplementations()
     throws UnsupportedEncodingException
   {
@@ -1085,7 +1068,8 @@ public class ArrayModule {
           generateAccess(i);
         }
         if (d_borrow) {
-          generateBorrow(i);
+          generateBorrow(i, false);
+          generateBorrow(i, true);
         }
       }
       generateCasts();  //have their own interfal for loop
@@ -1098,17 +1082,21 @@ public class ArrayModule {
     try {
       d_lw.println();
       if (d_is_generic_array) {
-        d_lw.printlnUnformatted("#include \"sidl_array_fAbbrev.h\"");      
+        //d_lw.printlnUnformatted("#include \"sidl_array_fAbbrev.h\"");      
+        d_lw.generateInclude("sidl_array_fAbbrev.h");      
       }
       d_lw.println("module " + Fortran.getArrayModule(d_id));
       d_lw.tab();
+      TreeMap emptyMap = new TreeMap();
+      d_lw.generateUse("sidl", emptyMap);
       if (!d_is_generic_array) { 
         if (!d_is_base_type) {
-          d_lw.println("use " + Fortran.getTypeModule(d_id));
+          d_lw.generateUse(Fortran.getTypeModule(d_id), emptyMap);
         }
       }
-      d_lw.println("use sidl_array_type"); //Everyone needs the generic module!
+      d_lw.generateUse("sidl_array_type", emptyMap); //Everyone needs the generic module!
       d_lw.println();
+      emptyMap = null;
       
       generateDeclarations();
 
