@@ -1,7 +1,7 @@
 //
 // File:        ArrayMethods.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 4434 $
+// Revision:    @(#) $Revision: 6196 $
 // Description: 
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -30,64 +30,68 @@
 
 
 package gov.llnl.babel.backend.fortran;
-import gov.llnl.babel.symbols.SymbolID;
+
+import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
+import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.c.ArrayMethods;
 import gov.llnl.babel.backend.fortran.Fortran;
-import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.BabelConfiguration;
-import gov.llnl.babel.symbols.Version;
 import gov.llnl.babel.backend.writers.LanguageWriter;
+import gov.llnl.babel.symbols.SymbolID;
+import gov.llnl.babel.symbols.Version;
 
 public class FortArrayMethods {
   static private final int s_maxArray = BabelConfiguration.getMaximumArray();
+
+  private Context d_context;
   
   private SymbolID d_id;
 
-  private boolean d_isEnum;
-
   private String d_implArrayType;
 
   private String d_implDataType;
 
+  private String d_valueType;
+
+  private String d_toFortranType;
+
   private SymbolID d_implID;
 
   public FortArrayMethods(SymbolID id,
-                          boolean isEnum) {
+                          boolean isEnum,
+                          Context context) {
     d_id = id;
-    d_isEnum = isEnum;
+    d_context = context;
     d_implArrayType = isEnum 
-      ? "struct sidl_int__array *" 
+      ? "struct sidl_long__array *" 
       : "struct sidl_interface__array *";
+    d_toFortranType = isEnum ? "" : "(ptrdiff_t)";
+    d_valueType = "int64_t";
     d_implDataType = isEnum 
-      ? "(int32_t)" 
-      : "(struct sidl_BaseInterface__object *)";
+      ? "" 
+      : "(struct sidl_BaseInterface__object *)(ptrdiff_t)";
     if (isEnum) {
-      d_implID = new SymbolID("sidl.int", new Version());
+      d_implID = new SymbolID("sidl.long", new Version());
     }
     else {
       d_implID = new SymbolID("sidl.interface", new Version());
     }
   }
 
-  private String getTypeName()
-  {
-    return d_isEnum ? IOR.getEnumName(d_id) : IOR.getObjectName(d_id);
-  }
-
-  private static void writeMethodName(LanguageWriter lw,
+  private void writeMethodName(LanguageWriter lw,
                                       String methodName)
   {
-    lw.println(Fortran.getFortranSymbol() + "(" + methodName.toLowerCase() + ",");
+    lw.println(Fortran.getFortranSymbol(d_context) + "(" + methodName.toLowerCase() + ",");
     lw.printSpaces(18);
     lw.println(methodName.toUpperCase() + ",");
     lw.printSpaces(18);
     lw.println(methodName + ")");
   }
 
-  private static String generateCreateName(SymbolID id, String shortType)
+  private String generateCreateName(SymbolID id, String shortType)
   {
     return IOR.getSymbolName(id) + "__array_create" + shortType 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateCreateSig(LanguageWriter lw,
@@ -121,10 +125,10 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateOneDName(SymbolID id)
+  private String generateOneDName(SymbolID id)
   {
     return IOR.getSymbolName(id) + "__array_create1d" 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateOneDSig(LanguageWriter lw)
@@ -148,10 +152,10 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateTwoDName(SymbolID id, String shortType)
+  private String generateTwoDName(SymbolID id, String shortType)
   {
     return IOR.getSymbolName(id) + "__array_create2d" + shortType 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateTwoDSig(LanguageWriter lw,
@@ -180,10 +184,10 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateDelRefName(SymbolID id)
+  private String generateDelRefName(SymbolID id)
   {
     return IOR.getSymbolName(id) + "__array_deleteRef" 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
   
   private void generateDelRefSig(LanguageWriter lw)
@@ -206,9 +210,9 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateAddRefName(SymbolID id)
+  private String generateAddRefName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_addRef" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_addRef" + Fortran.getMethodSuffix(d_context);
   }
   
   private void generateAddRefSig(LanguageWriter lw)
@@ -231,22 +235,21 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateGetName(SymbolID id)
+  private String generateGetName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_get" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_get" + Fortran.getMethodSuffix(d_context);
   }
 
-  private static String generateGetName(SymbolID id, int num)
+  private String generateGetName(SymbolID id, int num)
   {
     return IOR.getSymbolName(id) + "__array_get" + num 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateGetSig(LanguageWriter lw,
                               int num)
   {
     String methodName = generateGetName(d_id, num);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -254,7 +257,7 @@ public class FortArrayMethods {
     for(int i = 1; i <= num; ++i){
       lw.println(" int32_t *i" + i + ", ");
     }
-    lw.println(" int64_t *result)" );
+    lw.println(" " + d_valueType + " *result)" );
     lw.backTab();
   }
 
@@ -264,7 +267,7 @@ public class FortArrayMethods {
     generateGetSig(lw, num);
     lw.println("{");
     lw.tab();
-    lw.println("*result = (ptrdiff_t)");
+    lw.println("*result = " + d_toFortranType);
     lw.tab();
     lw.println(ArrayMethods.generateGetName(d_implID, num) + "((const " +
                d_implArrayType + ")(ptrdiff_t)*array");
@@ -280,13 +283,12 @@ public class FortArrayMethods {
   private void generateGetSig(LanguageWriter lw)
   {
     String methodName = generateGetName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
     lw.println("(int64_t *array,");
     lw.println(" int32_t indices[],");
-    lw.println(" int64_t *result)" );
+    lw.println(" " + d_valueType + " *result)" );
     lw.backTab();
   }
 
@@ -295,7 +297,7 @@ public class FortArrayMethods {
     generateGetSig(lw);
     lw.println("{");
     lw.tab();
-    lw.println("*result = (ptrdiff_t)");
+    lw.println("*result = " + d_toFortranType);
     lw.tab();
     lw.println(ArrayMethods.generateGetName(d_implID) + "((const " +
                d_implArrayType + ")(ptrdiff_t)*array, indices);");
@@ -304,22 +306,21 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateSetName(SymbolID id)
+  private String generateSetName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_set" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_set" + Fortran.getMethodSuffix(d_context);
   }
 
-  private static String generateSetName(SymbolID id, int num)
+  private String generateSetName(SymbolID id, int num)
   {
     return IOR.getSymbolName(id) + "__array_set" + num 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateSetSig(LanguageWriter lw,
                               int num)
   {
     String methodName = generateSetName(d_id, num);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -327,7 +328,7 @@ public class FortArrayMethods {
     for(int i = 1; i <= num; ++i){
       lw.println(" int32_t *i" + i + ",");
     }
-    lw.println(" int64_t *value)" );
+    lw.println(" " + d_valueType + " *value)" );
     lw.backTab();
   }
 
@@ -342,7 +343,7 @@ public class FortArrayMethods {
     for(int i = 1 ; i <= num; ++i){
       lw.print(", *i" + i);
     }
-    lw.println(", " + d_implDataType + "(ptrdiff_t)*value);");
+    lw.println(", " + d_implDataType + "*value);");
     lw.backTab();
     lw.println("}");
   }
@@ -350,13 +351,12 @@ public class FortArrayMethods {
   private void generateSetSig(LanguageWriter lw)
   {
     String methodName = generateSetName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
     lw.println("(int64_t *array,");
     lw.println("int32_t indices[],");
-    lw.println("int64_t *value)");
+    lw.println(d_valueType + " *value)");
     lw.backTab();
   }
 
@@ -367,20 +367,19 @@ public class FortArrayMethods {
     lw.tab();
     lw.println(ArrayMethods.generateSetName(d_implID) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array, indices, " +
-               d_implDataType + "(ptrdiff_t)*value);");
+               d_implDataType + "*value);");
     lw.backTab();
     lw.println("}");
   }
 
-  private static String generateDimenName(SymbolID id)
+  private String generateDimenName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_dimen" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_dimen" + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateDimenSig(LanguageWriter lw)
   {
     String methodName = generateDimenName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -402,11 +401,11 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateBoundName(SymbolID id,
+  private String generateBoundName(SymbolID id,
                                              String direction)
   {
     return IOR.getSymbolName(id) + "__array_" + direction 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateBoundSig(LanguageWriter lw,
@@ -415,7 +414,6 @@ public class FortArrayMethods {
                                 String bad)
   {
     String methodName = generateBoundName(d_id, direction);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -442,24 +440,24 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateOrderName(SymbolID id,
+  private String generateOrderName(SymbolID id,
                                          String order)
   {
     return IOR.getSymbolName(id) + "__array_is"
       + order.substring(0,1).toUpperCase()
-      + order.substring(1) + "Order" + Fortran.getMethodSuffix();
+      + order.substring(1) + "Order" + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateOrderSig(LanguageWriter lw,
                                 String order)
   {
     String methodName = generateOrderName(d_id, order);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
     lw.println("(int64_t *array,");
-    lw.println(" SIDL_F" + Fortran.getFortranVersion() + "_Bool *result)");
+    lw.println(" SIDL_F" + Fortran.getFortranVersion(d_context) +
+               "_Bool *result)");
     lw.backTab();
   }
 
@@ -476,15 +474,14 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateCopyName(SymbolID id)
+  private String generateCopyName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_copy" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_copy" + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateCopySig(LanguageWriter lw)
   {
     String methodName = generateCopyName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -506,15 +503,14 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateSliceName(SymbolID id)
+  private String generateSliceName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_slice" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_slice" + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateSliceSig(LanguageWriter lw)
   {
     String methodName = generateSliceName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -545,16 +541,15 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateSmartCopyName(SymbolID id)
+  private String generateSmartCopyName(SymbolID id)
   {
     return IOR.getSymbolName(id) + "__array_smartCopy" 
-           + Fortran.getMethodSuffix();
+           + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateSmartCopySig(LanguageWriter lw)
   {
     String methodName = generateSmartCopyName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
@@ -573,15 +568,14 @@ public class FortArrayMethods {
     lw.println("}");
   }
 
-  private static String generateEnsureName(SymbolID id)
+  private String generateEnsureName(SymbolID id)
   {
-    return IOR.getSymbolName(id) + "__array_ensure" + Fortran.getMethodSuffix();
+    return IOR.getSymbolName(id) + "__array_ensure" + Fortran.getMethodSuffix(d_context);
   }
 
   private void generateEnsureSig(LanguageWriter lw)
   {
     String methodName = generateEnsureName(d_id);
-    String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
     lw.tab();
