@@ -1,7 +1,7 @@
 //
 // File:        Fortran.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 4434 $
+// Revision:    @(#) $Revision: 6205 $
 // Description: Collection of static methods for the FORTRAN binding
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -31,21 +31,27 @@
 
 package gov.llnl.babel.backend.fortran;
 
-import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.mangler.NameMangler;
+import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.Comment;
+import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.FortranExprString;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
 import java.util.ArrayList;
-import java.util.Collection;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 
 /**
  * Provide a collection of static methods to provide the mapping of sidl
@@ -56,6 +62,8 @@ import java.util.List;
 public class Fortran implements CodeConstants 
 {
 
+  public static final String s_return = "retval";
+
   /**
    * The order of all of the following arrays must match the ordering of 
    * the constants in the Type class.  A separate version is maintained to
@@ -75,7 +83,8 @@ public class Fortran implements CodeConstants
     "integer*8",                // long
     "integer*8",                // opaque
     "character*(*)",            // string
-    "integer*4",                // enum
+    "integer*8",                // enum
+    "void",                     // struct
     "integer*8",                // class
     "integer*8",                // interface
     "void",                     // package
@@ -86,17 +95,33 @@ public class Fortran implements CodeConstants
     "void",  					// void
     "logical",  				// bool
     "character (len=1)",  			// character
-    "complex (selected_real_kind(15, 307))",	// dcomplex
-    "real (selected_real_kind(15, 307))",	// double
-    "complex (selected_real_kind(6, 37))",	// fcomplex
-    "real (selected_real_kind(6, 37))",		// float
-    "integer (selected_int_kind(9))",		// int
-    "integer (selected_int_kind(18))",		// long
-    "integer (selected_int_kind(18))",		// opaque
+    "complex (kind=sidl_dcomplex)",             // dcomplex
+    "real (kind=sidl_double)",          	// double
+    "complex (kind=sidl_fcomplex)",     	// fcomplex
+    "real (kind=sidl_float)",		        // float
+    "integer (kind=sidl_int)",		        // int
+    "integer (kind=sidl_long)",		        // long
+    "integer (kind=sidl_opaque)",		// opaque
     "character (len=*)",       			// string
-    "integer (selected_int_kind(9))"		// enum
+    "integer (kind=sidl_enum)",		        // enum
+    "void"                                      // struct
   };
   
+  private final static String s_f03_types[] = {
+    "void",  					// void
+    "logical",  				// bool
+    "character (len=1)",  			// character
+    "complex (kind=sidl_dcomplex)",             // dcomplex
+    "real (kind=sidl_double)",          	// double
+    "complex (kind=sidl_fcomplex)",     	// fcomplex
+    "real (kind=sidl_float)",		        // float
+    "integer (kind=sidl_int)",		        // int
+    "integer (kind=sidl_long)",		        // long
+    "integer (kind=sidl_opaque)",		// opaque
+    "character (len=*)",       			// string
+    "integer (kind=sidl_enum)"		        // enum
+  };
+
   /**
    * This array stores the name of the C type that corresponds to the FORTRAN
    * type for a particular sidl type.  For example, an opaque maps into
@@ -115,7 +140,8 @@ public class Fortran implements CodeConstants
     "int64_t",                  // long
     "int64_t",                  // opaque
     "char *",                   // string
-    "int32_t",                  // enum
+    "int64_t",                  // enum
+    "void",                   // struct
     "int64_t",                  // class
     "int64_t",                  // interface
     "void",                     // package
@@ -159,17 +185,17 @@ public class Fortran implements CodeConstants
    * with underline characters.
    */
   public static String getSymbolNameForFile(SymbolID id) {
-    BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
-		return id.getFullName().replace('.', '_');
+    return id.getFullName().replace('.', '_');
   }
 	
-	/** 
-	 * Same as above method, but use an extra argument for determining 
-	 * when the file whose name is to be generated corresponds to an impl.
-	 */
-  public static String getSymbolNameForFile(SymbolID id, boolean isImpl) {
-    BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
-    if (s_babel_config.makePackageSubdirs() && s_babel_config.getShortFileNames()) {
+  /** 
+   * Same as above method, but use an extra argument for determining 
+   * when the file whose name is to be generated corresponds to an impl.
+   */
+  public static String getSymbolNameForFile(SymbolID id, boolean isImpl,
+                                            Context context) {
+    if (context.getConfig().makePackageSubdirs() && 
+        context.getConfig().getShortFileNames()) {
       return id.getShortName().replace('.','_');
     } else {
       return id.getFullName().replace('.', '_');
@@ -193,6 +219,19 @@ public class Fortran implements CodeConstants
     return getSymbolNameForFile(id) + "_fAbbrev.h";
   }
 
+  /**
+   * Return the name of the stub header file for a particular symbol.
+   * The stub header file is a C file that globablly externs a few 
+   * special functions in the fStub.
+   *
+   * @param id  the symbol whose stub file will be returned
+   * @return the filename of a C file containing the implementation of
+   *         the FORTRAN stubs.
+   */
+  public static String getHeaderFile(SymbolID id) {
+    return getSymbolNameForFile(id) + "_fStub.h";
+  }
+
   /**
    * Return the name of the F77 stub documentation file for a particular
    * symbol. The documentation file is FORTRAN pseudo-code to
@@ -287,14 +326,15 @@ public class Fortran implements CodeConstants
     return result;
   }
 
-  static public String arrayIndices(Collection indices)
+  static public String arrayIndices(List indices)
   {
     StringBuffer result = new StringBuffer();
     Iterator i = indices.iterator();
     result.append('(');
     while (i.hasNext()) {
+      AssertionExpression ae = (AssertionExpression)i.next();
       result.append("0:");
-      result.append(i.next().toString());
+      result.append(ae.accept(new FortranExprString(), null).toString());
       result.append("-1");
       if (i.hasNext()) {
         result.append(", ");
@@ -342,7 +382,7 @@ public class Fortran implements CodeConstants
     case Type.CLASS:
       return "sidl_interface__array_ensure((struct sidl_interface__array *)";
     case Type.ENUM:
-      return "sidl_int__array_ensure((struct sidl_int__array *)";
+      return "sidl_long__array_ensure((struct sidl_long__array *)";
     default:
       return "sidl_" + arrayType.getTypeString() + "__array_ensure(";
     }
@@ -383,11 +423,16 @@ public class Fortran implements CodeConstants
    *
    * @return the FORTRAN file extension.
    */
-  public static String getImplExtension() {
-    if (isFortran90()) {
+  public static String getImplExtension(Context context) {
+    if (isFortran90(context) || hasBindC(context)) {
       return CodeConstants.C_F90_IMPL_EXTENSION;
     } else {
-      return CodeConstants.C_F77_IMPL_EXTENSION;
+      if (isFortran7731(context)) {
+        return CodeConstants.C_F7731_IMPL_EXTENSION;
+      }
+      else {
+        return CodeConstants.C_F77_IMPL_EXTENSION;
+      }
     }
   }
 
@@ -399,8 +444,9 @@ public class Fortran implements CodeConstants
    * @param id the name of the symbol
    * @return the filename for the sidl object implementation.
    */
-  public static String getImplFile(SymbolID id) {
-    return getSymbolNameForFile(id, true) + "_Impl." + getImplExtension();
+  public static String getImplFile(SymbolID id, Context context) {
+    return getSymbolNameForFile(id, true, context) + "_Impl." + 
+      getImplExtension(context);
   }
 
   /**
@@ -410,8 +456,21 @@ public class Fortran implements CodeConstants
    * @param id the name of the symbol
    * @return the filename for the sidl impl module file.
    */
-  public static String getImplModuleFile(SymbolID id) {
-    return getSymbolNameForFile(id, true) + "_Mod." + getImplExtension();
+  public static String getImplModuleFile(SymbolID id, Context context) {
+    return getSymbolNameForFile(id, true, context) + "_Mod." + 
+      getImplExtension(context);
+  }
+
+  /**
+   * Return the name of the file that hold the derived types for
+   * Fortran/BindC to C/C++ interoperability.
+   *
+   * @param id the name of the symbol
+   * @return the filename for the sidl struct module file.
+   */
+  public static String getStructModuleFile(SymbolID id, Context context) {
+    return getSymbolNameForFile(id, true, context) + "."+ 
+      getImplExtension(context);
   }
 
   /**
@@ -425,9 +484,11 @@ public class Fortran implements CodeConstants
    * @param  method information about the method to be named.
    * @return the name of the function to be used in the FORTRAN stub.
    */
-  public static String getMethodStubName(SymbolID id, Method method) {
+  public static String getMethodStubName(SymbolID id, 
+                                         Method method,
+                                         Context context) {
     return getSymbolName(id) + "_" + method.getLongMethodName() 
-           + getMethodSuffix();
+           + getMethodSuffix(context);
   }
 
   /**
@@ -448,7 +509,7 @@ public class Fortran implements CodeConstants
 
   /**
    * Get the function name that should be used for the FORTRAN super method
-   * in the skels (availible in this Impls) This method does not take into account the
+   * in the skels (availible in this Impls). This method does not take into account the
    * compiler specific issues (i.e. whether the symbol should be all upper
    * or lower case or whether it has underscores appended); it provides
    * the starting name with potentially mixed case.
@@ -457,9 +518,21 @@ public class Fortran implements CodeConstants
    * @param  method information about the method to be named.
    * @return the name of the function to be used in the FORTRAN stub.
    */
-  public static String getMethodSuperName(SymbolID id, Method method) {
-    return getSymbolName(id) + "_" + "super_" + method.getLongMethodName() 
-           + getMethodSuffix();
+  public static String getMethodSuperName(SymbolID id, 
+                                          Method method,
+                                          NameMangler mang,
+                                          Context context)
+    throws CodeGenerationException
+  {
+    
+    try {
+      return mang.shortName(id.getFullName(),
+                            "super_" + method.getLongMethodName(),
+                            getMethodSuffix(context));
+    }
+    catch (java.io.UnsupportedEncodingException uee) {
+      throw new CodeGenerationException(uee.getMessage());
+    }
   }
 
 
@@ -468,9 +541,9 @@ public class Fortran implements CodeConstants
    *
    * @return the version of Fortran being generated
    */
-  public static int getFortranVersion() {
+  public static int getFortranVersion(Context context) {
     int fversion;
-    if (isFortran90()) {
+    if (isFortran90(context) || hasBindC(context)) {
       fversion = CodeConstants.C_F90_VERSION;
     } else {
       fversion = CodeConstants.C_F77_VERSION;
@@ -483,8 +556,8 @@ public class Fortran implements CodeConstants
    *
    * @return the FORTRAN method name suffix.
    */
-  public static String getMethodSuffix() {
-    if (isFortran90()) {
+  public static String getMethodSuffix(Context context) {
+    if (isFortran90(context) || hasBindC(context)) {
       return CodeConstants.C_F90_METHOD_SUFFIX;
     } else {
       return CodeConstants.C_F77_METHOD_SUFFIX;
@@ -505,8 +578,8 @@ public class Fortran implements CodeConstants
    *
    * @return the FORTRAN impl method name suffix.
    */
-  public static String getImplMethodSuffix() {
-    if (isFortran90()) {
+  public static String getImplMethodSuffix(Context context) {
+    if (isFortran90(context) || hasBindC(context)) {
       return CodeConstants.C_F90_IMPL_METHOD_SUFFIX;
     } else {
       return CodeConstants.C_F77_IMPL_METHOD_SUFFIX;
@@ -518,80 +591,88 @@ public class Fortran implements CodeConstants
    *
    * @return the appropriate version of SIDLFortran##Symbol.
    */
-  public static String getFortranSymbol() {
-    return "SIDLFortran" + getFortranVersion() + "Symbol";
+  public static String getFortranSymbol(Context context) {
+    return "SIDLFortran" + getFortranVersion(context) + "Symbol";
   }
 
   /**
    * Generate the name of the array destructor function.
    */
-  public static String getArrayDestructor(SymbolID id) {
-    return getSymbolName(id) + "__array_deleteRef" + getMethodSuffix();
+  public static String getArrayDestructor(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_deleteRef" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array constructor function.
    */
-  public static String getArrayConstructor(SymbolID id) {
-    return getSymbolName(id) + "__array_createRow" + getMethodSuffix();
+  public static String getArrayConstructor(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_createRow" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array set element function.
    */
-  public static String getArraySet(SymbolID id) {
-    return getSymbolName(id) + "__array_set" + getMethodSuffix();
+  public static String getArraySet(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_set" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array set element function.
    */
-  public static String getArraySet(SymbolID id, int numArgs) {
+  public static String getArraySet(SymbolID id, int numArgs, Context context)
+  {
     return getSymbolName(id) + "__array_set" +
-      Integer.toString(numArgs) + getMethodSuffix();
+      Integer.toString(numArgs) + getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array get element function.
    */
-  public static String getArrayGet(SymbolID id) {
-    return getSymbolName(id) + "__array_get" + getMethodSuffix();
+  public static String getArrayGet(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_get" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array get element function.
    */
-  public static String getArrayGet(SymbolID id, int numArgs) {
+  public static String getArrayGet(SymbolID id, int numArgs,
+                                   Context context) {
     return getSymbolName(id) + "__array_get" +
-      Integer.toString(numArgs) + getMethodSuffix();
+      Integer.toString(numArgs) + getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array dimension access function.
    */
-  public static String getArrayDimen(SymbolID id) {
-    return getSymbolName(id) + "__array_dimen" + getMethodSuffix();
+  public static String getArrayDimen(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_dimen" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array lower bound access function.
    */
-  public static String getArrayLower(SymbolID id) {
-    return getSymbolName(id) + "__array_lower" + getMethodSuffix();
+  public static String getArrayLower(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_lower" + 
+      getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array upper bound access function.
    */
-  public static String getArrayUpper(SymbolID id) {
-    return getSymbolName(id) + "__array_upper" + getMethodSuffix();
+  public static String getArrayUpper(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_upper" + getMethodSuffix(context);
   }
 
   /**
    * Generate the name of the array length access function.
    */
-  public static String getArrayLength(SymbolID id) {
-    return getSymbolName(id) + "__array_length" + getMethodSuffix();
+  public static String getArrayLength(SymbolID id, Context context) {
+    return getSymbolName(id) + "__array_length" + getMethodSuffix(context);
   }
 
   /**
@@ -610,19 +691,34 @@ public class Fortran implements CodeConstants
    */
   public static String getMethodImplName(SymbolID id,
                                          Method method,
-                                         NameMangler mang)
+                                         NameMangler mang,
+                                         Context context)
     throws CodeGenerationException
   {
     try {
       return mang.shortName(id.getFullName(), 
                             method.getLongMethodName(),
-                            getImplMethodSuffix());
+                            getImplMethodSuffix(context));
     }
     catch (java.io.UnsupportedEncodingException uee) {
       throw new CodeGenerationException(uee.getMessage());
     }
   }
 
+  /**
+   * Return the extended function name (i.e., one with the full name
+   * prepended).
+   *
+   * @param  id       the name of the symbol who has the method.
+   * @param  baseName the base method name.
+   *
+   * @return the name of the FORTRAN subroutine that implements this method.
+   */
+  public static String getExtendedMethodName(SymbolID id, String baseName)
+  {
+    return id.getFullName().replace('.','_') + "__" + baseName;
+  }
+
   /**
    * Return the function name that should be used for the FORTRAN subroutine
    * that implements a particular SUPER method.  The returned string leaves
@@ -639,13 +735,14 @@ public class Fortran implements CodeConstants
    */
   public static String getMethodSuperImplName(SymbolID id,
                                          Method method,
-                                         NameMangler mang)
+                                         NameMangler mang,
+                                              Context context)
     throws CodeGenerationException
   {
     try {
       return mang.shortName(id.getFullName(), 
                             "super_"+method.getLongMethodName(),
-                            getImplMethodSuffix());
+                            getImplMethodSuffix(context));
     }
     catch (java.io.UnsupportedEncodingException uee) {
       throw new CodeGenerationException(uee.getMessage());
@@ -670,18 +767,18 @@ public class Fortran implements CodeConstants
    * need to pass to FORTRAN, or the type that a C subroutine could expect
    * to receive from a FORTRAN caller.
    *
-   * @parm type  the sidl type description.
+   * @param type  the sidl type description.
    * @return the C type corresponding to the FORTRAN type corresponding
    *         to the sidl type.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    the type is unsupported.
    */
-  public static String getFortranTypeInC(Type type)
+  public static String getFortranTypeInC(Type type, Context context)
     throws CodeGenerationException
   {
     final int t = type.getDetailedType();
     if (t >= 0 && t < s_cTypeMatchingFortran.length) {
-      if (isFortran90()) {
+      if (isFortran90(context) || hasBindC(context)) {
         if (t == s_Fortran_Bool) return s_F90_Bool;
         if (t == Type.ARRAY) return s_F90_Array;
       }
@@ -695,45 +792,58 @@ public class Fortran implements CodeConstants
    *
    * @return the sidl Fortran prefix
    */
-  public static String getFortranPrefix() {
-    return "SIDL_F" + getFortranVersion();
+  public static String getFortranPrefix(Context context) {
+    return "SIDL_F" + getFortranVersion(context);
   }
 
-  public static boolean needsAbbrev()
+  public static boolean needsAbbrev(Context context)
   {
     return ! "f77".
-      equals(BabelConfiguration.getInstance().getTargetLanguage());
+      equals(context.getConfig().getTargetLanguage());
   }
 
-  public static boolean isFortran90()
+  public static boolean isFortran90(Context context)
   {
-    return "f90".equals(BabelConfiguration.getInstance().getTargetLanguage());
+    return "f90".equals(context.getConfig().getTargetLanguage());
+  }
+
+  public static boolean hasBindC(Context context)
+  {
+    return "f03".equals(context.getConfig().getTargetLanguage());
   
   }
 
-  public static String arrayIOR()
+  public static boolean isFortran7731(Context context)
   {
-    return isFortran90() ? ".d_ior" : "";
+    return "f77_31".equals(context.getConfig().getTargetLanguage());
   }
 
-  public static Method createCast(SymbolID id)
+  public static String arrayIOR(Context context)
   {
-    Method m = new Method();
+    return (isFortran90(context) || hasBindC(context)) ? ".d_ior" : "";
+  }
+
+  public static Method createCast(Context context,
+                                  SymbolID id)
+  {
+    Method m = new Method(context);
     m.setMethodName("_cast");
     m.setDefinitionModifier(Method.STATIC);
     String[] s = new String[1];
     s[0] = "Cast method for interface and type conversions.";
     m.setComment(new Comment(s));
-    m.setReturnType(new Type(id));
+    m.setReturnType(new Type(id, context));
     Argument a = new
-      Argument(false, Argument.IN, new Type(Type.OPAQUE), "ref");
+      Argument(Argument.IN, new Type(Type.OPAQUE), "ref");
     m.addArgument(a);
+    m.addThrows(IOR.getRuntimeException(context));
     return m;
   }
 
-  public static  Method createCastTwo(SymbolID id) throws CodeGenerationException
+  public static  Method createCastTwo(Context context, SymbolID id)
+    throws CodeGenerationException
   {
-    Method m = IOR.getBuiltinMethod(IOR.CAST, id).cloneMethod();
+    Method m = IOR.getBuiltinMethod(IOR.CAST, id, context).cloneMethod();
     m.setMethodName("_cast2");
     return m;
   }
@@ -748,11 +858,12 @@ public class Fortran implements CodeConstants
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    the type is unsupported.
    */
-  public static String getReturnString(Type type)
+  public static String getReturnString(Type type,
+                                       Context context)
     throws CodeGenerationException
   {
     final int t = type.getDetailedType();
-    if (isFortran90()) {
+    if (isFortran90(context)) {
       if (t >= 0 && t < s_f90_types.length) {
         return s_f90_types[t];
       } else {
@@ -764,7 +875,43 @@ public class Fortran implements CodeConstants
           Type arrayType = type.getArrayType();
           if (null != arrayType) {
             if (type.isRarray()) {
-              return getReturnString(arrayType);
+              return getReturnString(arrayType, context);
+            }
+            else{
+              if (arrayType.getDetailedType() <= Type.STRING) {
+                return "type(" +
+                  getArrayName(new SymbolID("sidl." + arrayType.getTypeString(),
+                                            new Version()),
+                               type.getArrayDimension()) + ")";
+              }
+              else {
+                return "type(" + getArrayName(arrayType.getSymbolID(),
+                                              type.getArrayDimension()) + ")";
+              }
+            }
+          }
+          else {
+            return "type(sidl__array)";
+          }
+        default:
+          throw new CodeGenerationException("Unknown supported " + t);
+        }
+      }
+    } else if (hasBindC(context)) {
+      if (t >= 0 && t < s_f03_types.length) {
+        return s_f03_types[t];
+      } else {
+        switch(t) {
+        case Type.INTERFACE:
+        case Type.STRUCT:
+          return "type(" + getTypeName(type.getSymbolID()) + ")";
+        case Type.CLASS:
+          return "type(" + getTypeName(type.getSymbolID()) + ")";
+        case Type.ARRAY:
+          Type arrayType = type.getArrayType();
+          if (null != arrayType) {
+            if (type.isRarray()) {
+              return getReturnString(arrayType, context);
             }
             else{
               if (arrayType.getDetailedType() <= Type.STRING) {
@@ -791,9 +938,98 @@ public class Fortran implements CodeConstants
         return s_f77_types[t];
       }
       else if (type.isArray()) {
-        return type.isRarray() ? getReturnString(type.getArrayType()) : "integer*8";
+        return type.isRarray() ? getReturnString(type.getArrayType(), context) 
+          : "integer*8";
       }
       throw new CodeGenerationException("Unknown supported " + t);
     }
   }
+
+
+  /**
+   * Generates include directives for all the Babel clases used in this 
+   * fortran stub or skel
+   * 
+   * @param writer Language writer for C
+   * @param ext Extendible (Class or Interface) to generate dependencies
+    */
+  public static Set generateStubIncludes( LanguageWriterForC writer,
+                                            Extendable ext) 
+    throws CodeGenerationException 
+  {
+    Set includes = new HashSet();
+    includes.addAll(Utilities.sort(ext.getObjectDependencies()));
+   
+    /*    ArrayList dependencies = Utilities.sort(cls.getObjectDependencies());
+          for (Iterator i = dependencies.iterator(); i.hasNext(); ) {
+          
+          
+          for( Iterator i = ext.getMethods(true).iterator(); i.hasNext(); ) { 
+      Method method = (Method) i.next();
+      Set argTypes = method.getSymbolReferences();
+      //includes.addAll(method.getSymbolReferences());
+      for(Iterator j = argTypes.iterator(); j.hasNext();) {
+        SymbolID argID = (SymbolID) j.next();
+        Symbol symbol = (Symbol) Utilities.lookupSymbol(argID);
+        if(symbol.getSymbolType() == Type.CLASS ||
+           symbol.getSymbolType() == Type.INTERFACE)
+          includes.add(argID);
+      }
+    }
+    */
+    
+    if (!includes.isEmpty()){
+      writer.writeComment("Includes for all method dependencies.",false);
+      
+      List entries = Utilities.sort(includes);
+      
+      for (Iterator i = entries.iterator(); i.hasNext(); ) { 
+        String header = Fortran.getHeaderFile( (SymbolID) i.next());
+
+        writer.generateInclude( header, true );
+      }
+    } 
+    return includes;
+  } 
+  
+    public static Method createRemoteMethod(Extendable ext, 
+                                            Context context, 
+                                            boolean isF90) {
+    SymbolID id = ext.getSymbolID();
+    Method ret = new Method(context);
+    Argument url = new Argument(Argument.IN, new Type(Type.STRING), "url");
+    Argument self = new Argument(Argument.OUT, new Type(id, context), "self");
+    if(isF90) {
+      ret.setMethodName("new","Remote");
+    } else {
+      ret.setMethodName("_create","Remote");
+    }
+    ret.setDefinitionModifier(Method.STATIC);
+    ret.setReturnType(new Type(Type.VOID)); 
+    ret.addThrows(context.getSymbolTable().lookupSymbol("sidl.rmi.NetworkException").getSymbolID());
+    ret.addArgument(self);
+    ret.addArgument(url);
+    return ret;
+  }
+
+  public static Method connectRemoteMethod(Extendable ext, 
+                                           Context context,
+                                           boolean isF90) {
+    SymbolID id = ext.getSymbolID();
+    Method ret = new Method(context);
+    Argument url = new Argument(Argument.IN, new Type(Type.STRING), "url");
+    Argument self = new Argument(Argument.OUT, new Type(id, context), "self");
+    if(isF90) {
+      ret.setMethodName("rConnect");
+    } else {
+      ret.setMethodName("_connect");
+    }
+    ret.setDefinitionModifier(Method.STATIC);
+    ret.addThrows(context.getSymbolTable().lookupSymbol("sidl.rmi.NetworkException").getSymbolID());
+    ret.addArgument(self);
+    ret.setReturnType(new Type(Type.VOID)); 
+    ret.addArgument(url);
+    return ret;
+  }
+  
 }
