@@ -1,7 +1,7 @@
 //
 // File:        GenerateClient.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 4434 $
+// Revision:    @(#) $Revision: 6203 $
 // Description: generate C code to allow FORTRAN to access BABEL
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -31,13 +31,13 @@
 
 package gov.llnl.babel.backend.fortran;
 
-import gov.llnl.babel.BabelConfiguration;
-import gov.llnl.babel.CommandLineDriver;
+import gov.llnl.babel.Context;
+import gov.llnl.babel.Generator;
 import gov.llnl.babel.ResourceLoader;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeGenerator;
-import gov.llnl.babel.backend.FileManager;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.fortran.ModuleSource;
 import gov.llnl.babel.backend.fortran.StubDoc;
 import gov.llnl.babel.backend.fortran.StubSource;
@@ -48,9 +48,9 @@ import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
-import java.io.PrintWriter;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.PrintWriter;
 import java.io.Writer;
 import java.util.Iterator;
 import java.util.Set;
@@ -63,8 +63,12 @@ import java.util.TreeSet;
  * This generator creates C source code to provide the glue between
  * FORTRAN clients and the IOR.
  */
-public class GenerateClient implements CodeGenerator 
+public class GenerateClient implements CodeGenerator
 {
+  private String d_name = "f77";
+
+  private Context d_context = null;
+
   private final static int[] s_basic_array_types = {
     Type.BOOLEAN,
     Type.CHAR,
@@ -78,6 +82,21 @@ public class GenerateClient implements CodeGenerator
     Type.STRING
   };
 
+  private final static String [] s_generic_array_types = {
+    // ordering here must match the ordering in sidl_array_type in sidlArray.h
+    "bool",
+    "char",
+    "dcomplex",
+    "double", 
+    "fcomplex",
+    "float",
+    "int", 
+    "long",
+    "opaque",
+    "string",
+    "interface"
+  };
+
   private boolean d_writeBasicArrayStubs = false;
   
   /**
@@ -91,10 +110,9 @@ public class GenerateClient implements CodeGenerator
                              Writer out)
     throws CodeGenerationException
   {
-    final boolean isF90 = Fortran.isFortran90();
     final String context =
       "gov/llnl/babel/backend/fortran/" +
-      BabelConfiguration.getInstance().getTargetLanguage();
+      d_context.getConfig().getTargetLanguage();
     ResourceLoader rl = new ResourceLoader();
     InputStream in = null;
     byte[] buffer = new byte[1024];
@@ -123,7 +141,7 @@ public class GenerateClient implements CodeGenerator
     throws CodeGenerationException
   {
     final String context = "gov/llnl/babel/backend/fortran/" +
-      BabelConfiguration.getInstance().getTargetLanguage();
+      d_context.getConfig().getTargetLanguage();
     ResourceLoader rl = new ResourceLoader();
     InputStream in = null;
     byte[] buffer = new byte[1024];
@@ -148,6 +166,86 @@ public class GenerateClient implements CodeGenerator
     }
   }
 
+  private void generateSIDLTypesFile()
+    throws CodeGenerationException
+  {
+    final Symbol pkg = Utilities.lookupSymbol(d_context, "sidl");
+    final SymbolID id = pkg.getSymbolID();
+    final boolean includeExternal =
+      !d_context.getConfig().excludeExternal();
+    PrintWriter pw = null;
+    LanguageWriterForFortran forWriter = null;
+    if ((includeExternal || Generator.isIncluded(id, d_context)) &&
+        Generator.notExcluded(id, d_context)) {
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
+        try {
+          pw = d_context.getFileManager().createFile(id, Type.PACKAGE,
+                                                    "BASICMODULESRC",
+                                                    "sidl.F90");
+          forWriter = new LanguageWriterForFortran(pw, d_context);
+          forWriter.writeBanner(pkg, "sidl.F90", false,
+                                CodeConstants.C_FORTRAN_MODULE_PREFIX +
+                                id.getFullName());
+          forWriter.println();
+          forWriter.println("module sidl");
+          forWriter.tab();
+          forWriter.writeCommentLine("Define types for common numeric types");
+          forWriter.println("integer, parameter :: sidl_int=selected_int_kind(9)");
+          forWriter.println("integer, parameter :: sidl_long=selected_int_kind(18)");
+          forWriter.println("integer, parameter :: sidl_opaque=selected_int_kind(18)");
+          forWriter.println("integer, parameter :: sidl_arrayptr=selected_int_kind(18)");
+          forWriter.println("integer, parameter :: sidl_iorptr=selected_int_kind(18)");
+          
+          forWriter.println("integer, parameter :: sidl_enum=selected_int_kind(18)");
+          forWriter.println("integer, parameter :: sidl_dcomplex=selected_real_kind(15,307)");
+          forWriter.println("integer, parameter :: sidl_fcomplex=selected_real_kind(6,37)");
+          forWriter.println("integer, parameter :: sidl_double=selected_real_kind(15,307)");
+          forWriter.println("integer, parameter :: sidl_float=selected_real_kind(6,37)");
+          forWriter.println();
+          forWriter.writeCommentLine("Define generic array types");
+          for(int i = 0 ; i < s_generic_array_types.length; ++i) {
+            forWriter.println("integer, parameter :: sidl_" + 
+                              s_generic_array_types[i] +
+                              "_array_t=" + Integer.toString(i+1));
+          }
+          forWriter.backTab();
+          forWriter.println("end module sidl");
+        }
+        finally {
+          if (pw != null) {
+            pw.flush();
+            pw.close();
+          }
+        }
+      }
+      else {
+        try {
+          pw = d_context.getFileManager().createFile(id, Type.PACKAGE,
+                                                    "STUBFORTRANINC",
+                                                    "sidl.inc");
+          forWriter = new LanguageWriterForFortran(pw, d_context);
+          forWriter.writeBanner(pkg, "sidl.inc", false,
+                                CodeConstants.C_FORTRAN_MODULE_PREFIX + 
+                                id.getFullName());
+          forWriter.println();
+          for(int i = 0 ; i < s_generic_array_types.length ; ++i) {
+            final String param = "SIDL_" + 
+              s_generic_array_types[i].toUpperCase() + "_ARRAY";
+            forWriter.println("integer*4 " + param);
+            forWriter.println("parameter (" + param + " = " +
+                              Integer.toString(i+1) + ")");
+          }
+        }
+        finally {
+          if (pw != null) {
+            pw.flush();
+            pw.close();
+          }
+        }
+      }
+    }
+  }
+
   /**
    * Write F90array stubs for all the basic types.
    *
@@ -157,12 +255,12 @@ public class GenerateClient implements CodeGenerator
   private void generateBasicArrayStubs()
     throws CodeGenerationException
   {
-    BabelConfiguration config = BabelConfiguration.getInstance();
     PrintWriter pw = null;
     Writer w = null;
     LanguageWriterForFortran forWriter = null;
-    boolean isF90 = Fortran.isFortran90();
-    boolean includeExternal = !config.excludeExternal();
+    final boolean isF90 = Fortran.isFortran90(d_context);
+    final boolean includeExternal = 
+      !d_context.getConfig().excludeExternal();
 
 
     try {
@@ -174,26 +272,26 @@ public class GenerateClient implements CodeGenerator
         final String typeName = (new Type(type)).getTypeString();
         final SymbolID id = new SymbolID("sidl." + typeName,
                                          new Version());
-        if ((includeExternal || CommandLineDriver.isIncluded(id)) &&
-            CommandLineDriver.notExcluded(id)) {
-          if (isF90) {
+        if ((includeExternal || Generator.isIncluded(id, d_context)) &&
+            Generator.notExcluded(id, d_context)) {
+          if (isF90 || Fortran.hasBindC(d_context)) {
             final String filename = Fortran.getArrayFile(id);
-            pw = FileManager.getInstance().createFile(id, type, 
+            pw = d_context.getFileManager().createFile(id, type, 
                                                       "ARRAYMODULESRCS",
                                                       filename);
-            forWriter = new LanguageWriterForFortran(pw);
-            ArrayModule am = new ArrayModule(id, type, forWriter);
+            forWriter = new LanguageWriterForFortran(pw, d_context);
+            ArrayModule am = new ArrayModule(id, type, forWriter, d_context);
             am.generateStub();
             forWriter.close();
           }
-          w = FileManager.getInstance().createFile(id, type,
+          w = d_context.getFileManager().createFile(id, type,
                                                    "STUBSRCS",
                                                    Fortran.getStubFile(id));
           copyArrayStub(typeName, w);
           w.close();
 
-          if (Fortran.needsAbbrev()) {
-            w = FileManager.getInstance().
+          if (Fortran.needsAbbrev(d_context)) {
+            w = d_context.getFileManager().
               createFile(id, type, "STUBHDRS",
                          Fortran.getStubNameFile(id));
             copyArrayAbbrev(typeName, w);
@@ -233,42 +331,40 @@ public class GenerateClient implements CodeGenerator
     Writer w = null;
     LanguageWriterForFortran forWriter = null;
     try {
-
-      int i;
       final int type = Type.ARRAY;
       final String typeName = "array";
       final SymbolID id = new SymbolID("sidl.array",
                                        new Version());
-      if (Fortran.isFortran90()) {
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
         //final SymbolID id1 = new SymbolID("sidl.",
         //                                 new Version());
         String filename = "sidl_array_array."+CodeConstants.C_F90_IMPL_EXTENSION;
-        pw = FileManager.getInstance().createFile(id, type, 
+        pw = d_context.getFileManager().createFile(id, type, 
                                                   "ARRAYMODULESRCS",
                                                   filename);
-        forWriter = new LanguageWriterForFortran(pw);
-        ArrayModule am = new ArrayModule(forWriter);
+        forWriter = new LanguageWriterForFortran(pw, d_context);
+        ArrayModule am = new ArrayModule(forWriter, d_context);
         am.generateStub();
         forWriter.close();
 
         filename = "sidl_array_type."+CodeConstants.C_F90_IMPL_EXTENSION;
-        pw = FileManager.getInstance().createFile(id, type, 
+        pw = d_context.getFileManager().createFile(id, type, 
                                                   "TYPEMODULESRCS",
                                                   filename);
-        forWriter = new LanguageWriterForFortran(pw);
+        forWriter = new LanguageWriterForFortran(pw, d_context);
         TypeModule.generateCode(null, forWriter);
         //am.generateStub();
         forWriter.close();
 
       }
-      w = FileManager.getInstance().createFile(id, type,
+      w = d_context.getFileManager().createFile(id, type,
                                                "STUBSRCS",
                                                Fortran.getStubFile(id));
       copyArrayStub(typeName, w);
       w.close();
 
-      if (Fortran.needsAbbrev()) {
-        w = FileManager.getInstance().
+      if (Fortran.needsAbbrev(d_context)) {
+        w = d_context.getFileManager().
           createFile(id, type, "STUBHDRS",
                      "sidl_array_fAbbrev.h");
         copyArrayAbbrev(typeName, w);
@@ -314,52 +410,59 @@ public class GenerateClient implements CodeGenerator
     LanguageWriterForFortran forWriter;
     final SymbolID id = symbol.getSymbolID();
     final int type = symbol.getSymbolType();
-    boolean isF90 = Fortran.isFortran90();
+    boolean isF90 = Fortran.isFortran90(d_context);
     switch (type){
     case Symbol.CLASS:
     case Symbol.INTERFACE:
       try {
+        String h = Fortran.getHeaderFile(id);
+        pw       = d_context.getFileManager().createFile(id, type, 
+                                                        "STUBHDRS", h);
+        
+        LanguageWriterForC writer = new LanguageWriterForC(pw, d_context);
+        StubHeader.generateCode(symbol, writer);
+        writer.close();
+        
         String f = Fortran.getStubFile(id);
-        pw       = FileManager.getInstance().createFile(id, type, 
+        pw       = d_context.getFileManager().createFile(id, type, 
                                                         "STUBSRCS", f);
-
-        LanguageWriterForC writer = new LanguageWriterForC(pw);
-        StubSource.generateCode(symbol, writer);
+        writer = new LanguageWriterForC(pw, d_context);
+        StubSource.generateCode(symbol, writer, d_context);
         d_writeBasicArrayStubs = true;
 
-        if (Fortran.needsAbbrev()) {
+        if (Fortran.needsAbbrev(d_context)) {
           String nf = Fortran.getStubNameFile(id);
-          nw        = FileManager.getInstance().
+          nw        = d_context.getFileManager().
             createFile(id, type, "STUBHDRS", nf);
-          writer    = new LanguageWriterForC(nw);
-          AbbrevHeader.generateCode(symbol, writer);
+          writer    = new LanguageWriterForC(nw, d_context);
+          AbbrevHeader.generateCode(symbol, writer, d_context);
           writer.close();
         }
 
-        if (isF90) {
+        if (isF90 || Fortran.hasBindC(d_context)) {
           String mf = Fortran.getModuleFile(id);
-          mw        = FileManager.getInstance().
+          mw        = d_context.getFileManager().
             createFile(id, type, "STUBMODULESRCS", mf);
-	  forWriter = new LanguageWriterForFortran(mw);
+	  forWriter = new LanguageWriterForFortran(mw, d_context);
           forWriter.writeBanner(symbol, mf, false,
                                 CodeConstants.C_FORTRAN_TYPE_MODULE_PREFIX +
                                 symbol.getFullName());
-          ModuleSource.generateCode(symbol, forWriter);
-          ArrayModule am = new ArrayModule(id, type, forWriter);
+          ModuleSource.generateCode(symbol, forWriter, d_context);
+          ArrayModule am = new ArrayModule(id, type, forWriter, d_context);
           am.generateStub();
           forWriter.close();
           String tf = Fortran.getTypeFile(id);
-          mw        = FileManager.getInstance().
+          mw        = d_context.getFileManager().
             createFile(id, type, "TYPEMODULESRCS", tf);
-          forWriter = new LanguageWriterForFortran(mw);
+          forWriter = new LanguageWriterForFortran(mw, d_context);
           TypeModule.generateCode(symbol, forWriter);
           forWriter.close();
         } else {
           String df = Fortran.getStubDocFile(id);
-          dw        = FileManager.getInstance().
+          dw        = d_context.getFileManager().
             createFile(id, type, "STUBDOCS", df);
-	  forWriter = new LanguageWriterForFortran(dw);
-          StubDoc.generateCode(symbol, forWriter);
+	  forWriter = new LanguageWriterForFortran(dw, d_context);
+          StubDoc.generateCode(symbol, forWriter, d_context);
         }
       }
       finally {
@@ -382,43 +485,43 @@ public class GenerateClient implements CodeGenerator
         String nf = Fortran.getStubNameFile(id);
         String f = Fortran.getEnumStubImpl(id);
         d_writeBasicArrayStubs = true;
-        pw = FileManager.getInstance().createFile(id, type, "STUBSRCS", f);
-        LanguageWriterForC clw = new LanguageWriterForC(pw);
-        StubSource.generateCode(symbol, clw);
+        pw = d_context.getFileManager().createFile(id, type, "STUBSRCS", f);
+        LanguageWriterForC clw = new LanguageWriterForC(pw, d_context);
+        StubSource.generateCode(symbol, clw, d_context);
         clw.close();
-        if (Fortran.needsAbbrev()) {
-          nw  = FileManager.getInstance().
+        if (Fortran.needsAbbrev(d_context)) {
+          nw  = d_context.getFileManager().
             createFile(id, type, "STUBHDRS", nf);
-          clw = new LanguageWriterForC(nw);
-          AbbrevHeader.generateCode(symbol, clw);
+          clw = new LanguageWriterForC(nw, d_context);
+          AbbrevHeader.generateCode(symbol, clw, d_context);
           clw.close();
         }
-        if (isF90) {
+        if (isF90 || Fortran.hasBindC(d_context)) {
           String mf = Fortran.getModuleFile(id);
-          mw        = FileManager.getInstance().
+          mw        = d_context.getFileManager().
             createFile(id, type, "STUBMODULESRCS", mf);
-	  forWriter = new LanguageWriterForFortran(mw);
+	  forWriter = new LanguageWriterForFortran(mw, d_context);
           ArrayModule am = new ArrayModule(id, type,
-                                           forWriter);
+                                           forWriter, d_context);
           forWriter.writeBanner(symbol, mf, false,
                                 CodeConstants.C_FORTRAN_TYPE_MODULE_PREFIX +
                                 symbol.getFullName());
-          ModuleSource.generateCode(symbol, forWriter);
+          ModuleSource.generateCode(symbol, forWriter, d_context);
           am.generateStub();
           forWriter.close();
-
+//
           String tf = Fortran.getTypeFile(id);
-          mw        = FileManager.getInstance().
+          mw        = d_context.getFileManager().
             createFile(id, type, "TYPEMODULESRCS", tf);
-          forWriter = new LanguageWriterForFortran(mw);
+          forWriter = new LanguageWriterForFortran(mw, d_context);
           TypeModule.generateCode(symbol, forWriter);
         }
         else {
           f = Fortran.getEnumStubFile(id);
-          pw = FileManager.getInstance().
+          pw = d_context.getFileManager().
             createFile(id, type, "STUBFORTRANINC", f);
-          LanguageWriterForFortran writer = new LanguageWriterForFortran(pw);
-          StubSource.generateCode(symbol, writer);
+          LanguageWriterForFortran writer = new LanguageWriterForFortran(pw, d_context);
+          StubSource.generateCode(symbol, writer, d_context);
           writer.close();
         }
       }
@@ -433,6 +536,29 @@ public class GenerateClient implements CodeGenerator
           mw.close();
         }
       }
+    case Symbol.STRUCT:
+      try {
+        //String h = Fortran.getHeaderFile(id);
+        //pw       = d_context.getFileManager().createFile(id, type, 
+                                                        //"STUBHDRS", h);
+        if (Fortran.needsAbbrev(d_context)) {
+          String nf = Fortran.getStubNameFile(id);
+          nw        = d_context.getFileManager().
+            createFile(id, type, "STUBHDRS", nf);
+          LanguageWriterForC writer    = new LanguageWriterForC(nw, d_context);
+          AbbrevHeader.generateCode(symbol, writer, d_context);
+          writer.close();
+        }
+        
+        //LanguageWriterForC writer = new LanguageWriterForC(pw);
+        //StubHeader.generateCode(symbol, writer);
+        //writer.close();
+
+      } finally {
+        if (nw != null) {
+          nw.close();
+        }
+      }
       break;
     }
   }
@@ -455,7 +581,7 @@ public class GenerateClient implements CodeGenerator
     throws CodeGenerationException 
   {
     Iterator i = symbols.iterator();
-    SymbolTable table = SymbolTable.getInstance();
+    SymbolTable table = d_context.getSymbolTable();
     while (i.hasNext()) {
       SymbolID id     = (SymbolID)i.next();
       Symbol   symbol = table.lookupSymbol(id);
@@ -464,6 +590,7 @@ public class GenerateClient implements CodeGenerator
       }
     }
     if (d_writeBasicArrayStubs) {
+      generateSIDLTypesFile();
       generateBasicArrayStubs();
     }
   }
@@ -484,6 +611,26 @@ public class GenerateClient implements CodeGenerator
     result.add("f77");
     result.add("f77_31");
     result.add("f90");
+    result.add("f03");
     return result;
   }
+
+  public void setName(String name)
+    throws CodeGenerationException
+ {
+    if (getLanguages().contains(name)) {
+      d_name = name;
+    }
+    else {
+      throw new CodeGenerationException
+        ("\"" +name + "\" is not a valid name for the Fortran generator.");
+    }
+  }
+
+  public String getName() { return d_name; }
+
+  public void setContext(Context context)
+  {
+    d_context = context;
+  }
 }
