@@ -1,7 +1,7 @@
 //
 // File:        ImplSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 4454 $
+// Revision:    @(#) $Revision: 6205 $
 // Description: Generate a skeleton implementation in Fortran
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -31,11 +31,13 @@
 
 package gov.llnl.babel.backend.fortran;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.fortran.Fortran;
 import gov.llnl.babel.backend.fortran.StubSource;
 import gov.llnl.babel.backend.mangler.FortranMangler;
@@ -44,6 +46,7 @@ import gov.llnl.babel.backend.mangler.NonMangler;
 import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
+import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
@@ -52,11 +55,16 @@ import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
 import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.Comparator;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.TreeMap;
 
 /**
  * This class provides the ability to write a FORTRAN file with a
@@ -74,6 +82,8 @@ public class ImplSource
    */
   private CodeSplicer          d_splicer;
 
+  private Context              d_context;
+
   /**
    * This is the output device.
    */
@@ -84,11 +94,6 @@ public class ImplSource
    */
   private NameMangler d_mang;
 
-  /**
-   * The version of Fortran being generated.
-   */
-  private final boolean d_isF90 = Fortran.isFortran90();
-
   /**
    * Generate an instance to generate a FORTRAN implementation template.
    * 
@@ -100,12 +105,14 @@ public class ImplSource
    *                  thrown when the JVM has not SHA hash algorithm.
    */
   public ImplSource(LanguageWriterForFortran writer,
-                    CodeSplicer              splicer)
+                    CodeSplicer              splicer,
+                    Context                  context)
     throws java.security.NoSuchAlgorithmException
   {
     d_lw      = writer;
     d_splicer = splicer;
-    if (Fortran.needsAbbrev()) {
+    d_context = context;
+    if (Fortran.needsAbbrev(context)) {
       d_mang =  new FortranMangler(AbbrevHeader.MAXNAME,
                                    AbbrevHeader.MAXUNMANGLED);
     }
@@ -116,11 +123,12 @@ public class ImplSource
 
   public static void useStatementsForSupers(Method m, SymbolID id, 
                                             LanguageWriterForFortran writer,
-                                            CodeSplicer              splicer)
+                                            CodeSplicer              splicer,
+                                            Context                  context)
   throws CodeGenerationException {
     try{
       writer.tab();
-      ImplSource source = new ImplSource(writer, splicer);
+      ImplSource source = new ImplSource(writer, splicer, context);
       source.useStatementsForSupers(m,id);
       writer.backTab();
     } catch (java.security.NoSuchAlgorithmException ex) {
@@ -147,19 +155,26 @@ public class ImplSource
   {
     final Type argType = a.getType();
     StringBuffer result = 
-      new StringBuffer(Fortran.getReturnString(argType));
-    if (d_isF90) {
+      new StringBuffer(Fortran.getReturnString(argType, d_context));
+    if (Fortran.isFortran90(d_context) ) {
       if (argType.isRarray()) {
         result.append(", dimension");
-        result.append(Fortran.arrayIndices(argType.getArrayIndices()));
+        result.append(Fortran.arrayIndices(argType.getArrayIndexExprs()));
       }
       result.append(" :: ");
       result.append(a.getFormalName());
+    } else if (Fortran.hasBindC(d_context) ) {
+      if (argType.isRarray()) {
+        result.append(", dimension");
+        result.append(Fortran.arrayIndices(argType.getArrayIndexExprs()));
+      } 
+      result.append(" :: ");
+      result.append(a.getFormalName());
     } else {
       result.append(' ');
       result.append(a.getFormalName());
       if (argType.isRarray()) {
-        result.append(Fortran.arrayIndices(argType.getArrayIndices()));
+        result.append(Fortran.arrayIndices(argType.getArrayIndexExprs()));
       }
     }
     return result.toString();
@@ -175,7 +190,7 @@ public class ImplSource
     HashSet set = new HashSet(refs.size() + 2);
     set.addAll(refs);
     if (!m.getThrows().isEmpty()) {
-      Symbol baseInterface = SymbolTable.getInstance().lookupSymbol
+      Symbol baseInterface = d_context.getSymbolTable().lookupSymbol
         (BabelConfiguration.getBaseInterface());
       set.add(baseInterface.getSymbolID());
     }
@@ -229,33 +244,141 @@ public class ImplSource
     return result;
   }
 
+  private static String uniqueName(String name, int num)
+  {
+    final String numStr = Integer.toString(num);
+    if (name.length() + numStr.length() + 1 <= 31) {
+      return name + '_' + numStr;
+    }
+    return name.substring(0,30 - numStr.length()) + '_' + numStr;
+  }
+
+  private static F90Method convertMethod(Method m, SymbolID id, 
+                                         Context context)
+    throws CodeGenerationException
+  {
+    List extendedArgs = StubSource.extendArgs(id, m, context, true);
+    return new F90Method(m.getShortMethodName(), extendedArgs);
+  }
+
+  public Map findCollisions(Map methodsSeen,
+                            Symbol sym)
+    throws CodeGenerationException
+  {
+    HashMap result = new HashMap();
+    if (sym instanceof Extendable) {
+      final Extendable ext = (Extendable)sym;
+      final SymbolID id = ext.getSymbolID();
+      Collection methods = ext.getMethods(true);
+      Iterator i = methods.iterator();
+      while (i.hasNext()) {
+        F90Method f90m = convertMethod((Method)i.next(), id, d_context);
+        if (methodsSeen.containsKey(f90m)) {
+          final Integer value = (Integer)methodsSeen.get(f90m);
+          final int newValue = value.intValue() + 1;
+          methodsSeen.put(f90m, new Integer(newValue));
+          result.put(f90m.getName(), 
+                     uniqueName(f90m.getName(), newValue));
+        }
+        else {
+          methodsSeen.put(f90m, new Integer(1));
+        }
+      }
+    }
+    return result;
+  }
+
+  private class RefComp implements Comparator
+  {
+    public int compare(Object o1, Object o2)
+    {
+      final SymbolID sid1 = (SymbolID)o1;
+      final SymbolID sid2 = (SymbolID)o2;
+      final SymbolTable table = d_context.getSymbolTable();
+      final Symbol sym1 = table.lookupSymbol(sid1);
+      final Symbol sym2 = table.lookupSymbol(sid2);
+      final int depth1 = (sym1 == null) ? -1 : sym1.getDepth();
+      final int depth2 = (sym2 == null) ? -1 : sym2.getDepth();
+      final int depthCompare = depth1 - depth2;
+      if (depthCompare == 0) {
+        return sid1.compareTo(sid2);
+      }
+      return depthCompare;
+    }
+  }
+
+  private List sortReferences(Collection refs)
+  {
+    Object[] objs = refs.toArray();
+    Arrays.sort(objs, new RefComp());
+    ArrayList result = new ArrayList(objs.length);
+    for(int i = 0; i < objs.length; ++i) {
+      result.add(objs[i]);
+    }
+    return result;
+  }
+
   private void addUseForReferences(Method m,
                                    SymbolID id)
     throws CodeGenerationException
   {
-    Collection refs      = extendedReferences(m, id);
+    List refs      = sortReferences(extendedReferences(m, id));
     HashSet appeared = new HashSet(refs.size());
+    HashMap methodsSeen = new HashMap();
+    d_lw.generateUse("sidl", methodsSeen);
+    d_lw.generateUse(BabelConfiguration.getNotImplemented().replace('.', '_'), methodsSeen);
     Iterator i = refs.iterator();
     while (i.hasNext()) {
       final SymbolID refid = (SymbolID)i.next();
       if (!appeared.contains(refid)) {
-        d_lw.println("use " + Fortran.getModule(refid));
+        final Map renameMap = 
+          findCollisions(methodsSeen, Utilities.lookupSymbol(d_context,refid));
+        d_lw.generateUse(Fortran.getModule(refid), renameMap);
         appeared.add(refid);
       }
     }
     // do array references now
-    refs = arrayReferences(m);
+    refs = sortReferences(arrayReferences(m));
     appeared.clear();
     i = refs.iterator();
+    TreeMap emptyMap = new TreeMap();
     while (i.hasNext()) {
       final SymbolID refid = (SymbolID)i.next();
       if (!appeared.contains(refid)) {
-        d_lw.println("use " + Fortran.getArrayModule(refid));
+        d_lw.generateUse(Fortran.getArrayModule(refid), emptyMap);
         appeared.add(refid);
       }
     }
   }
 
+  private String[] initialContent(SymbolID id,
+                                  String method)
+  {
+    if (IOR.getBuiltinName(IOR.CONSTRUCTOR).equals(method)) {
+      final String fullName = id.getFullName().replace('.', '_');
+      String defaultCtor[] = {
+        "! boilerplate contructor",
+        "type(" + fullName + "_wrap" + ") :: dp",
+        "allocate(dp%d_private_data)",
+        "! initialize contents of dp%d_private_data here",
+        "call " + fullName + "__set_data_m(self, dp)"
+      };
+      return defaultCtor;
+    }
+    else if (IOR.getBuiltinName(IOR.DESTRUCTOR).equals(method)) {
+      final String fullName = id.getFullName().replace('.', '_');
+      String defaultDtor[] = {
+        "! boilerplate contructor",
+        "type(" + fullName + "_wrap" + ") :: dp",
+        "call " + fullName + "__get_data_m(self, dp)",
+        "! release resources in dp%d_private_data here",
+        "deallocate(dp%d_private_data)",
+      };
+      return defaultDtor;
+    }
+    return null;
+  }
+
   /**
    * Write the FORTRAN server side implementation subroutine 
    * that corresponds to a sidl method. This writes the method
@@ -274,12 +397,17 @@ public class ImplSource
                                  SymbolID id)
     throws CodeGenerationException
   {
-    List extendedArgs    = StubSource.extendArgs(id, m, true);
+    List extendedArgs    = StubSource.extendArgs(id, m, d_context, true);
     String splicerTag    = id.getFullName() + '.' + m.getLongMethodName();
     String splicerTagUse = splicerTag + s_use;
-    String methodImplName = Fortran.getMethodImplName(id, m, d_mang);
+    String methodImplName = Fortran.getMethodImplName(id, m, d_mang, 
+                                                      d_context);
+
+    d_lw.println();
+    d_lw.println();
+
     d_lw.writeComment(m, false);
-    if (d_isF90) {
+    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
       d_lw.print("recursive ");
     }
     d_lw.print("subroutine " + methodImplName + "(");
@@ -291,19 +419,37 @@ public class ImplSource
       }
     }
     d_lw.println(")");
-    if (d_isF90) {
+
+    if (Fortran.hasBindC(d_context)){
+      d_lw.tab();
+      d_lw.println("use, intrinsic :: iso_c_binding");
+      d_lw.backTab();
+    }
+
+    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
       d_lw.tab();
       addUseForReferences(m, id);
-      d_lw.println("use " + id.getFullName().replace('.','_') + "_impl");
+      d_lw.generateUse( id.getFullName().replace('.','_') + "_impl", new TreeMap());
       d_splicer.splice (splicerTagUse, d_lw, "use statements");
     }
+
     d_lw.println("implicit none");
     i = Fortran.reorderArguments(extendedArgs).iterator();
     while (i.hasNext()) {
       Argument a = (Argument)i.next();
       String decl = getArgumentDeclaration(a); 
-      if (d_isF90) {
-        d_lw.println(decl + " ! " + a.getModeString());
+
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
+        //Retarded special case to make ctor2 work
+        if(m.equals(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2,id,d_context)) &&
+           a.getType().getType() == Type.OPAQUE && 
+           a.getFormalName().equals("private_data")) {
+          d_lw.println("type(" + id.getFullName().replace('.', '_') + 
+                       "_wrap" + ") :: private_data");
+        } else {
+          d_lw.println(decl);
+          d_lw.writeCommentLine(a.getModeString());
+        }
       }
       else {
         d_lw.writeCommentLine(a.getArgumentString());
@@ -311,18 +457,73 @@ public class ImplSource
       }
     }
     d_lw.println();
-    if (d_isF90) {
+    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
       d_lw.backTab();
     }
-    d_splicer.splice(splicerTag, d_lw, 
-                     m.getShortMethodName() + " method");
-    if (d_isF90) {
+
+    spliceInitialContent(m, splicerTag);
+    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
       d_lw.println("end subroutine " + methodImplName);
     } else {
       d_lw.println("end");
     }
   }
 
+  private void spliceInitialContent(Method m, String splicerTab) {
+    String sName   = m.getShortMethodName();
+    String alt_msg          = sName + " method";
+    String defaultComment[] = {
+      "This method has not been implemented" 
+    };
+    
+    if (  (!sName.endsWith("_pre")) && (!sName.endsWith("_post")) ) {
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
+        String defaultCode[] = {};
+        String notImplExWithCCA[] = {
+          "  ! DO-DELETE-WHEN-IMPLEMENTING exception.begin(" + splicerTab + ")",
+          "  type(sidl_BaseInterface_t) :: throwaway",
+          "  type(sidl_NotImplementedException_t) :: notImpl",
+          "  call new(notImpl, exception)",
+          "  call setNote(notImpl, \'Not Implemented\', exception)",
+          "  call cast(notImpl, exception,throwaway)",
+          "  call deleteRef(notImpl,throwaway)",
+          "  return",
+          "  ! DO-DELETE-WHEN-IMPLEMENTING exception.end(" + splicerTab + ")",
+        };
+        if (  d_context.getConfig().getCCAMode() == true ) { 
+          defaultCode = notImplExWithCCA;
+        }
+        d_splicer.splice(splicerTab, d_lw, alt_msg, defaultComment, 
+                         defaultCode);
+      } else {
+        String defaultCode[] = {};
+        String notImplExWithCCA[] = {
+          "C       DO-DELETE-WHEN-IMPLEMENTING exception.begin(" + splicerTab + ")",
+          "        integer*8 throwaway",
+          "        call sidl_NotImplementedException__create_f",
+          "     $      (exception, throwaway)",
+          "        if (exception .ne. 0) then",
+          "           call sidl_NotImplementedException_setNote_f(",
+          "     $         exception,", 
+          "     $         \'This method has not been implemented\',",
+          "     $         throwaway)",
+          "        endif",
+          "        return",
+          "C       DO-DELETE-WHEN-IMPLEMENTING exception.end(" + splicerTab + ")"
+        };
+        if ( d_context.getConfig().getCCAMode() == true ) { 
+            defaultCode = notImplExWithCCA;
+        }
+        d_splicer.splice(splicerTab, d_lw, alt_msg, defaultComment, 
+                         defaultCode);
+      } 
+    } else {
+      d_splicer.splice(splicerTab, d_lw, alt_msg, defaultComment);
+    }
+
+  }
+  
+
   /**
    * Add the builtin methods to the list of methods that must be
    * implemented.
@@ -334,20 +535,20 @@ public class ImplSource
     final SymbolID id = ext.getSymbolID();
     ArrayList  extendedMethods = new ArrayList(localMethods.size()+2);
     extendedMethods.
-      add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id));
+      add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context));
     extendedMethods.
-      add(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id));
+      add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context));
     extendedMethods.
-	add(IOR.getBuiltinMethod(IOR.LOAD, id));
+      add(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context));
+    extendedMethods.
+	add(IOR.getBuiltinMethod(IOR.LOAD, id, d_context));
     extendedMethods.addAll(localMethods);
     return extendedMethods;
   }
 
   private void addMiscCodeSection(String sectionName)
   {
-    d_splicer.splice(sectionName, 
-                     d_lw, 
-                     "extra code");
+    d_splicer.splice(sectionName, d_lw, "extra code");
   }
 
   /**
@@ -357,20 +558,23 @@ public class ImplSource
   private void writeIncludes(Class cls)
     throws CodeGenerationException
   {
-    Iterator i = ModuleSource.extendedReferences(cls).iterator();
-    d_lw.disableLineBreak();
-    while (i.hasNext()) {
-      SymbolID id = (SymbolID)i.next();
-      d_lw.printlnUnformatted("#include \"" +
-                              Fortran.getStubNameFile(id) + "\"");
+    Iterator i = ModuleSource.extendedReferences(cls, d_context).iterator();
+    try {
+      d_lw.pushLineBreak(false);
+      d_lw.generateInclude( Fortran.getStubNameFile(d_context.getSymbolTable().lookupSymbol(BabelConfiguration.getNotImplemented()).getSymbolID()) );
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        d_lw.generateInclude( Fortran.getStubNameFile(id) );
+      }
+      i = cls.getBasicArrayRefs().iterator();
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        d_lw.generateInclude( Fortran.getStubNameFile(id) );
+      }
     }
-    i = cls.getBasicArrayRefs().iterator();
-    while (i.hasNext()) {
-      SymbolID id = (SymbolID)i.next();
-      d_lw.printlnUnformatted("#include \"" +
-                              Fortran.getStubNameFile(id) + "\"");
+    finally {
+      d_lw.popLineBreak();
     }
-    d_lw.enableLineBreak();
   }
 
   /**
@@ -390,13 +594,13 @@ public class ImplSource
     Collection methods = extendMethods(cls);
     final SymbolID id = cls.getSymbolID();
 
-    d_lw.writeBanner(cls, Fortran.getImplFile(id), true, 
+    d_lw.writeBanner(cls, Fortran.getImplFile(id, d_context), true, 
                      CodeConstants.C_DESC_IMPL_PREFIX + id.getFullName());
     d_lw.println();
     d_lw.writeComment(cls, false);
     d_lw.println();
 
-    if (Fortran.needsAbbrev()) {
+    if (Fortran.needsAbbrev(d_context)) {
       writeIncludes(cls);
     }
 
@@ -408,10 +612,13 @@ public class ImplSource
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        d_lw.println();
-        d_lw.println();
-
-        extendAndGenerate(m, id);
+        if ( (IOR.supportHooks(cls, d_context)) && (!m.isBuiltIn()) ) {
+          extendAndGenerate(m.spawnPreHook(), id);
+          extendAndGenerate(m, id);
+          extendAndGenerate(m.spawnPostHook(false, false), id);
+        } else {
+          extendAndGenerate(m, id);
+        }
       }
     }
     d_lw.println();
@@ -431,7 +638,7 @@ public class ImplSource
       d_lw.println(CodeConstants.C_UNREFERENCED_COMMENT2);
       d_lw.println(CodeConstants.C_UNREFERENCED_COMMENT3);
       d_lw.endBlockComment(true);
-      d_splicer.outputUnusedSymbolEdits(d_lw.getPrintWriter()); 
+      d_splicer.outputUnusedSymbolEdits(d_lw); 
       d_lw.writeCommentLine(CodeConstants.C_END_UNREFERENCED_METHODS);
     }
   }
@@ -450,10 +657,11 @@ public class ImplSource
   */
   public static void generateCode(Class cls,
                                   LanguageWriterForFortran writer,
-                                  CodeSplicer splicer)
+                                  CodeSplicer splicer,
+                                  Context context)
     throws CodeGenerationException, NoSuchAlgorithmException
   {
-    ImplSource source = new ImplSource(writer, splicer);
+    ImplSource source = new ImplSource(writer, splicer, context);
     source.generateCode(cls);
   }
 }
