@@ -1,7 +1,7 @@
 //
 // File:        ModuleSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 4434 $
+// Revision:    @(#) $Revision: 6205 $
 // Description: Generate a module file for FORTRAN 90 clients
 //
 // Copyright (c) 2002-2003, The Regents of the University of Calfornia.
@@ -31,18 +31,24 @@
 
 package gov.llnl.babel.backend.fortran;
 
-import gov.llnl.babel.backend.CodeConstants;
+import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
+import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.fortran.Fortran;
 import gov.llnl.babel.backend.fortran.StubSource;
-import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
 import gov.llnl.babel.backend.mangler.NameMangler;
+import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
 import gov.llnl.babel.symbols.Argument;
-import gov.llnl.babel.symbols.Comment;
-import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.FortranExprString;
 import gov.llnl.babel.symbols.Class;
+import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Enumeration;
+import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.Inverter;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
@@ -58,18 +64,16 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeMap;
 
 /**
  * This class provides the ability to write a FORTRAN 90 module for
  * a sidl class/interface. 
  */
 public class ModuleSource {
-  private static final int s_CASTS_PER_LINE = 7;
+  private Context d_context;
 
-  /**
-   * Indicator of version of FORTRAN being generated.
-   */
-  private final static boolean d_isF90 = Fortran.isFortran90();
+  private static final int s_CASTS_PER_LINE = 7;
 
   /**
    * If Fortran90 is being generated, we need to keep track of 
@@ -77,6 +81,12 @@ public class ModuleSource {
    */
   private static HashSet interfacesWritten = null;
 
+  /**
+   * If Fortran90 is being generated, we need to keep track of
+   * what explicit method dispatches have been already generated.
+   */
+  private static HashSet dispatchesWritten = null;
+
   /**
    * This is the output device.
    */
@@ -95,8 +105,10 @@ public class ModuleSource {
    * @param writer  the output device to which the FORTRAN 90 module
    *                  should be written.
    */
-  public ModuleSource(LanguageWriterForFortran writer) {
+  public ModuleSource(LanguageWriterForFortran writer,
+                      Context context) {
     d_lw = writer;
+    d_context = context;
   }
 
   /**
@@ -117,10 +129,11 @@ public class ModuleSource {
   public static void generateSupers(Class             cls,
                                     LanguageWriterForFortran writer,
                                     CodeSplicer              splicer,
+                                    Context                  context,
                                     NameMangler mang)
     throws CodeGenerationException
   {
-    ModuleSource source = new ModuleSource(writer);
+    ModuleSource source = new ModuleSource(writer, context);
     source.generateSupers(cls, splicer, mang);
   }  
 
@@ -163,7 +176,7 @@ public class ModuleSource {
   {
     Type argType = a.getType();
     d_lw.writeCommentLine(a.getArgumentString());
-    d_lw.print(Fortran.getReturnString(argType));
+    d_lw.print(Fortran.getReturnString(argType, d_context));
     if (showIntent) {
       d_lw.print(" " + s_intent_spec[a.getMode()]);
     }
@@ -197,19 +210,24 @@ public class ModuleSource {
   private void writeMethodStub(Method   m,
                                SymbolID id,
                                String suffix,
-                               boolean convertRarray)
+                               boolean convertRarray,
+                               boolean isBuiltin)
     throws CodeGenerationException
   {
 
-    List extendedArgs = StubSource.extendArgs(id, m, false);
+    List extendedArgs = StubSource.extendArgs(id, m, d_context, false);
     Map indexMap;
     final String methodName = m.getLongMethodName() + suffix;
-    final String stubName = 
+    if(isBuiltin) {
+      m.setMethodName("_"+m.getShortMethodName());
+    }
+    String stubName = 
       (convertRarray ? Fortran.getAltStubName(id, m) 
-       : Fortran.getMethodStubName(id, m));
+       : Fortran.getMethodStubName(id, m, d_context));
+
     d_lw.print("recursive subroutine " + methodName);
     if (convertRarray) {
-      extendedArgs = StubSource.convertRarrayToArray(extendedArgs);
+      extendedArgs = StubSource.convertRarrayToArray(extendedArgs, d_context);
       indexMap = new HashMap(0);
     }
     else {
@@ -218,6 +236,21 @@ public class ModuleSource {
     printArgs(extendedArgs);
 
     d_lw.tab();
+
+    if (Fortran.hasBindC(d_context)){
+      d_lw.println("use, intrinsic :: iso_c_binding");
+
+      Iterator i = extendedArgs.iterator();
+      while (i.hasNext()) {
+        Argument a = (Argument)i.next();
+        Type t=a.getType();
+        if (t.isStruct()){
+          SymbolID idTest=t.getSymbolID();
+          d_lw.generateUse(Fortran.getModule(idTest), new TreeMap());
+        }
+      }
+    }
+
     d_lw.println("implicit none");
 
     Iterator i = extendedArgs.iterator();
@@ -243,9 +276,15 @@ public class ModuleSource {
       Collection choice = (Collection)indexMap.get(indexVar);
       if (choice.size() > 0) {
         Method.RarrayInfo info = (Method.RarrayInfo)choice.iterator().next();
-        d_lw.println(indexVar + " = size(" +
-                     info.rarray.getFormalName() + ", " +
-                     (info.dim+1) + ')');
+        d_lw.print(indexVar + " = ");
+        d_lw.println(FortranExprString.
+                   toFortranString(Inverter.invertExpr
+                                   ((AssertionExpression)
+                                    info.rarray.getType().
+                                    getArrayIndexExprs().get(info.dim),
+                                    "size(" +
+                                    info.rarray.getFormalName() + ", " +
+                                    (info.dim+1) + ')', d_context)));
       }
     }
     d_lw.print("call " + stubName);
@@ -253,7 +292,7 @@ public class ModuleSource {
       printArgs(extendedArgs);
     }
     else {
-      printArgs(StubSource.extendArgs(id, m, true));
+      printArgs(StubSource.extendArgs(id, m, d_context, true));
     }
     d_lw.backTab();
     d_lw.println();
@@ -277,13 +316,16 @@ public class ModuleSource {
                                  SymbolID id)
     throws CodeGenerationException
   {
+    d_lw.println();
+    d_lw.println();
+
     if (m.hasRarray()) {
-      writeMethodStub(m, id, "_1s", true);
+      writeMethodStub(m, id, "_1s", true, false);
       d_lw.println();
-      writeMethodStub(m, id, "_2s", false);
+      writeMethodStub(m, id, "_2s", false, false);
     }
     else {
-      writeMethodStub(m, id, "_s", false);
+      writeMethodStub(m, id, "_s", false,false);
     }
   }
   
@@ -304,13 +346,14 @@ public class ModuleSource {
     throws CodeGenerationException
   {
     SymbolID id = cls.getSymbolID();
-    List extendedArgs = StubSource.extendArgs(id, m, false);
-    final String methodName = Fortran.getMethodSuperImplName(id,m,mang);//"super_"+ m.getLongMethodName() + "_s";
-    final String stubName = Fortran.getMethodSuperName(id, m);
+    List extendedArgs = StubSource.extendArgs(id, m, d_context, false);
+    final String methodName = Fortran.getMethodSuperImplName(id,m,mang,
+                                                             d_context);//"super_"+ m.getLongMethodName() + "_s";
+    final String stubName = Fortran.getMethodSuperName(id, m, mang, d_context);
     d_lw.print("recursive subroutine " + methodName);
     printArgs(extendedArgs);
 
-    ImplSource.useStatementsForSupers(m,id,d_lw, splicer);
+    ImplSource.useStatementsForSupers(m,id,d_lw, splicer, d_context);
 
     d_lw.tab();
     d_lw.println("implicit none");
@@ -319,7 +362,7 @@ public class ModuleSource {
     while (i.hasNext()) {
       Argument a = (Argument)i.next();
       d_lw.writeCommentLine(a.getArgumentString());
-      d_lw.println(Fortran.getReturnString(a.getType()) + " " +
+      d_lw.println(Fortran.getReturnString(a.getType(), d_context) + " " +
                    s_intent_spec[a.getMode()] + " :: " 
                    + a.getFormalName());
     }
@@ -342,20 +385,21 @@ public class ModuleSource {
    */
   private Method create(SymbolID id)
   {
-    Method m = new Method();
-    m.setMethodName("new");
+    Method m = new Method(d_context);
+    m.setMethodName("new", "Local");
     m.setDefinitionModifier(Method.STATIC);
     String[] s = new String[1];
     s[0] = "Create an instance of class " + id.getFullName();
     m.setComment(new Comment(s));
-    m.setReturnType(new Type(id));
+    m.setReturnType(new Type(id, d_context));
+    m.addImplicitThrows(IOR.getRuntimeException(d_context));
     return m;
   }
 
-  public static Set extendedReferences(Extendable ext)
+  public static Set extendedReferences(Extendable ext, Context context)
     throws CodeGenerationException
   {
-    Set s = StubSource.extendedReferences(ext);
+    Set s = StubSource.extendedReferences(ext, context);
     s.add(ext.getSymbolID());
     Iterator i = ext.getParents(true).iterator();
     while (i.hasNext()) {
@@ -371,14 +415,17 @@ public class ModuleSource {
    *
    * @param ext the class whose module file is being written.
    */
-  private Collection extendMethods(Extendable ext) {
+  private Collection extendMethods(Extendable ext)
+  {
     Collection allMethods = ext.getMethods(true);
     final SymbolID id = ext.getSymbolID();
     ArrayList  extendedMethods = new ArrayList(allMethods.size()+1);
     if (!ext.isAbstract()) {
       extendedMethods.add(create(id));
+      extendedMethods.add(Fortran.createRemoteMethod(ext, d_context, ( Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context) )));
     }
-
+    extendedMethods.add(Fortran.connectRemoteMethod(ext, d_context, Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)));
+    
     extendedMethods.addAll(allMethods);
     return extendedMethods;
   }
@@ -391,21 +438,24 @@ public class ModuleSource {
   private void writeIncludes(Extendable ext)
     throws CodeGenerationException
   {
-    Set s = extendedReferences(ext);
+    Set s = extendedReferences(ext, d_context);
     s.add(ext.getSymbolID());
     Iterator i = s.iterator();
-    d_lw.disableLineBreak();
-    while (i.hasNext()) {
-      SymbolID id = (SymbolID)i.next();
-      d_lw.printlnUnformatted("#include \"" +
-                              Fortran.getStubNameFile(id) + "\"");
+    try {
+      d_lw.pushLineBreak(false);
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        d_lw.generateInclude( Fortran.getStubNameFile(id) );
+      }
+    }
+    finally {
+      d_lw.popLineBreak();
     }
-    d_lw.enableLineBreak();
   }
 
   private void includeType(SymbolID id)
   {
-    d_lw.println("use " + Fortran.getTypeModule(id));
+    d_lw.generateUse( Fortran.getTypeModule(id), new TreeMap());
   }
 
   private static void checkType(Type t, Set result)
@@ -444,22 +494,28 @@ public class ModuleSource {
     throws CodeGenerationException
   {
     final SymbolID id = ext.getSymbolID();
-    final SymbolTable table = SymbolTable.getInstance();
-    Set s = extendedReferences(ext);
+    final SymbolTable table = d_context.getSymbolTable();
+    Set s = extendedReferences(ext, d_context);
+    d_lw.generateUse("sidl", new TreeMap());
     s.add(id);
     Iterator i = s.iterator();
     while (i.hasNext()) {
       SymbolID current = (SymbolID)i.next();
       Symbol sym = table.lookupSymbol(current);
-      if (sym instanceof Extendable) {
+      if ((sym instanceof Extendable) || (sym instanceof Enumeration)) {
         includeType(current);
       }
     }
+    includeType(table.lookupSymbol("sidl.rmi.Call").getSymbolID());
+    includeType(table.lookupSymbol("sidl.rmi.Return").getSymbolID());
+    includeType(table.lookupSymbol("sidl.rmi.Ticket").getSymbolID());
+
+    TreeMap emptyMap = new TreeMap();
     s = basicArrayReferences(ext);
     i = s.iterator();
     while (i.hasNext()) {
       SymbolID basic_array = (SymbolID)i.next();
-      d_lw.println("use " + Fortran.getArrayModule(basic_array));
+      d_lw.generateUse(Fortran.getArrayModule(basic_array),emptyMap);
     }
   }
 
@@ -469,23 +525,27 @@ public class ModuleSource {
   private void generateCast(SymbolID oldType,
                             SymbolID newType,
                             int      num)
+    throws CodeGenerationException
   {
+    SymbolID exceptType = Utilities.lookupSymbol(d_context, BabelConfiguration.getBaseExceptionType());
     String castMethod = Fortran.getMethodStubName
-      (newType, Fortran.createCast(newType));
+      (newType, Fortran.createCast(d_context, newType), d_context);
     d_lw.beginBlockComment(false);
     d_lw.println("Static function to cast from " + oldType.getFullName());
     d_lw.println("to " + newType.getFullName() + ".");
     d_lw.endBlockComment(false);
-    d_lw.println("subroutine cast_" + num + "(oldType, newType)");
+    d_lw.println("subroutine cast_" + num + "(oldType, newType, exception)");
     d_lw.tab();
     d_lw.println("implicit none");
     d_lw.println("type(" + Fortran.getTypeName(oldType) + 
                  "), intent(in) :: oldType");
     d_lw.println("type(" + Fortran.getTypeName(newType) + 
                  "), intent(out) :: newType");
+    d_lw.println("type(" + Fortran.getTypeName(exceptType) + 
+                 "), intent(out) :: exception");
     d_lw.println("external " + castMethod);
     d_lw.println();
-    d_lw.println("call " + castMethod + "(oldType, newType)");
+    d_lw.println("call " + castMethod + "(oldType, newType, exception)");
     d_lw.backTab();
     d_lw.println("end subroutine cast_" + num);
     d_lw.println();
@@ -493,6 +553,7 @@ public class ModuleSource {
 
   private void generateCastMethods(Extendable ext,
                                    Collection parents)
+    throws CodeGenerationException
   {
     /*
      * Experience has shown that we cannot count on parents to
@@ -531,7 +592,6 @@ public class ModuleSource {
   private void writeCastInterface(int numCasts)
   {
     if (numCasts > 0) {
-      int i;
       d_lw.print("private :: ");
       writeCastList(numCasts);
       d_lw.println();
@@ -579,10 +639,17 @@ public class ModuleSource {
     d_lw.println();
   }
   //write the interface for generated (non-user) methods
-  private void writeMethodInterface(String methodName)
+  private void writeMethodInterface(SymbolID id, String methodName, 
+                                    boolean extendIfcName)
   {
+    String sIfcName;
     d_lw.println("private :: " + methodName + "_s");
-    d_lw.println("interface " + methodName);
+    if (extendIfcName) {
+      sIfcName = Fortran.getExtendedMethodName(id, methodName);
+    } else {
+      sIfcName = methodName;
+    }
+    d_lw.println("interface " + sIfcName);
     d_lw.tab();
     d_lw.println("module procedure " + methodName + "_s");
     d_lw.backTab();
@@ -590,14 +657,24 @@ public class ModuleSource {
     d_lw.println();
   }
 
-  private void writeMethodInterface(Extendable ext, Method m)
+  private void writeMethodInterface(Extendable ext, Method m, 
+                                    boolean extendIfcName)
   {
-    String sName = m.getShortMethodName();
-    Collection overloadedMethods = ext.getOverloadedMethodsByName(sName);    
+    SymbolID id  = ext.getSymbolID();
+    String sName = m.getCorrectMethodName();
+    String sIfcName;
+    Collection overloadedMethods = ext.getOverloadedMethodsByName(sName);
+
+    if(sName.compareTo("new") == 0) {
+      overloadedMethods.add(create(ext.getSymbolID()));
+      overloadedMethods.add(Fortran.createRemoteMethod(ext, d_context, (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context) )));
+    }
+
+    //TODO: Is this irrelevent now?  I think so.
     //This is only empty in one wierd case for the method new.
-    if(overloadedMethods.isEmpty())
-      writeMethodInterface(sName);
-    else {
+    if(overloadedMethods.isEmpty()) {
+      writeMethodInterface(id, sName, extendIfcName);
+    } else {
       //This simply lists all the long method names as private (it only
       //looks complex)
       d_lw.print("private :: "); 
@@ -614,16 +691,21 @@ public class ModuleSource {
         }
         if(i.hasNext()) {
           d_lw.println(", &");
-        }
-        else {
+        } else {
           d_lw.println();
+          //d_lw.println();
         }
       }
       d_lw.backTab();
       d_lw.println();
       
       //This maps the method short name to all the long names.
-      d_lw.println("interface " + sName);
+      if (extendIfcName) {
+        sIfcName = Fortran.getExtendedMethodName(id, sName);
+      } else {
+        sIfcName = sName;
+      }
+      d_lw.println("interface " + sIfcName);
       d_lw.tab();
       d_lw.print("module procedure ");
       for(Iterator i = overloadedMethods.iterator(); i.hasNext();) {
@@ -642,12 +724,23 @@ public class ModuleSource {
       }
       d_lw.backTab();
       d_lw.println("end interface");
+      d_lw.println();
+      d_lw.println();
     } 
   }
 
-  private void writeMethodInterfaces(Extendable ext)
+
+  private void writeMethodInterface(Extendable ext, Method m)
   {
-    Collection methods = extendMethods(ext);
+    writeMethodInterface(ext, m, false);
+  }
+
+
+  private void writeMethodInterfaces(Extendable ext, boolean doLocal) 
+    throws CodeGenerationException 
+  {
+    Collection methods = doLocal ? ext.getMethods(false) : extendMethods(ext);
+
     Iterator i = methods.iterator();
     while (i.hasNext()) {
       Method m = (Method)i.next();
@@ -655,10 +748,132 @@ public class ModuleSource {
         writeMethodInterface(ext, m);
         interfacesWritten.add(m.getShortMethodName());
       }
+      if ( m.getCommunicationModifier() == Method.NONBLOCKING ) { 
+        Method send = m.spawnNonblockingSend();
+        if(!interfacesWritten.contains(send.getCorrectMethodName())) {
+          writeMethodInterface(ext, send); 
+          interfacesWritten.add(send.getCorrectMethodName());
+        }
+        Method recv = m.spawnNonblockingRecv();
+        if(!interfacesWritten.contains(recv.getCorrectMethodName())) {
+          writeMethodInterface(ext, recv); 
+          interfacesWritten.add(recv.getCorrectMethodName());
+        }
+      }
+    }
+  }
+  
+
+  private void writeBuiltinMethodInterfaces(Extendable ext) throws CodeGenerationException 
+  {
+    SymbolID id = ext.getSymbolID();
+    Method bi = IOR.getBuiltinMethod(IOR.EXEC,id,d_context,false);
+    bi.setMethodName("exec");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+    
+    bi = IOR.getBuiltinMethod(IOR.GETURL,id, d_context,false);
+    bi.setMethodName("getURL");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+    
+    bi = IOR.getBuiltinMethod(IOR.ISREMOTE,id,d_context,false);
+    bi.setMethodName("isRemote");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+
+    bi = IOR.getBuiltinMethod(IOR.ISREMOTE,id,d_context,false);
+    bi.setMethodName("isLocal");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+
+    bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,false);
+    bi.setMethodName("set_hooks");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+
+    if(ext.hasStaticMethod(true)) {
+      bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,true);
+      bi.setMethodName("set_hooks_static");
+      d_lw.println();
+      writeMethodInterface(ext, bi, true);
+      interfacesWritten.add(bi.getShortMethodName());
+
+    }
+
+  }
+
+  private void generateBuiltinMethods(Extendable ext) throws CodeGenerationException
+  {
+    SymbolID id = ext.getSymbolID();
+    Method bi = IOR.getBuiltinMethod(IOR.EXEC,id,d_context,false);
+    bi.setMethodName("exec");
+    d_lw.println();
+    writeMethodStub(bi,id,"_s",false,true);
+
+    
+    bi = IOR.getBuiltinMethod(IOR.GETURL,id,d_context,false);
+    bi.setMethodName("getURL");
+    d_lw.println();
+    //    extendAndGenerate(bi, id);
+    writeMethodStub(bi,id,"_s",false,true);
+    
+
+    bi = IOR.getBuiltinMethod(IOR.ISREMOTE,id,d_context,false);
+    bi.setMethodName("isRemote");
+    d_lw.println();
+    writeMethodStub(bi,id,"_s",false,true);
+
+    bi = IOR.getBuiltinMethod(IOR.ISREMOTE,id,d_context,false);
+    bi.setMethodName("isLocal");
+    d_lw.println();
+    writeMethodStub(bi,id,"_s",false,true);
+
+    bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,false);
+    bi.setMethodName("set_hooks");
+    d_lw.println();
+    writeMethodStub(bi,id,"_s",false,true);
+
+    if(ext.hasStaticMethod(true)) {
+      bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,true);
+      bi.setMethodName("set_hooks_static");
+      d_lw.println();
+      writeMethodStub(bi,id,"_s",false,true);
     }
   }
 
 
+  private void generateMethodDispatches(Extendable ext, boolean doLocal)
+    throws CodeGenerationException
+  {
+    Collection methods = doLocal ? ext.getMethods(false) : extendMethods(ext);
+    final SymbolID id  = ext.getSymbolID();
+
+    Iterator i = methods.iterator();
+    while (i.hasNext()) {
+      Method m = (Method)i.next();
+      if( (!dispatchesWritten.contains(m.getLongMethodName()))
+        && (!m.getLongMethodName().equals("new"))) {
+        extendAndGenerate(m, id);
+
+        dispatchesWritten.add(m.getLongMethodName());
+        if ( m.getCommunicationModifier() == Method.NONBLOCKING ) {
+          Method send = m.spawnNonblockingSend();
+          extendAndGenerate(send, id);
+          dispatchesWritten.add(send.getLongMethodName());
+          Method recv = m.spawnNonblockingRecv();
+          extendAndGenerate(recv, id);
+          dispatchesWritten.add(recv.getLongMethodName());
+        }
+      }
+    }
+  }
+
   /**
    * Generate the FORTRAN 90 module file for a sidl class.  
    * 
@@ -670,11 +885,11 @@ public class ModuleSource {
   public void generateCode(Extendable ext)
     throws CodeGenerationException
   {
-    Collection methods = extendMethods(ext);
     Collection parents = ext.getParents(true);
     final SymbolID id = ext.getSymbolID();
     final String name = Fortran.getModule(id);
     interfacesWritten = new HashSet();
+    dispatchesWritten = new HashSet();
     d_lw.println();
     d_lw.writeComment(ext, false);
     d_lw.println();
@@ -686,11 +901,16 @@ public class ModuleSource {
 
     includeTypes(ext);
 
+    d_lw.println();
     writeCastInterface(parents.size()*2);
-    writeMethodInterfaces(ext);
-    writeMethodInterface("not_null");
-    writeMethodInterface("is_null");
-    writeMethodInterface("set_null");
+    d_lw.println();
+    writeMethodInterfaces(ext, true); /* Write local method interfaces */
+    writeMethodInterfaces(ext, false); /* Write parent/extended interfaces */
+    writeBuiltinMethodInterfaces(ext);
+
+    writeMethodInterface(id, "not_null", false);
+    writeMethodInterface(id, "is_null", false);
+    writeMethodInterface(id, "set_null", false);
 
     d_lw.println();
     d_lw.backTab();
@@ -698,15 +918,11 @@ public class ModuleSource {
     d_lw.println();
 
     d_lw.tab();
-    Iterator i = methods.iterator();
-    while (i.hasNext()) {
-      d_lw.println();
-      d_lw.println();
-
-      extendAndGenerate((Method)i.next(), id);
-    }
+    generateMethodDispatches(ext, true);  /* Generate local methods first */
+    generateMethodDispatches(ext, false); /* Generate parent/extended methods */
     d_lw.println();
     generateCastMethods(ext, parents);
+    generateBuiltinMethods(ext);
     generateNullSubroutines(ext);
 
     d_lw.backTab();
@@ -732,12 +948,13 @@ public class ModuleSource {
     d_lw.writeComment(enm, false);
     d_lw.tab();
     d_lw.println();
+    d_lw.generateUse("sidl", new TreeMap());
     Iterator  i = enm.getEnumerators().iterator();
     while (i.hasNext()){
       String sym = (String)i.next();
       Comment cmt = enm.getEnumeratorComment(sym);
       d_lw.writeComment(cmt, true);
-      d_lw.print(Fortran.getReturnString(new Type(id)));
+      d_lw.print(Fortran.getReturnString(new Type(id, d_context), d_context));
       d_lw.print(", parameter :: ");
       d_lw.print(sym);
       d_lw.println(" = " + enm.getEnumeratorValue(sym));
@@ -761,16 +978,17 @@ public class ModuleSource {
    *   phase.
   */
   public static void generateCode(Symbol                   sym,
-                                  LanguageWriterForFortran writer)
+                                  LanguageWriterForFortran writer,
+                                  Context                  context)
     throws CodeGenerationException
   {
-    if (d_isF90) {
+    if (Fortran.isFortran90(context) || Fortran.hasBindC(context)) {
       if (sym instanceof Extendable) {
-        ModuleSource modFile = new ModuleSource(writer);
+        ModuleSource modFile = new ModuleSource(writer, context);
         modFile.generateCode((Extendable)sym);
       }
       else if (sym instanceof Enumeration) {
-        ModuleSource modFile = new ModuleSource(writer);
+        ModuleSource modFile = new ModuleSource(writer, context);
         modFile.generateCode((Enumeration)sym);
       }
     } else {  // Assuming this means F77
