@@ -1,7 +1,7 @@
 //
 // File:        IORHeader.java
 // Package:     gov.llnl.babel.backend.ior
-// Revision:    @(#) $Id: IORHeader.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: IORHeader.java 6201 2007-10-26 06:15:35Z kumfert $
 // Description: generate IOR header to a pretty writer stream
 //
 // Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -31,15 +31,15 @@
 package gov.llnl.babel.backend.ior;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.backend.c.C;
-import gov.llnl.babel.backend.ior.IORSource;
 import gov.llnl.babel.backend.SortComparator;
 import gov.llnl.babel.backend.Utilities;
+import gov.llnl.babel.backend.ior.IORSource;
+import gov.llnl.babel.backend.rmi.RMIIORHeader;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
-import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Enumeration;
@@ -47,8 +47,10 @@ import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Interface;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Package;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
+import gov.llnl.babel.symbols.Type;
 import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -67,6 +69,8 @@ public class IORHeader {
      BabelConfiguration.getBaseExceptionInterface();
 
    private LanguageWriterForC d_writer;
+  private boolean d_forwardNote = true;
+  private Context d_context;
 
    /**
     * This is a convenience utility function that writes the symbol
@@ -74,10 +78,12 @@ public class IORHeader {
     * The output stream is not closed on exit.  A code generation
     * exception is thrown if an error is detected.
     */
-   public static void generateCode(Symbol symbol, LanguageWriterForC writer)
+   public static void generateCode(Symbol symbol, 
+                                   LanguageWriterForC writer,
+                                   Context context)
       throws CodeGenerationException 
    {
-      IORHeader header = new IORHeader(writer);
+      IORHeader header = new IORHeader(writer, context);
       header.generateCode(symbol);
    }
 
@@ -85,8 +91,10 @@ public class IORHeader {
     * Create a <code>IORHeader</code> object that will write symbol information
     * to the provided output language writer stream.
     */
-   public IORHeader(LanguageWriterForC writer) {
+   public IORHeader(LanguageWriterForC writer,
+                    Context context) {
       d_writer = writer;
+      d_context = context;
    }
 
    /**
@@ -100,6 +108,9 @@ public class IORHeader {
    public void generateCode(Symbol symbol) throws CodeGenerationException {
       if (symbol != null) {
          switch (symbol.getSymbolType()) {
+         case Symbol.STRUCT:
+           generateStruct((Struct) symbol);
+           break;
          case Symbol.CLASS:
          case Symbol.INTERFACE:
             generateExtendable((Extendable) symbol);
@@ -114,6 +125,137 @@ public class IORHeader {
       }
    }
 
+  private void writeForwardNote()
+  {
+    if (d_forwardNote) {
+      d_writer.println();
+      d_writer.beginBlockComment(false);
+      d_writer.println("Forward declarations of structs");
+      d_writer.endBlockComment(false);
+      d_forwardNote = false;
+    }
+  }
+
+  private void writeStructForward(Struct strct)
+  {
+    Iterator i = strct.getItems().iterator();
+    while (i.hasNext()) {
+      final Struct.Item item = (Struct.Item)i.next();
+      final Type t = item.getType();
+      switch(t.getDetailedType()) {
+      case Type.ARRAY:
+        writeForwardNote();
+        final Type arrayType = t.getArrayType();
+        if (null != arrayType) {
+          if (arrayType.getType() != Type.SYMBOL){
+            d_writer.println(IOR.getArrayNameWithoutAsterix
+                             (arrayType.getType()) + ";");
+          }
+          else {
+            d_writer.println(IOR.getArrayName(arrayType.getSymbolID()) +
+                             ";");
+          }
+        }
+        else {
+          d_writer.println(IOR.getArrayName(null) + ";");
+        }
+        break;
+      case Type.INTERFACE:
+      case Type.CLASS:
+        writeForwardNote();
+        d_writer.println(IOR.getObjectName(t.getSymbolID()) + ";");
+        break;
+      }
+    }
+  }
+
+  private void writeStructIncludes(Struct strct)
+  {
+    HashSet set = new HashSet();
+    Iterator i = strct.getItems().iterator();
+    while (i.hasNext()) {
+      final Struct.Item item = (Struct.Item)i.next();
+      final Type t = item.getType();
+      if (t.getDetailedType() == Type.STRUCT) {
+        final String headerFile = IOR.getHeaderFile(t.getSymbolID());
+        if (!set.contains(headerFile)) {
+          set.add(headerFile);
+          d_writer.generateInclude(headerFile, true);
+        }
+      }
+    }
+  }
+
+  private String getStructItemType(Type t)
+    throws CodeGenerationException
+  {
+    return (t.getDetailedType() == Type.STRUCT) 
+      ? IOR.getStructName(t.getSymbolID())
+      : IOR.getReturnString(t, d_context, true, false);
+  }
+
+  private int maxStructType(Struct strct)
+    throws CodeGenerationException
+  {
+    int maxwidth = 0;
+    Iterator i = strct.getItems().iterator();
+    while (i.hasNext()) {
+      final Struct.Item item = (Struct.Item)i.next();
+      final String str = getStructItemType(item.getType());
+      final int len = str.length();
+      if (len > maxwidth) {
+        maxwidth = len;
+      }
+    }
+    return maxwidth;
+  }
+
+  private void writeStructDefn(Struct strct) 
+    throws CodeGenerationException
+  {
+    final int width = Math.min(maxStructType(strct), 40);
+    Iterator i = strct.getItems().iterator();
+    d_writer.writeComment(strct, false);
+    d_writer.println(IOR.getStructName(strct.getSymbolID()) + " {");
+    d_writer.tab();
+    while (i.hasNext()) {
+      final Struct.Item item = (Struct.Item)i.next();
+      d_writer.printAligned(getStructItemType(item.getType()), width);
+      d_writer.println(" " + item.getName() + ";");
+    }
+    if (strct.getItems().isEmpty()) {
+      d_writer.writeCommentLine("Avoid empty struct");
+      d_writer.println("char d_not_empty;");
+    }
+    d_writer.backTab();
+    d_writer.println("};");
+    d_writer.println();
+  }
+
+   /**
+    * Generate the IOR header for a SIDL struct type.  SIDL struct's
+    * are mapped to C structs.
+    */
+  private void generateStruct(Struct strct)
+    throws CodeGenerationException
+  {
+      SymbolID id = strct.getSymbolID();
+      String header = IOR.getHeaderFile(id);
+      
+      d_writer.writeBanner(strct, header, CodeConstants.C_IS_NOT_IMPL, 
+         CodeConstants.C_DESC_IOR_PREFIX + id.getFullName());
+      d_writer.openHeaderGuard(header);
+      d_writer.generateInclude("sidlType.h", true);
+      if (strct.hasArrayReference()) {
+        d_writer.generateInclude("sidlArray.h", true);
+      }
+      writeStructIncludes(strct);
+      writeStructForward(strct);
+      d_writer.println();
+      writeStructDefn(strct);
+      d_writer.closeHeaderGuard();
+  }
+
    /**
     * Generate the IOR header for a SIDL enumerated type.  Enumerated
     * types are mapped to C enumerated types, which are guaranteed to be
@@ -148,14 +290,13 @@ public class IORHeader {
        * assigned value.  For pretty output, find the maximum enumerator
        * length and then space over that much to align the equals signs.
        */
-      int maxlength = Utilities.getWidth(enm.getEnumerators());
-      final String namespace = id.getFullName().replace('.','_') + "_";
-      maxlength += namespace.length();
+      final int maxlength = Utilities.getWidth(enm.getEnumerators()) +
+        IOR.getEnumValueSymbol(id, "").length();
       for (Iterator e = enm.getIterator(); e.hasNext(); ) {
          String name = (String) e.next();
          Comment cmt = enm.getEnumeratorComment(name);
          d_writer.writeComment(cmt, true);
-         d_writer.printAligned(namespace + name, maxlength);
+         d_writer.printAligned(IOR.getEnumValueSymbol(id, name), maxlength);
          d_writer.print(" = ");
          d_writer.print(String.valueOf(enm.getEnumeratorValue(name)));
          if (e.hasNext()) {
@@ -234,6 +375,16 @@ public class IORHeader {
       }
       generateEPV(ext, false);
 
+      if(IOR.supportHooks(ext, d_context)) {
+        generatePreHooksEPV(ext, false);
+        generatePostHooksEPV(ext, false);
+        if (ext.hasStaticMethod(true)) {
+          generatePreHooksEPV(ext, true);
+          generatePostHooksEPV(ext, true);
+        }
+
+      }
+
       /*
        * Generate the class or interface object itself
        */
@@ -247,10 +398,21 @@ public class IORHeader {
          d_writer.println();
 
       }
+      //generate the external method declarations for this Extendable
+      generateExportedMethods(ext);
+
+      /*
+       * For interfaces: Generate the special anonymous class for RMI stuff.
+       *
+       * For both: forward declare skel RMI Accessor Function
+       */
+      RMIIORHeader.generateCode(ext, d_writer, d_context);
+      
       /*
        * Conclude the header file by closing the C++ extern block and
        * header include guard.
        */
+
       d_writer.closeCxxExtern();
       d_writer.closeHeaderGuard();
       
@@ -289,12 +451,14 @@ public class IORHeader {
       Class cls = (Class)sym;
       final SymbolID id = sym.getSymbolID();
       final String symbolType = IOR.getSymbolType(sym);
-      final int maxDim = BabelConfiguration.getMaximumArray();
       d_writer.println(IOR.getExternalName(id) + " {");
       d_writer.tab();
       if (!cls.isAbstract()) {
         d_writer.println(symbolType);
-        d_writer.println("(*createObject)(void);");
+        d_writer.println("(*createObject)("
+                         + "void* ddata, "
+                         + IOR.getExceptionFundamentalType() +
+                         "*_ex);");
         d_writer.println();
       }
       if (cls.hasStaticMethod(true)) {
@@ -309,6 +473,8 @@ public class IORHeader {
 	      d_writer.println("(*getSuperEPV)(void);");
 	  }
       }
+      d_writer.println("int d_ior_major_version;");
+      d_writer.println("int d_ior_minor_version;");
       d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
@@ -338,6 +504,11 @@ public class IORHeader {
          if (parent != null) {
             includes.add(parent.getSymbolID());
          }
+      } else {
+        Class anonCls = ((Interface)ext).generateAnonymousClass();
+        Set ifcs = Utilities.getUniqueInterfaceIDs(anonCls);
+        ifcs.remove(ext.getSymbolID()); //We don't want to include ourself.
+        includes.addAll(ifcs);
       }
 
       /*
@@ -347,7 +518,7 @@ public class IORHeader {
       if ((dependencies != null) && (!dependencies.isEmpty())) {
          for (Iterator i = dependencies.iterator(); i.hasNext(); ) {
             SymbolID id = (SymbolID) i.next();
-            Symbol symbol = Utilities.lookupSymbol(id);
+            Symbol symbol = Utilities.lookupSymbol(d_context, id);
             if (symbol.getSymbolType() == Symbol.ENUM) {
                includes.add(id);
             }
@@ -358,17 +529,18 @@ public class IORHeader {
        * Sort the entries and output the list of include files.
        */
       d_writer.generateInclude("sidl_header.h", true);
-      d_writer.generateInclude(C.getHeaderFile(
-        Utilities.lookupSymbol("sidl.io.Serializer").getSymbolID()), true);
-      d_writer.generateInclude(C.getHeaderFile(
-        Utilities.lookupSymbol("sidl.io.Deserializer").getSymbolID()), true);
-      if ( IOR.supportAssertions(ext) && (!ext.isInterface()) ) {
+      
+      d_writer.println(IOR.getObjectName(Utilities.lookupSymbol(d_context, "sidl.rmi.InstanceHandle").getSymbolID()) + ";");
+      if ( IOR.supportAssertions(ext, d_context) && (!ext.isInterface()) ) {
         d_writer.generateInclude("sidlAsserts.h", true);
-        if (IOR.ASSERTION_LEVEL != BabelConfiguration.FULL_CHECKING_STATS) {
-          d_writer.print("//");
+        if (!d_context.getConfig().getAssertionLevel().equals(BabelConfiguration.ADVANCED_CHECKING)) {
+          d_writer.println("/*");
         }
-        d_writer.println("#define " + IOR.S_FULL_STATS_MACRO);
-        d_writer.println("//#define " + IOR.S_TEXT_STATS_MACRO);
+        d_writer.printlnUnformatted("#define " + IOR.S_FULL_STATS_MACRO);
+        if (!d_context.getConfig().getAssertionLevel().equals(BabelConfiguration.ADVANCED_CHECKING)) {
+          d_writer.println("*/");
+        }
+        d_writer.printlnUnformatted("/* #define " + IOR.S_TEXT_STATS_MACRO + " */");
         d_writer.generateInclude("stdlib.h", true);
         d_writer.generateInclude("math.h", true);
       }
@@ -389,10 +561,10 @@ public class IORHeader {
     * symbols are the name of the object structure and its constructors
     * and initializers.
     */
-   private void generateExportedSymbols(Extendable ext) {
-      SymbolID id = ext.getSymbolID();
-      String type = IOR.getObjectName(id);
-      String sepv = IOR.getSEPVName(id);
+  private void generateExportedSymbols(Extendable ext) {
+    SymbolID id = ext.getSymbolID();
+    String type = IOR.getObjectName(id);
+    String sepv = IOR.getSEPVName(id);
   
       d_writer.writeComment(ext, false);
 
@@ -405,20 +577,28 @@ public class IORHeader {
          d_writer.println(sepv + ";");
       }
       d_writer.println();
-
-      /*
-       * Output the functions that will be defined in the implementation.
-       */
+   }
+  //JIM
+  private void generateExportedMethods(Extendable ext) throws CodeGenerationException {
+    SymbolID id = ext.getSymbolID();
+    String type = IOR.getObjectName(id);
+    String sepv = IOR.getSEPVName(id);
+    /*
+     * Output the functions that will be defined in the implementation.
+     */
       if (!ext.isAbstract()) {
          d_writer.println("extern " + type + "*");
-         d_writer.println(IOR.getNewName(id) + "(void);");
+         d_writer.println(IOR.getNewName(id) + "(void* ddata," +
+                          IOR.getExceptionFundamentalType() +
+                          "* _ex);");
          d_writer.println();
       }
       boolean doStaticTypes = false;
       if (ext.hasStaticMethod(true)) {
          d_writer.println("extern " + sepv + "*");
          d_writer.println(IOR.getStaticsName(id) + "(void);");
-         if (IOR.supportAssertions(ext) || IOR.supportInterceptors(ext)) {
+         if (IOR.supportAssertions(ext, d_context) || 
+             IOR.supportHooks(ext, d_context)) {
            d_writer.println();
            d_writer.println("extern " + sepv + "*");
            d_writer.println(IOR.getLocalStaticsName(id) + "(int type);");
@@ -427,34 +607,46 @@ public class IORHeader {
          d_writer.println();
       }
       if (!ext.isInterface()) {
-         d_writer.println("extern void " + IOR.getInitName(id) + "(");
-         d_writer.tab();
-         d_writer.println(type + "* self);");
-         d_writer.backTab();
-         d_writer.println("extern void " + IOR.getFiniName(id) + "(");
-         d_writer.tab();
-         d_writer.println(type + "* self);");
-         d_writer.backTab();
-         d_writer.println("extern void " + IOR.getVersionName(id) + "(int32_t *"
-           + "major, int32_t *minor);");
-         d_writer.println();
+        Class cls = (Class) ext;
+        d_writer.println("extern void " + IOR.getInitName(id) + "(");
+        d_writer.tab();
+        d_writer.println(type + "* self, void* ddata, " +
+                         IOR.getExceptionFundamentalType() +
+                         "* _ex);");
+        d_writer.backTab();
+        
+        d_writer.println("extern void "+IOR.getGetEPVsName(id)+"(");
+        d_writer.tab();
+        IOR.declareEPVsAsArgs(d_writer, cls, true);
+        d_writer.println(");");
+        d_writer.backTab();
+        
+        d_writer.println("extern void " + IOR.getFiniName(id) + "(");
+        d_writer.tab();
+        d_writer.println(type + "* self, " +
+                         IOR.getExceptionFundamentalType() +
+                         "* _ex);");
+        d_writer.backTab();
+        d_writer.println("extern void " + IOR.getVersionName(id) + "(int32_t *"
+                         + "major, int32_t *minor);");
+        d_writer.println();
       }
       if (doStaticTypes) {
          d_writer.writeComment("Define static structure options.", false);
          d_writer.println("static const int " 
            + IOR.getStaticTypeOption(id, IOR.SET_PUBLIC) + " = 0;");
          int i = 1;
-         if (IOR.supportAssertions(ext)) {
+         if (IOR.supportAssertions(ext, d_context)) {
             d_writer.println("static const int " 
               + IOR.getStaticTypeOption(id, IOR.SET_ASSERTIONS) + " = " + i 
               + ";");
             i++;
          }
-         if (IOR.supportInterceptors(ext)) {
-            d_writer.println("static const int " 
-              + IOR.getStaticTypeOption(id, IOR.SET_INTERCEPTORS) + " = " + i 
-              + ";");
-         }
+
+         d_writer.println("static const int " 
+                          + IOR.getStaticTypeOption(id, IOR.SET_HOOKS) + " = " + i 
+                          + ";");
+   
          d_writer.println();
       }
    }
@@ -480,10 +672,16 @@ public class IORHeader {
          Method method = (Method) i.next();
          references.addAll(method.getSymbolReferences());
          if (!method.getThrows().isEmpty()) {
-            Symbol symbol = Utilities.lookupSymbol(SIDL_EXCEPTION_INTERFACE);
+            Symbol symbol = Utilities.lookupSymbol(d_context, SIDL_EXCEPTION_INTERFACE);
             references.add(symbol.getSymbolID());
          }
       }
+      references.add(Utilities.
+                     lookupSymbol(d_context, "sidl.rmi.Return").getSymbolID());
+      references.add(Utilities.
+                     lookupSymbol(d_context, "sidl.rmi.Call").getSymbolID());
+      
+      
 
       /*
        * Remove all previously defined types and continue if there are
@@ -499,7 +697,7 @@ public class IORHeader {
          Set declare = new HashSet();
          for (Iterator i = references.iterator(); i.hasNext(); ) {
             SymbolID id = (SymbolID) i.next();
-            Symbol symbol = Utilities.lookupSymbol(id);
+            Symbol symbol = Utilities.lookupSymbol(d_context, id);
             int type = symbol.getSymbolType();
             if (type != Symbol.PACKAGE) {
                declare.add(id);
@@ -516,11 +714,16 @@ public class IORHeader {
             List entries = Utilities.sort(declare);
             for (Iterator i = entries.iterator(); i.hasNext(); ) {
                SymbolID id = (SymbolID) i.next();
-               Symbol symbol = Utilities.lookupSymbol(id);
+               Symbol symbol = Utilities.lookupSymbol(d_context, id);
                int type = symbol.getSymbolType();
-               d_writer.println(IOR.getArrayName(id) + ";");
-               if (type != Symbol.ENUM) {
-                 d_writer.println(IOR.getObjectName(id) + ";");
+               if (type == Symbol.STRUCT) {
+                 d_writer.println(IOR.getStructName(id) + ";");
+               }
+               else {
+                 d_writer.println(IOR.getArrayName(id) + ";");
+                 if (type != Symbol.ENUM) {
+                   d_writer.println(IOR.getObjectName(id) + ";");
+                 }
                }
             }
             d_writer.println();
@@ -534,34 +737,29 @@ public class IORHeader {
    * 
    * @param m           the method to be generated
    * @param self        a string with the type of the object
-   * @param addHooks    indicator of whether interceptors should be added.
-   * @param alreadySeen set of method names already generated.
-   *                    Each element is a <code>String</code>.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateEPVEntry(Method m, String self, String suffix) 
+  private void generateEPVEntry(Method m, String self) 
     throws CodeGenerationException
   {
-    if (suffix.equals("")) {
-      d_writer.print(IOR.getReturnString(m.getReturnType(), true, false));
-    } else {
-      d_writer.print("void");
-    }
+    d_writer.print(IOR.getReturnString(m.getReturnType(), d_context,
+                                       true, false));
+
     d_writer.print(" (*");
-    d_writer.print(IOR.getVectorEntry(m.getLongMethodName()) + suffix);
+    d_writer.print(IOR.getVectorEntry(m.getLongMethodName()));
     d_writer.println(")(");
     d_writer.tab();
 
     /* 
      * Generate the argument list.
      */
-    boolean has_throws = !m.getThrows().isEmpty() && (suffix.equals(""));
+    boolean has_throws = !m.getThrows().isEmpty();
     List args = m.getArgumentList();
-    C.generateArguments(d_writer, self, args, m.isStatic(), has_throws, 
-        (suffix.equals(IOR.GENERIC_POST_SUFFIX) ? m.getReturnType() : null), 
-        true, true, false);
+    IOR.generateArguments(d_writer, d_context,
+                          self, args, m.isStatic(), has_throws, 
+                        m.getReturnType(), true, true, false, false);
     d_writer.println(");");
     d_writer.backTab();
   }
@@ -572,7 +770,7 @@ public class IORHeader {
    * 
    * @param m           the method to be generated
    * @param self        a string with the type of the object
-   * @param addHooks    indicator of whether interceptors should be added.
+   * @param addHooks    indicator of whether hooks should be added.
    * @param alreadySeen set of method names already generated.
    *                    Each element is a <code>String</code>.
    * @exception gov.llnl.babel.backend.CodeGenerationException
@@ -586,13 +784,7 @@ public class IORHeader {
     String name = m.getLongMethodName();
     if (!alreadySeen.contains(name)) {
       alreadySeen.add(name);
-      if (addHooks) {
-        generateEPVEntry(m, self, IOR.GENERIC_PRE_SUFFIX);
-        generateEPVEntry(m, self, "");
-        generateEPVEntry(m, self, IOR.GENERIC_POST_SUFFIX);
-      } else {
-        generateEPVEntry(m, self, "");
-      }
+      generateEPVEntry(m, self);
     }
   }
 
@@ -606,7 +798,7 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should interceptor hooks be listed?
+   * @param addHooks           should hooks be listed?
    * @param methodsAlreadySeen set of method names already included.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
@@ -632,6 +824,16 @@ public class IORHeader {
     while (m.hasNext()) {
       Method method = (Method) m.next();
       generateEPVMethod(method, self, addHooks, methodsAlreadySeen);
+      if (method.getCommunicationModifier() == Method.NONBLOCKING) { 
+      	Method send = method.spawnNonblockingSend();
+      	if ( send != null ) { 
+      	  generateEPVMethod(send, self, addHooks, methodsAlreadySeen );
+      	} 
+      	Method recv = method.spawnNonblockingRecv();
+      	if ( recv != null ) {
+           	generateEPVMethod(recv, self, addHooks, methodsAlreadySeen );
+      	}
+      }
     }
   }
 
@@ -639,7 +841,7 @@ public class IORHeader {
    * Generate the implicit/builtin methods for this fundamental
    * class or interface. These methods always appear first in
    * entry point vector (EPV).  Note that the built-in checks and
-   * interceptor methods are skipped for SIDL symbols.
+   * hooks methods are skipped for SIDL symbols.
    *
    * @param ext         the class/interface whose parents will be
    *                    asked to write their EPV entries.
@@ -659,9 +861,6 @@ public class IORHeader {
     final int      numBuiltins = (ext.isInterface() 
                                  ? IOR.INTERFACE_BUILTIN_METHODS
                                  : IOR.CLASS_BUILTIN_METHODS);
-    final boolean  doAsserts   = IOR.supportAssertions(ext);
-    final boolean  doIceptors  = IOR.supportInterceptors(ext);
-
     d_writer.writeCommentLine("Implicit builtin methods");
 
     /*
@@ -669,25 +868,27 @@ public class IORHeader {
      */
     if (doStatic) {
       for(int i = 0; i < numBuiltins; ++i){
-        if (  IOR.hasStaticBuiltin(i)
-           && (  (IOR.isBuiltinAssert(i)  && IOR.supportAssertions(ext))
-              || ((i == IOR.INTERCEPTORS) && IOR.supportInterceptors(ext)) ) ) {
-          Method b = IOR.getBuiltinMethod(i, id, doStatic);
+          d_writer.writeCommentLine(""+i);
+        if (IOR.hasStaticBuiltin(i) && 
+            (!IOR.isBuiltinAssert(i) || 
+             (IOR.isBuiltinAssert(i) && 
+              IOR.supportAssertions(ext, d_context)))) {
+          Method b = IOR.getBuiltinMethod(i, id, d_context, doStatic);
           generateEPVMethod(b, self, false, alreadySeen); 
         }
       }
     } else {
       for(int i = 0; i < numBuiltins; ++i){
-        if (  IOR.isBuiltinBasic(i) 
-           || (IOR.isBuiltinAssert(i)  && IOR.supportAssertions(ext))
-           || ((i == IOR.INTERCEPTORS) && IOR.supportInterceptors(ext)) ) {
-          Method b = IOR.getBuiltinMethod(i, id, doStatic);
+        d_writer.writeCommentLine(""+i);
+        //(IOR.isBuiltinAssert(i) && 
+        if (  IOR.isBuiltinBasic(i) || IOR.supportAssertions(ext, d_context)) {
+          Method b = IOR.getBuiltinMethod(i, id, d_context, doStatic);
           generateEPVMethod(b, self, false, alreadySeen); 
+
         }
       }
     }
   }
-
   /**
    * Generate the entry point vector entries for the parent class and/or
    * interfaces of <code>ext</code>. This must write the methods in a
@@ -698,7 +899,7 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should the interceptor method entries be 
+   * @param addHooks           should the hook method entries be 
    *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     set of extendables whose methods have
@@ -750,7 +951,7 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should the interceptor method entries be
+   * @param addHooks           should the hook method entries be
    *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     a set of extendables whose methodss have
@@ -766,7 +967,13 @@ public class IORHeader {
   {
     if ((ext != null) && !extAlreadySeen.contains(ext)) {
       if (hasParents(ext)) {
-        generateParentEntries(ext, self, doStatic, addHooks, 
+        /* 
+         * It is assumed parent entries should not have pre/post method
+         * hooks added.  If that is not the case, then extra work must be 
+         * done to ensure pre/post methods are created and EPV entries 
+         * properly initialized for each.
+         */
+        generateParentEntries(ext, self, doStatic, false, 
                               methodsAlreadySeen, extAlreadySeen);
       }
       generateLocalEntries(ext, self, doStatic, addHooks, methodsAlreadySeen);
@@ -780,8 +987,8 @@ public class IORHeader {
     * access to the corresponding control information for assertion checking 
     * purposes.
     */
-   private void generateMethodDescData(Extendable ext) {
-     if (IOR.supportAssertions(ext)) {
+   private void generateMethodDescData(Extendable ext) throws CodeGenerationException {
+     if (IOR.supportAssertions(ext, d_context)) {
         d_writer.writeComment("Define method description data for assertion "
           + "control", false);
         int      num      = 0;
@@ -870,8 +1077,18 @@ public class IORHeader {
       d_writer.tab();
       Set methodsAlreadySeen = new HashSet();
       Set extAlreadySeen = new HashSet();
+
+      //Put the methods in from the "From Clauses" into methodsAlreadySeen
+      //so we don't generate them.
+      for(Iterator ms = ext.getRenamedMethods().iterator(); ms.hasNext();) {
+        Method renamedMethod = (Method) ms.next();
+        if(ext.lookupMethodByLongName(renamedMethod.getLongMethodName(), false) == null) {
+          methodsAlreadySeen.add(renamedMethod.getLongMethodName());
+        }
+      }
+
       generateBuiltinEntries(ext, self, doStatic, methodsAlreadySeen);
-      generateEPVEntries(ext, self, doStatic, IOR.supportInterceptors(ext),
+      generateEPVEntries(ext, self, doStatic, IOR.supportHooks(ext, d_context),
                          methodsAlreadySeen, extAlreadySeen);
       methodsAlreadySeen = null;
       extAlreadySeen = null;
@@ -880,6 +1097,133 @@ public class IORHeader {
       d_writer.println();
    }
 
+
+   /**
+    * Generate the pre EPV (entry point vector for for pre hooks) 
+    * data structure for the methods
+    * in the interface or class.  If the flag is true, then generate an EPV
+    * for the static members of the class.
+    *
+    * The EPV must be ordered correctly!  The parent entries must appear
+    * before child entries.  This is required for runtime features.
+    */
+   private void generatePreHooksEPV(Extendable ext, boolean doStatic)
+     throws CodeGenerationException 
+   {
+      /*
+       * Generate the name for this entry point vector as well as a pointer
+       * to "self" for this structure.  For classes, self will be the object
+       * structure whereas for interfaces it is void*.
+       */
+      final SymbolID id = ext.getSymbolID();
+      String self = null;
+      if (!doStatic) {
+        if (ext.isInterface()) {
+           self = "void* self";
+        } else {
+           self = IOR.getObjectName(id) + "* self";
+        }
+      }
+
+      /*
+       * Write a comment block to document the EPV structure.
+       */
+      String desc = doStatic ? "static " : "";
+      d_writer.writeComment("Declare the " + desc 
+        + "method pre hooks entry point vector.", false);
+
+      /*
+       * Begin definition of the EPV structure and increase tab indent.
+       */
+      if (doStatic) {
+         d_writer.print(IOR.getPreSEPVName(id));
+      } else {
+         d_writer.print(IOR.getPreEPVName(id));
+      }
+      d_writer.println(" {");
+      d_writer.tab();
+
+      Iterator m = null;
+      if (doStatic) {
+        m = ext.getStaticMethods(false).iterator();
+      } else {
+        m = ext.getNonstaticMethods(false).iterator();
+      }
+
+      while (m.hasNext()) {
+        Method method = (Method) m.next();
+        Method pre = method.spawnPreHook();        
+        generateEPVEntry(pre, self);
+      }
+
+      d_writer.backTab();
+      d_writer.println("};");
+      d_writer.println();
+   }
+
+   /**
+    * Generate the post EPV (entry point vector for for pre hooks) 
+    * data structure for the methods
+    * in the interface or class.  If the flag is true, then generate an EPV
+    * for the static members of the class.
+    *
+    * The EPV must be ordered correctly!  The parent entries must appear
+    * before child entries.  This is required for runtime features.
+    */
+   private void generatePostHooksEPV(Extendable ext, boolean doStatic)
+     throws CodeGenerationException 
+   {
+      /*
+       * Generate the name for this entry point vector as well as a pointer
+       * to "self" for this structure.  For classes, self will be the object
+       * structure whereas for interfaces it is void*.
+       */
+      final SymbolID id = ext.getSymbolID();
+      String self = null;
+      if (!doStatic) {
+        if (ext.isInterface()) {
+           self = "void* self";
+        } else {
+           self = IOR.getObjectName(id) + "* self";
+        }
+      }
+
+      /*
+       * Write a comment block to document the EPV structure.
+       */
+      String desc = doStatic ? "static " : "";
+      d_writer.writeComment("Declare the " + desc 
+        + "method post hooks entry point vector.", false);
+
+      /*
+       * Begin definition of the EPV structure and increase tab indent.
+       */
+      if (doStatic) {
+        d_writer.print(IOR.getPostSEPVName(id));
+      } else {
+        d_writer.print(IOR.getPostEPVName(id));
+      }
+      d_writer.println(" {");
+      d_writer.tab();
+
+      Iterator m = null;
+      if (doStatic) {
+        m = ext.getStaticMethods(false).iterator();
+      } else {
+        m = ext.getNonstaticMethods(false).iterator();
+      }
+
+      while (m.hasNext()) {
+        Method method = (Method) m.next();
+        Method post = method.spawnPostHook();        
+        generateEPVEntry(post, self);
+      }
+
+      d_writer.backTab();
+      d_writer.println("};");
+      d_writer.println();
+   }
+
    /*
     * Generate the object data structure for an interface.  This structure
     * contains an EPV, a pointer to the object implementation, and an integer
@@ -911,52 +1255,43 @@ public class IORHeader {
    /**
     * Generate the different versions of the assertion controls data structure.
     */
-   private void generateControls(Extendable ext) {
-      boolean doAsserts  = IOR.supportAssertions(ext);
-      boolean doIceptors = IOR.supportInterceptors(ext);
+   private void generateControls(Extendable ext) throws CodeGenerationException {
+      boolean doAsserts = IOR.supportAssertions(ext, d_context);
 
-      if (doAsserts || doIceptors) {
-         d_writer.writeComment("Define the controls structure.", false);
-
-         SymbolID id  = ext.getSymbolID();
-         int      num = ext.getNumberOfMethods(true);
-         d_writer.println();
-         d_writer.println(IOR.getControlsStruct(id) + " {");
-         d_writer.tab();
-         if (doAsserts) {
-           d_writer.println("int32_t " + IOR.D_CHECKS + ";");
-           d_writer.println("double  " + IOR.D_RATE + ";");
-           d_writer.println(IOR.getMethodControlsStruct(id) + " {");
-           d_writer.printlnUnformatted("#ifdef " + IOR.S_FULL_STATS_MACRO);
-           d_writer.tab();
-           d_writer.println("int32_t " + IOR.D_CALLS + ";");
-           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
-           d_writer.println("int32_t " + IOR.D_METHOD_EXCEPT + ";");
-           d_writer.println("int32_t " + IOR.D_SUCCESSES + ";");
-           d_writer.println("int32_t " + IOR.D_TRIES + ";");
-           d_writer.printlnUnformatted("#else /* !" + IOR.S_FULL_STATS_MACRO 
-             + " */");
-           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
-           d_writer.printlnUnformatted("#endif /* " + IOR.S_FULL_STATS_MACRO 
-             + " */");
-           d_writer.backTab();
-           d_writer.println("} " + IOR.D_METHOD_CONTROLS + "[" + num + "];");
-         }
-         if (doIceptors) {
-           d_writer.println("int     " + IOR.D_INTERCEPTORS + ";");
-         }
-         d_writer.backTab();
-         d_writer.println("};");
+      //if (doAsserts) {
+      d_writer.writeComment("Define the controls structure.", false);
+      
+      SymbolID id  = ext.getSymbolID();
+      int      num = ext.getNumberOfMethods(true);
+      d_writer.println();
+      d_writer.println(IOR.getControlsStruct(id) + " {");
+      d_writer.tab();
+      if (doAsserts) {
+        d_writer.println("int32_t " + IOR.D_CHECKS + ";");
+        d_writer.println("double  " + IOR.D_RATE + ";");
+        d_writer.println(IOR.getMethodControlsStruct(id) + " {");
+        d_writer.printlnUnformatted("#ifdef " + IOR.S_FULL_STATS_MACRO);
+        d_writer.tab();
+        d_writer.println("int32_t " + IOR.D_CALLS + ";");
+        d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
+        d_writer.println("int32_t " + IOR.D_METHOD_EXCEPT + ";");
+        d_writer.println("int32_t " + IOR.D_SUCCESSES + ";");
+        d_writer.println("int32_t " + IOR.D_TRIES + ";");
+        d_writer.printlnUnformatted("#else /* !" + IOR.S_FULL_STATS_MACRO 
+                                    + " */");
+        d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
+        d_writer.printlnUnformatted("#endif /* " + IOR.S_FULL_STATS_MACRO 
+                                    + " */");
+        d_writer.backTab();
+        d_writer.println("} " + IOR.D_METHOD_CONTROLS + "[" + num + "];");
       }
-   }
-
-   /**
-    * Generate a version of the structure based on whether or not all of
-    * the statistics are to be retained.
-    */
-   private void generateControlsStats(SymbolID id, int num, boolean doAsserts,
-                                      boolean doIceptors, boolean all) 
-   {
+      //if (doHooks) {
+      //We need the control structure all the time.
+      d_writer.println("int     " + IOR.D_HOOKS + ";");
+        //}
+      d_writer.backTab();
+      d_writer.println("};");
+      //}
    }
 
    /**
@@ -965,7 +1300,7 @@ public class IORHeader {
     * not found in parent objects.  It also contains an EPV, a data pointer,
     * and an integer identifier.
     */
-   private void generateClassObject(Class cls) {
+   private void generateClassObject(Class cls) throws CodeGenerationException {
       d_writer.writeComment("Define the class object structure.", false);
 
       /*
@@ -993,15 +1328,14 @@ public class IORHeader {
       /*
        * Extract information about the EPV structure and its width.
        */
-      boolean addAsserts  = IOR.supportAssertions(cls);
-      boolean addIceptors = IOR.supportInterceptors(cls);
-      String  ctrls       = null;
+      boolean addAsserts = IOR.supportAssertions(cls, d_context);
+      String  ctrls      = null;
 
       SymbolID id  = cls.getSymbolID();
       String   epv = IOR.getEPVName(id) + "*";
       int      len = epv.length();
       if (len > width) { width = len; }
-      if (addAsserts || addIceptors) {
+      if (addAsserts) {
         ctrls = IOR.getControlsStruct(id);
         len   = ctrls.length();
         if (len > width) { width = len; }
@@ -1031,12 +1365,12 @@ public class IORHeader {
        */
       d_writer.printAligned(epv, width);
       d_writer.println(IOR.getEPVVar(IOR.PUBLIC_EPV) + ";");
-      if (IOR.supportBaseEPVAttr(cls)) {
+      if (IOR.supportBaseEPVAttr(cls, d_context)) {
         d_writer.printAligned(epv, width);
         d_writer.println(IOR.getEPVVar(IOR.BASE_EPV) + ";");
       }
 
-      if (addAsserts || addIceptors) {
+      if (addAsserts) {
         d_writer.printAligned(ctrls, width);
         d_writer.println(IOR.D_CONTROLS + ";");
       }
