@@ -1,7 +1,7 @@
 //
 // File:        Java.java
 // Package:     gov.llnl.babel.backend.jdk
-// Revision:    @(#) $Id: Java.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: Java.java 6196 2007-10-24 23:42:37Z epperly $
 // Description: common Java binding routines shared by Java code generators
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,15 +30,22 @@
 
 package gov.llnl.babel.backend.jdk;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.backend.writers.LanguageWriterForJava;
 import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
+
+import java.util.Set;
+import java.util.List;
+import java.util.HashSet;
 import java.util.HashMap;
 import java.util.Iterator;
 
@@ -105,7 +112,7 @@ public class Java {
     s_java_val.put(s_long,     "long");
     s_java_val.put(s_opaque,   "long");
     s_java_val.put(s_string,   "java.lang.String");
-    s_java_val.put(s_enum,     "int");
+    s_java_val.put(s_enum,     "long");
 
     s_java_ref.put(s_boolean,  "sidl.Boolean$Holder");
     s_java_ref.put(s_char,     "sidl.Character$Holder");
@@ -130,7 +137,7 @@ public class Java {
     s_ior_name.put(s_long,     "long");
     s_ior_name.put(s_opaque,   "opaque");
     s_ior_name.put(s_string,   "string");
-    s_ior_name.put(s_enum,     "int");
+    s_ior_name.put(s_enum,     "long");
     s_ior_name.put(s_class,    "BaseInterface");
     s_ior_name.put(s_interface,    "BaseInterface");
 
@@ -206,8 +213,8 @@ public class Java {
     s_in.put(s_opaque,    "#1 = (#3) JLONG_TO_POINTER(#2);");
     s_in.put(s_string,    "#1 = sidl_Java_J2I_string(env, #2);");
     s_in.put(s_enum,      "#1 = (#3) #2;");
-    s_in.put(s_class,     "#1 = (#3) sidl_Java_J2I_cls(env, #2);");
-    s_in.put(s_interface, "#1 = (#3) sidl_Java_J2I_ifc(env, #2, \"#4\");");
+    s_in.put(s_class,     "#1 = (#3) sidl_Java_J2I_cls(env, #2, FALSE);JAVA_CHECK(env);");
+    s_in.put(s_interface, "#1 = (#3) sidl_Java_J2I_ifc(env, #2, \"#4\", FALSE);JAVA_CHECK(env);");
     s_in.put(s_array,     "#1 = (#3) sidl_Java_J2I_borrow_array(env, #2);");
 
     s_inout.put(s_boolean,   "#1 = sidl_Java_J2I_boolean_holder(env, #2);");
@@ -220,11 +227,11 @@ public class Java {
     s_inout.put(s_long,      "#1 = sidl_Java_J2I_long_holder(env, #2);");
     s_inout.put(s_opaque,    "#1 = sidl_Java_J2I_opaque_holder(env, #2);");
     s_inout.put(s_string,    "#1 = sidl_Java_J2I_string_holder(env, #2);");
-    s_inout.put(s_enum,      "#1 = (#3) sidl_Java_J2I_int_holder(env, #2);");
+    s_inout.put(s_enum,      "#1 = (#3) sidl_Java_J2I_long_holder(env, #2);");
     s_inout.put(s_class,     "#1 = (#3) sidl_Java_J2I_cls_holder(env, #2, "
-                             + "\"#4\");");
+                             + "\"#4\", TRUE);JAVA_CHECK(env);");
     s_inout.put(s_interface, "#1 = (#3) sidl_Java_J2I_ifc_holder(env, #2, "
-                             + "\"#4\");");
+                             + "\"#4\", TRUE);JAVA_CHECK(env);");
     s_inout.put(s_array,     "#1 = (#3) sidl_Java_J2I_array_holder(env, #2, "
                              + "\"#4\");");
 
@@ -238,9 +245,9 @@ public class Java {
     s_post.put(s_long,      "sidl_Java_I2J_long_holder(env, #2, #1);");
     s_post.put(s_opaque,    "sidl_Java_I2J_opaque_holder(env, #2, #1);");
     s_post.put(s_string,    "sidl_Java_I2J_string_holder(env, #2, #1);");
-    s_post.put(s_enum,      "sidl_Java_I2J_int_holder(env, #2, (int) #1);");
-    s_post.put(s_class,     "sidl_Java_I2J_cls_holder(env, #2, #1, \"#4\");");
-    s_post.put(s_interface, "sidl_Java_I2J_ifc_holder(env, #2, #1, \"#4\");");
+    s_post.put(s_enum,      "sidl_Java_I2J_long_holder(env, #2, (int) #1);");
+    s_post.put(s_class,     "sidl_Java_I2J_cls_holder(env, #2, #1, \"#4\", FALSE);JAVA_CHECK(env);");
+    s_post.put(s_interface, "sidl_Java_I2J_ifc_holder(env, #2, #1, \"#4\", FALSE);JAVA_CHECK(env);");
     s_post.put(s_array,     "#1 = sidl_Java_I2J_new_array(env, #2, \"#4\");");
     //sidl_Java_I2J_set_array(env, #2, #1);");
 
@@ -255,134 +262,134 @@ public class Java {
     s_return.put(s_opaque,    "#1 = (#3) POINTER_TO_JLONG(#2);");
     s_return.put(s_string,    "#1 = sidl_Java_I2J_string(env, #2);");
     s_return.put(s_enum,      "#1 = (#3) #2;");
-    s_return.put(s_class,     "#1 = sidl_Java_I2J_cls(env, #2, \"#4\");");
-    s_return.put(s_interface, "#1 = sidl_Java_I2J_ifc(env, #2, \"#4\");");
+    s_return.put(s_class,     "#1 = sidl_Java_I2J_cls(env, #2, \"#4\", FALSE);JAVA_CHECK(env);");
+    s_return.put(s_interface, "#1 = sidl_Java_I2J_ifc(env, #2, \"#4\", FALSE);JAVA_CHECK(env);");
     s_return.put(s_array,     "#1 = sidl_Java_I2J_new_array(env, #2, \"#4\");");
 
     s_server_in.put(s_boolean,   "#1 = (#3) #2;");
     s_server_in.put(s_char,      "#1 = (#3) #2;");
-    s_server_in.put(s_dcomplex,  "#1 = sidl_Java_I2J_dcomplex(data.env, &#2);");
+    s_server_in.put(s_dcomplex,  "#1 = sidl_Java_I2J_dcomplex(env, &#2);");
     s_server_in.put(s_double,    "#1 = (#3) #2;");
-    s_server_in.put(s_fcomplex,  "#1 = sidl_Java_I2J_fcomplex(data.env, &#2);");
+    s_server_in.put(s_fcomplex,  "#1 = sidl_Java_I2J_fcomplex(env, &#2);");
     s_server_in.put(s_float,     "#1 = (#3) #2;");
     s_server_in.put(s_int,       "#1 = (#3) #2;");
     s_server_in.put(s_long,      "#1 = (#3) #2;");
     s_server_in.put(s_opaque,    "#1 = (#3) POINTER_TO_JLONG(#2);");
-    s_server_in.put(s_string,    "#1 = sidl_Java_I2J_string(data.env, #2);");
+    s_server_in.put(s_string,    "#1 = sidl_Java_I2J_string(env, #2);");
     s_server_in.put(s_enum,      "#1 = (#3) #2;");
-    s_server_in.put(s_class,     "#1 = sidl_Java_I2J_cls(data.env, #2, \"#4\")"
-                                 + ";");
-    s_server_in.put(s_interface, "#1 = sidl_Java_I2J_ifc(data.env, #2, \"#4\")"
-                                 + ";");
-    s_server_in.put(s_array,     "#1 = sidl_Java_I2J_new_array_server(data.env"
+    s_server_in.put(s_class,     "#1 = sidl_Java_I2J_cls(env, #2, \"#4\", TRUE)"
+                                 + ";JAVA_CHECK(env);");
+    s_server_in.put(s_interface, "#1 = sidl_Java_I2J_ifc(env, #2, \"#4\", TRUE)"
+                                 + ";JAVA_CHECK(env);");
+    s_server_in.put(s_array,     "#1 = sidl_Java_I2J_new_array_server(env"
                                  + ", #2, \"#4\");");
 
-    s_server_out.put(s_boolean,   "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_boolean,   "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Boolean$Holder\");");
-    s_server_out.put(s_char,      "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_char,      "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Character$Holder\");");
-    s_server_out.put(s_dcomplex,  "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_dcomplex,  "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.DoubleComplex$Holder\");");
-    s_server_out.put(s_double,    "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_double,    "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Double$Holder\");");
-    s_server_out.put(s_fcomplex,  "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_fcomplex,  "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.FloatComplex$Holder\");");
-    s_server_out.put(s_float,     "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_float,     "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Float$Holder\");");
-    s_server_out.put(s_int,       "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_int,       "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Integer$Holder\");");
-    s_server_out.put(s_long,      "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_long,      "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Long$Holder\");");
-    s_server_out.put(s_opaque,    "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_opaque,    "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Opaque$Holder\");");
-    s_server_out.put(s_string,    "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_string,    "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.String$Holder\");");
-    s_server_out.put(s_enum,      "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_enum,      "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"sidl.Enum$Holder\");");
-    s_server_out.put(s_class,     "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_class,     "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"#4$Holder\");");
-    s_server_out.put(s_interface, "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_interface, "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"#4$Holder\");");
-    s_server_out.put(s_array,     "#1 = sidl_Java_create_empty_class(data.env,"
+    s_server_out.put(s_array,     "#1 = sidl_Java_create_empty_class(env,"
                                   + " \"#4$Holder\");");
 
-    s_server_inout.put(s_boolean,   "sidl_Java_I2J_boolean_holder(data.env, #1,"
+    s_server_inout.put(s_boolean,   "sidl_Java_I2J_boolean_holder(env, #1,"
                                     + " *#2);");
-    s_server_inout.put(s_char,      "sidl_Java_I2J_character_holder(data.env, #"
+    s_server_inout.put(s_char,      "sidl_Java_I2J_character_holder(env, #"
                                     + "1, *#2);");
-    s_server_inout.put(s_dcomplex,  "sidl_Java_I2J_dcomplex_holder(data.env, #1"
+    s_server_inout.put(s_dcomplex,  "sidl_Java_I2J_dcomplex_holder(env, #1"
                                     + ", #2);");
-    s_server_inout.put(s_double,    "sidl_Java_I2J_double_holder(data.env, #1, "
+    s_server_inout.put(s_double,    "sidl_Java_I2J_double_holder(env, #1, "
                                     + "*#2);");
-    s_server_inout.put(s_fcomplex,  "sidl_Java_I2J_fcomplex_holder(data.env, #1"
+    s_server_inout.put(s_fcomplex,  "sidl_Java_I2J_fcomplex_holder(env, #1"
                                     + ", #2);");
-    s_server_inout.put(s_float,     "sidl_Java_I2J_float_holder(data.env, #1, *"
+    s_server_inout.put(s_float,     "sidl_Java_I2J_float_holder(env, #1, *"
                                     + "#2);");
-    s_server_inout.put(s_int,       "sidl_Java_I2J_int_holder(data.env, #1, *#2"
+    s_server_inout.put(s_int,       "sidl_Java_I2J_int_holder(env, #1, *#2"
                                     + ");");
-    s_server_inout.put(s_long,      "sidl_Java_I2J_long_holder(data.env, #1, *#"
+    s_server_inout.put(s_long,      "sidl_Java_I2J_long_holder(env, #1, *#"
                                     + "2);");
-    s_server_inout.put(s_opaque,    "sidl_Java_I2J_opaque_holder(data.env, #1, "
+    s_server_inout.put(s_opaque,    "sidl_Java_I2J_opaque_holder(env, #1, "
                                     + "*#2);");
-    s_server_inout.put(s_string,    "sidl_Java_I2J_string_holder(data.env, #1, "
+    s_server_inout.put(s_string,    "sidl_Java_I2J_string_holder(env, #1, "
                                     + "*#2);");
-    s_server_inout.put(s_enum,      "sidl_Java_I2J_int_holder(data.env, #1,(int"
+    s_server_inout.put(s_enum,      "sidl_Java_I2J_long_holder(env, #1,(int"
                                     + ") *#2);");
-    s_server_inout.put(s_class,     "sidl_Java_I2J_cls_holder(data.env, #1, *#2"
-                                    + ", \"#4\");");
-    s_server_inout.put(s_interface, "sidl_Java_I2J_ifc_holder(data.env, #1, *#2"
-                                    + ", \"#4\");");
-    s_server_inout.put(s_array,     "sidl_Java_I2J_array_holder(data.env, #1, #"
+    s_server_inout.put(s_class,     "sidl_Java_I2J_cls_holder(env, #1, *#2"
+                                    + ", \"#4\", FALSE);JAVA_CHECK(env);");
+    s_server_inout.put(s_interface, "sidl_Java_I2J_ifc_holder(env, #1, *#2"
+                                    + ", \"#4\", FALSE);JAVA_CHECK(env);");
+    s_server_inout.put(s_array,     "sidl_Java_I2J_array_holder(env, #1, #"
                                     + "2, \"#4\");");
 
-    s_server_post.put(s_boolean,   "*#1 = sidl_Java_J2I_boolean_holder(data.env"
+    s_server_post.put(s_boolean,   "*#1 = sidl_Java_J2I_boolean_holder(env"
                                    + ", #2);");
-    s_server_post.put(s_char,      "*#1 = sidl_Java_J2I_character_holder(data.e"
+    s_server_post.put(s_char,      "*#1 = sidl_Java_J2I_character_holder(e"
                                    + "nv, #2);");
-    s_server_post.put(s_dcomplex,  "*#1 = sidl_Java_J2I_dcomplex_holder(data.en"
+    s_server_post.put(s_dcomplex,  "*#1 = sidl_Java_J2I_dcomplex_holder(en"
                                    + "v, #2);");
-    s_server_post.put(s_double,    "*#1 = sidl_Java_J2I_double_holder(data.env,"
+    s_server_post.put(s_double,    "*#1 = sidl_Java_J2I_double_holder(env,"
                                    + " #2);");
-    s_server_post.put(s_fcomplex,  "*#1 = sidl_Java_J2I_fcomplex_holder(data.en"
+    s_server_post.put(s_fcomplex,  "*#1 = sidl_Java_J2I_fcomplex_holder(en"
                                    + "v, #2);");
-    s_server_post.put(s_float,     "*#1 = sidl_Java_J2I_float_holder(data.env, "
+    s_server_post.put(s_float,     "*#1 = sidl_Java_J2I_float_holder(env, "
                                    + "#2);");
-    s_server_post.put(s_int,       "*#1 = sidl_Java_J2I_int_holder(data.env, #2"
+    s_server_post.put(s_int,       "*#1 = sidl_Java_J2I_int_holder(env, #2"
                                    + ");");
-    s_server_post.put(s_long,      "*#1 = sidl_Java_J2I_long_holder(data.env, #"
+    s_server_post.put(s_long,      "*#1 = sidl_Java_J2I_long_holder(env, #"
                                    + "2);");
-    s_server_post.put(s_opaque,    "*#1 = sidl_Java_J2I_opaque_holder(data.env,"
+    s_server_post.put(s_opaque,    "*#1 = sidl_Java_J2I_opaque_holder(env,"
                                    + " #2);");
-    s_server_post.put(s_string,    "*#1 = sidl_Java_J2I_string_holder(data.env,"
+    s_server_post.put(s_string,    "*#1 = sidl_Java_J2I_string_holder(env,"
                                    + " #2);");
-    s_server_post.put(s_enum,      "*#1 = (#3) sidl_Java_J2I_int_holder(data.en"
+    s_server_post.put(s_enum,      "*#1 = (#3) sidl_Java_J2I_long_holder(en"
                                    + "v, #2);");
-    s_server_post.put(s_class,     "*#1 = (#3) sidl_Java_J2I_cls_holder(data.en"
-                                   + "v, #2, \"#4\");");
-    s_server_post.put(s_interface, "*#1 = (#3) sidl_Java_J2I_ifc_holder(data.en"
-                                   + "v, #2, \"#4\");");
-    s_server_post.put(s_array,     "#1 = (#3) sidl_Java_J2I_array_holder(data.e"
+    s_server_post.put(s_class,     "*#1 = (#3) sidl_Java_J2I_cls_holder(en"
+                                   + "v, #2, \"#4\", TRUE);JAVA_CHECK(env);");
+    s_server_post.put(s_interface, "*#1 = (#3) sidl_Java_J2I_ifc_holder(en"
+                                   + "v, #2, \"#4\", TRUE);JAVA_CHECK(env);");
+    s_server_post.put(s_array,     "#1 = (#3) sidl_Java_J2I_array_holder(e"
                                    + "nv, #2, \"#4\");");
 
-    s_server_return.put(s_boolean,   "#1 = (#3) #2;");
+    s_server_return.put(s_boolean,   "#1 = ( (#2) ? TRUE : FALSE);");
     s_server_return.put(s_char,      "#1 = (#3) #2;");
-    s_server_return.put(s_dcomplex,  "#1 = sidl_Java_J2I_dcomplex(data.env, #2)"
+    s_server_return.put(s_dcomplex,  "#1 = sidl_Java_J2I_dcomplex(env, #2)"
                                      + ";");
     s_server_return.put(s_double,    "#1 = (#3) #2;");
-    s_server_return.put(s_fcomplex,  "#1 = sidl_Java_J2I_fcomplex(data.env, #2)"
+    s_server_return.put(s_fcomplex,  "#1 = sidl_Java_J2I_fcomplex(env, #2)"
                                      + ";");
     s_server_return.put(s_float,     "#1 = (#3) #2;");
     s_server_return.put(s_int,       "#1 = (#3) #2;");
     s_server_return.put(s_long,      "#1 = (#3) #2;");
     s_server_return.put(s_opaque,    "#1 = (#3) JLONG_TO_POINTER(#2);");
-    s_server_return.put(s_string,    "#1 = sidl_Java_J2I_string(data.env, #2)"
+    s_server_return.put(s_string,    "#1 = sidl_Java_J2I_string(env, #2)"
                                      + ";");
     s_server_return.put(s_enum,      "#1 = (#3) #2;");
-    s_server_return.put(s_class,     "#1 = (#3) sidl_Java_J2I_cls(data.env, #2"
-                                     + ");");
-    s_server_return.put(s_interface, "#1 = (#3) sidl_Java_J2I_ifc(data.env, #2"
-                                     + ", \"#4\");");
-    s_server_return.put(s_array,     "#1 = (#3) sidl_Java_J2I_take_array(data."
+    s_server_return.put(s_class,     "#1 = (#3) sidl_Java_J2I_cls(env, #2"
+                                     + ", TRUE);JAVA_CHECK(env);");
+    s_server_return.put(s_interface, "#1 = (#3) sidl_Java_J2I_ifc(env, #2"
+                                     + ", \"#4\", TRUE);JAVA_CHECK(env);");
+    s_server_return.put(s_array,     "#1 = (#3) sidl_Java_J2I_take_array("
                                      + "env, #2);");
 
   }
@@ -404,6 +411,15 @@ public class Java {
     return id.getFullName().replace('.', '_') + "_jniStub.c";
   }
 
+  /**
+   * Generate the JNI source filename for the client using the specified
+   * symbol identifier. Simply append the suffix "_jniStub.c" to the symbol
+   * name.
+   */
+  public static String getHeaderFile(SymbolID id) {
+    return id.getFullName().replace('.', '_') + "_jniStub.h";
+  }
+
   /**
    * Generate the Java filename for the server using the specified symbol
    * identifier.  Simply append the suffix ".java" to the symbol name.
@@ -566,7 +582,7 @@ public class Java {
    * according to the hash table map.  If the type is not one of the basic
    * types, then convert the symbol type according to JNI conventions.
    */
-  private static String getDescriptor(String type) {
+  public static String getDescriptor(String type) {
     String jni = (String) s_java_sig.get(type);
     if (jni == null) {
       jni = "L" + type.replace('.', '/') + ";";
@@ -588,9 +604,6 @@ public class Java {
     int     t    = type.getDetailedType();
     int     m    = arg.getMode();
     Integer T    = new Integer(t);
-    Type arrayType = type.getArrayType();
-    Integer arrayTypeInteger = null;
-    String arrayTypeString = null;
 
     if(type.isArray()) {         //Is it an array?
       if(type.isGenericArray()) {  //is it a generic array?
@@ -823,13 +836,15 @@ public class Java {
    * the sidl type into an IOR type string and substitutes the type string in
    * the initialization phrase from the initialization hash table.
    */
-  public static void declareIORVariable(LanguageWriterForC writer, Type type, 
-                                        String variable)
+  public static void declareIORVariable(LanguageWriterForC writer, 
+                                        Type type, 
+                                        String variable,
+                                        Context context)
     throws CodeGenerationException 
   {
     printSub(writer, 
              (String)s_init_ior.get(new Integer(type.getDetailedType())),
-             getReturnString(type), variable);
+             getReturnString(type, context), variable);
   }
 
   /**
@@ -837,14 +852,16 @@ public class Java {
    * the sidl type into an IOR type string and substitutes the type string in
    * the initialization phrase from the initialization hash table. (In JNI)
    */
-  public static void declareIORVariable(LanguageWriterForC writer, Argument arg,
-                                        String prefix)
+  public static void declareIORVariable(LanguageWriterForC writer, 
+                                        Argument arg,
+                                        String prefix,
+                                        Context context)
     throws CodeGenerationException 
   {
     Type type = arg.getType();
     printSub(writer, 
              (String)s_init_ior.get(new Integer(type.getDetailedType())),
-             getReturnString(type), prefix + arg.getFormalName());
+             getReturnString(type, context), prefix + arg.getFormalName());
   }
 
   /**
@@ -878,7 +895,6 @@ public class Java {
   public static void declareServerInOutVariable(LanguageWriterForC writer, 
                                                 Type type, String variable) 
   {
-    String rtype = Java.getJNIReturnType(type);
     printSub(writer, "#1 #2 = (#1) NULL;", "jobject", variable);
   }
 
@@ -918,7 +934,9 @@ public class Java {
    * string.
    */
   public static void preprocessJNIArgument(LanguageWriterForC writer, 
-                                             Argument arg, String prefix)
+                                           Argument arg,
+                                           String prefix, 
+                                           Context context)
     throws CodeGenerationException 
   {
       String convert = null;
@@ -948,7 +966,8 @@ public class Java {
 
       if (convert != null) {
         printSub(writer, convert, prefix + arg.getFormalName(), 
-                 "_arg_" + arg.getFormalName(), getReturnString(type),
+                 "_arg_" + arg.getFormalName(), 
+                 getReturnString(type, context),
                  type.isSymbol() ? type.getSymbolID().getFullName() 
                                  : arrayTypeString);
     } 
@@ -961,7 +980,9 @@ public class Java {
    * substitutes the appropriate variable names in the conversion string.
    */
   public static void postprocessJNIArgument(LanguageWriterForC writer, 
-                                            Argument arg, String prefix)
+                                            Argument arg,
+                                            String prefix,
+                                            Context context)
     throws CodeGenerationException 
   {
     if (arg.getMode() != Argument.IN) {
@@ -990,7 +1011,7 @@ public class Java {
         /*
         if (convert != null) {
           printSub(writer, convert, prefix + arg.getFormalName(), 
-                   "_arg_" + arg.getFormalName(), getReturnString(type),
+                   "_arg_" + arg.getFormalName(), getReturnString(type, context),
                    arrayTypeString);
         }
          */
@@ -1000,7 +1021,7 @@ public class Java {
           String fromName = /*"*" + */"_arg_" + arg.getFormalName();
           
           printSub(writer, convert, fromName, prefix + arg.getFormalName(),
-                   getReturnString(type), 
+                   getReturnString(type, context), 
                    type.isSymbol() ? type.getSymbolID().getFullName() 
                                    : arrayTypeString);
         }
@@ -1009,7 +1030,8 @@ public class Java {
                                                    type.getDetailedType()));
         if (convert != null) {
           printSub(writer, convert, prefix + arg.getFormalName(),
-                   "_arg_" + arg.getFormalName(), getReturnString(type),
+                   "_arg_" + arg.getFormalName(), 
+                   getReturnString(type, context),
                    type.isSymbol() ? type.getSymbolID().getFullName() 
                                    : null);
         }
@@ -1047,7 +1069,9 @@ public class Java {
    * string.
    */
     public static void preprocessServerJNIArgument(LanguageWriterForC writer, 
-                                                   Argument arg, String prefix)
+                                                   Argument arg,
+                                                   String prefix,
+                                                   Context context)
       throws CodeGenerationException 
    {
     String convert = null;
@@ -1060,7 +1084,7 @@ public class Java {
     String argFormalName = arg.getFormalName();
     String typeName      = type.isSymbol() ? type.getSymbolID().getFullName()
                                            : " ";
-    String returnString  = getReturnString(type);
+    String returnString  = getReturnString(type, context);
 
     if(type.isArray()) {
       if (null != arrayType) {
@@ -1110,6 +1134,22 @@ public class Java {
     }
   }
 
+  private static boolean hasJavaObjectProxy(int mode,
+                                            int type)
+  {
+    if (mode != Argument.IN) return true;
+    switch(type){
+    case Type.STRING:
+    case Type.STRUCT:
+    case Type.CLASS:
+    case Type.INTERFACE:
+    case Type.ARRAY:
+      return true;
+    default:
+      return false;
+    }
+  }
+
   /**
    * Method <code>postprocessServerJNIArgument</code> converts Java arguments to
    * IOR arguments.  Conversion routines are output to the language writer.
@@ -1117,20 +1157,17 @@ public class Java {
    * substitutes the appropriate variable names in the conversion string.
    */
   public static void postprocessServerJNIArgument(LanguageWriterForC writer, 
-                                                  Argument arg, String prefix)
+                                                  Argument arg, 
+                                                  String prefix,
+                                                  Context context)
     throws CodeGenerationException 
   {
+    Type type = arg.getType();
     if (arg.getMode() != Argument.IN) {
-      Type type = arg.getType();
       Type arrayType = type.getArrayType();
       Integer arrayTypeInteger = null;
       String arrayTypeString = null;
-
-      String argFormalName = arg.getFormalName();
-      String typeName      = type.isSymbol() ? type.getSymbolID().getFullName()
-                                             : " ";
-      String returnString  = getReturnString(type);
-
+      
       if(type.isArray()) {
         if (null != arrayType) {
           arrayTypeInteger = new Integer(arrayType.getDetailedType());
@@ -1157,11 +1194,22 @@ public class Java {
         }
 
         printSub(writer, convert, toName, prefix + arg.getFormalName(), 
-                 getReturnString(type),
+                 getReturnString(type, context),
                  type.isSymbol() ? type.getSymbolID().getFullName() 
                                  : arrayTypeString);
       }
     }
+    if (hasJavaObjectProxy(arg.getMode(), type.getDetailedType())) {
+      if (type.isRarray()) {
+        writer.println("sidl_Java_destroy_array(env, " + prefix + 
+                       arg.getFormalName() + ");");
+      }
+      else {
+        writer.println("(*env)->DeleteLocalRef(env, " + prefix +
+                       arg.getFormalName() + ");");
+      }
+      writer.println("JAVA_CHECK(env);");
+    }
   }
 
   /**
@@ -1172,25 +1220,25 @@ public class Java {
    * in the conversion string.
    */
   public static void postprocessServerJNIReturn(LanguageWriterForC writer, 
-                                                Type type, String java_result, 
-                                                String ior_result) 
+                                                Type type, 
+                                                String java_result, 
+                                                String ior_result,
+                                                Context context) 
+    throws CodeGenerationException
   {
     int t = type.getDetailedType();
     printSub(writer, (String) s_server_return.get(new Integer(t)), ior_result, 
-             java_result, getJNIReturnType(type), 
+             java_result, IOR.getReturnString(type, context),
              getJavaInternalReturnType(type));
+    if (hasJavaObjectProxy(Argument.IN, t)) {
+      writer.println("(*env)->DeleteLocalRef(env, " + java_result + ");");
+      writer.println("JAVA_CHECK(env);");
+    }
     //if (t == Type.STRING) {
     //  writer.println("sidl_String_free(" + ior_result + ");");
     //}
   }
 
-  /**
-   * Substitute string tokens of the form #n for two arguments.
-   */
-  private static String substitute(String s, String s0) {
-    return substitute(s, new String[] { s0 });
-  }
-
   /**
    * Substitute string tokens of the form #n for two arguments.
    */
@@ -1237,13 +1285,6 @@ public class Java {
     return sb.toString();
   }
 
-  /**
-   * Print the specified string, s, after conversion with argument s0.
-   */
-  private static void printSub(LanguageWriter lw, String s, String s0) {
-    lw.println(substitute(s, s0));
-  }
-
   /**
    * Print the specified string, s, after conversion with arguments s0 and s1.
    */
@@ -1263,22 +1304,100 @@ public class Java {
     lw.println(substitute(s, s0, s1, s2, s3));
   }
 
-  /**
-   * Print the specified string, s, after conversion with arguments in args.
-   */
-  private static void printSub(LanguageWriter lw, String s, String[] args) {
-    lw.println(substitute(s, args));
-  }
-
   /**
     * Generate a return string for the specified SIDL type.  Most
     * of the SIDL return strings are listed in the static structures defined
     * at the start of this class.  Symbol types and array types require
     * special processing.
     */
-   private static String getReturnString(Type type)
+   private static String getReturnString(Type type,
+                                         Context context)
      throws CodeGenerationException
    {
-     return IOR.getReturnString(type, true, false);
+     return IOR.getReturnString(type, context, true, false);
    }
+
+
+  /**
+   * Generates include directives for all the Babel clases used in this 
+   * fortran stub or skel
+   * 
+   * @param writer Language writer for C
+   * @param ext Extendible (Class or Interface) to generate dependencies
+   */
+  public static Set generateStubIncludes( LanguageWriterForC writer,
+                                            Extendable ext) 
+    throws CodeGenerationException 
+  {
+    Set includes = new HashSet();
+    includes.addAll(Utilities.sort(ext.getObjectDependencies()));
+    /*
+    includes.add(ext.getSymbolID()); //add self
+    for( Iterator i = ext.getMethods(true).iterator(); i.hasNext(); ) { 
+      Method method = (Method) i.next();
+      Set argTypes = method.getSymbolReferences();
+      //includes.addAll(method.getSymbolReferences());
+      for(Iterator j = argTypes.iterator(); j.hasNext();) {
+        SymbolID argID = (SymbolID) j.next();
+        Symbol symbol = (Symbol) Utilities.lookupSymbol(argID);
+        if(symbol.getSymbolType() == Type.CLASS ||
+           symbol.getSymbolType() == Type.INTERFACE)
+          includes.add(argID);
+      }
+    }
+    */
+    if (!includes.isEmpty()){
+      writer.writeComment("Includes for all method dependencies.",false);
+      
+      List entries = Utilities.sort(includes);
+      
+      for (Iterator i = entries.iterator(); i.hasNext(); ) { 
+        String header = Java.getHeaderFile( (SymbolID) i.next());
+
+        writer.generateInclude( header, true );
+      }
+    } 
+    return includes;
+  }
+
+  /**
+   * Return an upper bound on the number of Java local object references
+   * required in the Java skeleton for the indicated type and mode.
+   *
+   * @param mode  the argument passing mode IN, INOUT, or OUT
+   * @param type  the argument type
+   */
+  private static int localJavaVars(int mode, Type type)
+  {
+    switch (type.getDetailedType()) {
+    case Type.ARRAY:
+    case Type.INTERFACE:
+    case Type.CLASS:
+    case Type.STRUCT:
+    case Type.STRING:
+      return (mode == Argument.IN) ? 1 : 2;
+    default:
+      return 0;
+    }
+  }
+
+  /**
+   * Return an upper bound on the number of Java local object references
+   * required in the Java skeleton for the indicated method.
+   */
+  public static int localJavaVars(Method m)
+  {
+    int result = 8;             // padding
+    result += localJavaVars(Argument.OUT, m.getReturnType());
+    Iterator i = m.getArgumentList().iterator();
+    while (i.hasNext()) {
+      Argument a = (Argument)i.next();
+      result += localJavaVars(a.getMode(), a.getType());
+    }
+    if (!m.getThrows().isEmpty()) {
+      ++result;
+    }
+    return result;
+  }
+    
 }
