@@ -1,8 +1,8 @@
 //
 // File:        Python.java
 // Package:     gov.llnl.babel.backend.python
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:    @(#) $Revision: 6201 $
+// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
 // Description: Python naming conventions shared by Python code generators
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -32,18 +32,27 @@
 package gov.llnl.babel.backend.python;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
-import gov.llnl.babel.backend.FileManager;
 import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.Utilities;
+import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.backend.writers.LanguageWriterForPython;
+import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Comment;
+import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Iterator;
 
 /**
  * <p>
@@ -61,15 +70,6 @@ import gov.llnl.babel.symbols.Type;
  * </p>
  */
 public class Python {
-  /**
-   * The class is going to be generating a number of files, so we keep
-   * a reference to a file manager.  This is primarily a keystroke
-   * saving decision to avoid having FileManager.getInstance() everytime 
-   * a file manager is needed.  It also keeps this class from
-   * assuming that FileManager is a singleton.
-   */
-  private static FileManager s_fileMan = FileManager.getInstance();
-
   /**
    * Generate an include file for a symbol.
    */
@@ -155,7 +155,8 @@ public class Python {
 
   public static LanguageWriterForC createCHeader(Symbol symbol,
                                                  String modifier,
-                                                 String description)
+                                                 String description,
+                                                 Context context)
     throws CodeGenerationException
   {
     final SymbolID id = symbol.getSymbolID();
@@ -163,7 +164,8 @@ public class Python {
     String filename = headerFilename(symbol, modifier);
     LanguageWriterForC lw = null;
     lw = new LanguageWriterForC
-        (s_fileMan.createFile(id, type, "PYMOD_HDRS", filename));
+        (context.getFileManager().createFile(id, type, "PYMOD_HDRS", filename),
+         context);
     lw.writeBanner(symbol, filename, CodeConstants.C_IS_IMPL, 
                    description);
     return lw;
@@ -177,23 +179,25 @@ public class Python {
    *    generation phase.
    */
   public static LanguageWriterForC createStub(Symbol symbol,
-                                              String description)
+                                              String description,
+                                              Context context)
     throws CodeGenerationException
   {
-    final boolean saveState = s_fileMan.getJavaStylePackageGeneration();
+    final boolean saveState = context.getFileManager().getJavaStylePackageGeneration();
     final SymbolID id = symbol.getSymbolID();
     final int type = symbol.getSymbolType();
     String filename = sourceFilename(symbol, "Module");
     LanguageWriterForC lw = null;
     try {
-      s_fileMan.setJavaStylePackageGeneration(true);
+      context.getFileManager().setJavaStylePackageGeneration(true);
       lw = new LanguageWriterForC
-        (s_fileMan.createFile(id, type, "PYMOD_SRCS", filename));
+        (context.getFileManager().createFile(id, type, "PYMOD_SRCS", filename),
+         context);
       lw.writeBanner(symbol, filename, CodeConstants.C_IS_NOT_IMPL, 
                      description);
     }
     finally {
-      s_fileMan.setJavaStylePackageGeneration(saveState);
+      context.getFileManager().setJavaStylePackageGeneration(saveState);
     }
     return lw;
   }
@@ -207,7 +211,8 @@ public class Python {
    *    generation phase.
    */
   public static LanguageWriterForC createSkel(Symbol symbol,
-                                              String description)
+                                              String description,
+                                              Context context)
     throws CodeGenerationException
   {
     final SymbolID id = symbol.getSymbolID();
@@ -215,7 +220,8 @@ public class Python {
     String filename = skelFilename(symbol, "pSkel");
     LanguageWriterForC lw = null;
     lw = new LanguageWriterForC
-      (s_fileMan.createFile(id, type, "SKELSRCS", filename));
+      (context.getFileManager().createFile(id, type, "SKELSRCS", filename),
+       context);
     lw.writeBanner(symbol, filename, CodeConstants.C_IS_NOT_IMPL,
                    description);
     return lw;
@@ -230,7 +236,8 @@ public class Python {
    *    generation phase.
    */
   public static LanguageWriterForC createLaunch(Symbol symbol,
-                                                String description)
+                                                String description,
+                                                Context context)
     throws CodeGenerationException
   {
     String filename = skelFilename(symbol, "pLaunch");
@@ -238,7 +245,8 @@ public class Python {
     final int type = symbol.getSymbolType();
     LanguageWriterForC lw = null;
     lw = new LanguageWriterForC
-      (s_fileMan.createFile(id, type, "LAUNCHSRCS", filename));
+      (context.getFileManager().createFile(id, type, "LAUNCHSRCS", filename),
+       context);
     lw.writeBanner(symbol, filename, CodeConstants.C_IS_NOT_IMPL,
                    description);
     return lw;
@@ -281,18 +289,19 @@ public class Python {
    *   <code>IOException</code>.
    */
   public static CodeSplicer getPySplicer(Symbol symbol, 
-                                         String filename)
+                                         String filename,
+                                         Context context)
     throws java.io.IOException
   {
-    final boolean saveState = s_fileMan.getJavaStylePackageGeneration();
+    final boolean saveState = context.getFileManager().getJavaStylePackageGeneration();
     try {
-      s_fileMan.setJavaStylePackageGeneration(true);
-      return s_fileMan.getCodeSplicer(symbol.getSymbolID(),
+      context.getFileManager().setJavaStylePackageGeneration(true);
+      return context.getFileManager().getCodeSplicer(symbol.getSymbolID(),
                                       symbol.getSymbolType(),
-                                      filename);
+                                      filename, true, false);
     }
     finally {
-      s_fileMan.setJavaStylePackageGeneration(saveState);
+      context.getFileManager().setJavaStylePackageGeneration(saveState);
     }
   }
    
@@ -312,28 +321,29 @@ public class Python {
    */
   public static LanguageWriterForPython createPyWriter(Symbol symbol,
                                                        String file,
-                                                       String description)
+                                                       String description,
+                                                       Context context)
     throws CodeGenerationException
   {
-    final boolean saveState = s_fileMan.getJavaStylePackageGeneration();
+    final boolean saveState = context.getFileManager().getJavaStylePackageGeneration();
     LanguageWriterForPython out = null;
     try {
-      s_fileMan.setJavaStylePackageGeneration(true);
+      context.getFileManager().setJavaStylePackageGeneration(true);
       if (file.charAt(0) != '_') {
         out = new LanguageWriterForPython
-          (s_fileMan.createFile(symbol.getSymbolID(),
+          (context.getFileManager().createFile(symbol.getSymbolID(),
                                 symbol.getSymbolType(), 
-                                "PYTHONSRC", file));
+                                "PYTHONSRC", file), context);
       }
       else {
         out = new LanguageWriterForPython
-          (s_fileMan.createFile(symbol.getSymbolID(),
+          (context.getFileManager().createFile(symbol.getSymbolID(),
                                 symbol.getSymbolType(),
-                                "PYTHONADMIN", file));
+                                "PYTHONADMIN", file), context);
       }
     }
     finally {
-      s_fileMan.setJavaStylePackageGeneration(saveState);
+      context.getFileManager().setJavaStylePackageGeneration(saveState);
     }
     Comment comment = symbol.getComment();
     out.writeBanner(symbol, file, CodeConstants.C_IS_NOT_IMPL,
@@ -362,6 +372,15 @@ public class Python {
       return IOR.getSymbolName(id) + "__impl_set_epv";
    }
 
+   /**
+    * Convert a sidl symbol into the name of its associated get RMI
+    * Externals method, which is the symbol name appended with 
+    * "__impl_rmi_externals".
+    */
+   public static String getRMIExternName(SymbolID id) {
+      return IOR.getSymbolName(id) + "__impl_rmi_externals";
+   }
+
    /**
     * Convert a sidl symbol into the name of its associated set static
     * EPV method, which is the symbol name appended with "__set_sepv".
@@ -406,6 +425,19 @@ public class Python {
     return buf.toString();
   }
 
+  public static int maxNameLength(Collection items)
+  {
+    int nameLength = 0;
+    Iterator i = items.iterator();
+    while (i.hasNext()) {
+      final int size = ((Struct.Item)i.next()).getName().length();
+      if (size > nameLength) {
+        nameLength = size;
+      }
+    }
+    return nameLength;
+  }
+
   public static String getExtendableConverter(Symbol symbol) {
     String symbolName = symbol.getFullName();
     StringBuffer buf = new StringBuffer(symbolName.length() + 7);
@@ -430,6 +462,59 @@ public class Python {
     return buf.toString();
   }
 
+  public static String getPyStructType(Symbol symbol)
+  {
+    return symbol.getFullName().replace('.','_') + "__pyInst";
+  }
+
+  public static String getStructInit(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 13);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pyInit");
+    return buf.toString();
+  }
+
+  public static String getStructCopy(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 13);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pyCopy");
+    return buf.toString();
+  }
+
+  public static String getStructBorrow(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 13);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pyBorrow");
+    return buf.toString();
+  }
+
+  public static String getStructDestroy(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 13);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pyDestroy");
+    return buf.toString();
+  }
+
+  public static String getStructSerialize(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 13);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pySerialize");
+    return buf.toString();
+  }
+
+  public static String getStructDeserialize(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 15);
+    buf.append(symbolName.replace('.','_'))
+      .append("__pyDeserialize");
+    return buf.toString();
+  }
+
   public static String getExtendableAddRef(Symbol symbol) {
     String symbolName = symbol.getFullName();
     StringBuffer buf = new StringBuffer(symbolName.length() + 7);
@@ -438,6 +523,23 @@ public class Python {
     return buf.toString();
   }
 
+  public static String getExtendableConnect(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 10);
+    buf.append(symbolName.replace('.','_'))
+      .append("__connectI");
+    return buf.toString();
+  }
+  
+  /*  public static String getExtendableCast(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 7);
+    buf.append(symbolName.replace('.','_'))
+      .append("__cast");
+    return buf.toString();
+  }
+  */
+
   public static String getExtendableDeref(Symbol symbol) {
     return symbol.getFullName().replace('.','_') + "_deref";
   }
@@ -519,4 +621,150 @@ public class Python {
   public static String getDestroyArray(Type arrayType) {
     return "sidl_python_deleteRef_array";
   }
+
+
+  /**
+    * Convert a SIDL symbol into the name of its associated remote
+    * connector.  This requires both the SybmolID of the class this is being
+    * defined in (sourceid) and the SymbolID of the target class to be
+    * connected (targetid) 
+    */
+  public static String getPSkelFConnectName(SymbolID sourceid, SymbolID targetid) {
+    return "pskel_"+IOR.getSymbolName(sourceid)+"_fconnect_" + IOR.getSymbolName(targetid);
+   }
+
+  /**
+    * Convert a SIDL symbol into the name of its associated remote
+    * (de)serialize method.  This requires both the SybmolID of the class
+    * this is being defined in (sourceid) and the SymbolID of the target
+    * class to be connected (targetid) 
+    */
+  public static String getPSkelSerializeName(SymbolID sourceid, 
+                                             SymbolID targetid, 
+                                             boolean serialize,
+                                             boolean inLaunch) {
+    return (inLaunch ? "" : "p") +
+      "skel_"+IOR.getSymbolName(sourceid)+
+      (serialize ? "_serialize_" : "_deserialize_") +
+      IOR.getSymbolName(targetid);
+   }
+
+  /**
+    * Convert a SIDL symbol into the name of its associated remote
+    * connector.  This requires both the SybmolID of the class this is being
+    * defined in (sourceid) and the SymbolID of the target class to be
+    * connected (targetid) 
+    */
+  public static String getPSkelFCastName(SymbolID sourceid, SymbolID targetid) {
+    return "pskel_"+IOR.getSymbolName(sourceid)+"_fcast_" + IOR.getSymbolName(targetid);
+   }
+
+  public static Method createRemoteMethod(Extendable ext, Context context) {
+    SymbolID id = ext.getSymbolID();
+    Method ret = new Method(context);
+    Argument url = new Argument(Argument.IN, new Type(Type.STRING), "url");
+    Argument self = new Argument(Argument.OUT, new Type(id, context), "self");
+    //Type retType = new Type(ext.getSymbolID());
+    ret.setMethodName("_create","Remote");
+
+    ret.setDefinitionModifier(Method.STATIC);
+    ret.setReturnType(new Type(Type.VOID)); 
+    ret.addThrows(context.getSymbolTable().lookupSymbol("sidl.rmi.NetworkException").getSymbolID());
+    ret.addArgument(self);
+    ret.addArgument(url);
+    return ret;
+  }
+
+  public static Method connectRemoteMethod(Extendable ext, Context context) {
+    SymbolID id = ext.getSymbolID();
+    Method ret = new Method(context);
+    Argument url = new Argument(Argument.IN, new Type(Type.STRING), "url");
+    Argument self = new Argument(Argument.OUT, new Type(id, context), "self");
+    //Type retType = new Type(ext.getSymbolID());
+    ret.setMethodName("_connect");
+ 
+    ret.setDefinitionModifier(Method.STATIC);
+    ret.addThrows(context.getSymbolTable().lookupSymbol("sidl.rmi.NetworkException").getSymbolID());
+    ret.addArgument(self);
+    ret.setReturnType(new Type(Type.VOID)); 
+    ret.addArgument(url);
+    return ret;
+  }
+
+  private static void generateRMISerialize(Class cls, 
+                                           LanguageWriterForC lw,
+                                           Context context,
+                                           boolean serialize)
+    throws CodeGenerationException
+  {
+    SymbolID pipe = Utilities.
+      lookupSymbol(context, serialize ? "sidl.rmi.Return" : "sidl.rmi.Call");
+    ArrayList serializeSIDs = 
+      Utilities.sort(IOR.getStructSymbolIDs(cls, serialize));
+    ArrayList con_args = new ArrayList(4);
+    con_args.add(null);
+    con_args.add(new 
+                 Argument(Argument.IN, 
+                          new Type(pipe, Type.INTERFACE, null, 0, 0, context),
+                          "pipe"));
+    con_args.add(new Argument(Argument.IN, new Type(Type.STRING), "name"));
+    con_args.add(new Argument(Argument.IN, new Type(Type.BOOLEAN), "copy"));
+    for (Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
+      final SymbolID did = (SymbolID)i.next();
+      lw.println("void (*" +
+               IOR.getVectorEntry(getPSkelSerializeName(cls, did, serialize, false)) +
+               ")(");
+      lw.tab();
+      con_args.set(0,
+                   new Argument(serialize ? Argument.IN : Argument.OUT,
+                                new Type(did, Type.STRUCT, null, 0, 
+                                         0, context),
+                                "strct"));
+      IOR.generateArguments(lw, context,
+                            "", con_args, true, true, null, true, true, false, false);
+      lw.println(");");
+      lw.backTab();
+      lw.println("");
+    }
+
+  }
+
+  /** This method generates the rmi struct that allows connect and rmicast
+   * to be called through the pSkel.
+   */
+  public static void generateRMIExternStruct(Class cls, LanguageWriterForC lw,
+                                             Context context) 
+  throws CodeGenerationException {
+    SymbolID id = cls.getSymbolID();
+    ArrayList fconnectSIDs = Utilities.sort(IOR.getFConnectSymbolIDs(cls));
+    lw.println("struct "+IOR.getSymbolName(id)+"__rmiExternals {");
+    lw.tab();
+    ArrayList con_args = new ArrayList(2);
+    
+    con_args.add(new Argument(Argument.IN, new Type(Type.STRING), "url"));
+    con_args.add(new Argument(Argument.IN, new Type(Type.BOOLEAN), "ar"));
+    for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
+      SymbolID destination_id = (SymbolID) i.next();
+       
+      lw.print(C.getSymbolObjectPtr(destination_id));
+      lw.print(" (*");
+      lw.print(IOR.getVectorEntry(getPSkelFConnectName(id,destination_id)));
+      lw.println(")(");
+      lw.tab();
+      IOR.generateArguments(lw, context,
+                            "", con_args, true, true, null, true, true, false, false);
+      lw.println(");");
+      lw.backTab();
+
+      //Forward declaration for the used functions
+      lw.println();
+    }
+    fconnectSIDs = null;
+    generateRMISerialize(cls, lw, context, true);
+    generateRMISerialize(cls, lw, context, false);
+    lw.backTab();
+    lw.println("};");
+  }
+
+
 }
