@@ -1,8 +1,8 @@
 //
 // File:        PythonServerCSource.java
 // Package:     gov.llnl.babel.backend.python
-// Revision:    @(#) $Revision: 4462 $
-// Date:        $Date: 2005-03-23 11:29:24 -0800 (Wed, 23 Mar 2005) $
+// Revision:    @(#) $Revision: 6212 $
+// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
 // Description: Generate skeleton's to link to a Python implementation
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,26 +31,30 @@
 
 package gov.llnl.babel.backend.python;
 
+import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.LevelComparator;
 import gov.llnl.babel.backend.Utilities;
+import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.python.Python;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.Type;
-import gov.llnl.babel.BabelConfiguration;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Set;
 
 /**
@@ -73,20 +77,32 @@ public class PythonServerCSource {
 
   private Class d_cls = null;
 
-  public PythonServerCSource(Class cls) {
+  private Context d_context;
+
+  private static final String s_epv = "epv";
+  private static final String s_pre_epv = "pre_epv";
+  private static final String s_post_epv = "post_epv";
+  private static final String s_sepv = "sepv";
+  private static final String s_pre_sepv = "pre_sepv";
+  private static final String s_post_sepv = "post_sepv";
+
+  public PythonServerCSource(Class cls,
+                             Context context) {
     d_cls = cls;
+    d_context = context;
   }
 
   private Collection argSymbols() throws CodeGenerationException {
     final String exceptionFundamentalName = BabelConfiguration.
-                                                   getBaseExceptionType();
+      getBaseExceptionType();
     HashSet references = new HashSet();
-    for(Iterator i = d_cls.getMethods(false).iterator(); i.hasNext() ; ){
+    for(Iterator i = d_cls.getMethods(true).iterator(); i.hasNext() ; ){
       Method m = (Method)i.next();
       references.addAll(m.getSymbolReferences());
       if (!m.getThrows().isEmpty()) {
-        references.add(Utilities.lookupSymbol(
-                                   exceptionFundamentalName).getSymbolID());
+        references.add(Utilities.lookupSymbol(d_context, 
+                                              exceptionFundamentalName).
+                       getSymbolID());
       }
     }
     references.remove(d_cls.getSymbolID());
@@ -128,18 +144,29 @@ public class PythonServerCSource {
     addInclude(IOR.getHeaderFile(d_cls.getSymbolID()), true);
     addInclude("sidlObjA.h", false);
     addInclude("sidlPyArrays.h", false);
+    d_lw.printlnUnformatted("#ifdef SIDL_HAVE_NUMPY");
+    addInclude("oldnumeric.h", false);
+    d_lw.printlnUnformatted("#elif defined(SIDL_HAVE_NUMERIC_PYTHON)");
     addInclude("Numeric/arrayobject.h", false);
+    d_lw.printlnUnformatted("#else");
+    d_lw.printlnUnformatted("#error Neither Numeric Python nor NumPy installed");
+    d_lw.printlnUnformatted("#endif");
     addInclude("sidl_header.h", true);
     addInclude("sidl_Python.h", true);
     addInclude("sidl_BaseInterface_Module.h", false);
     addInclude(Python.getCHeaderPath(d_cls, "Module"), false);
-    Iterator i = Utilities.convertIdsToSymbols(argSymbols()).iterator();
+    // add argument dependencies
+    Iterator i = Utilities.sort(Utilities.convertIdsToSymbols(d_context, argSymbols())).iterator();
     while (i.hasNext()) {
       Symbol sym = (Symbol)i.next();
       if (Symbol.ENUM != sym.getSymbolType()) {
         addInclude(Python.getCHeaderPath(sym, "Module"), false);
       }
+      if (sym instanceof Struct) {
+        addInclude(IOR.getHeaderFile(sym), true);
+      }
     }
+
     d_lw.printlnUnformatted("#include <stdlib.h>");
     d_lw.printlnUnformatted("#include <string.h>");
     d_lw.println();
@@ -149,8 +176,8 @@ public class PythonServerCSource {
     SymbolID id = d_cls.getSymbolID();
     Iterator arguments = m.getArgumentList().iterator();
     boolean needComma = false;
-    d_lw.println("static " + IOR.getReturnString(m.getReturnType(), true, 
-                                                 false));
+    d_lw.println("static " + IOR.getReturnString(m.getReturnType(), 
+                                                 d_context, true, false));
     d_lw.println(Python.getSkelMethod(id, m) + "(");
     if (!m.isStatic() || arguments.hasNext() || !m.getThrows().isEmpty()) {
       d_lw.tab();
@@ -164,7 +191,8 @@ public class PythonServerCSource {
           d_lw.println(",");
         }
         d_lw.print("/* " + arg.getModeString() + " */ ");
-        d_lw.print(IOR.getArgumentWithFormal(arg, true, false, false));
+        d_lw.print(IOR.getArgumentWithFormal(arg, d_context,
+                                             true, false, false));
         needComma = true;
       }
       if (!m.getThrows().isEmpty()) {
@@ -192,13 +220,13 @@ public class PythonServerCSource {
   private void getFunction(boolean isStatic, String  methodName) {
     if (isStatic) {
       d_lw.println("if ((_context = PyImport_ImportModule(\"" 
-        + d_cls.getFullName() + "_Impl" + "\"))) {");
+                   + d_cls.getFullName() + "_Impl" + "\"))) {");
     } else {
       d_lw.println("if ((_context = (self ? self->d_data : NULL))) {");
     }
     d_lw.tab();
     d_lw.println("if ((_pfunc = PyObject_GetAttrString(_context, \"" 
-      + methodName + "\"))) {");
+                 + methodName + "\"))) {");
     d_lw.tab();
   }
 
@@ -213,7 +241,8 @@ public class PythonServerCSource {
     d_lw.println("import_SIDLObjA();");
     d_lw.println("import_SIDLPyArrays();");
     d_lw.println(Python.getExtendableImport(d_cls) + "();");
-    for(Iterator i = Utilities.convertIdsToSymbols(argSymbols()).iterator(); 
+    for(Iterator i = Utilities.convertIdsToSymbols(d_context,
+                                                   argSymbols()).iterator(); 
         i.hasNext(); ) {
 
       Symbol sym = (Symbol)i.next();
@@ -223,17 +252,17 @@ public class PythonServerCSource {
     }
     for(Iterator i = d_cls.getMethods(false).iterator(); i.hasNext() ; ){
       Method m = (Method)i.next();
-      ArrayList args = m.getArgumentList();
+      List args = m.getArgumentList();
       for(Iterator argIt = args.iterator(); argIt.hasNext();) {
         Argument arg = (Argument)argIt.next();
         Type argType = arg.getType();
         if(argType.getType() == Type.ARRAY && argType.getArrayType() == null)
-           usesGeneric = true;
+          usesGeneric = true;
       }
     }
     if(usesGeneric == true)
       d_lw.println(Python.getImport(BabelConfiguration.getBaseInterface())
-        + "();");
+                   + "();");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.backTab();
@@ -241,6 +270,19 @@ public class PythonServerCSource {
     d_lw.println();
   }
 
+  private void cleanupRarrays(Collection arguments)
+  {
+    Iterator i = arguments.iterator();
+    while (i.hasNext()) {
+      Argument arg = (Argument)i.next();
+      if ((arg.getMode() == Argument.INOUT) &&
+          arg.getType().isRarray()) {
+        d_lw.println("sidl_python_deleteRef_array((struct sidl__array *)(*" +
+                     arg.getFormalName() + "));");
+      }
+    }
+  }
+
   private void handleExceptions(Method m, Set throwTypes)
     throws CodeGenerationException
   {
@@ -249,44 +291,53 @@ public class PythonServerCSource {
      * Be sure to use the fundamental base exception type here.
      */
     final String baseType = IOR.getObjectName
-      (Utilities.lookupSymbol(BabelConfiguration.getBaseExceptionType()).
-                                                                 getSymbolID());
-    Arrays.sort(exceptions, new LevelComparator(SymbolTable.getInstance()));
+      (Utilities.lookupSymbol(d_context, 
+                              BabelConfiguration.getBaseExceptionType()).
+       getSymbolID());
+    Arrays.sort(exceptions, new LevelComparator(d_context.getSymbolTable()));
     int i;
     d_lw.println("else {");
     d_lw.tab();
+    cleanupRarrays(m.getArgumentList());
     d_lw.println("if (PyErr_Occurred()) {");
     d_lw.tab();
     d_lw.println("PyObject *_exType, *_exValue, *_exTrace;");
     d_lw.println("PyErr_Fetch(&_exType, &_exValue, &_exTrace);");
     for(i = 0; i < exceptions.length ; ++i){
       SymbolID id = (SymbolID)exceptions[i];
-      Symbol sym = Utilities.lookupSymbol(id);
+      Symbol sym = Utilities.lookupSymbol(d_context, id);
       if (i > 0) {
         d_lw.print("else ");
       }
       d_lw.println("if (PyErr_GivenExceptionMatches(_exType, " + 
                    Python.getExceptionType(sym) + ")) {");
       d_lw.tab();
-      d_lw.println(IOR.getObjectName(id) + " *_local_exception;");
+      d_lw.println(IOR.getObjectName(id) + " *_local_exception = NULL;");
+      d_lw.println("PyObject *_sidlPyException = ");
+      d_lw.tab();
+      d_lw.println("PyObject_GetAttrString(_exValue, \"exception\");");
+      d_lw.backTab();
       d_lw.println("(void)" + Python.getExtendableConverter(sym) + 
-                   "(_exValue , &_local_exception);");
+                   "(_sidlPyException , &_local_exception);");
       d_lw.println("if (_local_exception) {");
       d_lw.tab();
-      d_lw.println(Python.getExtendableAddRef(sym) + "(_local_exception);");
+      d_lw.println("sidl_AddTrace(_sidlPyException, \"" +
+                   m.getLongMethodName() + "\");");
       d_lw.println("*_exception = (" + baseType + " *)_local_exception;");
       d_lw.backTab();
       d_lw.println("}");
+      d_lw.println("Py_XDECREF(_sidlPyException);");
       d_lw.backTab();
       d_lw.println("}");
     }
     d_lw.println("else {");
     d_lw.tab();
     d_lw.println("PyErr_Restore(_exType, _exValue, _exTrace);");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"" +
+                 m.getLongMethodName() + "\");");
     d_lw.println("_exType = NULL;");
     d_lw.println("_exValue = NULL;");
     d_lw.println("_exTrace = NULL;");
-    d_lw.println("PyErr_Print();");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println("Py_XDECREF(_exType);");
@@ -307,23 +358,20 @@ public class PythonServerCSource {
     if (!throwTypes.isEmpty()) {
       handleExceptions(m, throwTypes);
     }
-    d_lw.println("if (PyErr_Occurred()) {");
-    d_lw.tab();
-    d_lw.println("PyErr_Print();");
-    d_lw.backTab();
-    d_lw.println("}");
     d_lw.println("Py_DECREF(_args);");
     d_lw.backTab();
-    d_lw.println("} else if (PyErr_Occurred()) {");
+    d_lw.println("} else {");
     d_lw.tab();
-    d_lw.println("PyErr_Print();");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"" +
+                 m.getLongMethodName() + "\");");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println("Py_DECREF(_pfunc);");
     d_lw.backTab();
-    d_lw.println("} else if (PyErr_Occurred()) {");
+    d_lw.println("} else {");
     d_lw.tab();
-    d_lw.println("PyErr_Print();");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"" +
+                 m.getLongMethodName() + "\");");
     d_lw.backTab();
     d_lw.println("}");
     if (m.isStatic()) {
@@ -332,9 +380,10 @@ public class PythonServerCSource {
     d_lw.backTab();
     d_lw.println("}");
     if (m.isStatic()) {
-      d_lw.println("else if (PyErr_Occurred()) {");
+      d_lw.println("else {");
       d_lw.tab();
-      d_lw.println("PyErr_Print();");
+      d_lw.println("*_exception = sidl_Handle_Unexpected(\"" +
+                   m.getLongMethodName() + "\");");
       d_lw.backTab();
       d_lw.println("}");
     }
@@ -343,51 +392,74 @@ public class PythonServerCSource {
   private void declareReturnVariable(Method m) throws CodeGenerationException {
     final Type t = m.getReturnType();
     if (t.getDetailedType() != Type.VOID) {
-      d_lw.print(IOR.getReturnString(t) + " " + TranslateArguments.RETURN_VAR);
+      d_lw.print(IOR.getReturnString(t, d_context) +
+                 " " + TranslateArguments.RETURN_VAR);
       if (Utilities.isPointer(t)) {
         d_lw.print(" = NULL");
       } else if (t.getDetailedType() == Type.DCOMPLEX) {
         d_lw.print(" = { 0.0, 0.0 }");
       } else if (t.getDetailedType() == Type.FCOMPLEX) {
         d_lw.print(" = { 0.0, 0.0 }");
-      } else {
-        d_lw.print(" = (" + IOR.getReturnString(t) + ") 0");
+      } else if (t.getDetailedType() != Type.STRUCT) {
+        d_lw.print(" = (" + IOR.getReturnString(t, d_context) + ") 0");
       }
       d_lw.println(";");
     }
   }
 
-  private void initializeOutVariables(Method m) {
+  private void initializeOutVariables(Method m) 
+    throws CodeGenerationException
+  {
     Iterator i = m.getArgumentList().iterator();
     while (i.hasNext()) {
       Argument arg = (Argument)i.next();
-      if ((arg.getMode() == Argument.OUT) && 
-          Utilities.isPointer(arg.getType())){
-        d_lw.println('*' + arg.getFormalName() + " = NULL;");
+      if (arg.getMode() == Argument.OUT) {
+        if (Utilities.isPointer(arg.getType())) {
+          d_lw.println('*' + arg.getFormalName() + " = NULL;");
+        }
+        else if (Type.STRUCT == arg.getType().getDetailedType()) {
+          Symbol sym = Utilities.lookupSymbol(d_context,
+                                              arg.getType().getSymbolID());
+          d_lw.println(Python.getStructInit(sym) + "(" +
+                       arg.getFormalName() + ");");
+          
+        }
       }
     }
+    if (Type.STRUCT == m.getReturnType().getDetailedType()) {
+      Symbol sym = Utilities.lookupSymbol(d_context,
+                                          m.getReturnType().getSymbolID());
+      d_lw.println(Python.getStructInit(sym) + "(&" +
+                   TranslateArguments.RETURN_VAR + ");");
+    }
     if (!m.getThrows().isEmpty()) {
       d_lw.println("*_exception = NULL;");
     }
   }
 
   private void releaseInOutParameters(Method m) throws CodeGenerationException {
-    Extendable ext;
+    Symbol sym;
     Iterator i = m.getArgumentList().iterator();
     while (i.hasNext()) {
       Argument arg = (Argument)i.next();
       if (Argument.INOUT == arg.getMode()) {
         final Type t = arg.getType();
         switch (t.getDetailedType()) {
+        case Type.STRUCT:
+          sym = (Symbol)
+            Utilities.lookupSymbol(d_context, t.getSymbolID());
+          d_lw.println(Python.getStructDestroy(sym) +
+                       "(" + arg.getFormalName() + ");");
+          break;
         case Type.STRING:
           d_lw.println("free((void *)*" + arg.getFormalName() + ");");
           d_lw.println("*" + arg.getFormalName() + " = NULL;");
           break;
         case Type.INTERFACE:
         case Type.CLASS:
-          ext = (Extendable)
-            Utilities.lookupSymbol(t.getSymbolID());
-          d_lw.println(Python.getExtendableDeref(ext) + "(*" +
+          sym = (Symbol)
+            Utilities.lookupSymbol(d_context, t.getSymbolID());
+          d_lw.println(Python.getExtendableDeref(sym) + "(*" +
                        arg.getFormalName() + ");");
           d_lw.println("*" + arg.getFormalName() + " = NULL;");
           break;
@@ -413,7 +485,8 @@ public class PythonServerCSource {
    *    violations of the data type invariants.
    */
   private void convertMethod(Method m) throws CodeGenerationException {
-    TranslateArguments trans = new TranslateArguments(d_lw, m, false, false);
+    TranslateArguments trans = new TranslateArguments(d_lw, m, d_context,
+                                                      false, false);
     writeMethodSignature(m);
     trans.setConvertIncoming(true);
     trans.setBorrowArrays(true);
@@ -429,6 +502,7 @@ public class PythonServerCSource {
     trans.convertSidlToPython();
     d_lw.println("if (_args) {");
     d_lw.tab();
+    releaseInOutParameters(m);
     d_lw.println("_result = PyObject_CallObject(_pfunc, _args);");
     d_lw.println("if (_result) {");
     d_lw.tab();
@@ -442,7 +516,6 @@ public class PythonServerCSource {
       d_lw.println("(void)PyTuple_SetItem(_tmp, 0, _result);");
       d_lw.println("_result = _tmp;");
     }
-    releaseInOutParameters(m);
     d_lw.print("_okay = ");
     trans.setBorrowArrays(false);
     trans.convertPythonToSidl("_result");
@@ -475,27 +548,48 @@ public class PythonServerCSource {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
         convertMethod(m);
+        if (IOR.supportHooks(d_cls, d_context)) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          convertMethod(pre);
+          convertMethod(post);
+        } 
       }
     }
   }
 
-  private void writeInitializeSEPV(Collection methods) {
+  private void writeInitializeSEPV(Collection methods) 
+    throws CodeGenerationException {
     SymbolID id = d_cls.getSymbolID();
     Iterator i = methods.iterator();
     d_lw.openCxxExtern();
     d_lw.println("void");
-    d_lw.println(Python.getSetSEPVName(id));
-    d_lw.tab();
-    d_lw.println("(" + IOR.getSEPVName(id) + " *sepv)");
-    d_lw.backTab();
+    if(IOR.supportHooks(id, d_context)) {
+      d_lw.println(Python.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *"+s_sepv+",");
+      d_lw.tab();
+      d_lw.println(IOR.getPreSEPVName(id) + " *"+s_pre_sepv+", "+
+                   IOR.getPostSEPVName(id) + " *"+s_post_sepv+")");
+      d_lw.backTab();
+    } else {
+      d_lw.println(Python.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) 
+                   + " *" + s_sepv + ")");
+
+    }
+
     d_lw.println("{");
     d_lw.tab();
+
     d_lw.println("_importModules();");
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (m.isStatic()) {
-        d_lw.println("sepv->" + IOR.getVectorEntry(m.getLongMethodName()) 
-          + " = " + Python.getSkelMethod(id, m) + ";");
+        initializeEPVPointer(id, m, s_sepv);
+        if (IOR.supportHooks(d_cls, d_context)) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          initializeEPVPointer(id, pre, s_pre_sepv);
+          initializeEPVPointer(id, post, s_post_sepv);
+        }
       }
     }
     d_lw.backTab();
@@ -504,33 +598,39 @@ public class PythonServerCSource {
     d_lw.println();
   }
 
-  private void writeInitializeEPV(Collection methods) {
+  private void writeInitializeEPV(Collection methods) 
+    throws CodeGenerationException {
     SymbolID id = d_cls.getSymbolID();
     Iterator i = methods.iterator();
     d_lw.openCxxExtern();
+
     d_lw.println("void");
-    d_lw.println(Python.getSetEPVName(id));
-    d_lw.tab();
-    d_lw.println("(" + IOR.getEPVName(id) + " *epv)");
-    d_lw.backTab();
+    if(IOR.supportHooks(id, d_context)) {
+      d_lw.println(Python.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+",");
+      d_lw.tab();
+      d_lw.println(IOR.getPreEPVName(id) + " *"+s_pre_epv+", "+
+                   IOR.getPostEPVName(id) + " *"+s_post_epv+")");
+      d_lw.backTab();
+    } else {
+      d_lw.println(Python.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+")");
+    }
     d_lw.println("{");
     d_lw.tab();
     d_lw.println("_importModules();");
+    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context), s_epv);
+    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context), s_epv);
+    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context), s_epv);
+
     while (i.hasNext()) {
       Method m = (Method)i.next();
-      switch(m.getDefinitionModifier()) {
-      case Method.FINAL:
-      case Method.NORMAL:
-        d_lw.println("epv->" + IOR.getVectorEntry(m.getLongMethodName()) + " = "
-          + Python.getSkelMethod(id, m) + ";");
-        break;
-      case Method.ABSTRACT:
-        d_lw.println("epv->" + IOR.getVectorEntry(m.getLongMethodName()) 
-          + " = NULL;");
-        break;
-      default:
-        /* do nothing */
-        break;
+      if(!m.isStatic()) {
+        initializeEPVPointer(id, m, s_epv);
+        if (IOR.supportHooks(d_cls, d_context)) {
+          Method pre = m.spawnPreHook();
+          Method post = m.spawnPostHook();
+          initializeEPVPointer(id, pre, s_pre_epv);
+          initializeEPVPointer(id, post, s_post_epv);
+        }
       }
     }
     d_lw.backTab();
@@ -539,6 +639,90 @@ public class PythonServerCSource {
     d_lw.println();
   }
 
+  private void initializeEPVPointer(SymbolID id, Method m, String epvname) {
+    switch(m.getDefinitionModifier()) {
+    case Method.FINAL:
+    case Method.NORMAL:
+    case Method.STATIC:
+      d_lw.println(epvname+"->" + IOR.getVectorEntry(m.getLongMethodName()) + " = "
+                   + Python.getSkelMethod(id, m) + ";");
+      break;
+    case Method.ABSTRACT:
+      d_lw.println(epvname+"->" + IOR.getVectorEntry(m.getLongMethodName()) 
+                   + " = NULL;");
+    default:
+      /* do nothing */
+      break;
+    }
+  }
+
+  private boolean listSerialize(List ids,
+                                boolean serialize,
+                                boolean output_comma)
+  {
+    SymbolID id = d_cls.getSymbolID();
+    Iterator i = ids.iterator();
+    while (i.hasNext()) { 
+      SymbolID did = (SymbolID)i.next();
+      if (!output_comma) {
+        output_comma = true;
+      }
+      else {
+        d_lw.println(",");
+      }
+      d_lw.print(Python.getPSkelSerializeName(id, did, serialize, false));
+    }
+    return output_comma;
+  }
+
+  private void writeInitializeRMIExtern() throws CodeGenerationException {
+    SymbolID id = d_cls.getSymbolID();
+    ArrayList fconnectSIDs = Utilities.sort(IOR.getFConnectSymbolIDs(d_cls));
+    ArrayList serializeSIDs = 
+      Utilities.sort(IOR.getStructSymbolIDs(d_cls, true));
+    ArrayList deserializeSIDs = 
+      Utilities.sort(IOR.getStructSymbolIDs(d_cls, false));
+
+    boolean output_comma = false;
+
+    d_lw.printlnUnformatted("#ifdef WITH_RMI");
+
+    if(!(fconnectSIDs.isEmpty() && serializeSIDs.isEmpty() &&
+         deserializeSIDs.isEmpty())) {
+      d_lw.println("static struct " + IOR.getSymbolName(id) + "__rmiExternals s_rmiExtern = {");
+      d_lw.tab();
+      for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
+        SymbolID destination_id = (SymbolID) i.next();
+        if(!output_comma) {
+          output_comma = true;
+        } else {
+          d_lw.println(",");
+        }
+        d_lw.print(Python.getPSkelFConnectName(id,destination_id));
+      }
+      output_comma = listSerialize(serializeSIDs, true, output_comma);
+      listSerialize(deserializeSIDs, false, output_comma);
+
+      d_lw.println();
+      d_lw.backTab();
+      d_lw.println("};");
+      d_lw.println();
+      
+      
+      d_lw.openCxxExtern();
+      d_lw.println("struct " + IOR.getSymbolName(id) + "__rmiExternals *");
+      d_lw.println(Python.getRMIExternName(id)+"(void) {");
+      d_lw.tab();
+      d_lw.println("return &s_rmiExtern;");
+      d_lw.backTab();
+      d_lw.println("}"); 
+      d_lw.closeCxxExtern();
+      d_lw.println();
+      
+    }
+    d_lw.printlnUnformatted("#endif /*WITH_RMI*/");
+  }
+
   /**
    * Add the implicit builtin methods to the list of methods. There are
    * implicit functions for the constructor and destructor.
@@ -548,19 +732,33 @@ public class PythonServerCSource {
   {
     final SymbolID id = d_cls.getSymbolID();
     ArrayList  extendedMethods = new ArrayList(localMethods.size()+2);
-    extendedMethods.add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id));
-    extendedMethods.add(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id));
-    extendedMethods.addAll(localMethods);
+    extendedMethods.add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context));
+    extendedMethods.add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context));
+    extendedMethods.add(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context));
+
+    if (IOR.supportHooks(d_cls, d_context)) {
+      for(Iterator i = localMethods.iterator(); i.hasNext();) {
+        Method m = (Method) i.next();
+        Method pre = m.spawnPreHook();
+        Method post = m.spawnPostHook();
+        extendedMethods.add(pre);
+        extendedMethods.add(m);
+        extendedMethods.add(post);
+      }
+    } else {
+      extendedMethods.addAll(localMethods);
+    }
     return extendedMethods;
   }
 
   private void writeConstructor() throws CodeGenerationException {
     final SymbolID id = d_cls.getSymbolID();
-    Method ctor = IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id);
+    Method ctor = IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context);
     writeMethodSignature(ctor);
     d_lw.println("{");
     d_lw.tab();
     declareLocalVariables();
+    d_lw.println("*_exception = NULL;");
     getFunction(true, id.getShortName());
     d_lw.println("_args = Py_BuildValue(\"(O&)\", (void *)");
     d_lw.tab();
@@ -572,42 +770,63 @@ public class PythonServerCSource {
     d_lw.println("if (_result) {");
     d_lw.tab();
     d_lw.println("self->d_data = _result;");
+    d_lw.println("sidl_Python_IncGlobalRef();");
     d_lw.backTab();
     d_lw.println("}");
+    d_lw.println("else {");
+    d_lw.tab();
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"_ctor\");");
+    d_lw.backTab();
+    d_lw.println("}");
+    
     d_lw.println("Py_DECREF(_args);");
     d_lw.backTab();
     d_lw.println("} else {");
     d_lw.tab();
-    d_lw.println("if (PyErr_Occurred()) PyErr_Print();");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"_ctor\");");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println("Py_DECREF(_pfunc);");
     d_lw.backTab();
     d_lw.println("} else {");
     d_lw.tab();
-    d_lw.println("if (PyErr_Occurred()) PyErr_Print();");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"_ctor\");");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println("Py_DECREF(_context);");
     d_lw.backTab();
     d_lw.println("} else {");
     d_lw.tab();
-    d_lw.println("if (PyErr_Occurred()) PyErr_Print();");
+    d_lw.println("*_exception = sidl_Handle_Unexpected(\"_ctor\");");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println();
+    
+    //Generate ctor2, which is empty for Python (doesn't need it)
+
+    ctor = IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context);
+    d_lw.writeCommentLine("ctor2, unused in Python.");
+    writeMethodSignature(ctor);
+    d_lw.println("{");
+    d_lw.tab();
+    d_lw.println("*_exception = NULL;");
+    d_lw.backTab();
+    d_lw.println("}");
+
   }
 
   private void writeDestructor() throws CodeGenerationException {
     final SymbolID id = d_cls.getSymbolID();
-    Method dtor = IOR.getBuiltinMethod(IOR.DESTRUCTOR, id);
+    Method dtor = IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context);
     writeMethodSignature(dtor);
     d_lw.println("{");
     d_lw.tab();
+    d_lw.println("*_exception = NULL;");
     d_lw.writeCommentLine("Remove only reference to Python object");
     d_lw.println("Py_XDECREF(((PyObject *)self->d_data));");
+    d_lw.println("sidl_Python_DecGlobalRef();");
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println();
@@ -624,19 +843,21 @@ public class PythonServerCSource {
   public synchronized void generateCode() throws CodeGenerationException {
     try {
       Collection methods = d_cls.getMethods(false);
-      Collection extendedMethods = extendMethods(methods);
       d_lw = Python.createSkel
-        (d_cls, "skeletons to link IOR to Python implementation");
+        (d_cls, "skeletons to link IOR to Python implementation", d_context);
       explainSkelSource();
       includeHeaderFiles();
+      Python.generateRMIExternStruct(d_cls, d_lw, d_context);
       generateInit();
       convertMethods(methods);
       writeConstructor();
       writeDestructor();
-      writeInitializeEPV(extendedMethods);
+      writeInitializeEPV(methods);
       if (d_cls.hasStaticMethod(false)) {
-        writeInitializeSEPV(extendedMethods);
+        writeInitializeSEPV(methods);
       }
+      writeRMIAccessFunctions(d_cls); 
+      writeInitializeRMIExtern();
     } finally {
       if (d_lw != null) {
         d_lw.close();
@@ -644,4 +865,113 @@ public class PythonServerCSource {
       }
     }
   }
+
+  private void writeRMISerialize(Extendable ext, 
+                                 boolean serialize)
+    throws CodeGenerationException
+  {
+    final String baseType = IOR.getSymbolType
+      (Utilities.lookupSymbol(d_context, BabelConfiguration.getBaseExceptionType()));
+    SymbolID pipe = Utilities.
+      lookupSymbol(d_context, serialize ? "sidl.rmi.Return" : "sidl.rmi.Call");
+    final String pipeName = serialize ? "sidl.io.Serializer"
+      : "sidl.io.Deserializer";
+    Symbol pipeUp = Utilities.lookupSymbol(d_context, pipeName);
+    final String pipeType = IOR.getSymbolType(pipeUp);
+    Iterator i = IOR.getStructSymbolIDs(ext, serialize).iterator();
+    ArrayList decl_args = new ArrayList(4);
+    decl_args.add(null);
+    decl_args.add(new 
+                 Argument(Argument.IN, 
+                          new Type(pipe, Type.INTERFACE, null, 0, 0, d_context),
+                          "pipe"));
+    decl_args.add(new Argument(Argument.IN, new Type(Type.STRING), "name"));
+    decl_args.add(new Argument(Argument.IN, new Type(Type.BOOLEAN), "copy"));
+    ArrayList call_args = (ArrayList)decl_args.clone();
+    call_args.set(1, new
+                  Argument(Argument.IN,
+                           new Type(pipeUp, Type.INTERFACE, null, 0, 0, d_context),
+                           "uppipe"));
+    while (i.hasNext()) {
+      final SymbolID id = (SymbolID)i.next();
+      final Symbol sym = Utilities.lookupSymbol(d_context,id);
+      Argument arg = new Argument(serialize ? Argument.IN : Argument.OUT,
+                                  new Type(id, Type.STRUCT, null, 0, 0,
+                                           d_context),
+                                  "strct");
+      d_lw.println("static void");
+      d_lw.println(Python.getPSkelSerializeName(ext, id, serialize, false) +
+                   "(");
+      d_lw.tab();
+      decl_args.set(0, arg);
+      call_args.set(0, arg);
+      IOR.generateArguments(d_lw, d_context,
+                            "", decl_args, true, true, null, true, true, false, false);
+      d_lw.println(")");
+      d_lw.backTab();
+      d_lw.println("{");
+      d_lw.tab();
+      d_lw.println(baseType + " throwaway_exception;");
+      d_lw.println(baseType + " bipipe =");
+      d_lw.tab();
+      d_lw.println("(" + baseType + ")pipe;");
+      d_lw.backTab();
+      d_lw.println(pipeType + " uppipe =");
+      d_lw.tab();
+      d_lw.println("(" + pipeType + ")");
+      d_lw.println("(*bipipe->d_epv->f__cast)(bipipe->d_object,");
+      d_lw.println("\"" + pipeName + "\", &throwaway_exception);");
+      d_lw.backTab();
+      d_lw.println(Python.getExtendableImport(Utilities.
+                                              lookupSymbol(d_context, id)) +
+                   "();");
+      d_lw.println((serialize 
+                    ? Python.getStructSerialize(sym)
+                    : Python.getStructDeserialize(sym)) + "(");
+      d_lw.tab();
+      IOR.generateArguments(d_lw, d_context,
+                            "", call_args, true, true, null, false, true, false, false);
+      d_lw.println(");");
+      d_lw.backTab();
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println();
+    }
+  }
+
+
+  /**
+   * Write functions to call each RMI fconnect function
+   *
+   * @param cls    the class for which an routine will be written.
+   * @param writer the output writer to which the routine will be written.
+   */
+  private void writeRMIAccessFunctions(gov.llnl.babel.symbols.Class cls) 
+    throws CodeGenerationException
+  {
+    SymbolID id = cls.getSymbolID();
+    SymbolID bi_id = Utilities.lookupSymbol(d_context, "sidl.BaseInterface");
+    String bi_name = C.getSymbolObjectPtr(bi_id);
+
+    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
+    SymbolTable table = d_context.getSymbolTable();
+
+    for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {        
+      SymbolID destination_id = (SymbolID) i.next();
+      d_lw.println("static " + C.getSymbolObjectPtr(destination_id) + " " + Python.getPSkelFConnectName(id,destination_id)+ 
+                   "(const char* url, sidl_bool ar, "+bi_name+" *_ex) { ");
+      d_lw.tab();
+      d_lw.println(Python.getExtendableImport(table.lookupSymbol(destination_id))+"();");
+      d_lw.println("return "+
+                   Python.getExtendableConnect(table.lookupSymbol(destination_id))+
+                   "(url, ar, _ex);");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println();
+
+    }
+    writeRMISerialize(cls, true);
+    writeRMISerialize(cls, false);
+  }
+
 }
