@@ -1,8 +1,8 @@
 //
 // File:        TranslateArguments.java
 // Package:     gov.llnl.babel.backend.pythong
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:    @(#) $Revision: 6196 $
+// Date:        $Date: 2007-10-24 16:42:37 -0700 (Wed, 24 Oct 2007) $
 // Description: Manage the translation of parameters to/from Python
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,6 +30,7 @@
 // Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 package gov.llnl.babel.backend.python;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
@@ -46,6 +47,7 @@ import java.util.List;
 public class TranslateArguments {
   private LanguageWriter d_lw;
   private Method d_m;
+  private Context d_context;
   private boolean d_convertIncoming = true;
   private boolean d_localVars;
   private boolean d_borrowArrays = false;
@@ -54,10 +56,12 @@ public class TranslateArguments {
 
   public TranslateArguments(LanguageWriter writer,
                             Method m,
+                            Context context,
                             boolean localVars,
                             boolean isClient)
   {
     d_lw = writer;
+    d_context = context;
     d_m = m;
     d_localVars = localVars;
     d_isClient = isClient;
@@ -171,10 +175,17 @@ public class TranslateArguments {
     case Type.CHAR:
       d_lw.print("(int)");
       break;
+    case Type.STRUCT:
+      {
+        Symbol sym = Utilities.lookupSymbol(d_context, t.getSymbolID());
+        d_lw.println("(void *)" +
+                     Python.getExtendableWrapper(sym) + ", ");
+      }
+      break;
     case Type.CLASS: 
     case Type.INTERFACE: 
       {
-        Symbol sym = Utilities.lookupSymbol(t.getSymbolID());
+        Symbol sym = Utilities.lookupSymbol(d_context, t.getSymbolID());
         d_lw.print("(void *)");
         if (d_convertIncoming) {
           d_lw.print(Python.getExtendableNewRef(sym));
@@ -225,10 +236,11 @@ public class TranslateArguments {
     throws CodeGenerationException
   {
     switch(t.getDetailedType()) {
+    case Type.STRUCT:
     case Type.CLASS: 
     case Type.INTERFACE: 
       {
-        Symbol sym = Utilities.lookupSymbol(t.getSymbolID());
+        Symbol sym = Utilities.lookupSymbol(d_context, t.getSymbolID());
         d_lw.print("(void *)");
         d_lw.print(Python.getExtendableConverter(sym));
         d_lw.print(", ");
@@ -250,26 +262,49 @@ public class TranslateArguments {
     }
   }
 
-  private void outputFormatString(boolean sidlToPython) 
-    throws CodeGenerationException
+  private int numArguments()
   {
-    d_lw.disableLineBreak();
-    d_lw.print((sidlToPython && d_convertIncoming) ? "\"(" : "\"");
-    if (!d_convertIncoming) {
-      argFormat(d_m.getReturnType(), sidlToPython);
+    int result = 0;
+    if (!d_convertIncoming && (Type.VOID !=
+                               d_m.getReturnType().getDetailedType())) {
+      ++result;
     }
     Iterator i = d_m.getArgumentList().iterator();
-    while (i.hasNext()) {
-      Argument arg = (Argument)i.next();
-      final int mode = arg.getMode();
+    while(i.hasNext()) {
+      final int mode = ((Argument)i.next()).getMode();
       if ((mode == Argument.INOUT) ||
           (d_convertIncoming && (mode == Argument.IN)) ||
           ((!d_convertIncoming) && (mode == Argument.OUT))) {
-        argFormat(arg.getType(), sidlToPython);
+        ++result;
       }
     }
-    d_lw.print((sidlToPython && d_convertIncoming) ? ")\"" : "\"");
-    d_lw.enableLineBreak();
+    return result;
+  }
+
+  private void outputFormatString(boolean sidlToPython) 
+    throws CodeGenerationException
+  {
+    try {
+      d_lw.pushLineBreak(false);
+      d_lw.print((sidlToPython && d_convertIncoming) ? "\"(" : "\"");
+      if (!d_convertIncoming) {
+        argFormat(d_m.getReturnType(), sidlToPython);
+      }
+      Iterator i = d_m.getArgumentList().iterator();
+      while (i.hasNext()) {
+        Argument arg = (Argument)i.next();
+        final int mode = arg.getMode();
+        if ((mode == Argument.INOUT) ||
+            (d_convertIncoming && (mode == Argument.IN)) ||
+            ((!d_convertIncoming) && (mode == Argument.OUT))) {
+          argFormat(arg.getType(), sidlToPython);
+        }
+      }
+      d_lw.print((sidlToPython && d_convertIncoming) ? ")\"" : "\"");
+    }
+    finally {
+      d_lw.popLineBreak();
+    }
   }
 
   private void addParameter(String        param,
@@ -280,30 +315,46 @@ public class TranslateArguments {
   {
     if ((d_convertIncoming && (mode != Argument.OUT)) ||
         !(d_convertIncoming || (mode == Argument.IN))) {
-      final boolean proxy = hasProxy(t, mode == Argument.INOUT, d_isClient);
-      final boolean isPointer =
-        !((proxy && !(t.isRarray() && sidlToPython)) || 
-          d_localVars || (mode == Argument.IN) ||
-          param.equals(RETURN_VAR));
-      d_lw.println(",");
-      if (proxy && !(t.isRarray() && sidlToPython)) {
-        param = proxyVariable(param);
-      }
-      if (sidlToPython) {
-        addSidlToPythonStart(t);
-        if (isPointer) {
-          d_lw.print("*");
+      if (t.getDetailedType() == Type.STRUCT) {
+        d_lw.println(",");
+        if (sidlToPython) {
+          addSidlToPythonStart(t);
+        }
+        else {
+          addPythonToSidlStart(t);
+        }
+        d_lw.print(((d_isClient || param.equals(RETURN_VAR)) ? "&" : "") + 
+                   param);
+        if (sidlToPython) {
+          addSidlToPythonEnd(t);
         }
       }
       else {
-        addPythonToSidlStart(t);
-        if (!isPointer) {
-          d_lw.print("&");
+        final boolean proxy = hasProxy(t, mode == Argument.INOUT, d_isClient);
+        final boolean isPointer =
+          !((proxy && !(t.isRarray() && sidlToPython)) || 
+            d_localVars || (mode == Argument.IN) ||
+            param.equals(RETURN_VAR));
+        d_lw.println(",");
+        if (proxy && !(t.isRarray() && sidlToPython)) {
+          param = proxyVariable(param);
+        }
+        if (sidlToPython) {
+          addSidlToPythonStart(t);
+          if (isPointer) {
+            d_lw.print("*");
+          }
+        }
+        else {
+          addPythonToSidlStart(t);
+          if (!isPointer) {
+            d_lw.print("&");
+          }
+        }
+        d_lw.print(param);
+        if (sidlToPython) {
+          addSidlToPythonEnd(t);
         }
-      }
-      d_lw.print(param);
-      if (sidlToPython) {
-        addSidlToPythonEnd(t);
       }
     }
   }
@@ -365,6 +416,7 @@ public class TranslateArguments {
     case Type.INT:
       d_lw.print("l");
       break;
+    case Type.ENUM:
     case Type.LONG:
       d_lw.println("\"");
       d_lw.printlnUnformatted("#ifdef HAVE_LONG_LONG");
@@ -377,12 +429,10 @@ public class TranslateArguments {
     case Type.STRING:
       d_lw.print("z");
       break;
-    case Type.ENUM:
-      d_lw.print("l");
-      break;
     case Type.VOID:
       /* don't print anything */
       break;
+    case Type.STRUCT:
     case Type.CLASS:
     case Type.INTERFACE:
     case Type.ARRAY:
@@ -445,7 +495,7 @@ public class TranslateArguments {
     ArrayList outList = new ArrayList(argList.size() + 1);
     if (returnType.getDetailedType() != Type.VOID) {
       outList.add(new 
-        Argument(false, Argument.OUT, returnType, RETURN_VAR));
+        Argument(Argument.OUT, returnType, RETURN_VAR));
     }
     while (args.hasNext()){
       Argument arg = (Argument)args.next();
@@ -492,12 +542,22 @@ public class TranslateArguments {
   public void convertSidlToPython()
     throws CodeGenerationException
   {
-    d_lw.println("Py_BuildValue(");
-    d_lw.tab();
-    outputFormatString(true);
-    outputParameters(true);
-    d_lw.println(");");
-    d_lw.backTab();
+    if (numArguments() > 0) {
+      d_lw.println("Py_BuildValue(");
+      d_lw.tab();
+      outputFormatString(true);
+      outputParameters(true);
+      d_lw.println(");");
+      d_lw.backTab();
+    }
+    else {
+      if (d_convertIncoming) {
+        d_lw.println(" PyTuple_New(0);");
+      }
+      else {
+        d_lw.println(" Py_None; Py_INCREF(Py_None);");
+      }
+    }
   }
 
   public void declareProxy(final Type t,
@@ -510,7 +570,8 @@ public class TranslateArguments {
       switch (typeCode) {
       case Type.ARRAY:
         if (t.isRarray()) {
-          d_lw.println(IOR.getReturnString(t) + proxyVariable(name) + " = NULL;");
+          d_lw.println(IOR.getReturnString(t, d_context) + 
+                       proxyVariable(name) + " = NULL;");
         }
         break;
       case Type.BOOLEAN:
@@ -526,9 +587,9 @@ public class TranslateArguments {
         d_lw.println("Py_complex " + proxyVariable(name) + ";");
         break;
       case Type.INT:
-      case Type.ENUM:
         d_lw.println("long " + proxyVariable(name) + ";");
         break;
+      case Type.ENUM:
       case Type.LONG:
         d_lw.printlnUnformatted("#ifdef HAVE_LONG_LONG");
         d_lw.println("long long int " + proxyVariable(name) + ";");
