@@ -1,7 +1,7 @@
 //
 // File:        SidlSource.java
 // Package:     gov.llnl.babel.backend.sidl
-// Revision:    @(#) $Revision: 4434 $
+// Revision:    @(#) $Revision: 6212 $
 // Description: Generate a sidl file for contents of symbol table
 //
 // Copyright (c) 2002-2003, The Regents of the University of Calfornia.
@@ -32,6 +32,7 @@
 package gov.llnl.babel.backend.sidl;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.sidl.ScopedName;
 import gov.llnl.babel.backend.sidl.Sidl;
@@ -40,6 +41,7 @@ import gov.llnl.babel.backend.writers.LanguageWriterForSidl;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Assertion;
+import gov.llnl.babel.symbols.Attributes;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Extendable;
@@ -47,13 +49,16 @@ import gov.llnl.babel.symbols.Enumeration;
 import gov.llnl.babel.symbols.Interface;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Package;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
-import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.SymbolUtilities;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Set;
 
 
@@ -63,27 +68,78 @@ import java.util.Set;
  */
 public class SidlSource {
 
+  private Context d_context;
+
   /**
    * This is the output device.
    */
   private LanguageWriterForSidl d_lw;
 
-  /**
-   * Also need a handle to the symbol table so we can obtain the symbol
-   * associated with a package's id.
-   */
-  private static SymbolTable d_table = SymbolTable.getInstance();
-
-
   /**
    * Generate an instance to write the SIDL file.
    * 
    * @param writer  the output device to which the SIDL should be written.
    */
-  public SidlSource(LanguageWriterForSidl writer) {
+  public SidlSource(LanguageWriterForSidl writer,
+                    Context context) {
     d_lw    = writer;
+    d_context = context;
   }
 
+  /**
+   * Generate the SIDL for a struct.
+   *
+   * @param enm  the SIDL struct to be written.
+   * @exception   gov.llnl.babel.backend.CodeGenerationException
+   *   a catch all exception to indicate problems in the code generation
+   *   phase.
+   */
+  public void generateCode(Struct st)
+  {
+    d_lw.writeComment(st, true, false);
+    generateCustomAttrs(st, null);
+    d_lw.println("struct " + st.getSymbolID().getShortName() + " {");
+    d_lw.tab();
+    Iterator i = st.getItems().iterator();
+    while(i.hasNext()) {
+      Struct.Item item = (Struct.Item)i.next();
+      d_lw.println(item.getType().getTypeString() + " " + item.getName() + ";");
+    }
+    d_lw.backTab();
+    d_lw.println("}");
+  }
+
+  private void generateCustomAttrs(Attributes a, String [] builtins)
+  {
+    if (a.getAttributes().size() > 0) {
+      HashSet all = new HashSet(a.getAttributes());
+      if (builtins != null) {
+        for(int i = 0; i < builtins.length; ++i) {
+          all.remove(builtins[i]);
+        }
+      }
+      if (all.size() > 0) {
+        Object [] attrs = all.toArray();
+        Arrays.sort(attrs);
+        d_lw.print("%attrib{");
+        for(int i = 0; i < attrs.length; ++i) {
+          final String key=attrs[i].toString();
+          String value = a.getAttribute(key);
+          d_lw.print(" ");
+          d_lw.print(key);
+          if (value != null) {
+            d_lw.print(" = \"");
+            d_lw.print(value);
+            d_lw.print("\"");
+          }
+          if ((i+1) < attrs.length) {
+            d_lw.print(",");
+          }
+        }
+        d_lw.print(" } ");
+      }
+    }
+  }
 
   /**
    * Generate the SIDL for the enumeration.
@@ -101,6 +157,7 @@ public class SidlSource {
 
     d_lw.println();
     d_lw.writeComment(enm, true, false);
+    generateCustomAttrs(enm, null);
     d_lw.println("enum " + ename + " {");
     d_lw.tab();
     int maxlength = Utilities.getWidth(enm.getEnumerators());
@@ -113,7 +170,7 @@ public class SidlSource {
         d_lw.print(" = ");
         d_lw.print(String.valueOf(enm.getEnumeratorValue(name)));
       }
-      d_lw.println(",");
+      d_lw.println(e.hasNext() ? "," : "");
       if ((cmt != null) && (!cmt.isEmpty()) ) {
         d_lw.println();
       }
@@ -127,7 +184,7 @@ public class SidlSource {
    * Return indication of whether or not the symbol name is the same as
    * the base class.
    *
-   * @param sym  the symbol name being checked
+   * @param name the symbol name being checked
    * @return     true if the name equals the base class name; else, false
    */
   public static boolean isBaseClassName(String name) {
@@ -147,7 +204,7 @@ public class SidlSource {
    * Return indication of whether or not the symbol name is the same as
    * the base interface.
    *
-   * @param sym  the symbol name being checked
+   * @param name  the symbol name being checked
    * @return     true if the name equals the base interface name; else, false
    */
   public static boolean isBaseInterfaceName(String name) {
@@ -227,10 +284,6 @@ public class SidlSource {
    * Print the assertion.
    *
    * @param lw           the language writer to which the signature is printed
-   * @param meth         the method whose signature is being printed
-   * @param parent_pkg   the string containing the parent package.  When not
-   *                     null, it is used to strip the package from the return
-   *                     string if it is in the specified package.
    */
   public void printAssertion(LanguageWriter lw, Assertion as)
   {
@@ -247,20 +300,19 @@ public class SidlSource {
    * in specified package.
    *
    * @param lw           the language writer to which the signature is printed
-   * @param is_interface True if interface and want the implicit definition
-   *                     modifier excluded from the result; otherwise, false
+   * @param ext          The Extendable this method is being generated for
    * @param meth         the method whose signature is being printed
    * @param parent_pkg   the string containing the parent package.  When not
    *                     null, it is used to strip the package from the return
    *                     string if it is in the specified package.
    */
-  public void printSignaturePlus(LanguageWriter lw, boolean is_interface,
+  public void printSignaturePlus(LanguageWriter lw, Extendable ext,
                              Method meth, String parent_pkg)
   {
-    lw.print(meth.getSignaturePreface(is_interface, parent_pkg) + "(");
+    lw.print(meth.getSignaturePreface(ext.isInterface(), parent_pkg) + "(");
 
     int       increased_tabs = 0;
-    ArrayList args           = meth.getArgumentListWithIndices();
+    List args           = meth.getArgumentListWithIndices();
     for (Iterator a = args.iterator(); a.hasNext(); ) {
       // ToDo...Once ScopedName's getScopedName() differs from the
       //     ...SymbolUtilities' getSymbolName method used by the 
@@ -277,9 +329,9 @@ public class SidlSource {
         }
       }
     }
-    lw.print(")" + meth.getCommunicationModifierString());
+    lw.print(")");// + meth.getCommunicationModifierString());
 
-    Set exceptions = meth.getThrows();
+    Set exceptions = meth.getExplicitThrows();
     if (!exceptions.isEmpty()) {
       lw.println();
       if (increased_tabs > 0) {
@@ -304,10 +356,21 @@ public class SidlSource {
         }
       }
     }
+
+    if(ext.methodWasRenamed(meth)) {
+      Method oldM = ext.getRenamedMethod(meth);
+      SymbolID sid = ext.getRenamedMethodSymbolID(oldM);
+      lw.print(" from " + sid.getFullName()+"."+oldM.getShortMethodName());
+      String extension = oldM.getNameExtension();
+      if ((extension != null) && (extension != "")) {
+        lw.print(" ["+extension+"] ");
+      }
+    }
+
     boolean   noAssertions  = true;
-    ArrayList preconditions = meth.getPreconditions();
+    List preconditions = meth.getPreconditions();
     if (!preconditions.isEmpty()) {
-      lw.println();
+      lw.println(";");
       for (int i=increased_tabs; i > 1; i--) {
         lw.backTab();
         increased_tabs = increased_tabs - 1;
@@ -322,11 +385,11 @@ public class SidlSource {
       lw.backTab();
       lw.backTab();
     }
-    ArrayList postconditions = meth.getPostconditions();
+    List postconditions = meth.getPostconditions();
     if (!postconditions.isEmpty()) {
       if (noAssertions) {
         noAssertions = false;
-        lw.println();
+        lw.println(";");
         for (int i=increased_tabs; i > 1; i--) {
           lw.backTab();
           increased_tabs = increased_tabs - 1;
@@ -370,7 +433,7 @@ public class SidlSource {
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method meth = (Method)m.next();
       d_lw.writeComment(meth, true, false);
-      printSignaturePlus(d_lw, ext.isInterface(), meth,
+      printSignaturePlus(d_lw, ext, meth,
                          SymbolUtilities.getParentPackage(name));
       if ( m.hasNext() ) {
         d_lw.println();
@@ -384,7 +447,7 @@ public class SidlSource {
   /**
    * Generate the SIDL for the class.
    * 
-   * @param ext  the SIDL class to be written.
+   * @param cls  the SIDL class to be written.
    * @exception  gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
@@ -392,6 +455,7 @@ public class SidlSource {
   public void generateCode(Class cls)
     throws CodeGenerationException
   {
+    String builtinAttrs[] = { "abstract" };
     SymbolID sid = cls.getSymbolID();
     String sname = sid.getShortName();
     String scope = SymbolUtilities.getParentPackage(sid.getFullName());
@@ -401,6 +465,7 @@ public class SidlSource {
     if (cls.isAbstract()) {
       d_lw.print("abstract ");
     }
+    generateCustomAttrs(cls, builtinAttrs);
     d_lw.print("class " + sname + " ");
     Class   parent          = cls.getParentClass();
     boolean printed_extends = false;
@@ -426,7 +491,7 @@ public class SidlSource {
   /**
    * Generate the SIDL for the interface.
    * 
-   * @param ext  the SIDL interface to be written.
+   * @param iface  the SIDL interface to be written.
    * @exception  gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
@@ -438,6 +503,7 @@ public class SidlSource {
 
     d_lw.println();
     d_lw.writeComment(iface, true, false);
+    generateCustomAttrs(iface, null);
     d_lw.print("interface " + name + " ");
     printParentInterfaces((Extendable)iface);
     printMethods((Extendable)iface);
@@ -452,14 +518,14 @@ public class SidlSource {
    */
   public void printRequires(Package pkg)
   {
-    ScopedName.printRequires(pkg, d_lw);
+    ScopedName.printRequires(d_context, pkg, d_lw);
   }
 
 
   /**
    * Generate the SIDL for a package.  
    * 
-   * @param ext  the SIDL package whose file is to be written.
+   * @param pkg  the SIDL package whose file is to be written.
    * @exception  gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
@@ -467,6 +533,7 @@ public class SidlSource {
   public void generateCode(Package pkg)
     throws CodeGenerationException
   {
+    final String builtinAttrs[] = { "final" };
     Collection symids  = pkg.getOrderedSymbolReferences();
     SymbolID id        = pkg.getSymbolID();
     String name        = id.getShortName();
@@ -488,8 +555,9 @@ public class SidlSource {
     if ( pkg.getFinal() ) {
       d_lw.print("final ");
     }
+    generateCustomAttrs(pkg, builtinAttrs);
     d_lw.print("package " + name);
-    if ( !SymbolUtilities.sameVersionAsParent(id) ) {
+    if ( !SymbolUtilities.sameVersionAsParent(d_context, id) ) {
       vstr = SymbolUtilities.getVersionString(id);
       if (vstr != null) {
         d_lw.print(" " + vstr);
@@ -501,8 +569,11 @@ public class SidlSource {
     d_lw.tab();
     for (Iterator s = symids.iterator(); s.hasNext(); ) {
       SymbolID sid  = (SymbolID)s.next();
-      Symbol sym    = d_table.lookupSymbol(sid);
+      Symbol sym    = d_context.getSymbolTable().lookupSymbol(sid);
       switch (sym.getSymbolType()) { 
+      case Symbol.STRUCT:
+        generateCode((Struct)sym);
+        break;
         case Symbol.CLASS:
           generateCode((Class)sym);
           break;
@@ -536,11 +607,12 @@ public class SidlSource {
    *   phase.
   */
   public static void generateCode(Symbol                sym,
-                                  LanguageWriterForSidl writer)
+                                  LanguageWriterForSidl writer,
+                                  Context               context)
     throws CodeGenerationException
   {
     if (sym instanceof Package) {
-      SidlSource sidlFile = new SidlSource(writer);
+      SidlSource sidlFile = new SidlSource(writer, context);
       sidlFile.generateCode((Package)sym);
     }
   }
