@@ -1,8 +1,8 @@
 //
 // File:        CxxStubHeader.java
 // Package:     gov.llnl.babel.backend.ucxx
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:    @(#) $Revision: 6201 $
+// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
 // Description: Write Cxx extension header file for a BABEL extendable
 // 
 // This is typically directed by GenCxxClient.
@@ -33,29 +33,28 @@
 package gov.llnl.babel.backend.ucxx;
 
 import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
-import gov.llnl.babel.backend.ucxx.Cxx;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.LevelComparator;
+import gov.llnl.babel.backend.Utilities;
+import gov.llnl.babel.backend.rmi.RMIStubHeader;
+import gov.llnl.babel.backend.ucxx.Cxx;
 import gov.llnl.babel.backend.writers.LanguageWriterForCxx;
-import gov.llnl.babel.symbols.Symbol;
-import gov.llnl.babel.symbols.Extendable;
-import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Class;
+import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.Interface;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
-import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.Type;
-import gov.llnl.babel.backend.Utilities;
-import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Iterator;
 import java.util.Collection;
 import java.util.HashSet;
-import java.util.Set;
+import java.util.Iterator;
 import java.util.List;
-
+import java.util.Map;
+import java.util.Set;
 
 /**
  * 
@@ -64,6 +63,7 @@ public class CxxStubHeader {
   private Extendable d_ext = null;
   private LanguageWriterForCxx d_writer = null;
   private String d_self = null;
+  private Context d_context;
 
   /**
    * Create an object capable of generating the header file for a
@@ -72,8 +72,10 @@ public class CxxStubHeader {
    * @param ext   an interface or class symbol that needs a header
    *              file for a Cxx C extension class.
    */
-  public CxxStubHeader(Extendable ext) {
+  public CxxStubHeader(Extendable ext,
+                       Context context) {
     d_ext = ext;
+    d_context = context;
   }
   
     /**
@@ -83,10 +85,12 @@ public class CxxStubHeader {
    * @param ext   an interface or class symbol that needs a header
    *              file for a Cxx C extension class.
    */
-  public CxxStubHeader(Extendable ext, LanguageWriterForCxx writer) {
+  public CxxStubHeader(Extendable ext, LanguageWriterForCxx writer,
+                       Context context) {
     d_ext = ext;
     d_writer = writer;
     d_self = "self";
+    d_context = context;
   }
 
   /**
@@ -95,22 +99,21 @@ public class CxxStubHeader {
    * The output stream is not closed on exit.  A code
    * generation exception is thrown if an error is detected.
    *
-   * @param methods is a collection of super methods to be generated.
-   *
+   * @param cls The class in which these supers are to be generated  
+   * 
    * @param writer the output writer to which the stub source will
    *               be written. This will not be closed.
-   *
-   * @param cls The class in which these supers are to be generated      
-   *
+   *    
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
   public static void generateSupers(Class             cls,
-                                  LanguageWriterForCxx writer)
+                                  LanguageWriterForCxx writer,
+                                    Context context)
     throws CodeGenerationException
   {
-    CxxStubHeader source = new CxxStubHeader(cls, writer);
+    CxxStubHeader source = new CxxStubHeader(cls, writer, context);
     source.generateSupers();
   }  
   
@@ -124,7 +127,6 @@ public class CxxStubHeader {
     throws CodeGenerationException {
     Class cls = (Class) d_ext;
     SymbolID clsID = cls.getSymbolID();
-    final String ext_name = IOR.getExternalName(clsID);
     d_writer.println("public:");
     d_writer.writeComment("Hold pointer to IOR functions.", false);
 
@@ -138,6 +140,9 @@ public class CxxStubHeader {
                      "* superEPV;");
     d_writer.println(/*"const " + */IOR.getObjectName(clsID) + "* superSelf;");
     d_writer.println();
+    
+    throwExceptionMethods(true);
+
     d_writer.backTab();
     d_writer.println("public:"); 
     d_writer.tab();
@@ -153,15 +158,59 @@ public class CxxStubHeader {
     Collection methods = cls.getOverwrittenClassMethods();
     for(Iterator mit = methods.iterator(); mit.hasNext();){
       Method method = (Method)mit.next();
-      generateInlineMethodDispatch(method, null, true, true);
+      //generateInlineMethodDispatch(method, null, true, true);
+      if(!Cxx.inlineStub(method)) {
+        Cxx.generateMethodSignature( d_writer, d_context, method, 
+                                     "user defined static method", 
+                                     Cxx.FILE_ROLE_STUB, true);
+        d_writer.println();
+        if(method.hasRarray()) {
+          Cxx.generateMethodSignature( d_writer, d_context, method, 
+                                       "user defined static method", 
+                                       Cxx.FILE_ROLE_STUB, false);
+          d_writer.println();
+        }
+      }
     }
-    
     d_writer.backTab();
     d_writer.backTab();
     d_writer.println("};");
 
   }
 
+  private void includeSet(Set symbols)
+  {
+    if (!symbols.isEmpty()) {
+      List entries = Utilities.sort(symbols);
+      for(Iterator i = entries.iterator(); i.hasNext(); ) {
+        d_writer.
+          generateInclude(Cxx.generateFilename((SymbolID)i.next(),
+                                               Cxx.FILE_ROLE_STUB,
+                                               Cxx.FILE_TYPE_CXX_HEADER,
+                                               d_context),
+                          true);
+      }
+    }
+  }
+
+  public void generateFrontIncludes()
+    throws CodeGenerationException
+  {
+    Set symbols = Cxx.getFrontIncludes(d_ext, d_context);
+    d_writer.generateInclude( "sidl_cxx.hxx", true);
+    d_writer.generateInclude(IOR.getHeaderFile(d_ext.getSymbolID()), true);
+    includeSet(symbols);
+  }
+
+  public void generateEndIncludes()
+    throws CodeGenerationException
+  {
+    Set symbols = Cxx.generateIncludeSet(d_ext, d_context);
+    symbols.removeAll(Cxx.getFrontIncludes(d_ext, d_context));
+    includeSet(symbols);
+  }
+
+  
 
   /**
    * Generate the header file for the extendable with which this object was
@@ -176,8 +225,8 @@ public class CxxStubHeader {
   {
     String filename = Cxx.generateFilename( d_ext.getSymbolID(), 
                                             Cxx.FILE_ROLE_STUB, 
-                                            Cxx.FILE_TYPE_CXX_HEADER );
-    //System.out.println("Create " + filename + "..." );
+                                            Cxx.FILE_TYPE_CXX_HEADER ,
+                                            d_context);
     
     if ( d_ext.isInterface() ) { 
       d_self = "loc_self->d_object";
@@ -186,23 +235,41 @@ public class CxxStubHeader {
     }
 
     try { 
-      d_writer = Cxx.createHeader( d_ext, Cxx.FILE_ROLE_STUB, "STUBHDRS");
-      if ( d_ext.getSymbolType() == Symbol.PACKAGE ) { 
-	  d_writer.skipIncludeGuard();
-      }
+      d_writer = Cxx.createHeader( d_ext, Cxx.FILE_ROLE_STUB, "STUBHDRS",
+                                   d_context );
       d_writer.println();
       d_writer.openHeaderGuard( filename );
 
       writeDefineNullIORException();
 
       writeClassDeclaration();
-      
-      Cxx.generateDependencyIncludes( d_writer, d_ext, true );
+
+      generateFrontIncludes();
+      Cxx.openUCxxNamespace(d_writer);
+      d_writer.println("namespace sidl {");
+      d_writer.tab();
+      d_writer.println("namespace rmi {");
+      d_writer.tab();
+      d_writer.println("class Call;");
+      d_writer.println("class Return;");
+      d_writer.println("class Ticket;");
+      d_writer.backTab();
+      d_writer.println("}");
+      d_writer.println("namespace rmi {");
+      d_writer.tab();
+      d_writer.println("class InstanceHandle;");
+      d_writer.backTab();
+      d_writer.println("}");
+      d_writer.backTab();
+      d_writer.println("}");
+      Cxx.closeUCxxNamespace(d_writer);
 
       Cxx.nestPackagesInNamespaces( d_writer, d_ext );
-      
+
       writeClassBegin();
-      
+
+      throwExceptionMethods(false);
+
       writeUserDefinedMethods();
       
       writeTypedefs();
@@ -213,15 +280,30 @@ public class CxxStubHeader {
 
       writeBindingSpecificMethods();
 
+      if(d_ext.isInterface()) {
+        writeLocalIOR();
+      }
+
       writeClassEnd(); 
       
       Cxx.unnestPackagesInNamespaces( d_writer, d_ext );
-      
+      Cxx.beginExternCRegion(d_writer); 
+      RMIStubHeader.generateCode(d_ext, d_writer);
+      Cxx.endExternCRegion(d_writer); 
+
       writeArrayDefinition();
 
+      generateEndIncludes();
+
       d_writer.closeHeaderGuard();
     } catch ( Exception ex ) { 
-      throw new CodeGenerationException("Exception : " + ex.getMessage() );
+      if(ex != null) {
+        System.out.println(ex.getMessage());
+        ex.printStackTrace();
+        throw new CodeGenerationException("Exception : " + ex.getMessage() );
+      } else {
+        System.out.println("ex is NULL?!?");
+      }
     } finally { 
       if (d_writer != null) {
         d_writer.close();
@@ -235,11 +317,10 @@ public class CxxStubHeader {
    * this file.
    */
   private void writeDefineNullIORException() { 
-    BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
     String flagname = d_ext.getSymbolID().getFullName().replace('.', '_').toUpperCase() + 
       "_NULL_IOR_EXCEPTION";
 
-    if(s_babel_config.makeCxxCheckNullIOR()) {
+    if(d_context.getConfig().makeCxxCheckNullIOR()) {
       d_writer.printlnUnformatted("#ifndef " + flagname);
       d_writer.printlnUnformatted("#define " + flagname + " 1");
       d_writer.printlnUnformatted("#endif ");
@@ -250,7 +331,9 @@ public class CxxStubHeader {
     SymbolID id = d_ext.getSymbolID();
     String name = id.getShortName();
     
-    d_writer.writeCommentLine("declare class before #includes");
+    d_writer.generateInclude("sidl_cxx.hxx", true );
+
+    d_writer.writeCommentLine("declare class before main #includes");
     d_writer.writeCommentLine("(this alleviates circular #include guard problems)[BUG#393]");
     Cxx.nestPackagesInNamespaces( d_writer, d_ext );
     d_writer.println("class " + name + ";");
@@ -259,7 +342,6 @@ public class CxxStubHeader {
     
 
     d_writer.writeCommentLine("Some compilers need to define array template before the specializations");
-    d_writer.generateInclude("sidl_ucxx.hh", true );
     Cxx.openUCxxNamespace(d_writer);
     d_writer.println("namespace sidl {");
     d_writer.tab();
@@ -271,10 +353,10 @@ public class CxxStubHeader {
     d_writer.backTab();
     Cxx.closeUCxxNamespace(d_writer);
     
-        //Forward declarations for method dependencies
+    //Forward declarations for method dependencies
 
     Set decls = new HashSet();
-    for( Iterator i = d_ext.getMethods(true).iterator(); i.hasNext(); ) { 
+    for( Iterator i = Cxx.getStubMethodList(d_ext).iterator(); i.hasNext(); ) {
       Method method = (Method) i.next();
       decls.addAll(method.getSymbolReferences()); 
     }
@@ -285,12 +367,21 @@ public class CxxStubHeader {
       
       for (Iterator i = entries.iterator(); i.hasNext(); ) { 
         SymbolID s = (SymbolID) i.next();
-        Symbol symbol = (Symbol) Utilities.lookupSymbol(s);
-        //Enumerated types are included, and hence shouldn't be forward declared.
-        if(symbol.getSymbolType() != Type.ENUM) {
+        Symbol symbol = (Symbol) Utilities.lookupSymbol(d_context, s);
+        //Enumerated types are included, and hence shouldn't be forward
+        //declared.
+        switch(symbol.getSymbolType()) {
+        case Type.CLASS:
+        case Type.INTERFACE:
           Cxx.nestPackagesInNamespaces( d_writer, s );
           d_writer.println("class " + s.getShortName() +";");
           Cxx.unnestPackagesInNamespaces( d_writer, s );
+          break;
+        case Type.STRUCT:
+          Cxx.nestPackagesInNamespaces( d_writer, s );
+          d_writer.println("struct " + s.getShortName() +";");
+          Cxx.unnestPackagesInNamespaces( d_writer, s );
+          break;
         }
       }
     }
@@ -298,13 +389,6 @@ public class CxxStubHeader {
   }
 
     private void writeClassBegin() { 
-    /*    SymbolID id = d_ext.getSymbolID();
-          String name = id.getShortName();
-
-          d_writer.writeComment(d_ext, true);    
-          d_writer.println("class " + name + " : public ::sidl::StubBase {");
-          d_writer.tab();
-    */
     SymbolID id = d_ext.getSymbolID();
     String name = id.getShortName();
     
@@ -312,28 +396,94 @@ public class CxxStubHeader {
     d_writer.print("class " + name);
     java.util.Collection parents = d_ext.getParents(false);
     boolean first = true;
-    //Hack to see if there are no parents.  If no parents, parent is
-    //stubbase 
-    Iterator i2 = parents.iterator();
-    if(!i2.hasNext()) {
-      d_writer.print(": public virtual "+Cxx.prependGlobalUCxx()+"::sidl::StubBase");
+
+    if(parents.size() == 0) {
+      d_writer.print(": public virtual " +
+                     Cxx.prependGlobalUCxx() + "::sidl::StubBase");
  
     } else {
+
+      Object [] parentArray = Utilities.sort(parents).toArray();
+      //Arrays.sort(parentArray);
  
-      for(Iterator i = parents.iterator(); i.hasNext(); ){
-        Extendable parent = (Extendable) i.next();
+      for(int i = 0; i < parentArray.length; ++i) {
+        Extendable parent = (Extendable) parentArray[i];
         String parentName = Cxx.getSymbolName(parent.getSymbolID(), "");
-        if(first)
+        if(first){
           d_writer.print(": ");
+          first = false;
+        }
         else
           d_writer.print(", ");
-        first = false;
         d_writer.print("public virtual " + parentName);
       }
     }
     d_writer.println(" {");
+    d_writer.tab();
     
   } 
+
+  /**Declares A set of methods for handling exceptions.  This takes the place
+   * of exception handing code in the stubs.  (To reduce code size) */
+  private void throwExceptionMethods(boolean isSuper) throws CodeGenerationException {
+    Map sets_to_nums = null;
+    if(isSuper) {
+      Class cls = (Class) d_ext;
+      sets_to_nums = Cxx.getExceptionSets(cls.getOverwrittenClassMethods());
+    } else {
+      sets_to_nums = Cxx.getExceptionSets(d_ext.getMethods(true));        
+    }
+    Set exp_sets = sets_to_nums.keySet();
+    d_writer.beginBoldComment();
+    d_writer.println("Special methods for throwing exceptions");
+    d_writer.endBoldComment();
+    
+    d_writer.backTab();
+    d_writer.println("private:");
+    d_writer.tab();
+    
+    //For each set of exceptions
+    for(Iterator i = exp_sets.iterator(); i.hasNext();) {
+      Set exp_set = (Set) i.next();
+      Integer num = (Integer)sets_to_nums.get(exp_set);
+      
+      d_writer.println("static ");
+      
+      Cxx.generateExceptionSetSignature(d_writer, d_ext.getSymbolID(), exp_set, num.intValue(), false, isSuper);
+      d_writer.println(";");
+      
+    }
+  }
+
+  /**
+   * Declares a local IOR pointer on this Stub.  This is used for caching
+   * the IOR pointer.
+   *
+   * This method is for interfaces ONLY.  We don't need to cache class stubs
+   * because they are always the same.
+   */
+  private void writeLocalIOR() throws CodeGenerationException {
+  
+    /*************************************************************
+     * Only interfaces have cache variables, and not BaseInterface
+     *************************************************************/
+    if(d_ext.isInterface() && 
+       !BabelConfiguration.getBaseInterface().equals(d_ext.getSymbolID().getFullName())) {
+
+      d_writer.beginBoldComment();
+      d_writer.println("Locally Cached IOR pointer");
+      d_writer.endBoldComment();
+    
+      d_writer.backTab();
+      d_writer.println("protected:");
+      d_writer.tab();
+      
+      d_writer.println("mutable ior_t* " + 
+                       Cxx.getIORCacheVariable((Interface)d_ext)+ ";");
+    }
+  }
+
+
   private void writeUserDefinedMethods() throws CodeGenerationException { 
     d_writer.beginBoldComment();
     d_writer.println("User Defined Methods");
@@ -344,48 +494,51 @@ public class CxxStubHeader {
     d_writer.tab();
     
     Iterator m = null;
-    m = d_ext.getStaticMethods(true).iterator();
+    m = Cxx.getStubMethodList(d_ext).iterator();
     while (m.hasNext()) {
-      Method method = (Method) m.next();
+      final Method method = (Method) m.next();
+      final boolean isStatic = method.isStatic();
+      final String desc = isStatic ? "user defined static method"
+        : "user defined non-static method";
       if(!Cxx.inlineStub(method)) {
-        Cxx.generateMethodSignature( d_writer, method, 
-                                     "user defined static method", 
-                                     Cxx.FILE_ROLE_STUB, true );
+        Cxx.generateMethodSignature( d_writer, d_context, method, desc,
+                                     Cxx.FILE_ROLE_STUB, true);
         d_writer.println();
-        if(method.hasRarray()) {
-          Cxx.generateMethodSignature( d_writer, method, 
-                                       "user defined static method", 
-                                       Cxx.FILE_ROLE_STUB, false );
+
+        if ( (!isStatic) &&
+            (method.getCommunicationModifier() == Method.NONBLOCKING) ) { 
+          Method send = method.spawnNonblockingSend();
+          Cxx.generateMethodSignature( d_writer, d_context, send, desc,
+                                       Cxx.FILE_ROLE_STUB, true);
+          d_writer.println();
+          Method recv = method.spawnNonblockingRecv();
+          Cxx.generateMethodSignature( d_writer, d_context, recv, desc,
+                                       Cxx.FILE_ROLE_STUB, true);
           d_writer.println();
         }
+
+        if(method.hasRarray()) {
+          Cxx.generateMethodSignature( d_writer, d_context, method, desc,
+                                       Cxx.FILE_ROLE_STUB, false);
+          d_writer.println();
+          if ((!isStatic) &&
+              (method.getCommunicationModifier() == Method.NONBLOCKING) ) { 
+            Method send = method.spawnNonblockingSend();
+            Cxx.generateMethodSignature( d_writer, d_context, send, desc,
+                                         Cxx.FILE_ROLE_STUB, false);
+            d_writer.println();
+            Method recv = method.spawnNonblockingRecv();
+            Cxx.generateMethodSignature( d_writer, d_context, recv, desc,
+                                         Cxx.FILE_ROLE_STUB, false);
+            d_writer.println();
+          }
+        }
       } else {
-        generateInlineMethodDispatch(method, 
-                                     "user defined static method", false, true);
+        generateInlineMethodDispatch(method, desc, false, true);
         d_writer.println();
       }
-
     }
     
-    m = d_ext.getNonstaticMethods(true).iterator();
-    while (m.hasNext()) {
-      Method method = (Method) m.next();
-      if(!Cxx.inlineStub(method)) {
-        Cxx.generateMethodSignature( d_writer, method, 
-                                     "user defined non-static method.", 
-                                     Cxx.FILE_ROLE_STUB, true );
-        d_writer.println();
-        if(method.hasRarray()) {
-          Cxx.generateMethodSignature( d_writer, method, 
-                                       "user defined static method", 
-                                       Cxx.FILE_ROLE_STUB, false );
-          d_writer.println();
-        }
-      } else {
-        generateInlineMethodDispatch(method, 
-                                     "user defined non-static method", false, true);
-        d_writer.println();
-      }
-    }
     d_writer.beginBoldComment();
     d_writer.println("End User Defined Methods");
     d_writer.println("(everything else in this file is specific to");
@@ -407,41 +560,82 @@ public class CxxStubHeader {
 
   private void writeConstructors() { 
     SymbolID id = d_ext.getSymbolID();
-    String ior_ptr = "ior_t*";
     String name = id.getShortName();
 
     d_writer.writeCommentLine("default constructor");
-    d_writer.println( name + "() { }");
+    d_writer.print( name + "() ");
+    if(d_ext.isInterface() && 
+       !BabelConfiguration.getBaseInterface().equals(d_ext.getSymbolID().getFullName())) {
+      d_writer.print(" : ");
+      Cxx.initializeLocalIOR(d_writer,(Interface)d_ext,"NULL",true); 
+    }
+    d_writer.print("{ }");
     d_writer.println();
     
     if( ! d_ext.isAbstract() ) {
       d_writer.writeCommentLine("static constructor");
       d_writer.println("static " + Cxx.getObjectName(id) + " _create();");
-    
       d_writer.println();
     }
 
+    d_writer.println();
+    d_writer.printlnUnformatted("#ifdef WITH_RMI");
+    d_writer.println();
+
+    if( ! d_ext.isAbstract() ) {
+      d_writer.writeCommentLine("RMI constructor");
+      d_writer.println("static " + Cxx.getObjectName(id) + 
+		       " _create( /*in*/ const std::string& url );");
+      d_writer.println();
+    }
+    
+    d_writer.writeCommentLine("RMI connect");
+    d_writer.println("static inline " + Cxx.getObjectName(id) + 
+		     " _connect( /*in*/ const std::string& url ) { ");
+    d_writer.tab();
+
+    d_writer.println("return _connect(url, true);");
+    d_writer.backTab();
+    d_writer.println("}");
+    d_writer.println();
+    
+    d_writer.writeCommentLine("RMI connect 2");
+    d_writer.println("static " + Cxx.getObjectName(id) + 
+		     " _connect( /*in*/ const std::string& url, /*in*/ const bool ar  );");
+    d_writer.println();
+
+    d_writer.println();
+    d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
+    d_writer.println();
+    
+
     d_writer.writeCommentLine("default destructor");
     d_writer.println("virtual ~" + name + " () { }");
     d_writer.println();
     
     d_writer.writeCommentLine("copy constructor");
-    d_writer.println( name + " ( const " + name + "& original );");// : StubBase(original) { }");
+    d_writer.println( name + " ( const " + name + "& original );");
     d_writer.println();
 
     d_writer.writeCommentLine("assignment operator");
     d_writer.println( name + "& operator= ( const " + name + "& rhs );");
     d_writer.println();
 
-    //d_writer.writeCommentLine("assignment operator from StubBase");
-    //d_writer.println( name + "& operator= ( const ::sidl::StubBase& rhs );");
-    //d_writer.println();
+    if( ! d_ext.isAbstract() && !IOR.isSIDLSymbol(id)) {
+      d_writer.println();
+      d_writer.println("protected:");
+      d_writer.writeCommentLine("Internal data wrapping method");
+      d_writer.println("static ior_t*  _wrapObj(void* private_data);");
+      d_writer.println();
+      d_writer.println();
+      d_writer.println("public:");
+    }
+
   }
   
   private void writeCastingOperators() { 
     SymbolID id = d_ext.getSymbolID();
     String name = id.getShortName();
-    String ior_ptr = "ior_t*";
 
     d_writer.writeCommentLine("conversion from ior to C++ class");
     d_writer.println( name + " ( " + name + "::ior_t* ior );");
@@ -453,50 +647,149 @@ public class CxxStubHeader {
     d_writer.println( name + " ( " + name + "::ior_t* ior, bool isWeak );");
     d_writer.println();
 
-    //d_writer.writeCommentLine("conversion from a StubBase");
-    //d_writer.println( name + " ( const ::sidl::StubBase& base );");
-    //d_writer.println();
   }
 
-  private void writeBindingSpecificMethods() { 
+  private void writeBindingSpecificMethods()  throws CodeGenerationException { 
     SymbolID id = d_ext.getSymbolID();
-    String name = id.getShortName();
     String ior_ptr = "ior_t*";
+      
+    d_writer.println("inline " + ior_ptr + " _get_ior() const throw() {");
+    d_writer.tab();
 
-    d_writer.println( ior_ptr + " _get_ior() { return " + 
-                      Cxx.reinterpretCast("ior_t*", "d_self") + "; }");
+    /*************************************************************
+     * Only interfaces have cache variables, and not BaseInterface
+     *************************************************************/
+    if(d_ext.isInterface() &&
+       !BabelConfiguration.getBaseInterface().equals(d_ext.getSymbolID().getFullName())) {
+      Interface ifc = (Interface) d_ext;
+      d_writer.println("if(!"+Cxx.getIORCacheVariable(ifc)+") { ");
+      d_writer.tab();
+      d_writer.println(Cxx.getIORCacheVariable(ifc)+
+                       " = " +  Cxx.getSymbolName(ifc.getSymbolID(),"") + 
+                       "::_cast((void*)d_self);");
+      d_writer.println("if ("+Cxx.getIORCacheVariable(ifc)+") {");
+      d_writer.tab();
+      d_writer.println(IOR.getExceptionFundamentalType() + "throwaway_exception;");
+      d_writer.println("("+Cxx.getIORCacheVariable(ifc)+ "->d_epv->" + 
+                       IOR.getVectorEntry("deleteRef") + 
+                       ")("+Cxx.getIORCacheVariable(ifc)+
+                       "->d_object, &throwaway_exception);  ");
+      d_writer.backTab();
+      d_writer.println("}  ");
+      
+      d_writer.backTab();
+      d_writer.println("}"); 
+      d_writer.println("return " + Cxx.getIORCacheVariable(ifc)+";");
+    } else {
+      d_writer.println("return " + Cxx.reinterpretCast(ior_ptr, "d_self") + ";");
+    }
+
+    d_writer.backTab();
+    d_writer.println("}");
     d_writer.println();
 
-    d_writer.println("const " + ior_ptr + " _get_ior() const { return " + 
-                     Cxx.reinterpretCast("ior_t*", "d_self")+ "; }");
+    d_writer.println( "inline void _set_ior( " + ior_ptr + " ptr ) throw () { ");
+    d_writer.tab();
+    //Don't clear the caches if the ptr hasn't changed.
+    if(d_ext.hasParentInterfaces()) {
+      d_writer.println("if(d_self == ptr) {return;}");
+    }
+    d_writer.println("d_self = " +
+                     Cxx.reinterpretCast("void*", "ptr") + ";");
+    if(d_ext.isInterface()) {
+      Cxx.initializeLocalIOR(d_writer, (Interface)d_ext, "ptr", false);
+    }
     d_writer.println();
 
-    d_writer.println( "void _set_ior( " + ior_ptr + " ptr ) { d_self = " +
-                      Cxx.reinterpretCast("void*", "ptr") + "; }");
+    Cxx.writeInterfaceCacheInitialization(d_writer, d_ext, "ptr");
+
+    d_writer.backTab();
+    d_writer.println("}");
     d_writer.println();
 
-    d_writer.println("bool _is_nil() const { return (d_self==0); }");
+    d_writer.println("virtual int _set_ior_typesafe( struct sidl_BaseInterface__object *obj,");
+    d_writer.println("                               const ::std::type_info &argtype );");
     d_writer.println();
 
-    d_writer.println("bool _not_nil() const { return (d_self!=0); }");
+    d_writer.println("bool _is_nil() const throw () { return (d_self==0); }");
     d_writer.println();
 
-    d_writer.println("bool operator !() const { return (d_self==0); }");
+    d_writer.println("bool _not_nil() const throw () { return (d_self!=0); }");
+    d_writer.println();
+
+    d_writer.println("bool operator !() const throw () { return (d_self==0); }");
     d_writer.println();
     
-    d_writer.println("static inline const char * type_name() { return \"" + 
+    d_writer.println("static inline const char * type_name() throw () { return \"" + 
                      id.getFullName()+"\";}");
+    d_writer.println();
 
-    d_writer.println("virtual void* _cast(const char* type) const;");
+    d_writer.println("static "+IOR.getObjectName(id)+"* _cast(const void* src);");
+    d_writer.println();
+
+    d_writer.writeCommentLine("execute member function by name");
+    d_writer.println("void _exec(const std::string& methodName,");
+    d_writer.println("           ::sidl::rmi::Call& inArgs,");
+    d_writer.println("           ::sidl::rmi::Return& outArgs);");
+  
+    if (d_ext.hasStaticMethod(true) ) { 
+	d_writer.writeCommentLine("exec static member function by name");
+	d_writer.println("static void _sexec(const std::string& methodName,");
+	d_writer.println("                   ::sidl::rmi::Call& inArgs,");
+	d_writer.println("                   ::sidl::rmi::Return& outArgs);");
+	d_writer.println();
+    }
+
+    Cxx.generateMethodSignature( d_writer, d_context, 
+                                 IOR.getBuiltinMethod(IOR.GETURL,id,d_context, false), 
+                                 "get URL of the implementation of this object", 
+                                 Cxx.FILE_ROLE_STUB, true);
+    d_writer.println();
+
+    boolean hasStatic = d_ext.hasStaticMethod(true);
+    Cxx.generateMethodSignature( d_writer, d_context,
+                                 IOR.getBuiltinMethod(IOR.HOOKS,id, d_context,false), 
+                                 "Set calling hooks on or off for this object",
+                                 Cxx.FILE_ROLE_STUB, true);
+    if (hasStatic) {
+      //Static set_hooks
+      Cxx.generateMethodSignature( d_writer, d_context,
+                                   IOR.getBuiltinMethod(IOR.HOOKS,id, d_context,true), 
+                                   "Set calling hooks on or off for class methods",
+                                   Cxx.FILE_ROLE_STUB, true );
+    }
+    
+    //d_writer.writeCommentLine("get URL of the implementation of this object");
+    //d_writer.println("std::string _getURL();");
+    //d_writer.println();
+
+    d_writer.writeCommentLine("return true iff object is remote");
+    d_writer.println("bool _isRemote() const { ");
+    d_writer.tab();
+    d_writer.println("ior_t* self = const_cast<ior_t*>(_get_ior() );");
+    d_writer.println(IOR.getExceptionFundamentalType() + "throwaway_exception;");
+    d_writer.println("return (*self->d_epv->f__isRemote)(self, &throwaway_exception) == TRUE;");
+    d_writer.backTab();
+    d_writer.println("}");
+    d_writer.println();
+
+    d_writer.writeCommentLine("return true iff object is local");
+    d_writer.println("bool _isLocal() const {");
+    d_writer.tab();
+    d_writer.println("return !_isRemote();");
+    d_writer.backTab();
+    d_writer.println("}");
     d_writer.println();
 
     d_writer.backTab();
     d_writer.println("protected:");
     d_writer.tab();
-    //    d_writer.println("template<typename Target, typename Source>");
-    //d_writer.println("friend Target babel_cast(Source& arg);");
-
-    d_writer.tab();
+  
+    /*
+    d_writer.writeCommentLine("perform addRef on remote object");
+    d_writer.println("void _raddRef();");
+    d_writer.println();
+    */
     d_writer.writeCommentLine("Pointer to external (DLL loadable) symbols (shared among instances)");
     d_writer.println("static const ext_t * s_ext;");
     d_writer.println();
@@ -640,18 +933,10 @@ public class CxxStubHeader {
   private void generateInlineMethodDispatch( Method m, String altcomment, boolean isSuper, boolean rarrays ) 
     throws CodeGenerationException { 
     if ( m == null ) { return; }
-    final ArrayList vArgs = m.getArgumentList();
-    final int nargs = vArgs.size();
-    String extra_close_paren = "";
     String shortMethodName = m.getShortMethodName();
-    String longMethodName = m.getLongMethodName();
     String className = d_ext.getSymbolID().getShortName();
-    Comment comment = m.getComment();
-    BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
     Type return_type = m.getReturnType();
-    SymbolID id = return_type.getSymbolID();
 
-    //d_writer.writeComment( comment, altcomment );
     if ( shortMethodName.equals(className) ) { 
       System.out.println("WARNING: gov.llnl.babel.backend.UCxx.CxxStubSource: sidl / C++ conflict!");
       System.out.println("         methodName == className is not allowed in C++");
@@ -661,27 +946,27 @@ public class CxxStubHeader {
     }
 
     //Output function deleration
-    Cxx.generateInlineMethodSignature( d_writer, m, 
+    Cxx.generateInlineMethodSignature( d_writer, d_context, m, 
                                      "user defined static method", 
-                                       Cxx.FILE_ROLE_STUB, isSuper, rarrays );
-    //d_writer.println(Cxx.generateFunctionDeclaration(m, d_ext, d_self));
-
-    //d_writer.println();
-    //d_writer.println("{");
+                                       Cxx.FILE_ROLE_STUB, isSuper, rarrays);
     d_writer.tab();
 
-    if ( (! m.isStatic()) &&  s_babel_config.makeCxxCheckNullIOR()) { 
+    if ( (! m.isStatic()) &&  d_context.getConfig().makeCxxCheckNullIOR()) { 
 
       d_writer.println("if ( d_self == 0 ) {");
       d_writer.tab();
       d_writer.println("throw "+Cxx.prependGlobalUCxx()+"::sidl::NullIORException( ::std::string (");
       d_writer.tab();
-      d_writer.disableLineBreak();
-      d_writer.println( "\"" + 
-                        "Null IOR Pointer in \\\"" + 
-                        Cxx.getMethodStubName(d_ext.getSymbolID(),shortMethodName) +
-                        "()\\\"\"");
-      d_writer.enableLineBreak();
+      try {
+        d_writer.pushLineBreak(false);
+        d_writer.println( "\"" + 
+                          "Null IOR Pointer in \\\"" + 
+                          Cxx.getMethodStubName(d_ext.getSymbolID(),shortMethodName, isSuper) +
+                          "()\\\"\"");
+      }
+      finally {
+        d_writer.popLineBreak();
+      }
       d_writer.backTab();
       d_writer.println("));");
       d_writer.backTab();
@@ -689,7 +974,7 @@ public class CxxStubHeader {
     }
 
     //Output almost nothing!
-    d_writer.println(Cxx.generateInitialization(m, d_ext, d_self));
+    d_writer.println(Cxx.generateInitialization(m, d_ext, d_self, d_context));
 
     if ( ! m.isStatic() ) { 
       if (shortMethodName.equals("addRef") ||
@@ -700,32 +985,35 @@ public class CxxStubHeader {
     }
 
     //Prepare for IOR call!
-    d_writer.println(Cxx.generatePreIORCall(m, d_ext, d_self, isSuper, rarrays));
+    d_writer.println(Cxx.generatePreIORCall(m, d_ext, d_self, isSuper, rarrays, d_context));
 
     if(isSuper) {
       Class cls = (Class) d_ext;
       String tmp_self = Cxx.reinterpretCast(IOR.getObjectName(cls.getParentClass().getSymbolID())
                                             + "*", "superSelf");
-      d_writer.println(Cxx.generateIORCall(m, d_ext, tmp_self, isSuper, rarrays));
+      d_writer.println(Cxx.generateIORCall(m, d_ext, d_context, tmp_self, isSuper, rarrays));
     } else {
-      d_writer.println(Cxx.generateIORCall(m, d_ext, d_self, isSuper, rarrays));
+      d_writer.println(Cxx.generateIORCall(m, d_ext, d_context, d_self, isSuper, rarrays));
     }
 
+    d_writer.println(Cxx.generatePostIORCleanup(m, rarrays));
     if (m.getThrows().size()>0) { // if throws exception
       d_writer.println("if (_exception != 0 ) {");      
       d_writer.tab();
       d_writer.println("void * _p = 0;");
+      d_writer.println(IOR.getExceptionFundamentalType() + "throwaway_exception;");
       Object [] exceptions = m.getThrows().toArray();
-      Arrays.sort(exceptions, new LevelComparator(SymbolTable.getInstance()));
+      Arrays.sort(exceptions, new LevelComparator(d_context.getSymbolTable()));
       
       for( int i=0; i<exceptions.length; ++i) {
         SymbolID exid = (SymbolID)exceptions[i];
-        d_writer.println("if ( (_p=(*(_exception->d_epv->f__cast))(_exception, \"" +
-                         exid.getFullName() + "\")) != 0 ) {");
+        d_writer.println("if ( (_p=(*(_exception->d_epv->f__cast))(_exception->d_object, \"" +
+                         exid.getFullName() + "\", &throwaway_exception)) != 0 ) {");
         d_writer.tab();
         d_writer.println(IOR.getObjectName(exid) + " * _realtype = " + 
                          Cxx.reinterpretCast(IOR.getObjectName(exid)+"*" ,
                                              "_p") + ";");
+        d_writer.println("(*_exception->d_epv->f_deleteRef)(_exception->d_object, &throwaway_exception);");
         d_writer.writeCommentLine("Note: alternate constructor does not increment refcount.");
         d_writer.println("throw " + Cxx.getObjectName(exid) + "( _realtype, false );");
         d_writer.backTab();
@@ -736,9 +1024,8 @@ public class CxxStubHeader {
     }
 
     //Clean up from return from IOR
-    d_writer.println(Cxx.generatePostIORCall(m, d_ext, d_self, rarrays));
+    d_writer.println(Cxx.generatePostIORCall(m, d_ext, d_self, isSuper, rarrays));
 
-    //d_writer.println( post_ior.toString().trim() );
     if ( ! m.isStatic() ) { 
       if (shortMethodName.equals("addRef") ||
           shortMethodName.equals("deleteRef") ) { 
