@@ -1,7 +1,7 @@
 //
 // File:        LanguageWriterForJava.java
 // Package:     gov.llnl.babel.backend.writers
-// Revision:    @(#) $Id: LanguageWriterForJava.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: LanguageWriterForJava.java 6196 2007-10-24 23:42:37Z epperly $
 // Description: Java language writer for backend code generation
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -30,6 +30,7 @@
 
 package gov.llnl.babel.backend.writers;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
@@ -49,8 +50,8 @@ public class LanguageWriterForJava extends LanguageWriter
     * stop at zero, a tab spacing of three, and line breaking at column
     * eighty.
     */
-   public LanguageWriterForJava(PrintWriter writer) {
-      super(writer);
+   public LanguageWriterForJava(PrintWriter writer, Context context) {
+      super(writer, context);
       setFirstTabStop(0);
       setTabSpacing(2);
       enableLineBreak(80, null, null);
@@ -87,6 +88,15 @@ public class LanguageWriterForJava extends LanguageWriter
       }
    }
 
+  /**
+   * Write the splicer tag as a language-specific comment.
+   */
+  public void writeSplicerTagLine(String tagLine) {
+    disableLineBreak();
+    writeCommentLine(tagLine);
+    enableLineBreak();
+  }
+
   /**
    * Convert a string into something that is safe to include inside a Java
    * comment string.
