@@ -1,7 +1,7 @@
 //
 // File:        PrettyWriter.java
 // Package:     gov.llnl.babel.backend.writers
-// Revision:    @(#) $Id: PrettyWriter.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: PrettyWriter.java 5885 2007-02-28 22:54:54Z epperly $
 // Description: a pretty writer class to aid in formatting backend output
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,6 +31,7 @@
 package gov.llnl.babel.backend.writers;
 
 import java.io.PrintWriter;
+import java.util.LinkedList;
 
 /**
  * The <code>PrettyWriter</code> class formats output for the backend code
@@ -74,6 +75,7 @@ public class PrettyWriter {
    private PrintWriter  d_writer;            // output print writer
   private boolean      d_strict_break = false; // No lines can be longer
                                                // than the break column.
+  private LinkedList d_lineBreakStack = new LinkedList();
 
    /**
     * Create a new <code>PrettyWriter</code> instance that will send
@@ -122,6 +124,13 @@ public class PrettyWriter {
       return d_writer;
    }
 
+   /**
+    * Flush the underlying print writer stream.
+    */
+   public void flushPrintWriter() {
+     d_writer.flush();
+   }
+
    /**
     * Define the pretty printer parameters for line breaking.  The
     * column argument specifies the maximum column number (with the left
@@ -160,6 +169,26 @@ public class PrettyWriter {
       d_break_enabled = false;
    }
 
+  /**
+   * Change the line break status to <code>linebreak</code> and save the
+   * previous value on a stack.
+   */
+  public void pushLineBreak(boolean linebreak)
+  {
+    final Boolean b = new Boolean(d_break_enabled);
+    d_lineBreakStack.addFirst(b);
+    d_break_enabled = linebreak;
+  }
+
+  /**
+   * Restore the line break status to its previous value.
+   */
+  public void popLineBreak()
+  {
+    final Boolean b = (Boolean)d_lineBreakStack.removeFirst();
+    d_break_enabled = b.booleanValue();
+  }
+
   public void setStrictBreaking()
   {
     d_strict_break = true;
@@ -412,6 +441,16 @@ public class PrettyWriter {
       d_line = new StringBuffer(BUFSIZE);
    }
 
+  private int firstBreakChar(String str, int start)
+  {
+    final int len = str.length();
+    int c;
+    for(c = start; c < len; ++c) {
+      if (d_break_string.indexOf(str.charAt(c)) >= 0) break;
+    }
+    return c;
+  }
+
    /**
     * Output remaining data in the line buffer, breaking lines if necessary.
     * The start argument gives the starting location from which to flush the
@@ -438,12 +477,7 @@ public class PrettyWriter {
              */
             int index = -1;
             int search = d_break_column - d_cursor;
-            for (int c = 0; c < d_break_string.length(); c++) {
-               index = findBreak(d_break_string.charAt(c), start, search);
-               if (index >= 0) {
-                  break;
-               }
-            }
+            index = findBreak(start, search);
 
             /*
              * If a breaking index was found, then print out data through
@@ -468,14 +502,25 @@ public class PrettyWriter {
                  if (d_strict_break && (search > 0)) {
                    String fragment = d_line.substring(start, start+search);
                    d_writer.print(fragment);
+                   d_cursor += fragment.length();
                    endContinuationLine();
                    beginContinuationLine();
                    breakLine(firstNonSpace(start+search), true);
                  }
                  else{
                    String fragment = d_line.substring(start);
-                   d_writer.print(fragment);
-                   d_cursor += fragment.length();
+                   int firstBreak = firstBreakChar(fragment, search);
+                   if (firstBreak < fragment.length()) {
+                     d_writer.print(fragment.substring(0, firstBreak));
+                     d_cursor += firstBreak;
+                     endContinuationLine();
+                     beginContinuationLine();
+                     breakLine(firstNonSpace(start+firstBreak), true);
+                   }
+                   else {
+                     d_writer.print(fragment);
+                     d_cursor += fragment.length();
+                   }
                  }
                } else {
                   endContinuationLine();
@@ -493,7 +538,8 @@ public class PrettyWriter {
     */
    private void endContinuationLine() {
       if (d_break_line_end != null) {
-         writeSpaces(d_break_column - d_cursor);
+         if (d_break_column > d_cursor)
+           writeSpaces(d_break_column - d_cursor);
          d_writer.print(d_break_line_end);
       }
       d_writer.print('\n');
@@ -523,13 +569,13 @@ public class PrettyWriter {
     * target character.  Return the index at which the character is found
     * or -1 if the character is not found.
     */
-   private int findBreak(char c, int start, int search) {
+   private int findBreak(int start, int search) {
       int index = -1;
       for (int s = start+search-1; s >= start; s--) {
-         if (d_line.charAt(s) == c) {
-            index = s;
-            break;
-         }
+        if (d_break_string.indexOf(d_line.charAt(s)) >= 0) {
+          index = s;
+          break;
+        }
       }
       return index;
    }
