@@ -1,7 +1,7 @@
 //
 // File:        ParseSymbolXML.java
 // Package:     gov.llnl.babel.parsers.xml
-// Revision:    @(#) $Id: ParseSymbolXML.java 4479 2005-03-25 00:47:40Z epperly $
+// Revision:    @(#) $Id: ParseSymbolXML.java 6201 2007-10-26 06:15:35Z kumfert $
 // Description: convert sidl symbol XML/DOM into a sidl symbol object
 //
 // Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -30,6 +30,7 @@
 
 package gov.llnl.babel.parsers.xml;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.parsers.xml.DTDManager;
 import gov.llnl.babel.parsers.xml.ParseSymbolException;
 import gov.llnl.babel.parsers.xml.StringXML;
@@ -37,6 +38,7 @@ import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Assertion;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.Attributes;
 import gov.llnl.babel.symbols.BinaryExpression;
 import gov.llnl.babel.symbols.BooleanLiteral;
 import gov.llnl.babel.symbols.CharacterLiteral;
@@ -57,10 +59,10 @@ import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.MethodCall;
 import gov.llnl.babel.symbols.Package;
 import gov.llnl.babel.symbols.StringLiteral;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolRedefinitionException;
-import gov.llnl.babel.symbols.SymbolTable;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.UnaryExpression;
 import gov.llnl.babel.symbols.Version;
@@ -68,12 +70,11 @@ import gov.llnl.babel.xml.ElementIterator;
 import gov.llnl.babel.xml.XMLUtilities;
 import java.io.IOException;
 import java.text.ParseException;
-import java.util.StringTokenizer;
 import java.util.Iterator;
-import org.xml.sax.InputSource;
-import org.xml.sax.SAXException;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
+import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
 
 /**
  * Class <code>ParseSymbolXML</code> converts an XML document into a SIDL
@@ -86,6 +87,8 @@ public class ParseSymbolXML {
 
    private Symbol d_symbol;
 
+  private Context d_context;
+
    /**
     * This is a convenience utility function that converts an XML input
     * stream into a SIDL symbol.  Any errors detected in the XML input
@@ -97,8 +100,10 @@ public class ParseSymbolXML {
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
     *              The exception raised if a parsing error is encountered.
     */
-   public static Symbol convert(InputSource is) throws ParseSymbolException {
-      ParseSymbolXML xml2sym = new ParseSymbolXML(is);
+   public static Symbol convert(InputSource is, Context context) 
+     throws ParseSymbolException 
+  {
+      ParseSymbolXML xml2sym = new ParseSymbolXML(is, context);
       return xml2sym.getSymbol();
    }
 
@@ -114,8 +119,10 @@ public class ParseSymbolXML {
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
     *               The exception raised if a parsing error is encountered.
     */
-   public static Symbol convert(Document doc) throws ParseSymbolException {
-      ParseSymbolXML doc2sym = new ParseSymbolXML(doc);
+   public static Symbol convert(Document doc, Context context)
+     throws ParseSymbolException 
+  {
+      ParseSymbolXML doc2sym = new ParseSymbolXML(doc, context);
       return doc2sym.getSymbol();
    }
 
@@ -130,7 +137,11 @@ public class ParseSymbolXML {
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
     *              The exception raised if a parsing error is encountered.
     */
-   public ParseSymbolXML(InputSource is) throws ParseSymbolException {
+   public ParseSymbolXML(InputSource is,
+                         Context context) 
+     throws ParseSymbolException 
+  {
+    d_context = context;
       Document document = null;
       try {
          document = XMLUtilities.parse(is, DTDManager.getInstance());
@@ -151,7 +162,11 @@ public class ParseSymbolXML {
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
     *               The exception raised if a parsing error is encountered.
     */
-   public ParseSymbolXML(Document doc) throws ParseSymbolException {
+   public ParseSymbolXML(Document doc,
+                         Context context)
+     throws ParseSymbolException 
+  {
+    d_context = context;
       parseSymbol(doc.getDocumentElement());
    }
 
@@ -162,6 +177,24 @@ public class ParseSymbolXML {
       return d_symbol;
    }
 
+  private void parseAttributes(Element node, Attributes a)
+    throws ParseSymbolException
+  {
+    if (hasChildElement(node, "Attributes")) {
+      Element attr = getElement(node, "Attributes");
+      Iterator i = new ElementIterator(attr, "Attribute");
+      while (i.hasNext()) {
+        Element item = (Element)i.next();
+        String name = getAttribute(item, "name");
+        String value = null;
+        if (item.hasAttribute("value")) {
+          value = item.getAttribute("value");
+        }
+        a.setAttribute(name, value);
+      }
+    }
+  }
+
 
    /**
     * Parse the DOM element (which must be an element of type "Symbol") and
@@ -182,26 +215,45 @@ public class ParseSymbolXML {
       Metadata md = parseMetadata(getElement(symbol, "Metadata"  ));
 
       if (hasChildElement(symbol, "Class")) {
-         d_symbol = new Class(id, cm, md);
+         d_symbol = new Class(id, cm, md, d_context);
          parseClass(getElement(symbol, "Class"));
 
       } else if (hasChildElement(symbol, "Enumeration")) {
-         d_symbol = new Enumeration(id, cm, md);
+         d_symbol = new Enumeration(id, cm, md, d_context);
          parseEnum(getElement(symbol, "Enumeration"));
          
       } else if (hasChildElement(symbol, "Interface")) {
-         d_symbol = new Interface(id, cm, md);
+         d_symbol = new Interface(id, cm, md, d_context);
          parseInterface(getElement(symbol, "Interface"));
          
       } else if (hasChildElement(symbol, "Package")) {
-         d_symbol = new Package(id, cm, md);
+         d_symbol = new Package(id, cm, md, d_context);
          parsePackage(getElement(symbol, "Package"));
-         
+      } else if (hasChildElement(symbol, "Struct")) {
+        d_symbol = new Struct(id, cm, md, d_context);
+        parseStruct(getElement(symbol, "Struct"));
       } else {
-         error("parseSymbol: Unknown SIDL symbol type in XML document.");
+        error("parseSymbol: Unknown SIDL symbol type in XML document.");
       }
    }
 
+  private void parseStruct(Element element) throws ParseSymbolException {
+    validateSymbolName(element, "Struct");
+    Struct s = (Struct) d_symbol;
+    parseAttributes(element, s);
+    Iterator i = new ElementIterator(element, "StructItem");
+    while (i.hasNext()) {
+      s.addItem(parseStructItem((Element)i.next()));
+    }
+  }
+
+  private Struct.Item parseStructItem(Element element) throws ParseSymbolException {
+    validateSymbolName(element, "StructItem");
+    String name = getAttribute(element, "name");
+    Type t = parseType(getElement(element, "Type"));
+    return new Struct.Item(name, t);
+  }
+
    /**
     * Convert a DOM class element into a SIDL class symbol.  A class
     * description consists of an extends symbol, an implements block,
@@ -214,6 +266,7 @@ public class ParseSymbolXML {
    private void parseClass(Element element) throws ParseSymbolException {
       validateSymbolName(element, "Class");
       Class cls = (Class) d_symbol;
+      parseAttributes(element, cls);
 
       /*
        * Parse the single optional extends symbol (which must be a class).
@@ -223,7 +276,7 @@ public class ParseSymbolXML {
       if (parent.hasNext()) {
          SymbolID id = parseSymbolID((Element) parent.next());
          try {
-            Symbol sym = SymbolTable.getInstance().resolveSymbol(id);
+            Symbol sym = d_context.getSymbolTable().resolveSymbol(id);
             if (sym == null) {
                error("parseClass: Class \"" + id.getSymbolName() 
                    + "\" not found.");
@@ -272,6 +325,7 @@ public class ParseSymbolXML {
    private void parseEnum(Element element) throws ParseSymbolException {
       validateSymbolName(element, "Enumeration");
       Enumeration enm = (Enumeration) d_symbol;
+      parseAttributes(element, enm);
 
       Iterator enumerators = new ElementIterator(element, "Enumerator");
       while (enumerators.hasNext()) {
@@ -305,6 +359,7 @@ public class ParseSymbolXML {
       validateSymbolName(element, "Interface");
       Interface ifc = (Interface) d_symbol;
 
+      parseAttributes(element, ifc);
       /*
        * Parse the extends block consisting of a collection of interfaces.
        */
@@ -351,7 +406,7 @@ public class ParseSymbolXML {
       validateSymbolName(element, "Package");
       Package p = (Package) d_symbol;
       Version pkgVersion = p.getSymbolID().getVersion();
-      p.setFinal("true".equals(getAttribute(element, "final")));
+      parseAttributes(element, p);
 
       try {
         Iterator entries = new ElementIterator(element, "PackageSymbol");
@@ -483,7 +538,7 @@ public class ParseSymbolXML {
       while (parents.hasNext()) {
          SymbolID id = parseSymbolID((Element) parents.next());
          try {
-            Symbol sym = SymbolTable.getInstance().resolveSymbol(id);
+            Symbol sym = d_context.getSymbolTable().resolveSymbol(id);
             if (sym == null) {
                error("parseParentInterfaceBlock: Interface \"" 
                    + id.getSymbolName() + "\" not found.");
@@ -500,6 +555,25 @@ public class ParseSymbolXML {
       }
     }
 
+  private void parseFromClause(Element fromClause, 
+                               Extendable ext,
+                               Method newMethod)
+    throws ParseSymbolException
+  {
+    Method oldMethod = newMethod.cloneMethod();
+    oldMethod.setMethodName(getAttribute(fromClause, "shortname"),
+                            getAttribute(fromClause, "extension"));
+    try {
+      Version parentVersion = new Version(getAttribute(fromClause, "version"));
+      SymbolID parentID = new SymbolID(getAttribute(fromClause, "parentname"), parentVersion, true);
+      ext.addRenamedMethod(newMethod, oldMethod, parentID);
+    }
+    catch (NumberFormatException ex) {
+      error("parseSymbolID: Invalid version format \"" + getAttribute(fromClause, "version")
+            + "\" in symbol name.");
+    }
+  }
+
    /**
     * Recover the methods block from its DOM representation.  
     *
@@ -514,8 +588,16 @@ public class ParseSymbolXML {
       Element  block   = getElement(e, "MethodsBlock");
       Iterator methods = new ElementIterator(block, "Method");
       while (methods.hasNext()) {
-         ext.addMethod(parseMethod((Element) methods.next(), 
-                                   ext.getFullName()));
+        Element method = (Element) methods.next();
+        Method m = parseMethod(method, ext.getFullName());
+        Element fromClause = XMLUtilities.lookupElement(method, "From");
+        //        if (fromClause == null) {
+        ext.addMethod(m);
+        //}
+        //else {
+        if (fromClause != null) {
+          parseFromClause(fromClause, ext, m);
+        }
       }
     }
 
@@ -564,19 +646,14 @@ public class ParseSymbolXML {
       throws ParseSymbolException 
    {
       validateSymbolName(e, "Method");
-      Method method = new Method();
+      Method method = new Method(d_context);
 
+      parseAttributes(e, method);
       /*
        * Set method name, modifiers, return value, and comment
        */
       method.setMethodName(getAttribute(e, "shortname"), 
                            getAttribute(e, "extension"));
-      method.setCommunicationModifier(
-         StringXML.fromComXML(getAttribute(e, "communication")));
-      method.setDefinitionModifier(
-         StringXML.fromDefXML(getAttribute(e, "definition")));
-      method.setReturnCopy(getAttribute(e, "copy").equals("true"));
-
       method.setComment(parseComment(getElement(e, "Comment")));
       method.setReturnType(parseType(getElement(e, "Type")));
 
@@ -588,6 +665,17 @@ public class ParseSymbolXML {
       while (args.hasNext()) {
          method.addArgument(parseArgument((Element) args.next()));
       }
+      args = method.getArgumentList().iterator();
+      while (args.hasNext()) {
+        Argument arg = (Argument)args.next();
+        if (arg.getType().isRarray()) {
+          if (arg.getMode() == Argument.OUT) {
+            error("parseMethod: rarray " +
+                  arg.getFormalName() + " in an out argument. Rarrays must be in or inout.");
+          }
+          method.addRarrayIndex(arg.getType().getArrayIndices());
+        }
+      }
 
       /*
        * Parse the throws clause from the XML document
@@ -598,6 +686,13 @@ public class ParseSymbolXML {
          method.addThrows(parseSymbolID((Element) thrws.next()));
       }
 
+      Element implthrowslist = getElement(e, "ImplicitThrowsList");
+      thrws = new ElementIterator(implthrowslist, "SymbolName");
+      while (thrws.hasNext()) {
+         method.addImplicitThrows(parseSymbolID((Element) thrws.next()));
+      }
+
+
       /*
        * Parse the Assertion List, if any, and add any preconditions
        * and/or postconditions to this method.
@@ -632,11 +727,12 @@ public class ParseSymbolXML {
     */
    private Argument parseArgument(Element e) throws ParseSymbolException {
       validateSymbolName(e, "Argument");
-      boolean copy = getAttribute(e, "copy").equals("true");
       int     mode = StringXML.fromModeXML(getAttribute(e, "mode"));
       String  name = getAttribute(e, "name");
       Type    type = parseType(getElement(e, "Type"));
-      return new Argument(copy, mode, type, name);
+      Argument arg = new Argument(mode, type, name);
+      parseAttributes(e, arg);
+      return arg;
    }
 
    /**
@@ -652,7 +748,8 @@ public class ParseSymbolXML {
 
       Type type = null;
       if (typeid == Type.SYMBOL) {
-         type = new Type(parseSymbolID(getElement(e, "SymbolName")));
+         type = new Type(parseSymbolID(getElement(e, "SymbolName")), 
+                         d_context);
       } else if (typeid == Type.ARRAY) {
          Element array = getElement(e, "Array");
          String sdim   = getAttribute(array, "dim"); // default "0"
@@ -670,17 +767,19 @@ public class ParseSymbolXML {
            }
          }
          if (null != arrayType) {
-           type = new Type(parseType(getElement(array, "Type")), dim, order);
+           type = new Type(parseType(getElement(array, "Type")), dim, order,
+                           d_context);
          } else {
-           type = new Type(null, dim, order); // dimensionless typeless array
+           type = new Type(null, dim, order,
+                           d_context); // dimensionless typeless array
          }
          //This if should only occur if the Array is an rarray, and has
          //index variables.
          if(index_elem != null) {
-           String index_list   = getAttribute(index_elem, "name");
-           StringTokenizer index_tok = new StringTokenizer(index_list, ",");
-           while (index_tok.hasMoreTokens()) {
-             type.addArrayIndex(index_tok.nextToken());
+           Iterator iter = new ElementIterator(index_elem, "Expression");
+           while (iter.hasNext()) {
+             Element expr = (Element)iter.next();
+             type.addArrayIndex(parseExpression(expr));
            }
          }
       } else {
@@ -729,8 +828,8 @@ public class ParseSymbolXML {
       int       type = StringXML.fromAssertionXML(getAttribute(e, "type"));
       Assertion as   = null;
       try {
-         Element             ae     = getElement(e, "AssertionExpression");
-         AssertionExpression expr   = parseAssertionExpression(ae);
+         Element             ae     = getElement(e, "Expression");
+         AssertionExpression expr   = parseExpression(ae);
          as                         = new Assertion(type, source, tag, cm);
          as.setExpression(expr);
       } catch (AssertionException aex) {
@@ -748,10 +847,10 @@ public class ParseSymbolXML {
     *             The exception raise if there are any format or data
     *             errors associated with the assertion expression element.
     */
-   private AssertionExpression parseAssertionExpression(Element e)
+   private AssertionExpression parseExpression(Element e)
       throws ParseSymbolException
    {
-      validateSymbolName(e, "AssertionExpression");
+      validateSymbolName(e, "Expression");
       AssertionExpression expr = null;
 
       boolean parens = getAttribute(e, "parens").equals("true");
@@ -764,14 +863,14 @@ public class ParseSymbolXML {
       } else if (hasChildElement(e, "MethodCall")) {
          Element mc = getElement(e, "MethodCall");
          expr       = (AssertionExpression)parseMethodCall(mc, parens);
-      } else if (hasChildElement(e, "SimpleExpression")) {
-         Element se = getElement(e, "SimpleExpression");
-         expr       = parseSimpleExpression(se, parens);
+      } else if (hasChildElement(e, "Terminal")) {
+         Element se = getElement(e, "Terminal");
+         expr       = parseTerminal(se, parens);
       } else if (hasChildElement(e, "UnaryExpression")) {
          Element ue = getElement(e, "UnaryExpression");
          expr       = parseUnaryExpression(ue, parens);
       } else {
-         error("parseAssertionExpression: Unable to locate a "
+         error("parseExpression: Unable to locate a "
              + "recognizable/supported child element of an Assertion "
              + "Expression.");
       }
@@ -798,15 +897,15 @@ public class ParseSymbolXML {
       AssertionExpression rhs  = null;
       BinaryExpression    expr = null;
       int                 op  = StringXML.fromBinaryOpXML(getAttribute(e,"op"));
-      Iterator            iter = new ElementIterator(e, "AssertionExpression");
+      Iterator            iter = new ElementIterator(e, "Expression");
       int                 num = 0;
       while (iter.hasNext()) {
         Element entry = (Element)iter.next();
         num           = num + 1;
         if (num == 1) {
-          lhs = parseAssertionExpression(entry);
+          lhs = parseExpression(entry);
         } else if (num == 2) {
-          rhs = parseAssertionExpression(entry);
+          rhs = parseExpression(entry);
         } else {
            error("parseBinaryExpression: Invalid number of assertion "
                + "expressions detected when attempting to parse a binary "
@@ -816,7 +915,7 @@ public class ParseSymbolXML {
 
       if ( (lhs != null) && (rhs != null) ) {
         try {
-          expr = new BinaryExpression(lhs, op, rhs);
+          expr = new BinaryExpression(lhs, op, rhs, d_context);
           expr.setParens(parens);
         } catch (AssertionException aex) {
           error("parseBinaryExpression: AssertionException: "
@@ -851,15 +950,19 @@ public class ParseSymbolXML {
       String              imag = getAttribute(e, "imaginary");
       try {
         if (type.equals("float")) {
-          FloatLiteral    fr = new FloatLiteral(Float.valueOf(real), real);
-          FloatLiteral    fi = new FloatLiteral(Float.valueOf(imag), imag);
-          FComplexLiteral f  = new FComplexLiteral(fr, fi);
+          FloatLiteral    fr = new FloatLiteral(Float.valueOf(real), real, 
+                                                d_context);
+          FloatLiteral    fi = new FloatLiteral(Float.valueOf(imag), imag,
+                                                d_context);
+          FComplexLiteral f  = new FComplexLiteral(fr, fi, d_context);
           f.setParens(parens);
           expr = (AssertionExpression)f;
         } else if (type.equals("double")) {
-          DoubleLiteral   dr = new DoubleLiteral(Double.valueOf(real), real);
-          DoubleLiteral   di = new DoubleLiteral(Double.valueOf(imag), imag);
-          DComplexLiteral d  = new DComplexLiteral(dr, di);
+          DoubleLiteral   dr = new DoubleLiteral(Double.valueOf(real), real,
+                                                 d_context);
+          DoubleLiteral   di = new DoubleLiteral(Double.valueOf(imag), imag,
+                                                 d_context);
+          DComplexLiteral d  = new DComplexLiteral(dr, di, d_context);
           d.setParens(parens);
           expr = (AssertionExpression)d;
         } else {
@@ -893,11 +996,11 @@ public class ParseSymbolXML {
       String     name = getAttribute(e, "name");
       MethodCall expr = null;
       try {
-        expr = new MethodCall(name);
+        expr = new MethodCall(name, d_context);
         expr.setParens(parens);
-        Iterator iter = new ElementIterator(e, "AssertionExpression");
+        Iterator iter = new ElementIterator(e, "Expression");
         while (iter.hasNext()) {
-          expr.addArgument(parseAssertionExpression((Element)iter.next()));
+          expr.addArgument(parseExpression((Element)iter.next()));
         }
       } catch (AssertionException aex) {
         error("parseMethodCall: AssertionException: " + aex.getMessage());
@@ -917,66 +1020,65 @@ public class ParseSymbolXML {
     *                  The exception raise if there are any errors detected
     *                  with the simple expression or its components.
     */
-   private AssertionExpression parseSimpleExpression(Element e, boolean parens)
+   private AssertionExpression parseTerminal(Element e, boolean parens)
       throws ParseSymbolException
    {
-      validateSymbolName(e, "SimpleExpression");
+      validateSymbolName(e, "Terminal");
       AssertionExpression expr  = null;
       String              etype = getAttribute(e, "etype");
       String              value = getAttribute(e, "value");
-      Type                type  = parseType(getElement(e, "Type"));
       try {
-        if (etype.equals("identifier")) {
-          expr = new IdentifierLiteral(value);
+        if ("identifier".equals(etype)) {
+          expr = new IdentifierLiteral(value, d_context);
           expr.setParens(parens);
-        } else if (etype.equals("constant")) {
-          switch (type.getType()) {
+        } else {
+          switch (StringXML.fromTypeXML(etype)) {
             case Type.BOOLEAN:
-              expr = new BooleanLiteral(value.equals("true"));
+              expr = new BooleanLiteral(value.equals("true"), d_context);
               expr.setParens(parens);
               break;
             case Type.CHAR:
-              expr = new CharacterLiteral(value.charAt(0));
+              expr = new CharacterLiteral(value.charAt(0), d_context);
               expr.setParens(parens);
               break;
             case Type.DOUBLE:
-              expr = new DoubleLiteral(Double.valueOf(value), value);
+              expr = new DoubleLiteral(Double.valueOf(value), value, d_context);
               expr.setParens(parens);
               break;
             case Type.FLOAT:
-              expr = new FloatLiteral(Float.valueOf(value), value);
+              expr = new FloatLiteral(Float.valueOf(value), value, d_context);
               expr.setParens(parens);
               break;
             case Type.INT:
-              expr = new IntegerLiteral(Integer.decode(value), value);
+              expr = new IntegerLiteral(Integer.decode(value), value, d_context);
               expr.setParens(parens);
               break;
             case Type.LONG:
-              expr = new LongLiteral(Long.decode(value), value);
+              expr = new LongLiteral(Long.decode(value), value, d_context);
               expr.setParens(parens);
               break;
             case Type.STRING:
-              expr = new StringLiteral(value);
+              expr = new StringLiteral(value, d_context);
               expr.setParens(parens);
               break;
             case Type.SYMBOL:
-              expr = new IdentifierLiteral(value);
+              expr = new IdentifierLiteral(value, d_context);
               expr.setParens(parens);
               break;
             case Type.ARRAY:
-              expr = new IdentifierLiteral(value);
+              expr = new IdentifierLiteral(value, d_context);
               expr.setParens(parens);
               break;
             default:
-              error("parseSimpleExpression: Unsupported Type value \""
-                  + type.getType() + "\" encountered.");
+              error("parseTerminal: Unsupported Type value \""
+                  + etype + "\" encountered.");
               break;
           }
         }
       } catch (AssertionException aex) {
-        error("parseSimpleExpression: AssertionException: " + aex.getMessage());
+        error("parseTerminal: AssertionException: " + aex.getMessage());
       } catch (NumberFormatException nfe) {
-        error("parseSimpleExpression: NumberFormatException: "
+        error("parseTerminal: NumberFormatException: "
             + nfe.getMessage());
       }
 
@@ -1001,9 +1103,9 @@ public class ParseSymbolXML {
       int             op    = StringXML.fromUnaryOpXML(getAttribute(e, "op"));
       UnaryExpression uexpr = null;
       try {
-        AssertionExpression expr  = parseAssertionExpression(getElement(e,
-                                                       "AssertionExpression"));
-        uexpr                     = new UnaryExpression(op, expr);
+        AssertionExpression expr  = parseExpression(getElement(e,
+                                                       "Expression"));
+        uexpr                     = new UnaryExpression(op, expr, d_context);
         uexpr.setParens(parens);
       } catch (AssertionException aex) {
         error("parseUnaryExpression: AssertionException: " + aex.getMessage());
@@ -1056,11 +1158,16 @@ public class ParseSymbolXML {
    private void validateSymbolName(Element e, String name)
       throws ParseSymbolException 
    {
-      String tag = e.getTagName();
-      if (!name.equals(tag)) {
+     if (e != null) {
+       String tag = e.getTagName();
+       if (!name.equals(tag)) {
          error("validateSymbolName: Invalid element name \"" + tag 
-             + "\" (expected \""+name+"\").");
-      }
+               + "\" (expected \""+name+"\").");
+       }
+     }
+     else {
+       error("validateSymbolName: null " + name + " element.");
+     }
    }
 
    /**
