@@ -1,7 +1,7 @@
 //
 // File:        StringXML.java
 // Package:     gov.llnl.babel.parsers.xml
-// Revision:    @(#) $Id: StringXML.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: StringXML.java 5381 2006-04-11 23:31:17Z epperly $
 // Description: simple utility class to convert between enums and XML strings
 //
 // Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -79,11 +79,13 @@ public class StringXML {
     s_symbol_int2xml.put(new Integer(Symbol.ENUM),      "enum");
     s_symbol_int2xml.put(new Integer(Symbol.INTERFACE), "interface");
     s_symbol_int2xml.put(new Integer(Symbol.PACKAGE),   "package");
+    s_symbol_int2xml.put(new Integer(Symbol.STRUCT),   "struct");
 
     s_symbol_xml2int.put("class",     new Integer(Symbol.CLASS));
     s_symbol_xml2int.put("enum",      new Integer(Symbol.ENUM));
     s_symbol_xml2int.put("interface", new Integer(Symbol.INTERFACE));
     s_symbol_xml2int.put("package",   new Integer(Symbol.PACKAGE));
+    s_symbol_xml2int.put("struct",   new Integer(Symbol.STRUCT));
 
     s_type_int2xml.put(new Integer(Type.VOID),     "void");
     s_type_int2xml.put(new Integer(Type.BOOLEAN),  "boolean");
@@ -134,10 +136,12 @@ public class StringXML {
     s_com_int2xml.put(new Integer(Method.NORMAL), "normal");
     s_com_int2xml.put(new Integer(Method.LOCAL),  "local");
     s_com_int2xml.put(new Integer(Method.ONEWAY), "oneway");
+    s_com_int2xml.put(new Integer(Method.NONBLOCKING), "nonblocking");
 
     s_com_xml2int.put("normal", new Integer(Method.NORMAL));
     s_com_xml2int.put("local",  new Integer(Method.LOCAL));
     s_com_xml2int.put("oneway", new Integer(Method.ONEWAY));
+    s_com_xml2int.put("nonblocking", new Integer(Method.NONBLOCKING));
 
     s_order_int2xml.put(new Integer(Type.UNSPECIFIED),  "unspecified");
     s_order_int2xml.put(new Integer(Type.COLUMN_MAJOR), "column-major");
@@ -173,10 +177,9 @@ public class StringXML {
     s_unary_xml2int.put("not",         new Integer(UnaryExpression.NOT));
     s_unary_xml2int.put("plus",        new Integer(UnaryExpression.PLUS));
 
-    s_binary_int2xml.put(new Integer(BinaryExpression.AND),        "and");
+    s_binary_int2xml.put(new Integer(BinaryExpression.LOGICAL_AND),        "and");
     s_binary_int2xml.put(new Integer(BinaryExpression.DIVIDE),     "divide");
     s_binary_int2xml.put(new Integer(BinaryExpression.EQUALS),     "equals"); 
-    s_binary_int2xml.put(new Integer(BinaryExpression.EXPON),      "expon"); 
     s_binary_int2xml.put(new Integer(BinaryExpression.GREATER_EQUAL),
                                                             "greater_equal");
     s_binary_int2xml.put(new Integer(BinaryExpression.GREATER_THAN),
@@ -185,8 +188,6 @@ public class StringXML {
     s_binary_int2xml.put(new Integer(BinaryExpression.IMPLIES),   "implies");
     s_binary_int2xml.put(new Integer(BinaryExpression.LESS_EQUAL), 
                                                                "less_equal");
-    s_binary_int2xml.put(new Integer(BinaryExpression.LESS_GREATER),
-                                                             "less_greater");
     s_binary_int2xml.put(new Integer(BinaryExpression.LESS_THAN),  
                                                                 "less_than");
     s_binary_int2xml.put(new Integer(BinaryExpression.MINUS),      "minus"); 
@@ -194,7 +195,7 @@ public class StringXML {
     s_binary_int2xml.put(new Integer(BinaryExpression.MULTIPLY), "multiply");
     s_binary_int2xml.put(new Integer(BinaryExpression.NOT_EQUAL),  
                                                                 "not_equal");
-    s_binary_int2xml.put(new Integer(BinaryExpression.OR),         "or"); 
+    s_binary_int2xml.put(new Integer(BinaryExpression.LOGICAL_OR),         "or"); 
     s_binary_int2xml.put(new Integer(BinaryExpression.PLUS),       "plus"); 
     s_binary_int2xml.put(new Integer(BinaryExpression.POWER),      "power"); 
     s_binary_int2xml.put(new Integer(BinaryExpression.REMAINDER),  
@@ -203,12 +204,14 @@ public class StringXML {
                                                                "shift_left");
     s_binary_int2xml.put(new Integer(BinaryExpression.SHIFT_RIGHT),
                                                               "shift_right");
-    s_binary_int2xml.put(new Integer(BinaryExpression.XOR),        "xor"); 
+    s_binary_int2xml.put(new Integer(BinaryExpression.LOGICAL_XOR),        "xor"); 
+    s_binary_int2xml.put(new Integer(BinaryExpression.BITWISE_XOR),        "bit-xor"); 
+    s_binary_int2xml.put(new Integer(BinaryExpression.BITWISE_AND),        "bit-and"); 
+    s_binary_int2xml.put(new Integer(BinaryExpression.BITWISE_OR),        "bit-or"); 
 
-    s_binary_xml2int.put("and",       new Integer(BinaryExpression.AND));
+    s_binary_xml2int.put("and",       new Integer(BinaryExpression.LOGICAL_AND));
     s_binary_xml2int.put("divide",    new Integer(BinaryExpression.DIVIDE));
     s_binary_xml2int.put("equals",    new Integer(BinaryExpression.EQUALS));
-    s_binary_xml2int.put("expon",     new Integer(BinaryExpression.EXPON));
     s_binary_xml2int.put("greater_equal", 
                                 new Integer(BinaryExpression.GREATER_EQUAL));
     s_binary_xml2int.put("greater_than", 
@@ -218,8 +221,6 @@ public class StringXML {
     s_binary_xml2int.put("implies",   new Integer(BinaryExpression.IMPLIES));
     s_binary_xml2int.put("less_equal",
                                    new Integer(BinaryExpression.LESS_EQUAL));
-    s_binary_xml2int.put("less_greater", 
-                                 new Integer(BinaryExpression.LESS_GREATER));
     s_binary_xml2int.put("less_than", 
                                     new Integer(BinaryExpression.LESS_THAN));
     s_binary_xml2int.put("minus",     new Integer(BinaryExpression.MINUS));
@@ -227,7 +228,7 @@ public class StringXML {
     s_binary_xml2int.put("multiply", new Integer(BinaryExpression.MULTIPLY));
     s_binary_xml2int.put("not_equal", 
                                     new Integer(BinaryExpression.NOT_EQUAL));
-    s_binary_xml2int.put("or",        new Integer(BinaryExpression.OR));
+    s_binary_xml2int.put("or",        new Integer(BinaryExpression.LOGICAL_OR));
     s_binary_xml2int.put("plus",      new Integer(BinaryExpression.PLUS));
     s_binary_xml2int.put("power",     new Integer(BinaryExpression.POWER));
     s_binary_xml2int.put("remainder", 
@@ -236,7 +237,10 @@ public class StringXML {
                                    new Integer(BinaryExpression.SHIFT_LEFT));
     s_binary_xml2int.put("shift_right", 
                                   new Integer(BinaryExpression.SHIFT_RIGHT));
-    s_binary_xml2int.put("xor", new Integer(BinaryExpression.XOR));
+    s_binary_xml2int.put("xor", new Integer(BinaryExpression.LOGICAL_XOR));
+    s_binary_xml2int.put("bit-xor", new Integer(BinaryExpression.BITWISE_XOR)); 
+    s_binary_xml2int.put("bit-and", new Integer(BinaryExpression.BITWISE_AND)); 
+    s_binary_xml2int.put("bit-or", new Integer(BinaryExpression.BITWISE_OR)); 
   }
 
   /**
