@@ -1,7 +1,7 @@
 //
 // File:        SymbolToDOM.java
 // Package:     gov.llnl.babel.parsers.xml
-// Revision:    @(#) $Id: SymbolToDOM.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: SymbolToDOM.java 6212 2007-11-01 00:07:17Z epperly $
 // Description: convert sidl symbols into an XML DOM document
 //
 // Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -30,13 +30,14 @@
 
 package gov.llnl.babel.parsers.xml;
 
-import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.parsers.xml.DTDManager;
 import gov.llnl.babel.parsers.xml.StringXML;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Assertion;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.Attributes;
 import gov.llnl.babel.symbols.BinaryExpression;
 import gov.llnl.babel.symbols.BooleanLiteral;
 import gov.llnl.babel.symbols.CharacterLiteral;
@@ -45,6 +46,7 @@ import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.DComplexLiteral;
 import gov.llnl.babel.symbols.DoubleLiteral;
 import gov.llnl.babel.symbols.Enumeration;
+import gov.llnl.babel.symbols.ExprVisitor;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.FComplexLiteral;
 import gov.llnl.babel.symbols.FloatLiteral;
@@ -57,23 +59,25 @@ import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.MethodCall;
 import gov.llnl.babel.symbols.Package;
 import gov.llnl.babel.symbols.StringLiteral;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.UnaryExpression;
 import gov.llnl.babel.xml.XMLUtilities;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.StringTokenizer;
+import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.DocumentType;
-import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.w3c.dom.Text;
-import java.util.StringTokenizer;
 
 /**
  * Class <code>SymbolToDOM</code> converts a SIDL symbol into an XML DOM
@@ -87,6 +91,7 @@ public class SymbolToDOM {
 
    private Document d_document;
    private int      d_indent;
+  private Context d_context;
 
    /**
     * This is a convenience utility function that converts the symbol into
@@ -95,8 +100,8 @@ public class SymbolToDOM {
     *
     * @param  symbol  The symbol to be converted.
     */
-   public static Document convert(Symbol symbol) {
-      SymbolToDOM sym2dom = new SymbolToDOM(symbol);
+   public static Document convert(Symbol symbol, Context context) {
+      SymbolToDOM sym2dom = new SymbolToDOM(symbol, context);
       return sym2dom.getDocument();
    }
 
@@ -106,13 +111,12 @@ public class SymbolToDOM {
     *
     * @param  symbol  The symbol to be converted.
     */
-   public static String convertToString(Symbol symbol) {
-      return XMLUtilities.getXMLString(convert(symbol));
+   public static String convertToString(Symbol symbol, Context context) {
+      return XMLUtilities.getXMLString(convert(symbol, context));
    }
 
-  public static String getDTDPath(String dtd) {
-    BabelConfiguration b_config = BabelConfiguration.getInstance();
-    String repoPath = b_config.getRepositoryPath();
+  public String getDTDPath(String dtd) {
+    String repoPath = d_context.getConfig().getRepositoryPath();
     for( StringTokenizer tokenizer = new StringTokenizer( repoPath, ";" );
          tokenizer.hasMoreTokens(); ) {
       String path = tokenizer.nextToken();
@@ -137,9 +141,11 @@ public class SymbolToDOM {
     *
     * @param  symbol  The symbol associated with an instance of this class.
     */
-  public SymbolToDOM(Symbol symbol) {
+  public SymbolToDOM(Symbol symbol, Context context) {
 
-    DOMImplementation dom = XMLUtilities.createDOMImplementation();
+    d_context = context;
+    DOMImplementation dom = XMLUtilities.createDocBuilder().
+      getDOMImplementation();
     DocumentType doctype = dom.createDocumentType("Symbol", DTDManager.SYMBOL_PUBLIC_ID, 
                                                   getDTDPath("sidl.dtd")+"sidl.dtd");
       d_document = dom.createDocument(null, "Symbol", doctype);
@@ -148,18 +154,21 @@ public class SymbolToDOM {
          d_indent = 0;
          Element docsymbol = d_document.getDocumentElement();
          switch (symbol.getSymbolType()) {
+         case Symbol.STRUCT:
+           convertStruct((Struct) symbol, docsymbol);
+           break;
          case Symbol.CLASS:
-            convertClass((Class) symbol, docsymbol);
-            break;
+           convertClass((Class) symbol, docsymbol);
+           break;
          case Symbol.ENUM:
-            convertEnumeration((Enumeration) symbol, docsymbol);
-            break;
+           convertEnumeration((Enumeration) symbol, docsymbol);
+           break;
          case Symbol.INTERFACE:
-            convertInterface((Interface) symbol, docsymbol);
-            break;
+           convertInterface((Interface) symbol, docsymbol);
+           break;
          case Symbol.PACKAGE:
-            convertPackage((Package) symbol, docsymbol);
-            break;
+           convertPackage((Package) symbol, docsymbol);
+           break;
          }
          docsymbol.appendChild(d_document.createTextNode(EOL));
       }
@@ -172,6 +181,25 @@ public class SymbolToDOM {
       return d_document;
    }
 
+  private void convertAttributes(Attributes a, Element parent)
+  {
+    Iterator i = a.getAttributes().iterator();
+    if (i.hasNext()) {
+      Element attributes_element = createElement("Attributes");
+      while(i.hasNext()) {
+        String key = (String)i.next();
+        String value = a.getAttribute(key);
+        Element elem = createElement("Attribute");
+        elem.setAttribute("name", key);
+        if (value != null) {
+          elem.setAttribute("value", value);
+        }
+        appendChild(elem, attributes_element);
+      }
+      appendChild(attributes_element, parent);
+    }
+  }
+
    /**
     * Convert a SIDL class description into its DOM representation.
     * A class consists of an extends symbol, an implements block of
@@ -186,7 +214,7 @@ public class SymbolToDOM {
       convertComment (cls.getComment() , parent);
 
       Element class_element = createElement("Class");
-      class_element.setAttribute("abstract",String.valueOf(cls.isAbstract()));
+      convertAttributes(cls, class_element);
 
       /*
        * Write the extends block of a single class identifier.
@@ -242,6 +270,7 @@ public class SymbolToDOM {
       convertComment (e.getComment() , parent);
 
       Element enumeration_element = createElement("Enumeration");
+      convertAttributes(e, enumeration_element);
       for (Iterator i = e.getIterator(); i.hasNext(); ) {
          String n = (String) i.next();
          Element enumerator_element = createElement("Enumerator");
@@ -274,6 +303,7 @@ public class SymbolToDOM {
 
       Element interface_element = createElement("Interface");
 
+      convertAttributes(ifc, interface_element);
       /*
        * Write the extends block of symbol identifiers.
        */
@@ -339,7 +369,7 @@ public class SymbolToDOM {
    private void convertMethods(Extendable ext, Node parent) {
       Element methodblock_element = createElement("MethodsBlock");
       for (Iterator m = ext.getMethods(false).iterator(); m.hasNext(); ) {
-         convertMethod((Method) m.next(), methodblock_element);
+         convertMethod((Method) m.next(), ext, methodblock_element);
       }
       appendChild(methodblock_element, parent);
    }
@@ -370,8 +400,7 @@ public class SymbolToDOM {
       convertComment (p.getComment() , parent);
 
       Element package_element = createElement("Package");
-      package_element.setAttribute("final",
-                                   p.getFinal() ? "true" : "false");
+      convertAttributes(p, package_element);
       Collection children = p.getOrderedSymbolReferences();
       Map map = p.getSymbols();
       for (Iterator i = children.iterator(); i.hasNext(); ) {
@@ -401,12 +430,11 @@ public class SymbolToDOM {
     */
    private void convertComment(Comment c, Node parent) {
       Element comment_element = null;
-      BabelConfiguration b_config = BabelConfiguration.getInstance();
       /*
        * If the comment is empty, then return an empty comment element.
        */
       
-      if (c.isEmpty()) {
+      if ((null == c) || (c.isEmpty())) {
          comment_element = createElement("Comment");
       } else {
          String[] lines = c.getComment();
@@ -465,7 +493,7 @@ public class SymbolToDOM {
     */
    private void convertMetadata(Metadata m, Node parent) {
       Element metadata_element = createElement("Metadata");
-      if ( BabelConfiguration.getInstance().suppressTimestamps() ) {
+      if ( d_context.getConfig().suppressTimestamps() ) {
         metadata_element.setAttribute("date", "20010101 00:00:00 GMT");
       } else { 
         metadata_element.setAttribute("date", m.getDateAsString());
@@ -488,21 +516,17 @@ public class SymbolToDOM {
     * return type, argument list, and throws list.
     *
     * @param  m       The method to be converted.
+    * @param  ext     To determine if method requires a from clause
     * @param  parent  The parent node of the corresponding DOM representation.
     */
-   private void convertMethod(Method m, Node parent) {
+   private void convertMethod(Method m, Extendable ext, Node parent) {
       /*
        * Create a method node and add attributes
        */
       Element method_element = createElement("Method");
       method_element.setAttribute("shortname", m.getShortMethodName());
       method_element.setAttribute("extension", m.getNameExtension());
-      method_element.setAttribute(
-         "copy", String.valueOf(m.isReturnCopy()));
-      method_element.setAttribute(
-         "definition", StringXML.toDefXML(m.getDefinitionModifier()));
-      method_element.setAttribute(
-         "communication", StringXML.toComXML(m.getCommunicationModifier()));
+      convertAttributes(m, method_element);
 
       /*
        * Create the comment and return type elements
@@ -514,21 +538,41 @@ public class SymbolToDOM {
        * Create the argument list element
        */
       Element arglist_element = createElement("ArgumentList");
-      ArrayList args = m.getArgumentListWithIndices();
-      for (int a = 0; a < args.size(); a++) {
-         convertArgument((Argument) args.get(a), arglist_element);
+      List args = m.getArgumentListWithIndices();
+      for (Iterator a = args.iterator(); a.hasNext(); ) {
+         convertArgument((Argument) a.next(), arglist_element);
       }
       appendChild(arglist_element, method_element);
 
       /*
-       * Create the throws list element
+       * Create the explicit throws list element
        */
       Element throwslist_element = createElement("ThrowsList");
-      for (Iterator t = m.getThrows().iterator(); t.hasNext(); ) {
+      for (Iterator t = m.getExplicitThrows().iterator(); t.hasNext(); ) {
          convertSymbolID((SymbolID) t.next(), throwslist_element);
       }
       appendChild(throwslist_element, method_element);
 
+      /*
+       * Create the implicit throws list element
+       */
+      Element implthrowslist_element = createElement("ImplicitThrowsList");
+      for (Iterator t = m.getImplicitThrows().iterator(); t.hasNext(); ) {
+         convertSymbolID((SymbolID) t.next(), implthrowslist_element);
+      }
+      appendChild(implthrowslist_element, method_element);
+
+      Method renamedMethod = ext.getRenamedMethod(m);
+      if (renamedMethod != null) {
+        Element fromClause = createElement("From");
+        SymbolID parentID = ext.getRenamedMethodSymbolID(renamedMethod);
+        fromClause.setAttribute("shortname", renamedMethod.getShortMethodName());
+        fromClause.setAttribute("extension", renamedMethod.getNameExtension());
+        fromClause.setAttribute("parentname", parentID.getFullName());
+        fromClause.setAttribute("parentversion", parentID.getVersion().getVersionString());
+        appendChild(fromClause, method_element);
+      }
+
       /*
        * Create the lists of preconditions followed by postconditions, if any.
        */
@@ -547,13 +591,39 @@ public class SymbolToDOM {
     */
    private void convertArgument(Argument arg, Node parent) {
       Element arg_element = createElement("Argument");
-      arg_element.setAttribute("copy", String.valueOf(arg.isCopy()));
       arg_element.setAttribute("mode", StringXML.toModeXML(arg.getMode()));
       arg_element.setAttribute("name", arg.getFormalName());
+      convertAttributes(arg, arg_element);
       convertType(arg.getType(), arg_element);
       appendChild(arg_element, parent);
    }
 
+
+  /**
+   * Convert a SIDL struct description into a DOM reprentation.
+   */
+  private void convertStruct(Struct s, Node parent) {
+    convertSymbolID(s.getSymbolID(), parent);
+    convertMetadata(s.getMetadata(), parent);
+    convertComment (s.getComment() , parent);
+    Element struct_element = createElement("Struct");
+    convertAttributes(s, struct_element);
+    Iterator i = s.getItems().iterator();
+    while(i.hasNext()) {
+      Struct.Item item = (Struct.Item)i.next();
+      convertStructItem(item, struct_element);
+    }
+    appendChild(struct_element, parent);
+  }
+
+  private void convertStructItem(Struct.Item item, Node parent)
+  {
+    Element struct_item = createElement("StructItem");
+    struct_item.setAttribute("name", item.getName());
+    convertType(item.getType(), struct_item);
+    appendChild(struct_item, parent);
+  }
+
    /**
     * Convert a SIDL type description into a DOM representation.  The
     * element is <code>Type</code> with attribute <code>type</code>.
@@ -582,7 +652,11 @@ public class SymbolToDOM {
            //variables for this rarray.
            if(type.isRarray()) {
              Element index_element = createElement("Index");
-             index_element.setAttribute("name", type.getIndexString());
+             Iterator i = type.getArrayIndexExprs().iterator();
+             while (i.hasNext()) {
+               AssertionExpression ae = (AssertionExpression)i.next();
+               convertExpr(ae, index_element);
+             }
              appendChild(index_element, array_element);
            }
          }
@@ -617,19 +691,19 @@ public class SymbolToDOM {
     * @param  parent  The parent node of the corresponding DOM representation.
     */
    private void convertAssertions(Method meth, Node parent) {
-      ArrayList list = meth.getPreconditions();
+      LinkedList list = new LinkedList(meth.getPreconditions());
       list.addAll(meth.getPostconditions());
       convertAssertions(list, parent);
    }
 
    /**
-    * Convert an <code>ArrayList</code> of assertions, if any, into its DOM 
+    * Convert an <code>List</code> of assertions, if any, into its DOM 
     * representation.
     *
     * @param  list    The list of SIDL <code>Assertion</code> entries.
     * @param  parent  The parent node of the corresponding DOM representation.
     */
-   private void convertAssertions(ArrayList list, Node parent) {
+   private void convertAssertions(List list, Node parent) {
       if (!list.isEmpty()) {
          Element list_elem = createElement("AssertionList");
          for (Iterator i = list.iterator(); i.hasNext(); ) {
@@ -662,6 +736,140 @@ public class SymbolToDOM {
       }
    }
 
+  private class ExprToDOM extends ExprVisitor {
+
+    
+    public Object visitBinaryExpression(BinaryExpression be, Object parent) {
+      Element e = createElement("BinaryExpression");
+      e.setAttribute("op", StringXML.toBinaryOpXML(be.getOp()));
+      Element child = createElement("Expression");
+      be.getLeftExpression().accept(this, child);
+      appendChild(child, e);
+      child = createElement("Expression");
+      be.getRightExpression().accept(this, child);
+      appendChild(child, e);
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+  
+    public Object visitBooleanLiteral(BooleanLiteral bl, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.BOOLEAN));
+      e.setAttribute("value", bl.getValue() ? "true" : "false");
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitCharacterLiteral(CharacterLiteral cl, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.CHAR));
+      e.setAttribute("value", String.valueOf(cl.getValue()));
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+    
+    public Object visitDComplexLiteral(DComplexLiteral dcl, Object parent) {
+      Element e  = createElement("ComplexNumber");
+      e.setAttribute("type", "float");
+      e.setAttribute("real", dcl.getRealImage());
+      e.setAttribute("imaginary", dcl.getImaginaryImage());
+
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitDoubleLiteral(DoubleLiteral dl, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.DOUBLE));
+      e.setAttribute("value", dl.getImage());
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+  
+    public Object visitFComplexLiteral(FComplexLiteral fcl, Object parent) {
+      Element e  = createElement("ComplexNumber");
+      e.setAttribute("type", "float");
+      e.setAttribute("real", fcl.getRealImage());
+      e.setAttribute("imaginary", fcl.getImaginaryImage());
+      appendChild(e, (Node)parent);
+      
+      return parent;
+    }
+
+    public Object visitFloatLiteral(FloatLiteral fl, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.FLOAT));
+      e.setAttribute("value", fl.getImage());
+
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitIdentifierLiteral(IdentifierLiteral il, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", "identifier");
+      e.setAttribute("value", il.getIdentifier());
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitIntegerLiteral(IntegerLiteral il, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.INT));
+      e.setAttribute("value", il.getImage());
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitLongLiteral(LongLiteral ll, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.LONG));
+      e.setAttribute("value", ll.getImage());
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitMethodCall(MethodCall mc, Object parent) {
+      Element e  = createElement("MethodCall");
+      e.setAttribute("name", mc.getMethodName());
+      Iterator i = mc.getArguments().iterator();
+      while (i.hasNext()) {
+        AssertionExpression ae = (AssertionExpression)i.next();
+        Element child = createElement("Expression");
+        ae.accept(this, child);
+        appendChild(child, e);
+      }
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitStringLiteral(StringLiteral sl, Object parent) {
+      Element e  = createElement("Terminal");
+      e.setAttribute("etype", StringXML.toTypeXML(Type.STRING));
+      e.setAttribute("value", sl.getValue());
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+
+    public Object visitUnaryExpression(UnaryExpression ue, Object parent) {
+      Element e  = createElement("UnaryExpression");
+      e.setAttribute("op", StringXML.toUnaryOpXML(ue.getOp()));
+      Element child = createElement("Expression");
+      ue.getExpression().accept(this,child);
+      appendChild(child, e);
+      appendChild(e, (Node)parent);
+      return parent;
+    }
+  }
+
+  public Node convertExpr(AssertionExpression ae, Node parent) {
+    Element e = createElement("Expression");
+    ae.accept(new ExprToDOM(), e);
+    appendChild(e, parent);
+    return e;
+  }
+
+
    /**
     * Convert an <code>AssertionExpression</code> into its DOM representation.
     *
@@ -674,280 +882,15 @@ public class SymbolToDOM {
    private void convertAssertionExpression(AssertionExpression ae, Node parent)
       throws AssertionException
    {
-      if (ae != null) {
-         if (ae.isValid()) {
-           boolean ok = true;
-           Element e  = createElement("AssertionExpression");
-           e.setAttribute("parens", String.valueOf(ae.hasParens()));
-  
-           String classname  = ae.getClass().getName();
-           String pathprefix = "gov.llnl.babel.symbols.";
-           if        (classname.equals(pathprefix+"BinaryExpression")) {
-              convertBinaryExpression((BinaryExpression)ae, e);
-           } else if (classname.equals(pathprefix+"BooleanLiteral")   ) {
-              convertBooleanLiteral((BooleanLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"CharacterLiteral")   ) {
-              convertCharacterLiteral((CharacterLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"DComplexLiteral")   ) {
-              convertDComplexLiteral((DComplexLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"DoubleLiteral")   ) {
-              convertDoubleLiteral((DoubleLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"FComplexLiteral")   ) {
-              convertFComplexLiteral((FComplexLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"FloatLiteral")   ) {
-              convertFloatLiteral((FloatLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"IdentifierLiteral")   ) {
-              convertIdentifierLiteral((IdentifierLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"IntegerLiteral")   ) {
-              convertIntegerLiteral((IntegerLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"LongLiteral")   ) {
-              convertLongLiteral((LongLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"MethodCall")   ) {
-              convertMethodCall((MethodCall)ae, e);
-           } else if (classname.equals(pathprefix+"StringLiteral")   ) {
-              convertStringLiteral((StringLiteral)ae, e);
-           } else if (classname.equals(pathprefix+"UnaryExpression")   ) {
-              convertUnaryExpression((UnaryExpression)ae, e);
-           } else {
-             /*
-              * The class is not currently recognized as a valid assertion
-              * expression (to which the translation is known) so it is not 
-              * going to be added to the DOM representations.
-              */
-             ok = false;
-           }
-  
-           if (ok) {
-              appendChild(e, parent);
-           }
-        } else {
-           throw new AssertionException("convertAssertionExpression: Cannot "
-                       + "perform conversion on the unvalidated assertion "
-                       + "expression \"" + ae.toString() + "\".");
-        }
-      }
-   }
-
-   /**
-    * Convert a <code>BinaryExpression</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>BinaryExpression</code> to be 
-    *                 converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertBinaryExpression(BinaryExpression expr, Node parent) {
-      AssertionExpression lhs = expr.getLeftExpression();
-      AssertionExpression rhs = expr.getRightExpression();
-      if ( (lhs != null) && (rhs != null) ) {
-         Element e  = createElement("BinaryExpression");
-         e.setAttribute("op", StringXML.toBinaryOpXML(expr.getOp()));
-         try {
-            convertAssertionExpression(lhs, e);
-            convertAssertionExpression(rhs, e);
-         } catch (AssertionException ex) {
-            System.err.println(ex.getMessage());
-            System.exit(1);
-         }
-
-         appendChild(e, parent);
-      }
-   }
-
-   /**
-    * Convert a <code>BooleanLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>BooleanLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertBooleanLiteral(BooleanLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getValue() ? "true" : "false");
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>CharacterLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>CharacterLiteral</code> to be 
-    *                 converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertCharacterLiteral(CharacterLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", String.valueOf(expr.getValue()));
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>DComplexLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>DComplexLiteral</code> to be 
-    *                 converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertDComplexLiteral(DComplexLiteral expr, Node parent) {
-      Element e  = createElement("ComplexNumber");
-      e.setAttribute("type", "float");
-      e.setAttribute("real", expr.getRealImage());
-      e.setAttribute("imaginary", expr.getImaginaryImage());
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>DoubleLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>DoubleLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertDoubleLiteral(DoubleLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getImage());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>FComplexLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>FComplexLiteral</code> to be 
-    *                 converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertFComplexLiteral(FComplexLiteral expr, Node parent) {
-      Element e  = createElement("ComplexNumber");
-      e.setAttribute("type", "float");
-      e.setAttribute("real", expr.getRealImage());
-      e.setAttribute("imaginary", expr.getImaginaryImage());
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>FloatLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>FloatLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertFloatLiteral(FloatLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getImage());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>IdentifierLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>IdentifierLiteral</code> to be 
-    *                 converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertIdentifierLiteral(IdentifierLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "identifier");
-      e.setAttribute("value", expr.getIdentifier());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>IntegerLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>IntegerLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertIntegerLiteral(IntegerLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getImage());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>LongLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>LongLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertLongLiteral(LongLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getImage());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>MethodCall</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>MethodCall</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertMethodCall(MethodCall expr, Node parent) {
-      Element e  = createElement("MethodCall");
-      e.setAttribute("name", expr.getMethodName());
-      try {
-         for (Iterator i = expr.getArguments().iterator(); i.hasNext(); ) {
-            convertAssertionExpression((AssertionExpression) i.next(), e);
-         }
-      } catch (AssertionException ex) {
-         System.err.println(ex.getMessage());
-         System.exit(1);
-      }
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>StringLiteral</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>StringLiteral</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertStringLiteral(StringLiteral expr, Node parent) {
-      Element e  = createElement("SimpleExpression");
-      e.setAttribute("etype", "constant");
-      e.setAttribute("value", expr.getValue());
-      convertType(expr.getReturnType(), e);
-
-      appendChild(e, parent);
-   }
-
-   /**
-    * Convert a <code>UnaryExpression</code> into its DOM representation.
-    *
-    * @param  expr    The non-null <code>UnaryExpression</code> to be converted.
-    * @param  parent  The parent node of the corresponding DOM representation.
-    */
-   private void convertUnaryExpression(UnaryExpression expr, Node parent) {
-      AssertionExpression ae = expr.getExpression();
-      if (ae != null) {
-        Element e  = createElement("UnaryExpression");
-        e.setAttribute("op", StringXML.toUnaryOpXML(expr.getOp()));
-        try {
-           convertAssertionExpression(ae, e);
-        } catch (AssertionException ex) {
-           System.err.println(ex.getMessage());
-           System.exit(1);
-        }
-
-        appendChild(e, parent);
-      }
+     if (ae != null) {
+       if (ae.isValid()) {
+         convertExpr(ae, parent);
+       } else {
+         throw new AssertionException("convertAssertionExpression: Cannot "
+                        + "perform conversion on the unvalidated assertion "
+                        + "expression \"" + ae.toString() + "\".");
+       }
+     }
    }
 
    /**
