@@ -1,7 +1,7 @@
 //
 // File:        SymbolWriter.java
 // Package:     gov.llnl.babel.repository
-// Revision:    @(#) $Id: SymbolWriter.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: SymbolWriter.java 6201 2007-10-26 06:15:35Z kumfert $
 // Description: interface for writing symbols to an external repository
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -43,5 +43,6 @@ public interface SymbolWriter {
     * to the repository.  Each set entry is a <code>SymbolID</code> of
     * the symbol to be written to the repository.
     */
-   public void writeSymbols(Set symbol_names) throws RepositoryException;
+  public void writeSymbols(Set symbol_names)
+    throws RepositoryException;
 }
