@@ -1,7 +1,7 @@
 //
 // File:        Argument.java
 // Package:     gov.llnl.babel.symbols
-// Revision:    @(#) $Id: Argument.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: Argument.java 6212 2007-11-01 00:07:17Z epperly $
 // Description: this class represents a sidl method argument
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,24 +31,32 @@
 package gov.llnl.babel.symbols;
 
 import gov.llnl.babel.symbols.SymbolUtilities;
+import gov.llnl.babel.symbols.ASTNode;
 import gov.llnl.babel.symbols.Type;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Map;
+import java.util.Set;
 
 /**
  * The <code>Argument</code> class defines a SIDL argument, which consists
  * of a parameter passing mode (IN, INOUT, or OUT with an optional COPY),
  * a type, and a formal parameter name.
  */
-public class Argument {
-   public static final int IN    = 0; 
-   public static final int INOUT = 1; 
-   public static final int OUT   = 2; 
+public class Argument extends ASTNode implements Attributes {
+  private Map d_attributes = new HashMap();
+  public static final int IN    = 0; 
+  public static final int INOUT = 1; 
+  public static final int OUT   = 2; 
+  
+  private static final String[] s_mode_string = { "in", "inout", "out" };
 
-   private static final String[] s_mode_string = { "in", "inout", "out" };
-
-   private boolean d_copy;
-   private String  d_formal_name;
-   private int     d_mode;
-   private Type    d_type;
+  private String  d_formal_name;
+  private int     d_mode;
+  private Type    d_type;
+  private Comment   d_comment;
 
    /**
     * Create an argument object.  
@@ -58,11 +66,11 @@ public class Argument {
     * @param  type  The SIDL type of the argument.
     * @param  name  The formal parameter name.
     */
-   public Argument(boolean copy, int mode, Type type, String name) {
-      d_copy        = copy;
+   public Argument(int mode, Type type, String name) {
       d_formal_name = name;
       d_mode        = mode;
       d_type        = type;
+      d_comment                = null;
    }
 
    /**
@@ -70,7 +78,7 @@ public class Argument {
     * otherwise, return FALSE.
     */
    public boolean isCopy() {
-      return d_copy;
+     return hasAttribute("copy");
    }
 
    /**
@@ -80,6 +88,10 @@ public class Argument {
       return d_mode;
    }
 
+  public static String[] getAllowableModes() {
+    return s_mode_string;
+  }
+
    /**
     * Return a string representing the argument mode.
     */
@@ -112,6 +124,32 @@ public class Argument {
     return d_type.hasArrayOrderSpec();
   }
 
+  private String customAttrStr()
+  {
+    StringBuffer result = new StringBuffer();
+    if (getAttributes().size() > 0) {
+      HashSet attrs = new HashSet(getAttributes());
+      attrs.remove("copy");
+      if (attrs.size() > 0) {
+        Object[] sorted = attrs.toArray();
+        Arrays.sort(sorted);
+        result.append("%attrib{");
+        for(int i = 0; i < sorted.length; ++i) {
+          final String key = sorted[i].toString();
+          final String value = getAttribute(key);
+          result.append(' ').append(key);
+          if (value != null) {
+            result.append(" = \"").append(value).append('"');
+          }
+          if (i+1 < sorted.length) {
+            result.append(',');
+          }
+        }
+        result.append(" }");
+      }
+    }
+    return result.toString();
+  }
 
    /**
     * Return a string representation of the argument for outputting the
@@ -121,9 +159,10 @@ public class Argument {
    public String getArgumentString(String abbrev_pkg) {
       StringBuffer arg = new StringBuffer();
       
-      if (d_copy) {
+      if (isCopy()) {
          arg.append("copy ");
       }
+      arg.append(customAttrStr()).append(' ');
       arg.append(getModeString() + " ");
 
       String rtype = d_type.getTypeString();
@@ -151,7 +190,7 @@ public class Argument {
    }
 
   public int hashCode() {
-    return d_mode*7 + (d_copy ? 1 : 0) + d_type.hashCode();
+    return d_mode*7 + (isCopy() ? 1 : 0) + d_type.hashCode();
   }
 
    /**
@@ -165,10 +204,79 @@ public class Argument {
       boolean eq = false;
       if ((object != null) && (object instanceof Argument)) {
          Argument arg = (Argument) object;
-         eq = ((d_copy == arg.d_copy)
+         eq = ((isCopy() == arg.isCopy())
             && (d_mode == arg.d_mode)
             && (d_type.equals(arg.d_type)));
       }
       return eq;
    }
+
+  /**
+   * Set the comment for the method.  
+   *
+   * @param  comment  The comment associated with the method.  May be null.
+   */
+  public void setComment(Comment comment) {
+    checkFrozen();
+    d_comment = comment;
+  }
+  
+  /**
+   * Return the comment for the method.  This may be null if there is no
+   * comment.
+   */
+  public Comment getComment() {
+    return d_comment;
+  }
+
+  public boolean hasAttribute(String key) 
+  {
+    return d_attributes.containsKey(key);
+  }
+  
+  public String getAttribute(String key) throws UnknownAttributeException
+  {
+    if (hasAttribute(key)){
+      return (String)d_attributes.get(key);
+    }
+    throw new UnknownAttributeException("Argument " + d_formal_name + 
+                                        " has no attribute named " + key);
+  }
+
+  public void setAttribute(String key)
+  {
+    setAttribute(key, null);
+  }
+
+  public void setAttribute(String key, String value)
+  {
+    d_attributes.put(key, value);
+  }
+
+  public void removeAttribute(String key)
+    throws UnknownAttributeException
+  {
+    checkFrozen();
+    if (hasAttribute(key)) {
+      d_attributes.remove(key);
+    }
+    else {
+      throw new UnknownAttributeException("Argument " + d_formal_name + 
+                                          " has no attribute named " + key);
+    }
+  }
+
+  public Set getAttributes()
+  {
+    return d_attributes.keySet();
+  }
+
+  public void freeze() {
+    if (!d_frozen) {
+      super.freeze();
+      d_type.freeze();
+      d_attributes = protectMap(d_attributes);
+      if (d_comment != null) d_comment.freeze();
+    }
+  }
 }
