@@ -30,6 +30,7 @@
 
 package gov.llnl.babel.symbols;
 
+import gov.llnl.babel.Context;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.Extendable;
@@ -49,35 +50,48 @@ public class BinaryExpression extends AssertionExpression {
    * Valid binary operators.
    */
   public final static int NOOP           = 0;
-  public final static int AND            = 1;
+  public final static int LOGICAL_AND    = 1;
   public final static int DIVIDE         = 2;
   public final static int EQUALS         = 3;
-  public final static int EXPON          = 4;
-  public final static int GREATER_EQUAL  = 5;
-  public final static int GREATER_THAN   = 6;
-  public final static int IF_AND_ONLY_IF = 7;
-  public final static int IMPLIES        = 8;
-  public final static int LESS_EQUAL     = 9;
-  public final static int LESS_GREATER   = 10;
-  public final static int LESS_THAN      = 11;
-  public final static int MINUS          = 12;
-  public final static int MODULUS        = 13;
-  public final static int MULTIPLY       = 14;
-  public final static int NOT_EQUAL      = 15;
-  public final static int OR             = 16;
-  public final static int PLUS           = 17;
-  public final static int POWER          = 18;
-  public final static int REMAINDER      = 19;
-  public final static int SHIFT_LEFT     = 20;
-  public final static int SHIFT_RIGHT    = 21;
-  public final static int XOR            = 22;
+  public final static int GREATER_EQUAL  = 4;
+  public final static int GREATER_THAN   = 5;
+  public final static int IF_AND_ONLY_IF = 6;
+  public final static int IMPLIES        = 7;
+  public final static int LESS_EQUAL     = 8;
+  public final static int LESS_THAN      = 9;
+  public final static int MINUS          = 10;
+  public final static int MODULUS        = 11;
+  public final static int MULTIPLY       = 12;
+  public final static int NOT_EQUAL      = 13;
+  public final static int LOGICAL_OR     = 14;
+  public final static int PLUS           = 15;
+  public final static int POWER          = 16;
+  public final static int REMAINDER      = 17;
+  public final static int SHIFT_LEFT     = 18;
+  public final static int SHIFT_RIGHT    = 19;
+  public final static int LOGICAL_XOR    = 20;
+  public final static int BITWISE_AND    = 21;
+  public final static int BITWISE_OR     = 22;
+  public final static int BITWISE_XOR    = 23;
 
   public final static int MIN_OP_VALUE  = 1;   // Minimum valid value 
-  public final static int MAX_OP_VALUE  = 22;  // Maximum valid value
+  public final static int MAX_OP_VALUE  = 23;  // Maximum valid value
 
   private final static String s_symbol[] = {
-    "", "and", "/", "==", "^", ">=", ">", "iff", "implies", "<=", "<>", "<", 
-    "-", "mod", "*", "!=", "or", "+", "**", "rem", "<<", ">>", "xor"
+    "", "and", "/", "==", ">=", ">", "iff", "implies", "<=", "<", 
+    "-", "mod", "*", "!=", "or", "+", "**", "rem", "<<", ">>", "xor",
+    "&", "|", "^"
+  };
+
+  /*
+   * These are supposed to be the equivalent symbols in C.  When there isn't
+   * a "general" symbol (or it could have multiple meanings), the symbol
+   * should be null.
+   */
+  private final static String s_Csymbol[] = {
+    "", "&&", "/", "==", ">=", ">", "", "", "<=", "<", 
+    "-", "%", "*", "!=", "||", "+", "", "", "<<", ">>", "",
+    "&", "|", "^"
   };
 
   /**
@@ -101,10 +115,11 @@ public class BinaryExpression extends AssertionExpression {
    *                  or right expression is null.
    */
   public BinaryExpression(AssertionExpression lhs, int op, 
-                          AssertionExpression rhs)
+                          AssertionExpression rhs,
+                          Context context)
        throws AssertionException
   {
-    super(false);
+    super(false, context);
     if ( (op < MIN_OP_VALUE) || (op > MAX_OP_VALUE) ) {
       throw new AssertionException("Cannot instantiate BinaryExpression since "
                   + "binary operator value \"" + op + "\" invalid, must be in " 
@@ -146,6 +161,16 @@ public class BinaryExpression extends AssertionExpression {
   }
 
 
+  /**
+   * Return the C equivalent, if any, of the symbol associated with the 
+   * operator.  It is assumed the operator is valid thanks to the check 
+   * in the constructor.
+   */
+  private String getOpCSymbol() {
+    return s_Csymbol[d_op];
+  }
+
+
   /**
    * Return the right-hand expression.
    */
@@ -452,6 +477,7 @@ public class BinaryExpression extends AssertionExpression {
     if (returnIsLong()) {
       isLong = true;
     } else if (  ( (lType == Type.INT ) && (rType == Type.INT ) )
+              || ( (lType == Type.INT)  && (rType == Type.LONG ) )
               || ( (lType == Type.LONG) && (rType == Type.INT ) )
               || ( (lType == Type.LONG) && (rType == Type.LONG) ) ) {
       isLong = true;
@@ -532,21 +558,6 @@ public class BinaryExpression extends AssertionExpression {
   }
 
 
-  /**
-   * Return TRUE if the return value is or should be character; otherwise,
-   * return FALSE.
-   */
-  private boolean returnCharacter(int lType, int rType) {
-    boolean isChar = false;
-    if (returnIsCharacter()) {
-      isChar = true;
-    } else if ((lType == Type.CHAR) && (rType == Type.CHAR) && (d_op != PLUS)) {
-      isChar = true;
-    }
-    return isChar;
-  }
-
-
   /**
    * Return TRUE if the return value is or should be string; otherwise,
    * return FALSE.
@@ -566,21 +577,6 @@ public class BinaryExpression extends AssertionExpression {
   }
 
 
-  /**
-   * Return TRUE if the return value is or should be opaque; otherwise, 
-   * return FALSE.
-   */
-  private boolean returnOpaque(int lType, int rType) {
-    boolean isOpaque = false;
-    if (returnIsOpaque()) {
-      isOpaque = true;
-    } else if ( (lType == Type.OPAQUE) && (rType == Type.OPAQUE) ) {
-      isOpaque = true;
-    }
-    return isOpaque;
-  }
-
-
   /**
    * Return a string containing the standard return type error message.
    *
@@ -749,30 +745,6 @@ public class BinaryExpression extends AssertionExpression {
     } 
   }
 
-
-  /**
-   * Set the return type to either boolean or integer based on the 
-   * return types of the subexpressions.
-   *
-   * @throws  gov.llnl.babel.symbols.AssertionException
-   *                   The exception raised if type not suitable.
-   */
-  private void setBooleanOrIntegerType() throws AssertionException {
-    int lType = d_lhs.getReturnTypeValue();
-    int rType = d_rhs.getReturnTypeValue();
-    if (returnBoolean(lType, rType)) {
-      setReturnToBoolean();
-    } else if (returnInteger(lType, rType)) {
-      setReturnToInteger();
-    } else if (returnLong(lType, rType)) {
-      setReturnToLong();
-    } else {
-      throw new AssertionException(
-                 getReturnExceptionMessage("either boolean, integer, or long"));
-    } 
-  }
-
-
   /**
    * Set the return type to either integer or long based on the return types
    * of the subexpressions.
@@ -838,129 +810,75 @@ public class BinaryExpression extends AssertionExpression {
      * along with the operator.
      */
     switch (d_op) {
-      case AND:
-        setBooleanOrIntegerType();
-        break;
-      case DIVIDE:
-        setNumericType();
-        break;
-      case EQUALS:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setSameTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case EXPON:
-        setNumericType();
-        break;
-      case GREATER_EQUAL:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setNumericTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case GREATER_THAN:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setNumericTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case IF_AND_ONLY_IF:
-        if (returnBoolean(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setReturnToBoolean();
-        } else {
-          throw new AssertionException(getReturnExceptionMessage("boolean"));
-        }
-        break;
-      case IMPLIES:
-        if (returnBoolean(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setReturnToBoolean();
-        } else {
-          throw new AssertionException(getReturnExceptionMessage("boolean"));
-        }
-        break;
-      case LESS_EQUAL:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setNumericTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case LESS_GREATER:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setNumericTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case LESS_THAN:
+    case IF_AND_ONLY_IF:
+    case IMPLIES:
+    case LOGICAL_OR:
+    case LOGICAL_AND:
+    case LOGICAL_XOR:
+      if (returnBoolean(d_lhs.getReturnTypeValue(), 
+                        d_rhs.getReturnTypeValue())) {
+        setReturnToBoolean();
+      } else {
+        throw new AssertionException(getReturnExceptionMessage("boolean"));
+      }
+      break;
+    case BITWISE_AND:
+    case MULTIPLY:
+    case MINUS:
+    case BITWISE_OR:
+    case BITWISE_XOR:
+    case POWER:
+    case DIVIDE:
+      setNumericType();
+      break;
+    case GREATER_EQUAL:
+    case GREATER_THAN:
+    case LESS_EQUAL:
+      if (!returnString(d_lhs.getReturnTypeValue(), 
+                        d_rhs.getReturnTypeValue())) {
         setNumericTypeToBoolean();
-        break;
-      case MINUS:
+      } else {
+        throw new AssertionException(getStringCompareExceptionMessage());
+      }
+      break;
+    case EQUALS:
+    case NOT_EQUAL:
+      if (!returnString(d_lhs.getReturnTypeValue(), 
+                        d_rhs.getReturnTypeValue())) {
+        setSameTypeToBoolean();
+      } else {
+        throw new AssertionException(getStringCompareExceptionMessage());
+      }
+      break;
+    case LESS_THAN:
+      setNumericTypeToBoolean();
+      break;
+    case PLUS:
+      if (returnString(d_lhs.getReturnTypeValue(), 
+                       d_rhs.getReturnTypeValue())) {
+        throw new AssertionException("Unable to validate expression semantics"
+                                     + " for \"" + getExceptionPrefix(ext, m) 
+                                     + "\" since do not currently support string concatentation"
+                                     + ", which is being attempted with the expressions \""
+                                     + d_lhs.toString() + "\" and \"" + d_rhs.toString() 
+                                     + "\".");
+      } else {
         setNumericType();
-        break;
-      case MODULUS:
-        setIntegerType();
-        break;
-      case MULTIPLY:
-        setNumericType();
-        break;
-      case NOT_EQUAL:
-        if (!returnString(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          setSameTypeToBoolean();
-        } else {
-          throw new AssertionException(getStringCompareExceptionMessage());
-        }
-        break;
-      case OR:
-        setBooleanOrIntegerType();
-        break;
-      case PLUS:
-        if (returnString(d_lhs.getReturnTypeValue(), 
-                         d_rhs.getReturnTypeValue())) {
-          throw new AssertionException("Unable to validate expression semantics"
-                    + " for \"" + getExceptionPrefix(ext, m) 
-                    + "\" since do not currently support string concatentation"
-                    + ", which is being attempted with the expressions \""
-                    + d_lhs.toString() + "\" and \"" + d_rhs.toString() 
-                    + "\".");
-        } else {
-          setNumericType();
-        }
-        break;
-      case POWER:
-        setNumericType();
-        break;
-      case REMAINDER:
-        setIntegerType();
-        break;
-      case SHIFT_LEFT:
-        setIntegerType();
-        break;
-      case SHIFT_RIGHT:
-        setIntegerType();
-        break;
-      case XOR:
-        setBooleanOrIntegerType();
-        break;
-      default:
-        throw new AssertionException("Unable to validate expression semantics "
-                  + "for \"" + getExceptionPrefix(ext, m) 
-                  + "\" since invalid or unrecognized operator value \"" + d_op 
-                  + "\" for an expression whose left expression returns " 
-                  + d_lhs.getReturnTypeName() + " values and right returns " 
-                  + d_rhs.getReturnTypeName() + " values.");
+      }
+      break;
+    case MODULUS:
+    case REMAINDER:
+    case SHIFT_LEFT:
+    case SHIFT_RIGHT:
+      setIntegerType();
+      break;
+    default:
+      throw new AssertionException("Unable to validate expression semantics "
+               + "for \"" + getExceptionPrefix(ext, m) 
+               + "\" since invalid or unrecognized operator value \"" + d_op 
+               + "\" for an expression whose left expression returns " 
+               + d_lhs.getReturnTypeName() + " values and right returns " 
+               + d_rhs.getReturnTypeName() + " values.");
     } 
   }
 
@@ -1025,62 +943,44 @@ public class BinaryExpression extends AssertionExpression {
     String lhs  = d_lhs.cExpression(epvVar, startInd);
     String rhs  = d_rhs.cExpression(epvVar, getNextStartInd(startInd));
     switch (d_op) {
-      case AND:
-        if (returnBoolean(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          expr = "( " + lhs + " && " + rhs + " )";
-        } else {
-          expr = "( (" + lhs + ") & (" + rhs + ") )";
-        }
-        break;
-      case DIVIDE:
-      case EQUALS:
-      case GREATER_EQUAL:
-      case GREATER_THAN:
-      case LESS_EQUAL:
-      case LESS_GREATER:
-      case LESS_THAN:
-      case MINUS:
-      case MULTIPLY:
-      case NOT_EQUAL:
-      case PLUS:
-      case SHIFT_LEFT:
-      case SHIFT_RIGHT:
-        expr = "( (" + lhs + ") " + getOpSymbol() + " (" + rhs + ") )";
-        break;
-      case EXPON:
-      case POWER:
-        expr = "pow(" + lhs + ", " + rhs + ")";
-        break;
-      case IF_AND_ONLY_IF:
-        expr = "(  ( " + lhs + " && " + rhs + " ) "
-             + "|| ( (!" + lhs + ") && (!" + rhs + ") )  )";
-        break;
-      case IMPLIES:
-        expr = "( (!" + lhs + ") || " + rhs + " )";
-        break;
-      case MODULUS:
-      case REMAINDER:
-        expr = "( (" + lhs + ") % (" + rhs + ") )";
-        break;
-      case OR:
-        if (returnBoolean(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          expr = "( " + lhs + " || " + rhs + " )";
-        } else {
-          expr = "( (" + lhs + ") | (" + rhs + ") )";
-        }
-        break;
-      case XOR:
-        if (returnBoolean(d_lhs.getReturnTypeValue(), 
-                          d_rhs.getReturnTypeValue())) {
-          expr = "( ( " + lhs + " || " + rhs + " ) "
-               + "&& !( " + lhs + " && " + rhs + " ) )";
-        } else {
-          expr = "( (" + lhs + ") ^ (" + rhs + ") )";
-        }
-        break;
-    } 
+    case BITWISE_OR:
+    case BITWISE_XOR:
+    case BITWISE_AND:
+    case LOGICAL_OR:
+    case LOGICAL_AND:
+    case DIVIDE:
+    case EQUALS:
+    case GREATER_EQUAL:
+    case GREATER_THAN:
+    case LESS_EQUAL:
+    case LESS_THAN:
+    case MINUS:
+    case MULTIPLY:
+    case NOT_EQUAL:
+    case PLUS:
+    case MODULUS:
+    case SHIFT_LEFT:
+    case SHIFT_RIGHT:
+      expr = "( (" + lhs + ") " + getOpCSymbol() + " (" + rhs + ") )";
+      break;
+    case POWER:
+      expr = "pow(" + lhs + ", " + rhs + ")";
+      break;
+    case IF_AND_ONLY_IF:
+      expr = "(  ( " + lhs + " && " + rhs + " ) "
+        + "|| ( (!" + lhs + ") && (!" + rhs + ") )  )";
+      break;
+    case IMPLIES:
+      expr = "( (!" + lhs + ") || (" + rhs + ") )";
+      break;
+    case REMAINDER:
+      expr = "( ( (" + lhs + ") / (" + rhs + ") ) * (" + rhs + ") )";
+      break;
+    case LOGICAL_XOR:
+      expr = "( ( (" + lhs + ") || (" + rhs + ") ) "
+        + "&& !( (" + lhs + ") && (" + rhs + ") ) )";
+      break;
+    }
     return expr;
   }
 
@@ -1092,4 +992,11 @@ public class BinaryExpression extends AssertionExpression {
     String expr = d_lhs + " " + getOpSymbol() + " " + d_rhs;
     return hasParens() ?  "(" + expr + ")"  :  expr;
   }
+
+  /**
+   * Implement the "visitor pattern".
+   */
+  public Object accept(ExprVisitor ev, Object data) {
+    return ev.visitBinaryExpression(this, data);
+  }
 }
