@@ -1,7 +1,7 @@
 //
 // File:        Extendable.java
 // Package:     gov.llnl.babel.symbols
-// Revision:    @(#) $Id: Extendable.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: Extendable.java 6212 2007-11-01 00:07:17Z epperly $
 // Description: sidl base symbol for both classes and interfaces
 //
 // Copyright (c) 2000-2003, The Regents of the University of Calfornia.
@@ -30,6 +30,9 @@
 
 package gov.llnl.babel.symbols;
 
+import gov.llnl.babel.Context;
+import gov.llnl.babel.backend.CodeGenerationException;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.symbols.Assertion;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.Comment;
@@ -43,7 +46,9 @@ import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.ListIterator;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 /**
@@ -65,24 +70,35 @@ import java.util.Set;
  * - ensure original ordering of methods preserved when returning lists;
  */
 public abstract class Extendable extends Symbol {
-  private ArrayList d_all_abstract_methods;
-  private HashSet   d_all_basicarrays;
-  private HashMap   d_all_interfaces;
-  private ArrayList d_all_methods;
-  private HashMap   d_all_methods_long;
-  private HashMap   d_all_methods_short;
-  private HashSet   d_all_references;
-  private ArrayList d_all_nonstatic_methods;
-  private ArrayList d_all_static_methods;
+  private List d_all_abstract_methods;
+  private Set   d_all_basicarrays;
+  private Map   d_all_interfaces;
+  private List d_all_methods;
+  private List d_all_methods_with_nonblocking;
+  private Map   d_all_methods_long;
+  private Map   d_all_methods_short;
+  private Set   d_all_references;
+  private List d_all_nonstatic_methods;
+  private List d_all_static_methods;
 
-  private HashMap   d_local_interfaces;
-  private ArrayList d_local_methods;
-  private HashMap   d_local_methods_long;
-  private HashMap   d_local_methods_short;
-  private ArrayList d_local_nonstatic_methods;
-  private ArrayList d_local_static_methods;
+  private Map   d_local_interfaces;
+  private List d_local_methods;
+  private List d_local_methods_with_nonblocking;
+  private Map   d_local_methods_long;
+  private Map   d_local_methods_short;
+  private List d_local_nonstatic_methods;
+  private List d_local_static_methods;
 
-  private ArrayList d_invariants;		// Local invariants ONLY
+  private Map   d_renamed_parent_methods; //For from clauses, maps from
+                                              //new Method to old Method
+  private Map   d_renamed_method_symbolID;//For from clauses, maps from  
+                                              //old Method to it's enclosing 
+                                              //SymbolID
+  private Map  d_old_name_new_method;  //Maps from the fully qualified
+                                            //name of the old method, to the
+                                            //complete new Method.
+   
+  private List d_invariants;		// Local invariants ONLY
 
 
   /**
@@ -93,8 +109,11 @@ public abstract class Extendable extends Symbol {
    * @param  type     The value of the type of the new object
    * @param  comment  The comment associated with the object
    */
-  public Extendable(SymbolID id, int type, Comment comment) {
-    super(id, type, comment);
+  public Extendable(SymbolID id, 
+                    int type, 
+                    Comment comment, 
+                    Context context) {
+    super(id, type, comment, context);
     d_all_abstract_methods    = new ArrayList();
     d_all_basicarrays         = new HashSet();
     d_all_interfaces          = new HashMap();
@@ -112,6 +131,10 @@ public abstract class Extendable extends Symbol {
     d_local_nonstatic_methods = new ArrayList();
     d_local_static_methods    = new ArrayList();
 
+    d_renamed_parent_methods = new HashMap();
+    d_renamed_method_symbolID = new HashMap();
+    d_old_name_new_method = new HashMap();
+
     d_invariants              = new ArrayList();
 
     d_all_references.add(id);
@@ -126,8 +149,9 @@ public abstract class Extendable extends Symbol {
    * @param  comment  The comment associated with the object
    * @param  m        Object metadata
    */
-  public Extendable(SymbolID id, int type, Comment comment, Metadata m) {
-    super(id, type, comment, m);
+  public Extendable(SymbolID id, int type, Comment comment, 
+                    Metadata m, Context context) {
+    super(id, type, comment, m, context);
     d_all_abstract_methods    = new ArrayList();
     d_all_basicarrays         = new HashSet();
     d_all_interfaces          = new HashMap();
@@ -145,6 +169,10 @@ public abstract class Extendable extends Symbol {
     d_local_nonstatic_methods = new ArrayList();
     d_local_static_methods    = new ArrayList();
 
+    d_renamed_parent_methods = new HashMap();
+    d_renamed_method_symbolID = new HashMap();
+    d_old_name_new_method = new HashMap();
+
     d_invariants              = new ArrayList();
 
     d_all_references.add(id);
@@ -173,11 +201,71 @@ public abstract class Extendable extends Symbol {
    * @param  method  The method to be added to this object
    */
   public void addMethod(Method method) {
+    checkFrozen();
     addToMethodLists(method, false);
     d_all_references.addAll(method.getSymbolReferences());
     d_all_basicarrays.addAll(method.getBasicArrays());
   }
 
+  /** 
+   * For From clauses:Add the specified "new" method to the normal method lists, and the
+   * "old" method to the d_renamed_parent_methods HashMap.  newM and oldM
+   * are assumed to have the same signatures and only differ by name.
+   */
+  public void addRenamedMethod(Method newM, Method oldM, SymbolID old_sid ) {
+    checkFrozen();
+    removeFromMethodLists(oldM);  //Remove the renamed method if it exists.
+    addMethod(newM);
+    d_renamed_parent_methods.put(newM, oldM);
+    d_renamed_method_symbolID.put(oldM, old_sid);
+    d_old_name_new_method.put(old_sid.getFullName()+"."+oldM.getLongMethodName(), newM);
+  }
+  
+
+  /** 
+   * For From clauses:Find the Parent Method Object for a renamed method.
+   */
+  public Method getRenamedMethod(Method newM) {
+    return (Method)d_renamed_parent_methods.get(newM);
+  }
+
+  /** 
+   * For From clauses:Get all the orginal Method Objects of all the renamed methods
+   */
+  public Set getRenamedMethods() {
+    return d_renamed_method_symbolID.keySet();
+  }
+
+  /** 
+   * For From clauses:Get all the new method objects of all the renamed methods
+   */
+  public Set getNewMethods() {
+    return d_renamed_parent_methods.keySet();
+  }
+
+
+  /** 
+   * For From clauses:Check a parent has a different name for this method.
+   */
+  public boolean methodWasRenamed(Method newM) {
+    return d_renamed_parent_methods.get(newM) != null;
+  }
+
+  /** 
+   * For From clauses: Get the old old method's enclosing symbolID. 
+   */
+  public SymbolID getRenamedMethodSymbolID(Method oldM) {
+    return (SymbolID) d_renamed_method_symbolID.get(oldM);
+  }
+
+  /** 
+   * For From clauses: Take the FQN of the renamed method and return the new
+   * Method.
+   */
+  public Method getNewMethod(String oldM_name) {
+    return (Method) d_old_name_new_method.get(oldM_name);
+  }
+
   /**
    * Return the number of methods associated with this extendable.
    *
@@ -195,10 +283,51 @@ public abstract class Extendable extends Symbol {
    * @param  all  If TRUE, then return local and parent methods; otherwise,
    *              return only local methods.
    */
-  public Collection getMethods(boolean all) {
+  public List getMethods(boolean all) {
     return all ? d_all_methods : d_local_methods;
   }
 
+  /**
+   * Return the methods in this interface as a <code>Collection</code>.
+   * Each element in the collection is of type <code>Method</code>.
+   * This list includes methods NONBLOCKING_SEND and NONBLOCKING_RECV
+   * methods spawned in response to NONBLOCKING methods.
+   *
+   * @param  all  If TRUE, then return local and parent methods; otherwise,
+   *              return only local methods.
+   */
+  public List getMethodsWithNonblocking(boolean all) throws CodeGenerationException {
+    if(d_all_methods_with_nonblocking == null) {
+      d_all_methods_with_nonblocking = new ArrayList();
+      d_local_methods_with_nonblocking = new ArrayList();
+      for(Iterator allMethods = d_all_nonstatic_methods.iterator(); allMethods.hasNext();) {
+        Method method = (Method) allMethods.next();
+
+        findExistingEntry(d_all_methods_with_nonblocking, method, true);
+        if(method.getCommunicationModifier() == Method.NONBLOCKING) {
+          Method send = method.spawnNonblockingSend();
+          Method recv = method.spawnNonblockingRecv();
+          findExistingEntry(d_all_methods_with_nonblocking, send, true);
+          findExistingEntry(d_all_methods_with_nonblocking, recv, true);
+        }
+      }
+      for(Iterator localMethods = d_local_nonstatic_methods.iterator(); localMethods.hasNext();) {
+        Method method = (Method) localMethods.next();
+        
+        findExistingEntry(d_local_methods_with_nonblocking, method, true);
+        if(method.getCommunicationModifier() == Method.NONBLOCKING) {
+          Method send = method.spawnNonblockingSend();
+          Method recv = method.spawnNonblockingRecv();
+          findExistingEntry(d_local_methods_with_nonblocking, send, true);
+          findExistingEntry(d_local_methods_with_nonblocking, recv, true);
+        }
+      }
+    }
+    return all ? d_all_methods_with_nonblocking 
+      : d_local_methods_with_nonblocking;
+  }
+
+
   /**
    * Return the <code>Collection</code> non-static methods in this interface.
    * Each element in the collection is of type <code>Method</code>.
@@ -206,8 +335,9 @@ public abstract class Extendable extends Symbol {
    * @param  all  If TRUE, then return local and parent non-static methods; 
    *              otherwise, return only local non-static methods.
    */
-  public Collection getNonstaticMethods(boolean all) {
-    return all ? d_all_nonstatic_methods : d_local_nonstatic_methods;
+  public List getNonstaticMethods(boolean all) {
+    return all ? d_all_nonstatic_methods
+      : d_local_nonstatic_methods;
   }
 
   /**
@@ -217,8 +347,9 @@ public abstract class Extendable extends Symbol {
    * @param  all  If TRUE, then return local and parent static methods; 
    *              otherwise, return only local static methods.
    */
-  public Collection getStaticMethods(boolean all) {
-    return all ? d_all_static_methods : d_local_static_methods;
+  public List getStaticMethods(boolean all) {
+    return all ? d_all_static_methods
+      : d_local_static_methods;
   }
 
   /**
@@ -226,7 +357,7 @@ public abstract class Extendable extends Symbol {
    * includes all parent classes and interfaces.  Each element in the
    * collection is of type <code>Method</code>.
    */
-  public Collection getAbstractMethods() {
+  public List getAbstractMethods() {
     return d_all_abstract_methods;
   }
 
@@ -235,8 +366,8 @@ public abstract class Extendable extends Symbol {
    *  This is a special case for Java abstract classes, although maybe it
    *  will be useful elsewhere.
    */
-  public Collection getAbstractAndLocalMethods() {
-    ArrayList ret = (ArrayList) d_local_methods.clone();
+  public List getAbstractAndLocalMethods() {
+    LinkedList ret = new LinkedList(d_local_methods);
     //Add all Abstracts that don't collide with locals
     for (Iterator abs = d_all_abstract_methods.iterator(); abs.hasNext(); ) {
       Method absM = (Method) abs.next();
@@ -292,8 +423,9 @@ public abstract class Extendable extends Symbol {
    *               be searched.
    */
   public Collection lookupMethodByShortName(String name, boolean all) {
-    return (Collection) (all ? d_all_methods_short.get(name)
-                         : d_local_methods_short.get(name));
+    return protectCollection((Collection)
+                             (all ? d_all_methods_short.get(name)
+                              : d_local_methods_short.get(name)));
   }
 
   /**
@@ -363,6 +495,15 @@ public abstract class Extendable extends Symbol {
    */
   abstract public void addParentInterface(Interface parent);
 
+  /**
+   * Return true if this extendable inherts from interfaces. 
+   * BaseInterface does not count for this test.
+   *
+   */
+  public boolean hasParentInterfaces() {
+    return (d_all_interfaces.size() > 1);
+  }
+
   /**
    * Return the parent interfaces in a <code>Collection</code>.  Each
    * member of the collection is an <code>Interface</code>.  
@@ -371,48 +512,57 @@ public abstract class Extendable extends Symbol {
    *              only direct parents are to be returned.
    */
   public Collection getParentInterfaces(boolean all) {
-    return all ? d_all_interfaces.values() : d_local_interfaces.values();
-  }
-
-  /**
-   * Return TRUE if the specified extendable can be found in the given 
-   * list; otherwise, return FALSE.
-   *
-   * @param  list  The list being searched.
-   * @param  ext   The extendable that is to be located.
-   */
-  private boolean findExistingEntry(ArrayList list, Extendable ext)
-  {
-    boolean    found = false;
-    Extendable item  = null;
-    for (ListIterator iter = list.listIterator(); iter.hasNext() && !found; )
-    {
-      item = (Extendable) iter.next();
-      if (item == ext) {
-        found = true;
-      }
-    }
-    return found;
+    return all 
+      ? d_all_interfaces.values() 
+      : d_local_interfaces.values();
   }
 
   /**
    * Return the parent interfaces and/or classes in hierarchical order
-   * from top-most down to direct parents in an <code>ArrayList</code>.  
+   * from top-most down to direct parents in an <code>List</code>.  
    * Each member of the list is an <code>Extendable</code> and there
    * are NO duplicates.
+   * Does a DFS to return ancestors in "reverse" topolgical order.
+   * by Jim Leek
    */
-  public ArrayList getAllParentsInOrder() {
-    ArrayList  ancestors = new ArrayList();
-    Collection parents   = getParents(false);
-    Iterator   i         = parents.iterator();
-    while (i.hasNext()) {
-      Extendable parent  = (Extendable)i.next();
-      if (!findExistingEntry(ancestors, parent)) {
-        ancestors.addAll(parent.getAllParentsInOrder());
-        ancestors.add(parent);
+  public List getAllParentsInOrder() throws CodeGenerationException {
+    LinkedList ancestors = new LinkedList();
+    HashSet visited = new HashSet();
+    visited.add(this);  //We actually don't want this in the ancestor list.
+
+    getAllParentsInOrderHelper(this, ancestors, visited);
+    return ancestors;
+  }
+
+  /**
+   * getAllParentsInOrderHelper is a recursive DFS call.  It is started
+   * by getAllParentsInOrder.  It returns the post order list of ancestors.
+   * (a "reverse" topological sort)
+   */
+  private void getAllParentsInOrderHelper(Extendable current, List ancestors, Set visited) 
+    throws CodeGenerationException {
+
+    Collection parents = Utilities.sort(current.getParents(false));
+
+    for(Iterator pIter = parents.iterator(); pIter.hasNext();) {
+      Extendable parent = (Extendable) pIter.next();
+      if(!visited.contains(parent)) {
+        getAllParentsInOrderHelper(parent, ancestors, visited);
+      }
+    }
+    
+    //Keep "this" out of the ancestor list
+    if(!visited.contains(current)) {
+      //This node sould never have been visited before, assert that.
+      if(!ancestors.contains(current)) {
+        ancestors.add(current); 
+        visited.add(current);  //Finished visiting this node
+      } else {
+        throw new CodeGenerationException 
+          ("Extendable: " + getFullName() + " Had ancestor " + 
+           current.getFullName() + " appear in the ancestor list twice.");
       }
     }
-    return ancestors;
   }
 
   /**
@@ -424,7 +574,7 @@ public abstract class Extendable extends Symbol {
    *              direct parents.  
    */
   public Collection getParents(boolean all) {
-    return getParentInterfaces(all);
+    return protectCollection(getParentInterfaces(all));
   }
 
   /**
@@ -464,6 +614,19 @@ public abstract class Extendable extends Symbol {
       : d_local_interfaces.containsKey(id);
   }
 
+  /**
+   * Return Interface if this class has the specified parent interface; otherwise,
+   * return FALSE.  
+   *
+   * @param  id   The parent interface to be located
+   * @param  all  Indicator of whether all parents (TRUE) or only direct
+   *              parents (FALSE) are to be searched
+   */
+  public Interface getParentInterface(SymbolID id, boolean all) {
+    return (Interface) (all ? d_all_interfaces.get(id)
+                        : d_local_interfaces.get(id));
+  }
+
   /**
    * Return the <code>Set</code> of symbol references for this object, each
    * element of which is in <code>SymbolID</code> form.  These are
@@ -499,6 +662,7 @@ public abstract class Extendable extends Symbol {
    *              added
    */
   protected void addParentData(Extendable ext) {
+    checkFrozen();
     /*
      * Add all parent methods to the list of all methods in this
      * class.  Do not override an existing method in the methods
@@ -509,7 +673,10 @@ public abstract class Extendable extends Symbol {
      */
     for (Iterator m = ext.getMethods(true).iterator(); m.hasNext(); ) {
       Method method = (Method) m.next();
-      addToMethodLists(method, true);
+      //Only add the method if it hasn't already been renamed.
+      if(!d_renamed_method_symbolID.containsKey(method)) {
+        addToMethodLists(method, true);
+      }
     }
 
     /*
@@ -538,7 +705,7 @@ public abstract class Extendable extends Symbol {
    *  This is necessary for F90 method overloading
    */
   public Collection getOverloadedMethodsByName(String shortName) {
-    ArrayList ret = new ArrayList();
+    LinkedList ret = new LinkedList();
     for(Iterator i = d_all_methods.iterator(); i.hasNext();) {
       Method m = (Method) i.next();
       if(m.getShortMethodName().compareTo(shortName) == 0) {
@@ -563,14 +730,14 @@ public abstract class Extendable extends Symbol {
    * @param  replaceIt  Indicator of whether or not the existing method is
    *                    to be replaced
    */
-  private void findExistingEntry(ArrayList list, Method method, 
+  private void findExistingEntry(List list, Method method, 
                                  boolean replaceIt)
   {
     String  name  = method.getLongMethodName();
     boolean found = false;
     Method  item  = null;
 
-    for (ListIterator iter = list.listIterator(); iter.hasNext() && !found; )
+    for (Iterator iter = list.iterator(); iter.hasNext() && !found; )
       {
         item = (Method) iter.next();
         if (item.getLongMethodName().equals(name)) {
@@ -589,11 +756,12 @@ public abstract class Extendable extends Symbol {
    * @param  list    The short-name method list
    * @param  method  The method to be added to the list
    */
-  private void addToShortMethodList(HashMap list, Method method) {
+  private void addToShortMethodList(Map list, Method method) {
+    checkFrozen();
     String name = method.getShortMethodName();
 
     if (list.containsKey(name)) {
-      findExistingEntry((ArrayList) list.get(name), method, true);
+      findExistingEntry((List) list.get(name), method, true);
     } else {
       ArrayList sublist = new ArrayList();
       sublist.add(method);
@@ -601,6 +769,21 @@ public abstract class Extendable extends Symbol {
     }
   }
 
+
+  /**
+   * Remove the given method from the specified short-name method list.
+   *
+   * @param  list    The short-name method list
+   * @param  method  The method to be added to the list
+   */
+  private void removeFromShortMethodList(Map list, Method method) {
+    String name = method.getShortMethodName();
+
+    if (list.containsKey(name)) {
+      findExistingEntry((List) list.get(name), method, false);
+    } 
+  }
+
   /**
    * Add the specified method to all and, if not isParent, local method lists
    * provided the method does not exist or is not abstract.  Note the latter 
@@ -622,6 +805,7 @@ public abstract class Extendable extends Symbol {
       findExistingEntry(d_all_methods, method, true);
       d_all_methods_long.put(long_name, method);
       addToShortMethodList(d_all_methods_short, method);
+
       if (is_static) {
         findExistingEntry(d_all_static_methods, method, true);
       } else {
@@ -650,6 +834,47 @@ public abstract class Extendable extends Symbol {
     }
   }
 
+  /**
+   * Remove the specified method from all method lists.  This is used in
+   * From clauses
+   *
+   * @param  method    The method to be added
+   */
+  private void removeFromMethodLists(Method method) {
+    String long_name = method.getLongMethodName();
+
+    if (!(hasMethodByLongName(long_name, true) && method.isAbstract())) {
+      boolean is_static  = method.isStatic();
+
+      /* 
+       * First process the lists of local and parent methods (i.e., the
+       * "all" lists).
+       */
+      findExistingEntry(d_all_methods, method, false);
+      d_all_methods_long.remove(long_name);
+      removeFromShortMethodList(d_all_methods_short, method);
+      if (is_static) {
+        findExistingEntry(d_all_static_methods, method, false);
+      } else {
+        findExistingEntry(d_all_nonstatic_methods, method, false);
+      }
+      findExistingEntry(d_all_abstract_methods, method, false);
+
+      /* 
+       * Now process the lists of local methods
+       */
+      findExistingEntry(d_local_methods, method, false);
+      d_local_methods_long.remove(long_name);
+      removeFromShortMethodList(d_local_methods_short, method);
+      if (is_static) {
+        findExistingEntry(d_local_static_methods, method, false);
+      } else {
+        findExistingEntry(d_local_nonstatic_methods, method, false);
+        
+      }
+    }
+  }
+
   /**
    * Add the invariant to this object.  
    *
@@ -662,6 +887,7 @@ public abstract class Extendable extends Symbol {
    *                     The exception raised if the assertion cannot be added.
    */
   public void addInvariant(Assertion assertion) throws AssertionException {
+    checkFrozen();
     if (!assertion.isInvariant()) {
       throw new AssertionException("Extendable: " + getFullName(), "Cannot add "
                 + "a(n) \"" + assertion.getTypeName() + "\" assertion to the "
@@ -677,7 +903,7 @@ public abstract class Extendable extends Symbol {
    * @param   all   If TRUE then check inherited and local; otherwise, check
    *                only local invariants.
    */
-  public boolean hasInvariants(boolean all) {
+  public boolean hasInvariants(boolean all)  throws CodeGenerationException {
     boolean has = false;
     if (getNumberInvariants(false) > 0) {
       has = true;
@@ -693,26 +919,26 @@ public abstract class Extendable extends Symbol {
    * @param   all   If TRUE then return the number of inherited and local
    *                invariants; otherwise, return only the number of local ones.
    */
-  public int getNumberInvariants(boolean all) {
+  public int getNumberInvariants(boolean all) throws CodeGenerationException {
     return all ? d_invariants.size() : getAllInvariants().size();
   }
 
   /**
-   * Return the invariants in this interface as an <code>ArrayList</code>,
+   * Return the invariants in this interface as an <code>List</code>,
    * each element of which is in <code>Assertion</code> form.
    */
-  public ArrayList getInvariants() {
+  public List getInvariants() {
     return d_invariants;
   }
 
   /**
    * Return the list of all invariants associated with this object as an 
-   * <code>ArrayList</code>, each element of which is in <code>Assertion</code>
+   * <code>List</code>, each element of which is in <code>Assertion</code>
    * form.
    */
-  public ArrayList getAllInvariants() {
-    ArrayList  list    = new ArrayList();
-    ArrayList  parents = getAllParentsInOrder();
+  public List getAllInvariants() throws CodeGenerationException {
+    LinkedList  list    = new LinkedList();
+    List  parents = getAllParentsInOrder();
     Iterator i = parents.iterator();
     while (i.hasNext()) {
       Extendable parent = (Extendable)i.next();
@@ -733,9 +959,9 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validateInvariants(boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
-    ArrayList list = (all ? getAllInvariants() : d_invariants);
+    List list = (all ? getAllInvariants() : d_invariants);
     Iterator i = list.iterator();
     while (i.hasNext()) {
       Assertion inv = (Assertion)i.next();
@@ -746,7 +972,7 @@ public abstract class Extendable extends Symbol {
   /**
    * Return TRUE if the extendable has any assertions; otherwise, return FALSE.
    */
-  public boolean hasAssertions() {
+  public boolean hasAssertions()  throws CodeGenerationException {
     boolean has = false;
     if (hasInvariants(true)) {
       has = true;
@@ -773,7 +999,7 @@ public abstract class Extendable extends Symbol {
    * @param  longname  The long name of the method the number of whose 
    *                   assertions is to be returned.
    */
-  public int getNumberAssertions(String longname) {
+  public int getNumberAssertions(String longname)  throws CodeGenerationException {
     return getNumberPreconditions(longname) 
          + 2 * getNumberInvariants(true)
          + getNumberPostconditions(longname);
@@ -787,13 +1013,14 @@ public abstract class Extendable extends Symbol {
    * @param  longname  The long name of the method the number of whose 
    *                   preconditions is to be returned.
    */
-  public int getNumberPreconditions(String longname) {
+  public int getNumberPreconditions(String longname) 
+    throws CodeGenerationException {
     return getAllPreconditions(longname).size();
   }
 
   /**
    * Return all preconditions associated with the method in this object,
-   * if there is one, in an <code>ArrayList</code> with each entry being
+   * if there is one, in a <code>List</code> with each entry being
    * in <code>Assertion</code> form.  Will return an empty list if no such 
    * method is in this object's hierarchy.
    *
@@ -804,10 +1031,11 @@ public abstract class Extendable extends Symbol {
    * @param  longname  The long name of the method whose preconditions are to 
    *                   be returnede
    */
-  public ArrayList getAllPreconditions(String longname) {
-    ArrayList list    = new ArrayList();
-    ArrayList parents = getAllParentsInOrder();
-    Iterator  i       = parents.iterator();
+  public List getAllPreconditions(String longname) 
+    throws CodeGenerationException {
+    LinkedList list    = new LinkedList();
+    List parents       = getAllParentsInOrder();
+    Iterator  i        = parents.iterator();
     while (i.hasNext()) {
       Extendable parent = (Extendable)i.next();
       list.addAll(parent.getAllPreconditions(longname));
@@ -834,10 +1062,10 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validatePreconditions(Method meth, boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     if (meth != null) {
-      ArrayList list;
+      List list;
       if (all) {
         String longname = meth.getLongMethodName();
         list = getAllPreconditions(longname);
@@ -870,7 +1098,7 @@ public abstract class Extendable extends Symbol {
    *                    The exception raised if errors are detected.
    */
   public void validatePreconditions(String longname, boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     validatePreconditions(lookupMethodByLongName(longname, true), all, skip);
   }
@@ -886,7 +1114,7 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validatePreconditions(boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     Iterator im;
     if (all) {
@@ -908,13 +1136,13 @@ public abstract class Extendable extends Symbol {
    * @param  longname  The long name of the method the number of whose 
    *                   postconditions is to be returned.
    */
-  public int getNumberPostconditions(String longname) {
+  public int getNumberPostconditions(String longname) throws CodeGenerationException {
     return getAllPostconditions(longname).size();
   }
 
   /**
    * Return all postconditions associated with the method in this object,
-   * if there is one, in an <code>ArrayList</code> with each element in
+   * if there is one, in an <code>List</code> with each element in
    * <code>Assertion</code> form.  Will return an empty list if no such 
    * method in this object's hierarchy.
    *
@@ -925,9 +1153,10 @@ public abstract class Extendable extends Symbol {
    * @param  longname  The long name of the method whose postconditions are to 
    *                   be returned.
    */
-  public ArrayList getAllPostconditions(String longname) {
-    ArrayList  list    = new ArrayList();
-    ArrayList  parents = getAllParentsInOrder();
+  public List getAllPostconditions(String longname) 
+    throws CodeGenerationException {
+    LinkedList  list    = new LinkedList();
+    List  parents = getAllParentsInOrder();
     Iterator   i       = parents.iterator();
     while (i.hasNext()) {
       Extendable parent = (Extendable)i.next();
@@ -955,10 +1184,10 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validatePostconditions(Method meth, boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     if (meth != null) {
-      ArrayList list;
+      List list;
       if (all) {
         String longname = meth.getLongMethodName();
         list = getAllPostconditions(longname);
@@ -977,6 +1206,28 @@ public abstract class Extendable extends Symbol {
     }
   }
 
+  /**
+   * This function returns the set of symbols that this Extendable is
+   * dependent on.  It should include any classes or interfaces that are
+   * passed by any method in the Extendable, but no enums, and no self.
+   */
+  public Set getObjectDependencies() throws CodeGenerationException {
+    HashSet ret = new HashSet();
+    if(d_all_references.size() > 1) {
+      for (Iterator i = d_all_references.iterator(); i.hasNext(); ) {
+        SymbolID l_id = (SymbolID) i.next();
+        Symbol l_sym  = (Symbol) Utilities.lookupSymbol(d_context, l_id);
+        //if(l_id != this.getSymbolID() && l_sym.getSymbolType() != Symbol.ENUM) {
+        if((l_sym.getSymbolType() != Symbol.ENUM) &&
+           (l_sym.getSymbolType() != Symbol.STRUCT)) {
+          ret.add(l_id);
+        }
+      }
+    }
+    return ret;
+  }
+
+
   /**
    * Validate the postconditions associated with this object and the 
    * specified method.  
@@ -992,7 +1243,7 @@ public abstract class Extendable extends Symbol {
    *                    The exception raised if errors are detected.
    */
   public void validatePostconditions(String longname, boolean all, boolean skip)
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     validatePostconditions(lookupMethodByLongName(longname, true), all, skip);
   }
@@ -1008,7 +1259,7 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validatePostconditions(boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     Iterator im;
     if (all) {
@@ -1036,7 +1287,7 @@ public abstract class Extendable extends Symbol {
    *                The exception raised if errors are detected.
    */
   public void validateAssertions(boolean all, boolean skip) 
-       throws AssertionException 
+    throws AssertionException, CodeGenerationException
   {
     validateInvariants(all, skip);
     Iterator im;
@@ -1056,9 +1307,9 @@ public abstract class Extendable extends Symbol {
    * Returns true if the specified reserved method call is present in any
    * of the assertions; otherwise, returns false.
    */
-  public boolean hasReservedMethodAssertion(int type) {
+  public boolean hasReservedMethodAssertion(int type) throws CodeGenerationException {
     boolean   hasIt = false;
-    ArrayList list  = getAllInvariants();
+    List list  = getAllInvariants();
     Iterator i = list.iterator();
     while (i.hasNext() && !hasIt) {
       Assertion inv = (Assertion)i.next();
@@ -1078,4 +1329,53 @@ public abstract class Extendable extends Symbol {
     }
     return hasIt;
   }
+
+
+  public void freeze()
+  {
+    if (!d_frozen) {
+      super.freeze();
+      Iterator i = d_all_methods.iterator();
+      while (i.hasNext()) {
+        Method m = (Method)i.next();
+        m.freeze();
+      }
+      i = d_all_references.iterator();
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        id.freeze();
+      }
+      i = d_all_basicarrays.iterator();
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        id.freeze();
+      }
+      i = d_renamed_method_symbolID.values().iterator();
+      while (i.hasNext()) {
+        SymbolID id = (SymbolID)i.next();
+        id.freeze();
+      }
+      d_all_abstract_methods = protectList(d_all_abstract_methods);
+      d_all_basicarrays = protectSet(d_all_basicarrays);
+      d_all_interfaces = protectMap(d_all_interfaces);
+      d_all_methods = protectList(d_all_methods);
+      d_all_methods_with_nonblocking = protectList(d_all_methods_with_nonblocking);
+      d_all_methods_long = protectMap(d_all_methods_long);
+      d_all_methods_short = protectMap(d_all_methods_short);
+      d_all_references = protectSet(d_all_references);
+      d_all_nonstatic_methods = protectList(d_all_nonstatic_methods);
+      d_all_static_methods = protectList(d_all_static_methods);
+      d_local_interfaces = protectMap(d_local_interfaces);
+      d_local_methods = protectList(d_local_methods);
+      d_local_methods_with_nonblocking = protectList(d_local_methods_with_nonblocking);
+      d_local_methods_long = protectMap(d_local_methods_long);
+      d_local_methods_short = protectMap(d_local_methods_short);
+      d_local_nonstatic_methods = protectList(d_local_nonstatic_methods);
+      d_local_static_methods = protectList(d_local_static_methods);
+      d_renamed_parent_methods = protectMap(d_renamed_parent_methods);
+      d_renamed_method_symbolID = protectMap(d_renamed_method_symbolID);
+      d_old_name_new_method = protectMap(d_old_name_new_method);
+      d_invariants = protectList(d_invariants);
+    }
+  }
 }
