@@ -35,16 +35,17 @@ package gov.llnl.babel.symbols;
 // (or worse, assume) the naming scheme for IOR method names.  Obviously 
 // this needs more thought for a better dependency situation.
 //
+import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.ExprVisitor;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Type;
 import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Iterator;
+import java.util.List;
 import java.util.ListIterator;
 
 
@@ -128,18 +129,18 @@ public class MethodCall extends AssertionExpression {
   public static final int  METHOD_ARRAY_ANY        = 2;
   public static final int  METHOD_ARRAY_COUNT      = 3;
   public static final int  METHOD_ARRAY_DIMEN      = 4;
-  public static final int  METHOD_ARRAY_DOWN       = 5;
-  public static final int  METHOD_ARRAY_IRANGE     = 6;
-  public static final int  METHOD_ARRAY_LOWER      = 7;
-  public static final int  METHOD_ARRAY_MAX        = 8;
-  public static final int  METHOD_ARRAY_MIN        = 9;
-  public static final int  METHOD_ARRAY_NEAR_EQUAL = 10;
-  public static final int  METHOD_ARRAY_NONE       = 11;
-  public static final int  METHOD_ARRAY_RANGE      = 12;
-  public static final int  METHOD_ARRAY_SIZE       = 13;
-  public static final int  METHOD_ARRAY_STRIDE     = 14;
-  public static final int  METHOD_ARRAY_SUM        = 15;
-  public static final int  METHOD_ARRAY_UP         = 16;
+  public static final int  METHOD_ARRAY_IRANGE     = 5;
+  public static final int  METHOD_ARRAY_LOWER      = 6;
+  public static final int  METHOD_ARRAY_MAX        = 7;
+  public static final int  METHOD_ARRAY_MIN        = 8;
+  public static final int  METHOD_ARRAY_NEAR_EQUAL = 9;
+  public static final int  METHOD_ARRAY_NON_DECR   = 10;
+  public static final int  METHOD_ARRAY_NON_INCR   = 11;
+  public static final int  METHOD_ARRAY_NONE       = 12;
+  public static final int  METHOD_ARRAY_RANGE      = 13;
+  public static final int  METHOD_ARRAY_SIZE       = 14;
+  public static final int  METHOD_ARRAY_STRIDE     = 15;
+  public static final int  METHOD_ARRAY_SUM        = 16;
   public static final int  METHOD_ARRAY_UPPER      = 17;
   public static final int  METHOD_IRANGE           = 18;
   public static final int  METHOD_NEAR_EQUAL       = 19;
@@ -155,18 +156,18 @@ public class MethodCall extends AssertionExpression {
     "any",
     "count",
     "dimen",
-    "down",
     "irange",
     "lower",
     "max",
     "min",
     "nearequal",
+    "nondecr",
+    "nonincr",
     "none",
     "range",
     "size",
     "stride",
     "sum",
-    "up",
     "upper",
     "irange",
     "nearequal",
@@ -179,18 +180,18 @@ public class MethodCall extends AssertionExpression {
     "SIDL_ARRAY_ANY",
     "SIDL_ARRAY_COUNT",
     "SIDL_ARRAY_DIMEN",
-    "SIDL_ARRAY_DOWN",
     "SIDL_ARRAY_IRANGE",
     "SIDL_ARRAY_LOWER",
     "SIDL_ARRAY_MAX",
     "SIDL_ARRAY_MIN",
     "SIDL_ARRAY_NEAR_EQUAL",
+    "SIDL_ARRAY_NON_DECR",
+    "SIDL_ARRAY_NON_INCR",
     "SIDL_ARRAY_NONE",
     "SIDL_ARRAY_RANGE",
     "SIDL_ARRAY_SIZE",
     "SIDL_ARRAY_STRIDE",
     "SIDL_ARRAY_SUM",
-    "SIDL_ARRAY_UP",
     "SIDL_ARRAY_UPPER",
     "SIDL_IRANGE",
     "SIDL_NEAR_EQUAL",
@@ -239,8 +240,11 @@ public class MethodCall extends AssertionExpression {
    * @throws  gov.llnl.babel.symbols.AssertionException
    *                  The exception raised if error during any validation.
    */
-  public MethodCall(String name) throws AssertionException {
-    super(false);
+  public MethodCall(String name,
+                    Context context)
+    throws AssertionException 
+  {
+    super(false, context);
     if ( (name == null) || (name.equals("")) ) {
       throw new AssertionException("Cannot instantiate a method call without "
                 + "the name of the method to be called.");
@@ -515,7 +519,7 @@ public class MethodCall extends AssertionExpression {
   public ArrayList getArgumentReturnTypes(Method meth) {
     ArrayList list  = null;
     if (meth != null) {
-      ArrayList args = meth.getArgumentList();
+      List args = meth.getArgumentList();
 
       if ( (args != null) && (!args.isEmpty()) ) {
         list = new ArrayList();
@@ -841,8 +845,8 @@ public class MethodCall extends AssertionExpression {
         }
         setReturnType(Type.INT);
         break;
-      case METHOD_ARRAY_DOWN: 
-      case METHOD_ARRAY_UP: 
+      case METHOD_ARRAY_NON_DECR: 
+      case METHOD_ARRAY_NON_INCR: 
         rList = new ArrayList();
         rList.add(new Integer(Type.ARRAY));    /* array var */
         if (!hasCompatibleArgumentTypes(rList)) {
@@ -1050,8 +1054,8 @@ public class MethodCall extends AssertionExpression {
         call.append(", " + ARRAY_INTEGER_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_INTEGER_IND] + "])");
         break;
-      case METHOD_ARRAY_DOWN: 
-      case METHOD_ARRAY_UP: 
+      case METHOD_ARRAY_NON_DECR: 
+      case METHOD_ARRAY_NON_INCR: 
         call.append(s_builtin_macro[d_reserved_method] + "(");
         call.append(getCArgList(epvVar, null, true, false, true));
         call.append(", " + ARRAY_DOUBLE_RESULT_VAR);
@@ -1109,12 +1113,12 @@ public class MethodCall extends AssertionExpression {
     if (type == MACRO_RETURN_TYPE[MACRO_RETURNS_BOOLEAN_IND]) {
       if (  (d_reserved_method == METHOD_ARRAY_ALL)
          || (d_reserved_method == METHOD_ARRAY_ANY)
-         || (d_reserved_method == METHOD_ARRAY_DOWN)
          || (d_reserved_method == METHOD_ARRAY_IRANGE)
          || (d_reserved_method == METHOD_ARRAY_NEAR_EQUAL)
+         || (d_reserved_method == METHOD_ARRAY_NON_DECR)
+         || (d_reserved_method == METHOD_ARRAY_NON_INCR)
          || (d_reserved_method == METHOD_ARRAY_NONE)
-         || (d_reserved_method == METHOD_ARRAY_RANGE)
-         || (d_reserved_method == METHOD_ARRAY_UP) ) {
+         || (d_reserved_method == METHOD_ARRAY_RANGE) ) {
         num = 1;
       }
     } else if (type == MACRO_RETURN_TYPE[MACRO_RETURNS_DOUBLE_IND]) {
@@ -1134,6 +1138,7 @@ public class MethodCall extends AssertionExpression {
   }
 
 
+
   /**
    * Return the C version of the expression.
    *
@@ -1146,12 +1151,12 @@ public class MethodCall extends AssertionExpression {
     switch (d_reserved_method) {
       case METHOD_ARRAY_ALL: 
       case METHOD_ARRAY_ANY: 
-      case METHOD_ARRAY_DOWN: 
       case METHOD_ARRAY_IRANGE: 
       case METHOD_ARRAY_NEAR_EQUAL: 
+      case METHOD_ARRAY_NON_INCR: 
+      case METHOD_ARRAY_NON_DECR: 
       case METHOD_ARRAY_NONE: 
       case METHOD_ARRAY_RANGE: 
-      case METHOD_ARRAY_UP: 
         call.append(ARRAY_BOOLEAN_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_BOOLEAN_IND] + "]");
         break;
@@ -1190,4 +1195,8 @@ public class MethodCall extends AssertionExpression {
 
     return call.toString();
   }
+
+  public Object accept(ExprVisitor ev, Object data) {
+    return ev.visitMethodCall(this, data);
+  }
 }
