@@ -1,7 +1,7 @@
 //
 // File:        Version.java
 // Package:     gov.llnl.babel.symbols
-// Revision:    @(#) $Id: Version.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: Version.java 5246 2006-02-26 23:21:24Z kumfert $
 // Description: symbol version description of the form "V1.V2...Vn"
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -65,13 +65,8 @@ public class Version {
       while (tokens.hasMoreTokens()) {
          Integer part = null;
          
-         try {
-            part = new Integer(tokens.nextToken());
-         } catch (NumberFormatException ex) {
-            throw new NumberFormatException(
-               "Invalid integer in version \"" + ex.getMessage() + "\"");
-         }
-         
+         part = new Integer(tokens.nextToken());
+
          if (part.intValue() < 0) {
             throw new NumberFormatException(
                "Negative integer in version \"" + version + "\"");
