@@ -1,7 +1,7 @@
 //
 // File:        URLUtilities.java
 // Package:     gov.llnl.babel.url
-// Revision:    @(#) $Id: URLUtilities.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: URLUtilities.java 4744 2005-07-13 23:29:05Z kumfert $
 // Description: a collection of common URL utility functions
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -79,13 +79,14 @@ public class URLUtilities {
     * to be a file reference and is expanded as such.
     */
    public static String expandURL(String url) {
+	  URL u;
       if ((url != null) && (url.length() > 0)) {
          try {
-            URL u = new URL(url);
+            u = new URL(url);
          } catch (MalformedURLException ex1) {
             File file = new File(url);
             try {
-               URL u = file.toURL();
+               u = file.toURL();
                url = u.toString();
             } catch (MalformedURLException ex2) {
             }
