@@ -1,7 +1,7 @@
 //
 // File:        Cookie.java
 // Package:     gov.llnl.babel.url.cookie
-// Revision:    @(#) $Id: Cookie.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: Cookie.java 4763 2005-07-26 17:32:19Z kumfert $
 // Description: A class to store a HTTP Cookie for a User Agent
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -49,6 +49,11 @@ import java.util.StringTokenizer;
  * specification (RFC 2616).
  */
 public class Cookie implements Serializable {
+   /**
+	* 
+	*/
+   private static final long serialVersionUID = 6608089316513898159L;
+
    /**
     * The cookie's name.
     */
@@ -374,7 +379,7 @@ public class Cookie implements Serializable {
     */
    public synchronized String getUrlComment()
    {
-      return d_comment;
+      return d_urlComment;
    }
 
    /**
@@ -711,7 +716,7 @@ public class Cookie implements Serializable {
     * Return <code>true</code> iff <code>port</code> is allowed to
     * send/receive this cookie.
     *
-    * @param <code>port</code> the port whose access should be checked.
+    * @param port the port whose access should be checked.
     * @return <code>true</code> means that <code>port</code> may receive
     *         the cookie; <code>false</code> means <code>port</code>
     *         must not receive the cookie.
