@@ -1,7 +1,7 @@
 //
 // File:        XMLTest.java
 // Package:     gov.llnl.babel.xml
-// Revision:    @(#) $Id: XMLTest.java 4434 2005-03-17 17:05:29Z epperly $
+// Revision:    @(#) $Id: XMLTest.java 4744 2005-07-13 23:29:05Z kumfert $
 // Description: unit test for the XML routines
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -84,9 +84,6 @@ public class XMLTest {
       Document document = null;
       if (argv.length == 1) {
          document = XMLUtilities.checkXML(xml);
-      } else {
-         String dtd_uri = URLUtilities.expandURL(argv[0]);
-         //         document = XMLUtilities.validateXML(dtd_uri, argv[1], xml);
       }
 
       if (document == null) {
