@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include "enums.h"
 #include "synch.h"
+#include "sidl_Exception.h"
 
 void declare_part(synch_RegOut tracker, int * part_no );
 void end_part(synch_RegOut tracker,  int part_no, 
@@ -25,82 +26,83 @@ int clearstack(int magicNumber) {
 #define MYASSERT( AAA ) \
   declare_part(tracker,  &part_no ); \
   magicNumber = clearstack(magicNumber); \
-  synch_RegOut_writeComment(tracker, #AAA); \
+  synch_RegOut_writeComment(tracker, #AAA, &exception);	SIDL_REPORT(exception);\
   if ( AAA ) result = synch_ResultType_PASS; \
   else result = synch_ResultType_FAIL;  \
+  SIDL_REPORT(exception);\
   end_part( tracker, part_no, result);
 
 #define MYXFAIL( AAA ) \
   declare_part( tracker, &part_no ); \
   magicNumber = clearstack(magicNumber); \
-  synch_RegOut_writeComment(tracker, #AAA); \
+  synch_RegOut_writeComment(tracker, #AAA, &exception); SIDL_REPORT(exception);\
   if ( AAA ) result = synch_ResultType_XPASS; \
   else result = synch_ResultType_XFAIL;  \
+  SIDL_REPORT(exception);\
   end_part( tracker, part_no, result);
 
 #define MYBROKEN( AAA ) \
   declare_part( tracker, &part_no ); \
-  synch_RegOut_writeComment(tracker, #AAA); \
+  synch_RegOut_writeComment(tracker, #AAA, &exception); SIDL_REPORT(exception);\
   end_part( tracker, part_no, sync_ResultType_FAIL );
 
 int main(int argc, char** argv)
 {
+  sidl_BaseInterface exception = NULL;
   enum synch_ResultType__enum result = synch_ResultType_PASS; 
   int magicNumber = 1;
   int part_no = 0;
-  synch_RegOut tracker = synch_RegOut__create();
+  synch_RegOut tracker = synch_RegOut__create(&exception); SIDL_REPORT(exception);
 
   (void) argc;
   (void) argv;
 
-  synch_RegOut_setExpectations(tracker, -1);
+  synch_RegOut_setExpectations(tracker, -1, &exception); SIDL_REPORT(exception);
   
   { /* undefined integer values */
     enum enums_color__enum out = -5;
     enum enums_color__enum inout = enums_color_green;
-    enums_colorwheel obj = enums_colorwheel__create();
+    enums_colorwheel obj = enums_colorwheel__create(&exception); SIDL_REPORT(exception);
  
-    MYASSERT( enums_colorwheel_returnback( obj ) == enums_color_violet );
-    MYASSERT( enums_colorwheel_passin( obj, enums_color_blue ) == TRUE );
-    MYASSERT( enums_colorwheel_passout( obj, &out ) == TRUE && out == enums_color_violet );
-    MYASSERT( enums_colorwheel_passinout( obj, &inout ) == TRUE && inout == enums_color_red );
+    MYASSERT( enums_colorwheel_returnback( obj, &exception) == enums_color_violet );
+    MYASSERT( enums_colorwheel_passin( obj, enums_color_blue , &exception) == TRUE );
+    MYASSERT( enums_colorwheel_passout( obj, &out, &exception ) == TRUE && out == enums_color_violet );
+    MYASSERT( enums_colorwheel_passinout( obj, &inout, &exception ) == TRUE && inout == enums_color_red );
     out = enums_color_red;
-    MYASSERT( enums_colorwheel_passeverywhere( obj, enums_color_blue, &out, &inout ) == enums_color_violet &&
+    MYASSERT( enums_colorwheel_passeverywhere( obj, enums_color_blue, &out, &inout, 
+					       &exception ) == enums_color_violet &&
 	      out == enums_color_violet && inout == enums_color_green );
-    
-    enums_colorwheel_deleteRef( obj );
+    enums_colorwheel_deleteRef( obj, &exception );SIDL_REPORT(exception);
   }
 
   { /* fully defined integer values */
     enum enums_car__enum out = -5;
     enum enums_car__enum inout = enums_car_ford;
-    enums_cartest obj = enums_cartest__create();
+    enums_cartest obj = enums_cartest__create( &exception);SIDL_REPORT(exception);
  
-    MYASSERT( enums_cartest_returnback( obj ) == enums_car_porsche );
-    MYASSERT( enums_cartest_passin( obj, enums_car_mercedes ) == TRUE );
-    MYASSERT( enums_cartest_passout( obj, &out ) == TRUE && out == enums_car_ford );
-    MYASSERT( enums_cartest_passinout( obj, &inout ) == TRUE && inout == enums_car_porsche );
+    MYASSERT( enums_cartest_returnback( obj , &exception) == enums_car_porsche );
+    MYASSERT( enums_cartest_passin( obj, enums_car_mercedes , &exception) == TRUE );
+    MYASSERT( enums_cartest_passout( obj, &out , &exception) == TRUE && out == enums_car_ford );
+    MYASSERT( enums_cartest_passinout( obj, &inout , &exception) == TRUE && inout == enums_car_porsche );
     out = -5;
-    MYASSERT( enums_cartest_passeverywhere( obj, enums_car_mercedes, &out, &inout ) == enums_car_porsche
+    MYASSERT( enums_cartest_passeverywhere( obj, enums_car_mercedes, &out, &inout , &exception) == enums_car_porsche
 	      && out == enums_car_ford && inout == enums_car_mercedes );
-    
-    enums_cartest_deleteRef( obj );
+    enums_cartest_deleteRef( obj , &exception);SIDL_REPORT(exception);
   }
 
   { /* partially defined integer values */
     enum enums_number__enum out=-5;
     enum enums_number__enum inout = enums_number_zero;
-    enums_numbertest obj = enums_numbertest__create();
- 
-    MYASSERT( enums_numbertest_returnback( obj ) == enums_number_notOne );
-    MYASSERT( enums_numbertest_passin( obj, enums_number_notZero ) == TRUE );
-    MYASSERT( enums_numbertest_passout( obj, &out ) == TRUE && out == enums_number_negOne );
-    MYASSERT( enums_numbertest_passinout( obj, &inout ) == TRUE && inout == enums_number_notZero );
+    enums_numbertest obj = enums_numbertest__create(&exception);
+    SIDL_REPORT(exception);
+    MYASSERT( enums_numbertest_returnback( obj , &exception) == enums_number_notOne );
+    MYASSERT( enums_numbertest_passin( obj, enums_number_notZero , &exception) == TRUE );
+    MYASSERT( enums_numbertest_passout( obj, &out , &exception) == TRUE && out == enums_number_negOne );
+    MYASSERT( enums_numbertest_passinout( obj, &inout , &exception) == TRUE && inout == enums_number_notZero );
     out = -5;
-    MYASSERT( enums_numbertest_passeverywhere( obj, enums_number_notZero, &out, &inout ) == enums_number_notOne &&
+    MYASSERT( enums_numbertest_passeverywhere( obj, enums_number_notZero, &out, &inout , &exception) == enums_number_notOne &&
 	      out == enums_number_negOne && inout == enums_number_zero );
-    
-    enums_numbertest_deleteRef( obj );
+    enums_numbertest_deleteRef( obj , &exception);SIDL_REPORT(exception);
   }
 
   {
@@ -109,7 +111,6 @@ int main(int argc, char** argv)
     struct enums_car__array *enumArray = enums_car__array_create1d(4);
     struct enums_car__array *slicedArray = NULL;
     struct enums_car__array *cpArray = NULL;
-
     MYASSERT( enumArray );
     enums_car__array_set1(enumArray, 0, enums_car_porsche);
     enums_car__array_set1(enumArray, 1, enums_car_ford);
@@ -132,18 +133,35 @@ int main(int argc, char** argv)
     enums_car__array_deleteRef(slicedArray);
     enums_car__array_deleteRef(enumArray);
   }
-  synch_RegOut_close(tracker);
-  synch_RegOut_deleteRef(tracker);
+  synch_RegOut_close(tracker, &exception);SIDL_REPORT(exception);
+  synch_RegOut_deleteRef(tracker, &exception);SIDL_REPORT(exception);
   return 0;
+ EXIT:
+  {
+    sidl_BaseInterface throwaway_exception = NULL;
+    if (tracker) {
+      sidl_BaseInterface exception2 = NULL;
+      synch_RegOut_forceFailure(tracker, &exception2);
+      if (exception2) {
+        puts("TEST_RESULT FAIL\n");
+        sidl_BaseInterface_deleteRef(exception2, &throwaway_exception);
+      }
+      synch_RegOut_deleteRef(tracker, &throwaway_exception);
+    }
+    sidl_BaseInterface_deleteRef(exception, &throwaway_exception);
+    return -1;
+  }
 }
 
 void declare_part( synch_RegOut tracker,  int * part_no ) {
-  synch_RegOut_startPart(tracker, ++(*part_no));
+  sidl_BaseInterface throwaway = NULL;
+  synch_RegOut_startPart(tracker, ++(*part_no), &throwaway);
 }
 
 void end_part( synch_RegOut tracker,
               int part_no, 
                enum synch_ResultType__enum result) 
 {
-  synch_RegOut_endPart(tracker, part_no, result);
+  sidl_BaseInterface throwaway = NULL;
+  synch_RegOut_endPart(tracker, part_no, result, &throwaway);
 }
