@@ -1,8 +1,8 @@
 //
 // File:	BaseArray.java
 // Package:	gov.llnl.sidl
-// Revision:	$Revision: 4434 $
-// Modified:	$Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:	$Revision: 5764 $
+// Modified:	$Date: 2006-12-05 14:15:10 -0800 (Tue, 05 Dec 2006) $
 // Description:	base array for the sidl Java runtime system
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -450,6 +450,13 @@ public abstract class BaseArray {
     public gov.llnl.sidl.BaseArray get() {
       return d_obj;
     }
+
+    /**
+     * Destroy the held array.
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
 } 
