@@ -1,8 +1,8 @@
 //
 // File:	BaseInterface.java
 // Package:	gov.llnl.sidl
-// Revision:	$Revision: 4434 $
-// Modified:	$Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Revision:	$Revision: 5219 $
+// Modified:	$Date: 2006-02-01 11:00:36 -0800 (Wed, 01 Feb 2006) $
 // Description:	base interface for the sidl Java runtime system
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -54,4 +54,26 @@ public interface BaseInterface {
    * base object class.
    */
   public abstract void addRef();
+
+  /**
+   * Returns the URL of this object
+   */
+  public abstract String _getURL();
+
+  /**
+   * Returns true if this object is remotely implemented, false otherwise
+   */
+  public abstract boolean _isRemote();
+
+  /**
+   * Returns true if this object is locally implemented, false otherwise
+   */
+  public abstract boolean _isLocal();
+
+  /**
+   * calls a method by name
+   */
+  //public abstract void _exec(String methodName, sidl.rmi.Call inArgs, sidl.rmi.Return outArgs);
+
+
 }
