@@ -1,9 +1,9 @@
 //
 // File:	Integer.java
 // Package:	sidl
-// Copyright:	(c) 2000-2001 The Regents of the University of California
-// Revision:	$Revision: 4434 $
-// Modified:	$Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Copyright:	(c) 2000-2001 Lawrence Livermore National Security, LLC
+// Revision:	$Revision: 6171 $
+// Modified:	$Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
 // Description:	holder and array classes for built-in data types
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -250,6 +250,13 @@ public class Integer {
     public Array get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
 }
@@ -447,6 +454,13 @@ public class Integer {
     public Array1 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
   }
 
@@ -652,6 +666,14 @@ public class Integer {
     public Array2 get() {
       return d_obj;
     }
+
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
     }
   }
 
@@ -865,6 +887,13 @@ public class Integer {
     public Array3 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
   }
@@ -1091,6 +1120,13 @@ public class Integer {
     public Array4 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
   }
@@ -1326,6 +1362,13 @@ public class Integer {
     public Array5 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
   }
@@ -1568,6 +1611,13 @@ public class Integer {
     public Array6 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
   }
 
@@ -1824,6 +1874,13 @@ public class Integer {
     public Array7 get() {
       return d_obj;
     }
+
+    /**
+     * Destroy held array
+     */
+    public void destroy() {
+      if (d_obj != null) { d_obj.destroy(); d_obj = null; }
+    }
   }
 
   }
