@@ -1,4 +1,4 @@
-# Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+# Copyright (c) 2000-2007, The Regents of the University of Calfornia.
 # Produced at the Lawrence Livermore National Laboratory.
 # Written by the Components Team <components@llnl.gov>
 # UCRL-CODE-2002-054
@@ -21,4 +21,4 @@
 # You should have recieved a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-all = [ "sidl", "sidlObjA" "sidlPyArrays" ]
+all = [ "sidl", "sidlObjA", "sidlStruct", "sidlPyArrays" ]
