@@ -1,8 +1,8 @@
 /*
  * File:        sidlPyArrays.h
- * Copyright:   (c) 2001 The Regents of the University of California
- * Revision:    @(#) $Revision: 4434 $
- * Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+ * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
+ * Revision:    @(#) $Revision: 6171 $
+ * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
  * Description: Runtime support for routines to convert arrays to/from Python
  *
  * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -904,7 +904,27 @@ static void **sidlPyArrays_API;
    sidl_array__extract_python_info_PROTO) \
  sidlPyArrays_API[sidl_array__extract_python_info_NUM])
 
-
+#ifdef HAVE_PTHREAD
+#include <pthread.h>
+#define import_SIDLPyArrays() \
+{ \
+  pthread_mutex_t __sidl_pyapi_mutex = PTHREAD_MUTEX_INITIALIZER; \
+  pthread_mutex_lock(&__sidl_pyapi_mutex); \
+  PyObject *module = PyImport_ImportModule("sidlPyArrays"); \
+  if (module != NULL) { \
+    PyObject *module_dict = PyModule_GetDict(module); \
+    PyObject *c_api_object = PyDict_GetItemString(module_dict, "_C_API"); \
+    if (PyCObject_Check(c_api_object)) { \
+       sidlPyArrays_API = (void **)PyCObject_AsVoidPtr(c_api_object); \
+    } \
+    else { fprintf(stderr, "babel: import_sidlPyArrays failed to lookup _C_API (%p).\n", c_api_object); }\
+    Py_DECREF(module); \
+  } \
+  else { fprintf(stderr, "babel: import_sidlPyArrays failed to import its module.\n"); }\
+  pthread_mutex_unlock(&__sidl_pyapi_mutex); \
+  pthread_mutex_destroy(&__sidl_pyapi_mutex); \
+}
+#else
 #define import_SIDLPyArrays() \
 { \
   PyObject *module = PyImport_ImportModule("sidlPyArrays"); \
@@ -919,6 +939,7 @@ static void **sidlPyArrays_API;
   } \
   else { fprintf(stderr, "babel: import_sidlPyArrays failed to import its module.\n"); }\
 }
+#endif
 
 #endif /* sidlPyArrays_MODULE */
 #endif /* included_sidlPyArrays_h */
