@@ -1,7 +1,7 @@
 /*
  * File:        sidlArray.h
- * Revision:    @(#) $Revision: 4434 $
- * Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+ * Revision:    @(#) $Revision: 5884 $
+ * Date:        $Date: 2007-02-28 11:43:31 -0800 (Wed, 28 Feb 2007) $
  * Description: Generic array data types & convenience macros
  *
  * Copyright (c) 2000-2004, The Regents of the University of Calfornia.
@@ -46,11 +46,12 @@
 
 enum sidl_array_ordering {
   sidl_general_order=0, /* this must be zero (i.e. a false value) */
-  sidl_column_major_order,
-  sidl_row_major_order
+  sidl_column_major_order=1,
+  sidl_row_major_order=2
 };
 
 enum sidl_array_type {
+  sidl_undefined_array = 0,
   /* these values must match values used in F77 & F90 too */
   sidl_bool_array      = 1,
   sidl_char_array      = 2,
@@ -538,6 +539,15 @@ sidl__array_isRowOrder(const struct sidl__array* array);
 int32_t
 sidl__array_type(const struct sidl__array* array);
 
+/**
+ * The following two functions are used for low level array reference
+ * count debugging. They are not intended for Babel end-users.
+ */
+void
+sidl__array_add(struct sidl__array * const array);
+
+void 
+sidl__array_remove(struct sidl__array * const array);
 #ifdef __cplusplus
 }
 #endif
